@@ -31,9 +31,11 @@
 #define __GST_DASH_DEMUX_H__
 
 #include <gst/gst.h>
+#include <gst/adaptivedemux/gstadaptivedemux.h>
 #include <gst/base/gstadapter.h>
 #include <gst/base/gstdataqueue.h>
 #include "gstmpdparser.h"
+#include "gstisoff.h"
 #include <gst/uridownloader/gsturidownloader.h>
 
 G_BEGIN_DECLS
@@ -56,41 +58,19 @@ typedef struct _GstDashDemuxClass GstDashDemuxClass;
 
 struct _GstDashDemuxStream
 {
-  GstPad *pad;
-
-  GstDashDemux *demux;
+  GstAdaptiveDemuxStream parent;
 
   gint index;
   GstActiveStream *active_stream;
 
-  GstCaps *input_caps;
-
-  GstFlowReturn last_ret;
-  GstClockTime position;
-  gboolean restart_download;
-
-  GstEvent *pending_segment;
-
-  gboolean stream_eos;
-  gboolean need_header;
-
-  /* Download task */
-  GMutex download_mutex;
-  GCond download_cond;
-  GstTask *download_task;
-  GRecMutex download_task_lock;
-
-  /* download tooling */
-  GstElement *src;
-  GstPad *src_srcpad;
-  GMutex fragment_download_lock;
-  GCond fragment_download_cond;
   GstMediaFragmentInfo current_fragment;
-  gboolean starting_fragment;
-  gint64 download_start_time;
-  gint64 download_total_time;
-  gint64 download_total_bytes;
-  gint current_download_rate;
+
+  /* index parsing */
+  GstSidxParser sidx_parser;
+  gsize sidx_current_remaining;
+  gint sidx_index;
+  gint64 sidx_base_offset;
+  GstClockTime pending_seek_ts;
 };
 
 /**
@@ -100,20 +80,10 @@ struct _GstDashDemuxStream
  */
 struct _GstDashDemux
 {
-  GstBin parent;
-  GstPad *sinkpad;
+  GstAdaptiveDemux parent;
 
-  gboolean have_group_id;
-  guint group_id;
-
-  GSList *streams;
   GSList *next_periods;
 
-  GstSegment segment;
-  GstClockTime timestamp_offset;
-
-  GstBuffer *manifest;
-  GstUriDownloader *downloader;
   GstMpdClient *client;         /* MPD client */
   GMutex client_lock;
 
@@ -122,18 +92,15 @@ struct _GstDashDemux
 
   /* Properties */
   GstClockTime max_buffering_time;      /* Maximum buffering time accumulated during playback */
-  gfloat bandwidth_usage;       /* Percentage of the available bandwidth to use       */
   guint64 max_bitrate;          /* max of bitrate supported by target decoder         */
 
-  gboolean cancelled;
-
-  /* Manifest update */
-  GstClockTime last_manifest_update;
+  gint n_audio_streams;
+  gint n_video_streams;
 };
 
 struct _GstDashDemuxClass
 {
-  GstBinClass parent_class;
+  GstAdaptiveDemuxClass parent_class;
 };
 
 GType gst_dash_demux_get_type (void);
