@@ -42,15 +42,13 @@
 #include <openssl/err.h>
 #include <openssl/ssl.h>
 
-#ifdef G_OS_WIN32
-#include <winsock2.h>
-#else
-#include <string.h>
-#include <errno.h>
-#endif
-
 GST_DEBUG_CATEGORY_STATIC (gst_dtls_connection_debug);
 #define GST_CAT_DEFAULT gst_dtls_connection_debug
+G_DEFINE_TYPE_WITH_CODE (GstDtlsConnection, gst_dtls_connection, G_TYPE_OBJECT,
+    GST_DEBUG_CATEGORY_INIT (gst_dtls_connection_debug, "dtlsconnection", 0,
+        "DTLS Connection"));
+
+#define GST_DTLS_CONNECTION_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE((obj), GST_TYPE_DTLS_CONNECTION, GstDtlsConnectionPrivate))
 
 #define SRTP_KEY_LEN 16
 #define SRTP_SALT_LEN 14
@@ -69,7 +67,6 @@ enum
 {
   PROP_0,
   PROP_AGENT,
-  PROP_CONNECTION_STATE,
   NUM_PROPERTIES
 };
 
@@ -77,6 +74,7 @@ static GParamSpec *properties[NUM_PROPERTIES];
 
 static int connection_ex_index;
 
+static GstClock *system_clock;
 static void handle_timeout (gpointer data, gpointer user_data);
 
 struct _GstDtlsConnectionPrivate
@@ -88,42 +86,25 @@ struct _GstDtlsConnectionPrivate
   gboolean is_alive;
   gboolean keys_exported;
 
-  GstDtlsConnectionState connection_state;
-  gboolean sent_close_notify;
-  gboolean received_close_notify;
-
   GMutex mutex;
   GCond condition;
   gpointer bio_buffer;
   gint bio_buffer_len;
   gint bio_buffer_offset;
 
-  GstDtlsConnectionSendCallback send_callback;
-  gpointer send_callback_user_data;
-  GDestroyNotify send_callback_destroy_notify;
-  GstFlowReturn syscall_flow_return;
+  GClosure *send_closure;
 
   gboolean timeout_pending;
   GThreadPool *thread_pool;
 };
 
-G_DEFINE_TYPE_WITH_CODE (GstDtlsConnection, gst_dtls_connection,
-    GST_TYPE_OBJECT, G_ADD_PRIVATE (GstDtlsConnection)
-    GST_DEBUG_CATEGORY_INIT (gst_dtls_connection_debug, "dtlsconnection", 0,
-        "DTLS Connection"));
-
 static void gst_dtls_connection_finalize (GObject * gobject);
 static void gst_dtls_connection_set_property (GObject *, guint prop_id,
     const GValue *, GParamSpec *);
-static void gst_dtls_connection_get_property (GObject *, guint prop_id,
-    GValue *, GParamSpec *);
 
 static void log_state (GstDtlsConnection *, const gchar * str);
-static gboolean export_srtp_keys (GstDtlsConnection *, GError ** err);
-static GstFlowReturn openssl_poll (GstDtlsConnection *, gboolean * notify_state,
-    GError ** err);
-static GstFlowReturn handle_error (GstDtlsConnection * self, int ret,
-    GstResourceError error_type, gboolean * notify_state, GError ** err);
+static void export_srtp_keys (GstDtlsConnection *);
+static void openssl_poll (GstDtlsConnection *);
 static int openssl_verify_callback (int preverify_ok,
     X509_STORE_CTX * x509_ctx);
 
@@ -139,8 +120,9 @@ gst_dtls_connection_class_init (GstDtlsConnectionClass * klass)
 {
   GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
 
+  g_type_class_add_private (klass, sizeof (GstDtlsConnectionPrivate));
+
   gobject_class->set_property = gst_dtls_connection_set_property;
-  gobject_class->get_property = gst_dtls_connection_get_property;
 
   connection_ex_index =
       SSL_get_ex_new_index (0, (gpointer) "gstdtlsagent connection index", NULL,
@@ -148,17 +130,20 @@ gst_dtls_connection_class_init (GstDtlsConnectionClass * klass)
 
   signals[SIGNAL_ON_DECODER_KEY] =
       g_signal_new ("on-decoder-key", G_TYPE_FROM_CLASS (klass),
-      G_SIGNAL_RUN_LAST, 0, NULL, NULL, NULL,
-      G_TYPE_NONE, 3, G_TYPE_POINTER, G_TYPE_UINT, G_TYPE_UINT);
+      G_SIGNAL_RUN_LAST, 0, NULL, NULL,
+      g_cclosure_marshal_generic, G_TYPE_NONE, 3,
+      G_TYPE_POINTER, G_TYPE_UINT, G_TYPE_UINT);
 
   signals[SIGNAL_ON_ENCODER_KEY] =
       g_signal_new ("on-encoder-key", G_TYPE_FROM_CLASS (klass),
-      G_SIGNAL_RUN_LAST, 0, NULL, NULL, NULL,
-      G_TYPE_NONE, 3, G_TYPE_POINTER, G_TYPE_UINT, G_TYPE_UINT);
+      G_SIGNAL_RUN_LAST, 0, NULL, NULL,
+      g_cclosure_marshal_generic, G_TYPE_NONE, 3,
+      G_TYPE_POINTER, G_TYPE_UINT, G_TYPE_UINT);
 
   signals[SIGNAL_ON_PEER_CERTIFICATE] =
       g_signal_new ("on-peer-certificate", G_TYPE_FROM_CLASS (klass),
-      G_SIGNAL_RUN_LAST, 0, NULL, NULL, NULL, G_TYPE_BOOLEAN, 1, G_TYPE_STRING);
+      G_SIGNAL_RUN_LAST, 0, NULL, NULL,
+      g_cclosure_marshal_generic, G_TYPE_BOOLEAN, 1, G_TYPE_STRING);
 
   properties[PROP_AGENT] =
       g_param_spec_object ("agent",
@@ -167,30 +152,26 @@ gst_dtls_connection_class_init (GstDtlsConnectionClass * klass)
       GST_TYPE_DTLS_AGENT,
       G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY | G_PARAM_STATIC_STRINGS);
 
-  properties[PROP_CONNECTION_STATE] =
-      g_param_spec_enum ("connection-state",
-      "Connection State",
-      "Current connection state",
-      GST_DTLS_TYPE_CONNECTION_STATE,
-      GST_DTLS_CONNECTION_STATE_NEW, G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
-
   g_object_class_install_properties (gobject_class, NUM_PROPERTIES, properties);
 
   _gst_dtls_init_openssl ();
 
   gobject_class->finalize = gst_dtls_connection_finalize;
+
+  system_clock = gst_system_clock_obtain ();
 }
 
 static void
 gst_dtls_connection_init (GstDtlsConnection * self)
 {
-  GstDtlsConnectionPrivate *priv;
-
-  self->priv = priv = gst_dtls_connection_get_instance_private (self);
+  GstDtlsConnectionPrivate *priv = GST_DTLS_CONNECTION_GET_PRIVATE (self);
+  self->priv = priv;
 
   priv->ssl = NULL;
   priv->bio = NULL;
 
+  priv->send_closure = NULL;
+
   priv->is_client = FALSE;
   priv->is_alive = TRUE;
   priv->keys_exported = FALSE;
@@ -222,8 +203,10 @@ gst_dtls_connection_finalize (GObject * gobject)
   SSL_free (priv->ssl);
   priv->ssl = NULL;
 
-  if (priv->send_callback_destroy_notify)
-    priv->send_callback_destroy_notify (priv->send_callback_user_data);
+  if (priv->send_closure) {
+    g_closure_unref (priv->send_closure);
+    priv->send_closure = NULL;
+  }
 
   g_mutex_clear (&priv->mutex);
   g_cond_clear (&priv->condition);
@@ -233,38 +216,6 @@ gst_dtls_connection_finalize (GObject * gobject)
   G_OBJECT_CLASS (gst_dtls_connection_parent_class)->finalize (gobject);
 }
 
-#if OPENSSL_VERSION_NUMBER < 0x10100001L
-static void
-BIO_set_data (BIO * bio, void *ptr)
-{
-  bio->ptr = ptr;
-}
-
-static void *
-BIO_get_data (BIO * bio)
-{
-  return bio->ptr;
-}
-
-static void
-BIO_set_shutdown (BIO * bio, int shutdown)
-{
-  bio->shutdown = shutdown;
-}
-
-static void
-BIO_set_init (BIO * bio, int init)
-{
-  bio->init = init;
-}
-
-static X509 *
-X509_STORE_CTX_get0_cert (X509_STORE_CTX * ctx)
-{
-  return ctx->cert;
-}
-#endif
-
 static void
 gst_dtls_connection_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec)
@@ -288,7 +239,7 @@ gst_dtls_connection_set_property (GObject * object, guint prop_id,
       priv->bio = BIO_new (BIO_s_gst_dtls_connection ());
       g_return_if_fail (priv->bio);
 
-      BIO_set_data (priv->bio, self);
+      priv->bio->ptr = self;
       SSL_set_bio (priv->ssl, priv->bio, priv->bio);
 
       SSL_set_verify (priv->ssl,
@@ -303,37 +254,16 @@ gst_dtls_connection_set_property (GObject * object, guint prop_id,
   }
 }
 
-static void
-gst_dtls_connection_get_property (GObject * object, guint prop_id,
-    GValue * value, GParamSpec * pspec)
-{
-  GstDtlsConnection *self = GST_DTLS_CONNECTION (object);
-  GstDtlsConnectionPrivate *priv = self->priv;
-
-  switch (prop_id) {
-    case PROP_CONNECTION_STATE:
-      g_mutex_lock (&priv->mutex);
-      g_value_set_enum (value, priv->connection_state);
-      g_mutex_unlock (&priv->mutex);
-      break;
-    default:
-      G_OBJECT_WARN_INVALID_PROPERTY_ID (self, prop_id, pspec);
-  }
-}
-
-gboolean
-gst_dtls_connection_start (GstDtlsConnection * self, gboolean is_client,
-    GError ** err)
+void
+gst_dtls_connection_start (GstDtlsConnection * self, gboolean is_client)
 {
   GstDtlsConnectionPrivate *priv;
-  gboolean ret;
-  gboolean notify_state = FALSE;
 
   priv = self->priv;
 
-  g_return_val_if_fail (priv->send_callback, FALSE);
-  g_return_val_if_fail (priv->ssl, FALSE);
-  g_return_val_if_fail (priv->bio, FALSE);
+  g_return_if_fail (priv->send_closure);
+  g_return_if_fail (priv->ssl);
+  g_return_if_fail (priv->bio);
 
   GST_TRACE_OBJECT (self, "locking @ start");
   g_mutex_lock (&priv->mutex);
@@ -345,43 +275,20 @@ gst_dtls_connection_start (GstDtlsConnection * self, gboolean is_client,
   priv->bio_buffer_offset = 0;
   priv->keys_exported = FALSE;
 
-  priv->sent_close_notify = FALSE;
-  priv->received_close_notify = FALSE;
-
-  /* Client immediately starts connecting, the server waits for a client to
-   * start the handshake process */
   priv->is_client = is_client;
   if (priv->is_client) {
-    priv->connection_state = GST_DTLS_CONNECTION_STATE_CONNECTING;
-    notify_state = TRUE;
     SSL_set_connect_state (priv->ssl);
   } else {
-    if (priv->connection_state != GST_DTLS_CONNECTION_STATE_NEW) {
-      priv->connection_state = GST_DTLS_CONNECTION_STATE_NEW;
-      notify_state = TRUE;
-    }
     SSL_set_accept_state (priv->ssl);
   }
   log_state (self, "initial state set");
 
-  ret = openssl_poll (self, &notify_state, err);
-  if (ret == GST_FLOW_EOS && err) {
-    *err =
-        g_error_new_literal (GST_RESOURCE_ERROR, GST_RESOURCE_ERROR_OPEN_WRITE,
-        "Connection closed");
-  }
+  openssl_poll (self);
 
   log_state (self, "first poll done");
 
   GST_TRACE_OBJECT (self, "unlocking @ start");
   g_mutex_unlock (&priv->mutex);
-
-  if (notify_state) {
-    g_object_notify_by_pspec (G_OBJECT (self),
-        properties[PROP_CONNECTION_STATE]);
-  }
-
-  return ret == GST_FLOW_OK;
 }
 
 static void
@@ -390,7 +297,6 @@ handle_timeout (gpointer data, gpointer user_data)
   GstDtlsConnection *self = user_data;
   GstDtlsConnectionPrivate *priv;
   gint ret;
-  gboolean notify_state = FALSE;
 
   priv = self->priv;
 
@@ -406,16 +312,11 @@ handle_timeout (gpointer data, gpointer user_data)
       GST_WARNING_OBJECT (self, "handling timeout failed");
     } else if (ret > 0) {
       log_state (self, "handling timeout before poll");
-      openssl_poll (self, &notify_state, NULL);
+      openssl_poll (self);
       log_state (self, "handling timeout after poll");
     }
   }
   g_mutex_unlock (&priv->mutex);
-
-  if (notify_state) {
-    g_object_notify_by_pspec (G_OBJECT (self),
-        properties[PROP_CONNECTION_STATE]);
-  }
 }
 
 static gboolean
@@ -449,29 +350,21 @@ gst_dtls_connection_check_timeout_locked (GstDtlsConnection * self)
   priv = self->priv;
 
   if (DTLSv1_get_timeout (priv->ssl, &timeout)) {
-    wait_time = ((gint64) timeout.tv_sec) * G_USEC_PER_SEC + timeout.tv_usec;
+    wait_time = timeout.tv_sec * G_USEC_PER_SEC + timeout.tv_usec;
 
     GST_DEBUG_OBJECT (self, "waiting for %" G_GINT64_FORMAT " usec", wait_time);
     if (wait_time) {
-      GstClock *system_clock = gst_system_clock_obtain ();
       GstClockID clock_id;
-#ifndef G_DISABLE_ASSERT
       GstClockReturn clock_return;
-#endif
 
       end_time = gst_clock_get_time (system_clock) + wait_time * GST_USECOND;
 
       clock_id = gst_clock_new_single_shot_id (system_clock, end_time);
-#ifndef G_DISABLE_ASSERT
       clock_return =
-#else
-      (void)
-#endif
           gst_clock_id_wait_async (clock_id, schedule_timeout_handling,
           g_object_ref (self), (GDestroyNotify) g_object_unref);
       g_assert (clock_return == GST_CLOCK_OK);
       gst_clock_id_unref (clock_id);
-      gst_object_unref (system_clock);
     } else {
       if (self->priv->is_alive && !self->priv->timeout_pending) {
         self->priv->timeout_pending = TRUE;
@@ -506,8 +399,6 @@ gst_dtls_connection_check_timeout (GstDtlsConnection * self)
 void
 gst_dtls_connection_stop (GstDtlsConnection * self)
 {
-  gboolean notify_state = FALSE;
-
   g_return_if_fail (GST_IS_DTLS_CONNECTION (self));
   g_return_if_fail (self->priv->ssl);
   g_return_if_fail (self->priv->bio);
@@ -519,11 +410,6 @@ gst_dtls_connection_stop (GstDtlsConnection * self)
   GST_TRACE_OBJECT (self, "locked @ stop");
 
   self->priv->is_alive = FALSE;
-  if (self->priv->connection_state != GST_DTLS_CONNECTION_STATE_FAILED
-      && self->priv->connection_state != GST_DTLS_CONNECTION_STATE_CLOSED) {
-    self->priv->connection_state = GST_DTLS_CONNECTION_STATE_CLOSED;
-    notify_state = TRUE;
-  }
   GST_TRACE_OBJECT (self, "signaling @ stop");
   g_cond_signal (&self->priv->condition);
   GST_TRACE_OBJECT (self, "signaled @ stop");
@@ -532,18 +418,11 @@ gst_dtls_connection_stop (GstDtlsConnection * self)
   g_mutex_unlock (&self->priv->mutex);
 
   GST_DEBUG_OBJECT (self, "stopped connection");
-
-  if (notify_state) {
-    g_object_notify_by_pspec (G_OBJECT (self),
-        properties[PROP_CONNECTION_STATE]);
-  }
 }
 
 void
 gst_dtls_connection_close (GstDtlsConnection * self)
 {
-  gboolean notify_state = FALSE;
-
   g_return_if_fail (GST_IS_DTLS_CONNECTION (self));
   g_return_if_fail (self->priv->ssl);
   g_return_if_fail (self->priv->bio);
@@ -559,64 +438,41 @@ gst_dtls_connection_close (GstDtlsConnection * self)
     g_cond_signal (&self->priv->condition);
   }
 
-  if (self->priv->connection_state != GST_DTLS_CONNECTION_STATE_FAILED
-      && self->priv->connection_state != GST_DTLS_CONNECTION_STATE_CLOSED) {
-    self->priv->connection_state = GST_DTLS_CONNECTION_STATE_CLOSED;
-    notify_state = TRUE;
-  }
-
   GST_TRACE_OBJECT (self, "unlocking @ close");
   g_mutex_unlock (&self->priv->mutex);
 
   GST_DEBUG_OBJECT (self, "closed connection");
-
-  if (notify_state) {
-    g_object_notify_by_pspec (G_OBJECT (self),
-        properties[PROP_CONNECTION_STATE]);
-  }
 }
 
 void
 gst_dtls_connection_set_send_callback (GstDtlsConnection * self,
-    GstDtlsConnectionSendCallback callback, gpointer user_data,
-    GDestroyNotify destroy_notify)
+    GClosure * closure)
 {
-  GstDtlsConnectionPrivate *priv;
-
   g_return_if_fail (GST_IS_DTLS_CONNECTION (self));
 
-  priv = self->priv;
-
   GST_TRACE_OBJECT (self, "locking @ set_send_callback");
-  g_mutex_lock (&priv->mutex);
+  g_mutex_lock (&self->priv->mutex);
   GST_TRACE_OBJECT (self, "locked @ set_send_callback");
 
-  if (priv->send_callback_destroy_notify)
-    priv->send_callback_destroy_notify (priv->send_callback_user_data);
-  priv->send_callback = callback;
-  priv->send_callback_user_data = user_data;
-  priv->send_callback_destroy_notify = destroy_notify;
+  if (self->priv->send_closure) {
+    g_closure_unref (self->priv->send_closure);
+    self->priv->send_closure = NULL;
+  }
+  self->priv->send_closure = closure;
+
+  if (closure && G_CLOSURE_NEEDS_MARSHAL (closure)) {
+    g_closure_set_marshal (closure, g_cclosure_marshal_generic);
+  }
 
   GST_TRACE_OBJECT (self, "unlocking @ set_send_callback");
-  g_mutex_unlock (&priv->mutex);
+  g_mutex_unlock (&self->priv->mutex);
 }
 
-void
-gst_dtls_connection_set_flow_return (GstDtlsConnection * self,
-    GstFlowReturn flow_ret)
+gint
+gst_dtls_connection_process (GstDtlsConnection * self, gpointer data, gint len)
 {
-  g_return_if_fail (GST_IS_DTLS_CONNECTION (self));
-  self->priv->syscall_flow_return = flow_ret;
-}
-
-GstFlowReturn
-gst_dtls_connection_process (GstDtlsConnection * self, gpointer data, gsize len,
-    gsize * written, GError ** err)
-{
-  GstFlowReturn flow_ret = GST_FLOW_OK;
   GstDtlsConnectionPrivate *priv;
-  int ret;
-  gboolean notify_state = FALSE;
+  gint result;
 
   g_return_val_if_fail (GST_IS_DTLS_CONNECTION (self), 0);
   g_return_val_if_fail (self->priv->ssl, 0);
@@ -628,23 +484,6 @@ gst_dtls_connection_process (GstDtlsConnection * self, gpointer data, gsize len,
   g_mutex_lock (&priv->mutex);
   GST_TRACE_OBJECT (self, "locked @ process");
 
-  if (self->priv->received_close_notify
-      || self->priv->connection_state == GST_DTLS_CONNECTION_STATE_CLOSED) {
-    GST_DEBUG_OBJECT (self, "Already received close_notify");
-    g_mutex_unlock (&priv->mutex);
-    return GST_FLOW_EOS;
-  }
-
-  if (self->priv->connection_state == GST_DTLS_CONNECTION_STATE_FAILED) {
-    GST_ERROR_OBJECT (self, "Had a fatal error before");
-    g_mutex_unlock (&priv->mutex);
-    if (err)
-      *err =
-          g_error_new_literal (GST_RESOURCE_ERROR, GST_RESOURCE_ERROR_READ,
-          "Had fatal error before");
-    return GST_FLOW_ERROR;
-  }
-
   g_warn_if_fail (!priv->bio_buffer);
 
   priv->bio_buffer = data;
@@ -654,84 +493,30 @@ gst_dtls_connection_process (GstDtlsConnection * self, gpointer data, gsize len,
   log_state (self, "process start");
 
   if (SSL_want_write (priv->ssl)) {
-    flow_ret = openssl_poll (self, &notify_state, err);
+    openssl_poll (self);
     log_state (self, "process want write, after poll");
-    if (flow_ret != GST_FLOW_OK) {
-      g_mutex_unlock (&priv->mutex);
-      return flow_ret;
-    }
   }
 
-  /* If we're a server and were in new state then by receiving the first data
-   * we would start the connection process */
-  if (!priv->is_client) {
-    if (self->priv->connection_state == GST_DTLS_CONNECTION_STATE_NEW) {
-      priv->connection_state = GST_DTLS_CONNECTION_STATE_CONNECTING;
-      notify_state = TRUE;
-    }
-  }
-
-  ret = SSL_read (priv->ssl, data, len);
-  *written = ret >= 0 ? ret : 0;
-  GST_DEBUG_OBJECT (self, "read result: %d", ret);
-
-  flow_ret =
-      handle_error (self, ret, GST_RESOURCE_ERROR_READ, &notify_state, err);
-  if (flow_ret == GST_FLOW_EOS) {
-    self->priv->received_close_notify = TRUE;
-    if (self->priv->connection_state != GST_DTLS_CONNECTION_STATE_FAILED
-        && self->priv->connection_state != GST_DTLS_CONNECTION_STATE_CLOSED) {
-      self->priv->connection_state = GST_DTLS_CONNECTION_STATE_CLOSED;
-      notify_state = TRUE;
-    }
-    /* Notify about the connection being properly closed now if both
-     * sides did so */
-    if (self->priv->sent_close_notify && self->priv->send_callback)
-      self->priv->send_callback (self, NULL, 0, NULL);
-
-    g_mutex_unlock (&priv->mutex);
-
-    if (notify_state) {
-      g_object_notify_by_pspec (G_OBJECT (self),
-          properties[PROP_CONNECTION_STATE]);
-    }
-
-    return flow_ret;
-  } else if (flow_ret != GST_FLOW_OK) {
-    g_mutex_unlock (&priv->mutex);
-
-    if (notify_state) {
-      g_object_notify_by_pspec (G_OBJECT (self),
-          properties[PROP_CONNECTION_STATE]);
-    }
-
-    return flow_ret;
-  }
+  result = SSL_read (priv->ssl, data, len);
 
   log_state (self, "process after read");
 
-  flow_ret = openssl_poll (self, &notify_state, err);
+  openssl_poll (self);
 
   log_state (self, "process after poll");
 
+  GST_DEBUG_OBJECT (self, "read result: %d", result);
+
   GST_TRACE_OBJECT (self, "unlocking @ process");
   g_mutex_unlock (&priv->mutex);
 
-  if (notify_state) {
-    g_object_notify_by_pspec (G_OBJECT (self),
-        properties[PROP_CONNECTION_STATE]);
-  }
-
-  return flow_ret;
+  return result;
 }
 
-GstFlowReturn
-gst_dtls_connection_send (GstDtlsConnection * self, gconstpointer data,
-    gsize len, gsize * written, GError ** err)
+gint
+gst_dtls_connection_send (GstDtlsConnection * self, gpointer data, gint len)
 {
-  GstFlowReturn flow_ret;
   int ret = 0;
-  gboolean notify_state = FALSE;
 
   g_return_val_if_fail (GST_IS_DTLS_CONNECTION (self), 0);
 
@@ -742,78 +527,20 @@ gst_dtls_connection_send (GstDtlsConnection * self, gconstpointer data,
   g_mutex_lock (&self->priv->mutex);
   GST_TRACE_OBJECT (self, "locked @ send");
 
-  if (self->priv->connection_state == GST_DTLS_CONNECTION_STATE_FAILED) {
-    GST_ERROR_OBJECT (self, "Had a fatal error before");
-    g_mutex_unlock (&self->priv->mutex);
-    if (err)
-      *err =
-          g_error_new_literal (GST_RESOURCE_ERROR, GST_RESOURCE_ERROR_WRITE,
-          "Had fatal error before");
-    return GST_FLOW_ERROR;
-  }
-
-  if (self->priv->sent_close_notify) {
-    len = 0;
-    GST_DEBUG_OBJECT (self, "Not sending new data after close_notify");
-  }
-
-  if (len == 0) {
-    if (written)
-      *written = 0;
-    GST_DEBUG_OBJECT (self, "Sending close_notify");
-    ret = SSL_shutdown (self->priv->ssl);
-    if (self->priv->connection_state != GST_DTLS_CONNECTION_STATE_CLOSED &&
-        self->priv->connection_state != GST_DTLS_CONNECTION_STATE_FAILED) {
-      self->priv->connection_state = GST_DTLS_CONNECTION_STATE_CLOSED;
-      notify_state = TRUE;
-    }
-    if (ret == 1) {
-      GST_LOG_OBJECT (self, "received peer close_notify already");
-      self->priv->received_close_notify = TRUE;
-      flow_ret = GST_FLOW_EOS;
-    } else if (ret == 0) {
-      GST_LOG_OBJECT (self, "did not receive peer close_notify yet");
-      flow_ret = GST_FLOW_OK;
-    } else {
-      flow_ret =
-          handle_error (self, ret, GST_RESOURCE_ERROR_WRITE, &notify_state,
-          err);
-    }
-  } else if (SSL_is_init_finished (self->priv->ssl)) {
-    GST_DEBUG_OBJECT (self, "sending data of %" G_GSIZE_FORMAT " B", len);
+  if (SSL_is_init_finished (self->priv->ssl)) {
     ret = SSL_write (self->priv->ssl, data, len);
-    if (ret <= 0) {
-      if (written)
-        *written = 0;
-      flow_ret =
-          handle_error (self, ret, GST_RESOURCE_ERROR_WRITE, &notify_state,
-          err);
-    } else {
-      if (written)
-        *written = ret;
-      flow_ret = GST_FLOW_OK;
-    }
+    GST_DEBUG_OBJECT (self, "data sent: input was %d B, output is %d B", len,
+        ret);
   } else {
-    if (written)
-      *written = ret;
     GST_WARNING_OBJECT (self,
         "tried to send data before handshake was complete");
-    if (err)
-      *err =
-          g_error_new_literal (GST_RESOURCE_ERROR, GST_RESOURCE_ERROR_WRITE,
-          "Tried to send data before handshake was complete");
-    flow_ret = GST_FLOW_ERROR;
+    ret = 0;
   }
 
   GST_TRACE_OBJECT (self, "unlocking @ send");
   g_mutex_unlock (&self->priv->mutex);
 
-  if (notify_state) {
-    g_object_notify_by_pspec (G_OBJECT (self),
-        properties[PROP_CONNECTION_STATE]);
-  }
-
-  return flow_ret;
+  return ret;
 }
 
 /*
@@ -832,15 +559,14 @@ log_state (GstDtlsConnection * self, const gchar * str)
   GstDtlsConnectionPrivate *priv = self->priv;
   guint states = 0;
 
-  states |= (!!SSL_is_init_finished (priv->ssl) << 0);
-  states |= (!!SSL_in_init (priv->ssl) << 4);
-  states |= (!!SSL_in_before (priv->ssl) << 8);
-  states |= (!!SSL_in_connect_init (priv->ssl) << 12);
-  states |= (!!SSL_in_accept_init (priv->ssl) << 16);
-  states |= (!!SSL_want_write (priv->ssl) << 20);
-  states |= (!!SSL_want_read (priv->ssl) << 24);
+  states |= (! !SSL_is_init_finished (priv->ssl) << 0);
+  states |= (! !SSL_in_init (priv->ssl) << 4);
+  states |= (! !SSL_in_before (priv->ssl) << 8);
+  states |= (! !SSL_in_connect_init (priv->ssl) << 12);
+  states |= (! !SSL_in_accept_init (priv->ssl) << 16);
+  states |= (! !SSL_want_write (priv->ssl) << 20);
+  states |= (! !SSL_want_read (priv->ssl) << 24);
 
-#if OPENSSL_VERSION_NUMBER < 0x10100001L
   GST_LOG_OBJECT (self, "%s: role=%s buf=(%d,%p:%d/%d) %x|%x %s",
       str,
       priv->is_client ? "client" : "server",
@@ -849,19 +575,10 @@ log_state (GstDtlsConnection * self, const gchar * str)
       priv->bio_buffer_offset,
       priv->bio_buffer_len,
       states, SSL_get_state (priv->ssl), SSL_state_string_long (priv->ssl));
-#else
-  GST_LOG_OBJECT (self, "%s: role=%s buf=(%p:%d/%d) %x|%x %s",
-      str,
-      priv->is_client ? "client" : "server",
-      priv->bio_buffer,
-      priv->bio_buffer_offset,
-      priv->bio_buffer_len,
-      states, SSL_get_state (priv->ssl), SSL_state_string_long (priv->ssl));
-#endif
 }
 
-static gboolean
-export_srtp_keys (GstDtlsConnection * self, GError ** err)
+static void
+export_srtp_keys (GstDtlsConnection * self)
 {
   typedef struct
   {
@@ -899,26 +616,12 @@ export_srtp_keys (GstDtlsConnection * self, GError ** err)
       NULL, 0, 0);
 
   if (!success) {
-    GST_WARNING_OBJECT (self, "Failed to export SRTP keys");
-    if (err)
-      *err =
-          g_error_new_literal (GST_RESOURCE_ERROR, GST_RESOURCE_ERROR_READ,
-          "Failed to export SRTP keys");
-    return FALSE;
+    GST_WARNING_OBJECT (self, "failed to export srtp keys");
+    return;
   }
 
   profile = SSL_get_selected_srtp_profile (self->priv->ssl);
 
-  if (!profile) {
-    GST_WARNING_OBJECT (self,
-        "No SRTP capabilities negotiated during handshake");
-    if (err)
-      *err =
-          g_error_new_literal (GST_RESOURCE_ERROR, GST_RESOURCE_ERROR_READ,
-          "No SRTP capabilities negotiated during handshake");
-    return FALSE;
-  }
-
   GST_INFO_OBJECT (self, "keys received, profile is %s", profile->name);
 
   switch (profile->id) {
@@ -931,13 +634,8 @@ export_srtp_keys (GstDtlsConnection * self, GError ** err)
       auth = GST_DTLS_SRTP_AUTH_HMAC_SHA1_32;
       break;
     default:
-      GST_WARNING_OBJECT (self,
-          "Invalid/unsupported crypto suite set by handshake");
-      if (err)
-        *err =
-            g_error_new_literal (GST_RESOURCE_ERROR, GST_RESOURCE_ERROR_READ,
-            "Invalid/unsupported crypto suite set by handshake");
-      return FALSE;
+      GST_WARNING_OBJECT (self, "invalid crypto suite set by handshake");
+      goto beach;
   }
 
   client_key.key = exported_keys.client_key;
@@ -957,140 +655,67 @@ export_srtp_keys (GstDtlsConnection * self, GError ** err)
         auth);
   }
 
+beach:
   self->priv->keys_exported = TRUE;
-
-  return TRUE;
 }
 
-static int
-ssl_warn_cb (const char *str, size_t len, void *u)
-{
-  GstDtlsConnection *self = u;
-  GST_WARNING_OBJECT (self, "ssl error: %s", str);
-  return 0;
-}
-
-static int
-ssl_err_cb (const char *str, size_t len, void *u)
-{
-  GstDtlsConnection *self = u;
-  GST_ERROR_OBJECT (self, "ssl error: %s", str);
-  return 0;
-}
-
-static GstFlowReturn
-handle_error (GstDtlsConnection * self, int ret, GstResourceError error_type,
-    gboolean * notify_state, GError ** err)
+static void
+openssl_poll (GstDtlsConnection * self)
 {
+  int ret;
+  char buf[512];
   int error;
 
+  log_state (self, "poll: before handshake");
+
+  ret = SSL_do_handshake (self->priv->ssl);
+
+  log_state (self, "poll: after handshake");
+
+  if (ret == 1) {
+    if (!self->priv->keys_exported) {
+      GST_INFO_OBJECT (self,
+          "handshake just completed successfully, exporting keys");
+      export_srtp_keys (self);
+    } else {
+      GST_INFO_OBJECT (self, "handshake is completed");
+    }
+    return;
+  } else {
+    if (ret == 0) {
+      GST_DEBUG_OBJECT (self, "do_handshake encountered EOF");
+    } else if (ret == -1) {
+      GST_WARNING_OBJECT (self, "do_handshake encountered BIO error");
+    } else {
+      GST_DEBUG_OBJECT (self, "do_handshake returned %d", ret);
+    }
+  }
+
   error = SSL_get_error (self->priv->ssl, ret);
 
   switch (error) {
     case SSL_ERROR_NONE:
-      GST_TRACE_OBJECT (self, "No error");
-      return GST_FLOW_OK;
+      GST_WARNING_OBJECT (self, "no error, handshake should be done");
+      break;
     case SSL_ERROR_SSL:
-      GST_ERROR_OBJECT (self, "Fatal SSL error");
-      if (self->priv->connection_state != GST_DTLS_CONNECTION_STATE_FAILED) {
-        self->priv->connection_state = GST_DTLS_CONNECTION_STATE_FAILED;
-        *notify_state = TRUE;
-      }
-      ERR_print_errors_cb (ssl_err_cb, self);
-      if (err)
-        *err =
-            g_error_new_literal (GST_RESOURCE_ERROR, error_type,
-            "Fatal SSL error");
-      return GST_FLOW_ERROR;
-    case SSL_ERROR_ZERO_RETURN:
-      GST_LOG_OBJECT (self, "Connection was closed");
-      return GST_FLOW_EOS;
+      GST_LOG_OBJECT (self, "SSL error %d: %s", error,
+          ERR_error_string (ERR_get_error (), buf));
+      break;
     case SSL_ERROR_WANT_READ:
       GST_LOG_OBJECT (self, "SSL wants read");
-      return GST_FLOW_OK;
+      break;
     case SSL_ERROR_WANT_WRITE:
       GST_LOG_OBJECT (self, "SSL wants write");
-      return GST_FLOW_OK;
+      break;
     case SSL_ERROR_SYSCALL:{
-      GstFlowReturn rc = GST_FLOW_OK;
-      /* OpenSSL shouldn't be making real system calls, so we can safely
-       * ignore syscall errors. System interactions should happen through
-       * our BIO.
-       */
-      if (error_type == GST_RESOURCE_ERROR_WRITE) {
-        rc = self->priv->syscall_flow_return;
-      }
-      GST_DEBUG_OBJECT (self,
-          "OpenSSL reported a syscall error. flow_return=%i", rc);
-      return rc;
+      GST_LOG_OBJECT (self, "SSL syscall (error) : %lu", ERR_get_error ());
+      break;
     }
     default:
-      if (self->priv->connection_state != GST_DTLS_CONNECTION_STATE_FAILED) {
-        self->priv->connection_state = GST_DTLS_CONNECTION_STATE_FAILED;
-        *notify_state = TRUE;
-      }
-      GST_ERROR_OBJECT (self, "Unknown SSL error: %d, ret: %d", error, ret);
-      if (err)
-        *err =
-            g_error_new (GST_RESOURCE_ERROR, error_type,
-            "Unknown SSL error: %d, ret: %d", error, ret);
-      return GST_FLOW_ERROR;
+      GST_WARNING_OBJECT (self, "Unknown SSL error: %d, ret: %d", error, ret);
   }
 }
 
-static GstFlowReturn
-openssl_poll (GstDtlsConnection * self, gboolean * notify_state, GError ** err)
-{
-  int ret;
-  GstFlowReturn flow_ret;
-
-  log_state (self, "poll: before handshake");
-
-  ERR_clear_error ();
-  ret = SSL_do_handshake (self->priv->ssl);
-
-  log_state (self, "poll: after handshake");
-
-  switch (ret) {
-    case 1:
-      if (!self->priv->keys_exported) {
-        GST_INFO_OBJECT (self,
-            "handshake just completed successfully, exporting keys");
-
-        if (!export_srtp_keys (self, err))
-          return GST_FLOW_ERROR;
-
-        if (self->priv->connection_state != GST_DTLS_CONNECTION_STATE_FAILED
-            && self->priv->connection_state != GST_DTLS_CONNECTION_STATE_CLOSED
-            && self->priv->connection_state !=
-            GST_DTLS_CONNECTION_STATE_CONNECTED) {
-          self->priv->connection_state = GST_DTLS_CONNECTION_STATE_CONNECTED;
-          *notify_state = TRUE;
-        }
-      } else {
-        GST_INFO_OBJECT (self, "handshake is completed");
-      }
-      return GST_FLOW_OK;
-    case 0:
-      GST_DEBUG_OBJECT (self, "do_handshake encountered EOF");
-      break;
-    case -1:
-      GST_DEBUG_OBJECT (self, "do_handshake encountered potential BIO error");
-      break;
-    default:
-      GST_DEBUG_OBJECT (self, "do_handshake returned %d", ret);
-      break;
-  }
-
-  flow_ret =
-      handle_error (self, ret, GST_RESOURCE_ERROR_OPEN_WRITE, notify_state,
-      err);
-
-  ERR_print_errors_cb (ssl_warn_cb, self);
-
-  return flow_ret;
-}
-
 static int
 openssl_verify_callback (int preverify_ok, X509_STORE_CTX * x509_ctx)
 {
@@ -1106,7 +731,7 @@ openssl_verify_callback (int preverify_ok, X509_STORE_CTX * x509_ctx)
   self = SSL_get_ex_data (ssl, connection_ex_index);
   g_return_val_if_fail (GST_IS_DTLS_CONNECTION (self), FALSE);
 
-  pem = _gst_dtls_x509_to_pem (X509_STORE_CTX_get0_cert (x509_ctx));
+  pem = _gst_dtls_x509_to_pem (x509_ctx->cert);
 
   if (!pem) {
     GST_WARNING_OBJECT (self,
@@ -1118,8 +743,7 @@ openssl_verify_callback (int preverify_ok, X509_STORE_CTX * x509_ctx)
       gint len;
 
       len =
-          X509_NAME_print_ex (bio,
-          X509_get_subject_name (X509_STORE_CTX_get0_cert (x509_ctx)), 1,
+          X509_NAME_print_ex (bio, X509_get_subject_name (x509_ctx->cert), 1,
           XN_FLAG_MULTILINE);
       BIO_read (bio, buffer, len);
       buffer[len] = '\0';
@@ -1147,7 +771,6 @@ openssl_verify_callback (int preverify_ok, X509_STORE_CTX * x509_ctx)
     ########  ####  #######
 */
 
-#if OPENSSL_VERSION_NUMBER < 0x10100001L
 static BIO_METHOD custom_bio_methods = {
   BIO_TYPE_BIO,
   "stream",
@@ -1166,50 +789,36 @@ BIO_s_gst_dtls_connection (void)
 {
   return &custom_bio_methods;
 }
-#else
-static BIO_METHOD *custom_bio_methods;
-
-static BIO_METHOD *
-BIO_s_gst_dtls_connection (void)
-{
-  if (custom_bio_methods != NULL)
-    return custom_bio_methods;
-
-  custom_bio_methods = BIO_meth_new (BIO_TYPE_BIO, "stream");
-  if (custom_bio_methods == NULL
-      || !BIO_meth_set_write (custom_bio_methods, bio_method_write)
-      || !BIO_meth_set_read (custom_bio_methods, bio_method_read)
-      || !BIO_meth_set_ctrl (custom_bio_methods, bio_method_ctrl)
-      || !BIO_meth_set_create (custom_bio_methods, bio_method_new)
-      || !BIO_meth_set_destroy (custom_bio_methods, bio_method_free)) {
-    BIO_meth_free (custom_bio_methods);
-    return NULL;
-  }
-
-  return custom_bio_methods;
-}
-#endif
 
 static int
 bio_method_write (BIO * bio, const char *data, int size)
 {
-  GstDtlsConnection *self = GST_DTLS_CONNECTION (BIO_get_data (bio));
-  gboolean ret = TRUE;
+  GstDtlsConnection *self = GST_DTLS_CONNECTION (bio->ptr);
 
   GST_LOG_OBJECT (self, "BIO: writing %d", size);
-  self->priv->syscall_flow_return = GST_FLOW_OK;
 
-  if (self->priv->send_callback)
-    ret = self->priv->send_callback (self, data, size,
-        self->priv->send_callback_user_data);
+  if (self->priv->send_closure) {
+    GValue values[3] = { G_VALUE_INIT };
 
-  return ret ? size : -1;
+    g_value_init (&values[0], GST_TYPE_DTLS_CONNECTION);
+    g_value_set_object (&values[0], self);
+
+    g_value_init (&values[1], G_TYPE_POINTER);
+    g_value_set_pointer (&values[1], (gpointer) data);
+
+    g_value_init (&values[2], G_TYPE_INT);
+    g_value_set_int (&values[2], size);
+
+    g_closure_invoke (self->priv->send_closure, NULL, 3, values, NULL);
+  }
+
+  return size;
 }
 
 static int
 bio_method_read (BIO * bio, char *out_buffer, int size)
 {
-  GstDtlsConnection *self = GST_DTLS_CONNECTION (BIO_get_data (bio));
+  GstDtlsConnection *self = GST_DTLS_CONNECTION (bio->ptr);
   GstDtlsConnectionPrivate *priv = self->priv;
   guint internal_size;
   gint copy_size;
@@ -1253,7 +862,7 @@ bio_method_read (BIO * bio, char *out_buffer, int size)
 static long
 bio_method_ctrl (BIO * bio, int cmd, long arg1, void *arg2)
 {
-  GstDtlsConnection *self = GST_DTLS_CONNECTION (BIO_get_data (bio));
+  GstDtlsConnection *self = GST_DTLS_CONNECTION (bio->ptr);
   GstDtlsConnectionPrivate *priv = self->priv;
 
   switch (cmd) {
@@ -1269,7 +878,7 @@ bio_method_ctrl (BIO * bio, int cmd, long arg1, void *arg2)
       GST_LOG_OBJECT (self, "BIO: EOF reset");
       return 1;
     case BIO_CTRL_EOF:{
-      gint eof = priv->is_alive == FALSE;
+      gint eof = !(priv->bio_buffer_len - priv->bio_buffer_offset);
       GST_LOG_OBJECT (self, "BIO: EOF query returned %d", eof);
       return eof;
     }
@@ -1301,8 +910,8 @@ bio_method_new (BIO * bio)
 {
   GST_LOG_OBJECT (NULL, "BIO: new");
 
-  BIO_set_shutdown (bio, 0);
-  BIO_set_init (bio, 1);
+  bio->shutdown = 0;
+  bio->init = 1;
 
   return 1;
 }
@@ -1315,27 +924,6 @@ bio_method_free (BIO * bio)
     return 0;
   }
 
-  GST_LOG_OBJECT (GST_DTLS_CONNECTION (BIO_get_data (bio)), "BIO free");
+  GST_LOG_OBJECT (GST_DTLS_CONNECTION (bio->ptr), "BIO free");
   return 0;
 }
-
-GType
-gst_dtls_connection_state_get_type (void)
-{
-  static GType type = 0;
-  static const GEnumValue values[] = {
-    {GST_DTLS_CONNECTION_STATE_NEW, "New connection", "new"},
-    {GST_DTLS_CONNECTION_STATE_CLOSED, "Closed connection on either side",
-        "closed"},
-    {GST_DTLS_CONNECTION_STATE_FAILED, "Failed connection", "failed"},
-    {GST_DTLS_CONNECTION_STATE_CONNECTING, "Connecting", "connecting"},
-    {GST_DTLS_CONNECTION_STATE_CONNECTED, "Successfully connected",
-        "connected"},
-    {0, NULL, NULL},
-  };
-
-  if (!type) {
-    type = g_enum_register_static ("GstDtlsConnectionState", values);
-  }
-  return type;
-}
