@@ -27,21 +27,10 @@
 #include "config.h"
 #endif
 
-#include "gstdtlselements.h"
 #include "gstdtlsdec.h"
 
 #include "gstdtlscertificate.h"
 
-/**
- * SECTION: element-dtlsdec
- * @title: dtlsdec
- *
- * This element decodes DTLS packets. Before 1.28 the default X509 PEM
- * certificate was encoded using a RSA 2048 bits private key. Since 1.28 the
- * default certificate is encoded using a ECDSA P-256 private key.
- *
- */
-
 static GstStaticPadTemplate sink_template = GST_STATIC_PAD_TEMPLATE ("sink",
     GST_PAD_SINK,
     GST_PAD_ALWAYS,
@@ -59,8 +48,6 @@ GST_DEBUG_CATEGORY_STATIC (gst_dtls_dec_debug);
 #define gst_dtls_dec_parent_class parent_class
 G_DEFINE_TYPE_WITH_CODE (GstDtlsDec, gst_dtls_dec, GST_TYPE_ELEMENT,
     GST_DEBUG_CATEGORY_INIT (gst_dtls_dec_debug, "dtlsdec", 0, "DTLS Decoder"));
-GST_ELEMENT_REGISTER_DEFINE_WITH_CODE (dtlsdec, "dtlsdec", GST_RANK_NONE,
-    GST_TYPE_DTLS_DEC, dtls_element_init (plugin));
 
 enum
 {
@@ -76,10 +63,10 @@ enum
   PROP_CONNECTION_ID,
   PROP_PEM,
   PROP_PEER_PEM,
+
   PROP_DECODER_KEY,
   PROP_SRTP_CIPHER,
   PROP_SRTP_AUTH,
-  PROP_CONNECTION_STATE,
   NUM_PROPERTIES
 };
 
@@ -141,7 +128,8 @@ gst_dtls_dec_class_init (GstDtlsDecClass * klass)
 
   signals[SIGNAL_ON_KEY_RECEIVED] =
       g_signal_new ("on-key-received", G_TYPE_FROM_CLASS (klass),
-      G_SIGNAL_RUN_LAST, 0, NULL, NULL, NULL, G_TYPE_NONE, 0);
+      G_SIGNAL_RUN_LAST, 0, NULL, NULL,
+      g_cclosure_marshal_generic, G_TYPE_NONE, 0);
 
   properties[PROP_CONNECTION_ID] =
       g_param_spec_string ("connection-id",
@@ -152,9 +140,8 @@ gst_dtls_dec_class_init (GstDtlsDecClass * klass)
   properties[PROP_PEM] =
       g_param_spec_string ("pem",
       "PEM string",
-      "A string containing a X509 certificate and private key in PEM format",
-      DEFAULT_PEM,
-      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | GST_PARAM_DOC_SHOW_DEFAULT);
+      "A string containing a X509 certificate and RSA private key in PEM format",
+      DEFAULT_PEM, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
 
   properties[PROP_PEER_PEM] =
       g_param_spec_string ("peer-pem",
@@ -184,17 +171,12 @@ gst_dtls_dec_class_init (GstDtlsDecClass * klass)
       0, GST_DTLS_SRTP_AUTH_HMAC_SHA1_80, DEFAULT_SRTP_AUTH,
       G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
 
-  properties[PROP_CONNECTION_STATE] =
-      g_param_spec_enum ("connection-state",
-      "Connection State",
-      "Current connection state",
-      GST_DTLS_TYPE_CONNECTION_STATE,
-      GST_DTLS_CONNECTION_STATE_NEW, G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
-
   g_object_class_install_properties (gobject_class, NUM_PROPERTIES, properties);
 
-  gst_element_class_add_static_pad_template (element_class, &src_template);
-  gst_element_class_add_static_pad_template (element_class, &sink_template);
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&src_template));
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&sink_template));
 
   gst_element_class_set_static_metadata (element_class,
       "DTLS Decoder",
@@ -321,13 +303,6 @@ gst_dtls_dec_get_property (GObject * object, guint prop_id, GValue * value,
     case PROP_SRTP_AUTH:
       g_value_set_uint (value, self->srtp_auth);
       break;
-    case PROP_CONNECTION_STATE:
-      if (self->connection)
-        g_object_get_property (G_OBJECT (self->connection), "connection-state",
-            value);
-      else
-        g_value_set_enum (value, GST_DTLS_CONNECTION_STATE_CLOSED);
-      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (self, prop_id, pspec);
   }
@@ -435,7 +410,7 @@ static void
 on_key_received (GstDtlsConnection * connection, gpointer key, guint cipher,
     guint auth, GstDtlsDec * self)
 {
-  GstBuffer *new_decoder_key;
+  gpointer key_dup;
   gchar *key_str;
 
   g_return_if_fail (GST_IS_DTLS_DEC (self));
@@ -443,13 +418,15 @@ on_key_received (GstDtlsConnection * connection, gpointer key, guint cipher,
   self->srtp_cipher = cipher;
   self->srtp_auth = auth;
 
-  new_decoder_key =
-      gst_buffer_new_memdup (key, GST_DTLS_SRTP_MASTER_KEY_LENGTH);
+  key_dup = g_memdup (key, GST_DTLS_SRTP_MASTER_KEY_LENGTH);
 
-  if (self->decoder_key)
+  if (self->decoder_key) {
     gst_buffer_unref (self->decoder_key);
+    self->decoder_key = NULL;
+  }
 
-  self->decoder_key = new_decoder_key;
+  self->decoder_key =
+      gst_buffer_new_wrapped (key_dup, GST_DTLS_SRTP_MASTER_KEY_LENGTH);
 
   key_str = g_base64_encode (key, GST_DTLS_SRTP_MASTER_KEY_LENGTH);
   GST_INFO_OBJECT (self, "received key: %s", key_str);
@@ -458,10 +435,31 @@ on_key_received (GstDtlsConnection * connection, gpointer key, guint cipher,
   g_signal_emit (self, signals[SIGNAL_ON_KEY_RECEIVED], 0);
 }
 
+static gboolean
+signal_peer_certificate_received (GWeakRef * ref)
+{
+  GstDtlsDec *self;
+
+  self = g_weak_ref_get (ref);
+  g_weak_ref_clear (ref);
+  g_free (ref);
+  ref = NULL;
+
+  if (self) {
+    g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_PEER_PEM]);
+    g_object_unref (self);
+    self = NULL;
+  }
+
+  return FALSE;
+}
+
 static gboolean
 on_peer_certificate_received (GstDtlsConnection * connection, gchar * pem,
     GstDtlsDec * self)
 {
+  GWeakRef *ref;
+
   g_return_val_if_fail (GST_IS_DTLS_DEC (self), TRUE);
 
   GST_DEBUG_OBJECT (self, "Received peer certificate PEM: \n%s", pem);
@@ -472,110 +470,70 @@ on_peer_certificate_received (GstDtlsConnection * connection, gchar * pem,
   }
   self->peer_pem = g_strdup (pem);
 
-  g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_PEER_PEM]);
+  ref = g_new (GWeakRef, 1);
+  g_weak_ref_init (ref, self);
+
+  g_idle_add ((GSourceFunc) signal_peer_certificate_received, ref);
 
   return TRUE;
 }
 
-static GstFlowReturn
+static gint
 process_buffer (GstDtlsDec * self, GstBuffer * buffer)
 {
-  GstFlowReturn flow_ret;
   GstMapInfo map_info;
-  GError *err = NULL;
-  gsize written = 0;
+  gint size;
 
   if (!gst_buffer_map (buffer, &map_info, GST_MAP_READWRITE))
-    return GST_FLOW_ERROR;
+    return 0;
 
   if (!map_info.size) {
     gst_buffer_unmap (buffer, &map_info);
-    return GST_FLOW_ERROR;
+    return 0;
   }
 
-  flow_ret =
+  size =
       gst_dtls_connection_process (self->connection, map_info.data,
-      map_info.size, &written, &err);
+      map_info.size);
   gst_buffer_unmap (buffer, &map_info);
 
-  switch (flow_ret) {
-    case GST_FLOW_OK:
-      GST_LOG_OBJECT (self,
-          "Decoded buffer of size %" G_GSIZE_FORMAT " B to %" G_GSIZE_FORMAT,
-          map_info.size, written);
-      gst_buffer_set_size (buffer, written);
-      break;
-    case GST_FLOW_EOS:
-      gst_buffer_set_size (buffer, written);
-      GST_DEBUG_OBJECT (self, "Peer closed the connection");
-      break;
-    case GST_FLOW_ERROR:
-      GST_ERROR_OBJECT (self, "Error processing buffer: %s", err->message);
-      GST_ELEMENT_ERROR (self, RESOURCE, READ, (NULL), ("%s", err->message));
-      g_clear_error (&err);
-      break;
-    default:
-      g_assert_not_reached ();
-  }
-  g_assert (err == NULL);
+  if (size <= 0)
+    return size;
 
-  return flow_ret;
+  gst_buffer_set_size (buffer, size);
+
+  return size;
 }
 
-typedef struct
-{
-  GstDtlsDec *self;
-  GstFlowReturn flow_ret;
-  guint processed;
-} ProcessListData;
-
 static gboolean
 process_buffer_from_list (GstBuffer ** buffer, guint idx, gpointer user_data)
 {
-  ProcessListData *process_list_data = user_data;
-  GstDtlsDec *self = GST_DTLS_DEC (process_list_data->self);
-  GstFlowReturn flow_ret;
+  GstDtlsDec *self = GST_DTLS_DEC (user_data);
+  gint size;
 
   *buffer = gst_buffer_make_writable (*buffer);
-  flow_ret = process_buffer (self, *buffer);
-
-  process_list_data->flow_ret = flow_ret;
-  if (gst_buffer_get_size (*buffer) == 0)
+  size = process_buffer (self, *buffer);
+  if (size <= 0)
     gst_buffer_replace (buffer, NULL);
-  else if (flow_ret != GST_FLOW_ERROR)
-    process_list_data->processed++;
 
-  return flow_ret == GST_FLOW_OK;
+  return TRUE;
 }
 
 static GstFlowReturn
 sink_chain_list (GstPad * pad, GstObject * parent, GstBufferList * list)
 {
   GstDtlsDec *self = GST_DTLS_DEC (parent);
+  GstFlowReturn ret = GST_FLOW_OK;
   GstPad *other_pad;
-  ProcessListData process_list_data = { self, GST_FLOW_OK, 0 };
 
   list = gst_buffer_list_make_writable (list);
-  gst_buffer_list_foreach (list, process_buffer_from_list, &process_list_data);
-
-  /* If we successfully processed at least some buffers then forward those */
-  if (process_list_data.flow_ret != GST_FLOW_OK
-      && process_list_data.processed == 0) {
-    GST_ERROR_OBJECT (self, "Failed to process buffer list: %s",
-        gst_flow_get_name (process_list_data.flow_ret));
-    gst_buffer_list_unref (list);
-    return process_list_data.flow_ret;
-  }
-
-  /* Remove all buffers after the first one that failed to be processed */
-  gst_buffer_list_remove (list, process_list_data.processed,
-      gst_buffer_list_length (list) - process_list_data.processed);
+  gst_buffer_list_foreach (list, process_buffer_from_list, self);
 
   if (gst_buffer_list_length (list) == 0) {
     GST_DEBUG_OBJECT (self, "Not produced any buffers");
     gst_buffer_list_unref (list);
 
-    return process_list_data.flow_ret;
+    return GST_FLOW_OK;
   }
 
   g_mutex_lock (&self->src_mutex);
@@ -585,25 +543,17 @@ sink_chain_list (GstPad * pad, GstObject * parent, GstBufferList * list)
   g_mutex_unlock (&self->src_mutex);
 
   if (other_pad) {
-    gboolean was_eos = process_list_data.flow_ret == GST_FLOW_EOS;
-
-    GST_LOG_OBJECT (self, "pushing buffer list with length %u",
+    GST_LOG_OBJECT (self, "decoded buffer list with length %u, pushing",
         gst_buffer_list_length (list));
-    process_list_data.flow_ret = gst_pad_push_list (other_pad, list);
-
-    /* If the peer closed the connection, signal that we're done here now */
-    if (was_eos)
-      gst_pad_push_event (other_pad, gst_event_new_eos ());
-
+    ret = gst_pad_push_list (other_pad, list);
     gst_object_unref (other_pad);
   } else {
-    GST_LOG_OBJECT (self,
-        "dropping buffer list with length %d, have no source pad",
+    GST_LOG_OBJECT (self, "dropped buffer list with length %d, not linked",
         gst_buffer_list_length (list));
     gst_buffer_list_unref (list);
   }
 
-  return process_list_data.flow_ret;
+  return ret;
 }
 
 static GstFlowReturn
@@ -611,6 +561,7 @@ sink_chain (GstPad * pad, GstObject * parent, GstBuffer * buffer)
 {
   GstDtlsDec *self = GST_DTLS_DEC (parent);
   GstFlowReturn ret = GST_FLOW_OK;
+  gint size;
   GstPad *other_pad;
 
   if (!self->agent) {
@@ -618,17 +569,16 @@ sink_chain (GstPad * pad, GstObject * parent, GstBuffer * buffer)
     return GST_FLOW_OK;
   }
 
-  GST_DEBUG_OBJECT (self,
-      "received buffer from %s with length %" G_GSIZE_FORMAT,
+  GST_DEBUG_OBJECT (self, "received buffer from %s with length %zd",
       self->connection_id, gst_buffer_get_size (buffer));
 
   buffer = gst_buffer_make_writable (buffer);
-  ret = process_buffer (self, buffer);
-  if (ret == GST_FLOW_ERROR) {
-    GST_ERROR_OBJECT (self, "Failed to process buffer: %s",
-        gst_flow_get_name (ret));
+  size = process_buffer (self, buffer);
+
+  if (size <= 0) {
     gst_buffer_unref (buffer);
-    return ret;
+
+    return GST_FLOW_OK;
   }
 
   g_mutex_lock (&self->src_mutex);
@@ -638,25 +588,11 @@ sink_chain (GstPad * pad, GstObject * parent, GstBuffer * buffer)
   g_mutex_unlock (&self->src_mutex);
 
   if (other_pad) {
-    gboolean was_eos = (ret == GST_FLOW_EOS);
-
-    if (gst_buffer_get_size (buffer) > 0) {
-      GST_LOG_OBJECT (self, "pushing buffer");
-      ret = gst_pad_push (other_pad, buffer);
-    } else {
-      gst_buffer_unref (buffer);
-    }
-
-    /* If the peer closed the connection, signal that we're done here now */
-    if (was_eos) {
-      gst_pad_push_event (other_pad, gst_event_new_eos ());
-      if (ret == GST_FLOW_OK)
-        ret = GST_FLOW_EOS;
-    }
-
+    GST_LOG_OBJECT (self, "decoded buffer with length %d, pushing", size);
+    ret = gst_pad_push (other_pad, buffer);
     gst_object_unref (other_pad);
   } else {
-    GST_LOG_OBJECT (self, "dropping buffer, have no source pad");
+    GST_LOG_OBJECT (self, "dropped buffer with length %d, not linked", size);
     gst_buffer_unref (buffer);
   }
 
@@ -676,12 +612,9 @@ get_agent_by_pem (const gchar * pem)
   if (!pem) {
     if (g_once_init_enter (&generated_cert_agent)) {
       GstDtlsAgent *new_agent;
-      GObject *certificate;
 
-      certificate = g_object_new (GST_TYPE_DTLS_CERTIFICATE, NULL);
       new_agent = g_object_new (GST_TYPE_DTLS_AGENT, "certificate",
-          certificate, NULL);
-      g_object_unref (certificate);
+          g_object_new (GST_TYPE_DTLS_CERTIFICATE, NULL), NULL);
 
       GST_DEBUG_OBJECT (generated_cert_agent,
           "no agent with generated cert found, creating new");
@@ -704,12 +637,9 @@ get_agent_by_pem (const gchar * pem)
     agent = GST_DTLS_AGENT (g_hash_table_lookup (agent_table, pem));
 
     if (!agent) {
-      GObject *certificate;
-
-      certificate = g_object_new (GST_TYPE_DTLS_CERTIFICATE, "pem", pem, NULL);
-      agent = g_object_new (GST_TYPE_DTLS_AGENT, "certificate", certificate,
-          NULL);
-      g_object_unref (certificate);
+      agent = g_object_new (GST_TYPE_DTLS_AGENT,
+          "certificate", g_object_new (GST_TYPE_DTLS_CERTIFICATE, "pem", pem,
+              NULL), NULL);
 
       g_object_weak_ref (G_OBJECT (agent), (GWeakNotify) agent_weak_ref_notify,
           (gpointer) g_strdup (pem));
@@ -768,15 +698,6 @@ gst_dtls_dec_fetch_connection (gchar * id)
   return connection;
 }
 
-static void
-on_connection_state_changed (GObject * object, GParamSpec * pspec,
-    gpointer user_data)
-{
-  GstDtlsDec *self = GST_DTLS_DEC (user_data);
-
-  g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_CONNECTION_STATE]);
-}
-
 static void
 create_connection (GstDtlsDec * self, gchar * id)
 {
@@ -784,8 +705,6 @@ create_connection (GstDtlsDec * self, gchar * id)
   g_return_if_fail (GST_IS_DTLS_AGENT (self->agent));
 
   if (self->connection) {
-    g_signal_handlers_disconnect_by_func (self->connection,
-        on_connection_state_changed, self);
     g_object_unref (self->connection);
     self->connection = NULL;
   }
@@ -805,10 +724,6 @@ create_connection (GstDtlsDec * self, gchar * id)
 
   self->connection =
       g_object_new (GST_TYPE_DTLS_CONNECTION, "agent", self->agent, NULL);
-  g_signal_connect_object (self->connection,
-      "notify::connection-state", G_CALLBACK (on_connection_state_changed),
-      self, 0);
-  on_connection_state_changed (NULL, NULL, self);
 
   g_object_weak_ref (G_OBJECT (self->connection),
       (GWeakNotify) connection_weak_ref_notify, g_strdup (id));
