@@ -20,16 +20,16 @@
 
 /**
  * SECTION:element-faac
- * @title: faac
  * @see_also: faad
  *
  * faac encodes raw audio to AAC (MPEG-4 part 3) streams.
  *
- * ## Example launch line
+ * <refsect2>
+ * <title>Example launch line</title>
  * |[
- * gst-launch-1.0 audiotestsrc wave=sine num-buffers=100 ! audioconvert ! faac ! matroskamux ! filesink location=sine.mkv
+ * gst-launch audiotestsrc wave=sine num-buffers=100 ! audioconvert ! faac ! matroskamux ! filesink location=sine.mkv
  * ]| Encode a sine beep as aac and write to matroska container.
- *
+ * </refsect2>
  */
 
 #ifdef HAVE_CONFIG_H
@@ -55,6 +55,12 @@
                     "64000, " \
                     "88200, " \
                     "96000"
+#define SINK_CAPS \
+    "audio/x-raw, "                \
+    "format = (string) "GST_AUDIO_NE (S16) ", "  \
+    "layout = (string) interleaved, " \
+    "rate = (int) {" SAMPLE_RATES "}, "   \
+    "channels = (int) [ 1, 6 ] "
 
 /* these don't seem to work? */
 #if 0
@@ -90,6 +96,11 @@ static GstStaticPadTemplate src_template = GST_STATIC_PAD_TEMPLATE ("src",
     GST_PAD_ALWAYS,
     GST_STATIC_CAPS (SRC_CAPS));
 
+static GstStaticPadTemplate sink_template = GST_STATIC_PAD_TEMPLATE ("sink",
+    GST_PAD_SINK,
+    GST_PAD_ALWAYS,
+    GST_STATIC_CAPS (SINK_CAPS));
+
 enum
 {
   PROP_0,
@@ -113,9 +124,9 @@ static void gst_faac_set_property (GObject * object,
 static void gst_faac_get_property (GObject * object,
     guint prop_id, GValue * value, GParamSpec * pspec);
 
-static GstCaps *gst_faac_enc_generate_sink_caps (void);
 static gboolean gst_faac_configure_source_pad (GstFaac * faac,
     GstAudioInfo * info);
+static GstCaps *gst_faac_getcaps (GstAudioEncoder * enc, GstCaps * filter);
 
 static gboolean gst_faac_stop (GstAudioEncoder * enc);
 static gboolean gst_faac_set_format (GstAudioEncoder * enc,
@@ -135,7 +146,6 @@ GST_DEBUG_CATEGORY_STATIC (faac_debug);
 
 #define gst_faac_parent_class parent_class
 G_DEFINE_TYPE (GstFaac, gst_faac, GST_TYPE_AUDIO_ENCODER);
-GST_ELEMENT_REGISTER_DEFINE (faac, "faac", GST_RANK_SECONDARY, GST_TYPE_FAAC);
 
 #define GST_TYPE_FAAC_RATE_CONTROL (gst_faac_brtype_get_type ())
 static GType
@@ -184,33 +194,24 @@ gst_faac_class_init (GstFaacClass * klass)
   GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
   GstElementClass *gstelement_class = GST_ELEMENT_CLASS (klass);
   GstAudioEncoderClass *base_class = GST_AUDIO_ENCODER_CLASS (klass);
-  GstCaps *sink_caps;
-  GstPadTemplate *sink_templ;
 
   gobject_class->set_property = gst_faac_set_property;
   gobject_class->get_property = gst_faac_get_property;
 
-  GST_DEBUG_CATEGORY_INIT (faac_debug, "faac", 0, "AAC encoding");
-
-  gst_element_class_add_static_pad_template (gstelement_class, &src_template);
-
-  sink_caps = gst_faac_enc_generate_sink_caps ();
-  sink_templ = gst_pad_template_new ("sink",
-      GST_PAD_SINK, GST_PAD_ALWAYS, sink_caps);
-  gst_element_class_add_pad_template (gstelement_class, sink_templ);
-  gst_caps_unref (sink_caps);
+  gst_element_class_add_pad_template (gstelement_class,
+      gst_static_pad_template_get (&src_template));
+  gst_element_class_add_pad_template (gstelement_class,
+      gst_static_pad_template_get (&sink_template));
 
   gst_element_class_set_static_metadata (gstelement_class, "AAC audio encoder",
       "Codec/Encoder/Audio",
       "Free MPEG-2/4 AAC encoder",
       "Ronald Bultje <rbultje@ronald.bitfreak.net>");
 
-  gst_type_mark_as_plugin_api (GST_TYPE_FAAC_RATE_CONTROL, 0);
-  gst_type_mark_as_plugin_api (GST_TYPE_FAAC_SHORTCTL, 0);
-
   base_class->stop = GST_DEBUG_FUNCPTR (gst_faac_stop);
   base_class->set_format = GST_DEBUG_FUNCPTR (gst_faac_set_format);
   base_class->handle_frame = GST_DEBUG_FUNCPTR (gst_faac_handle_frame);
+  base_class->getcaps = GST_DEBUG_FUNCPTR (gst_faac_getcaps);
 
   /* properties */
   g_object_class_install_property (gobject_class, PROP_QUALITY,
@@ -241,12 +242,13 @@ gst_faac_class_init (GstFaacClass * klass)
           "Block type encorcing",
           GST_TYPE_FAAC_SHORTCTL, FAAC_DEFAULT_SHORTCTL,
           G_PARAM_CONSTRUCT | G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+
+  GST_DEBUG_CATEGORY_INIT (faac_debug, "faac", 0, "AAC encoding");
 }
 
 static void
 gst_faac_init (GstFaac * faac)
 {
-  GST_PAD_SET_ACCEPT_TEMPLATE (GST_AUDIO_ENCODER_SINK_PAD (faac));
 }
 
 static void
@@ -297,67 +299,58 @@ static const GstAudioChannelPosition aac_channel_positions[][8] = {
 };
 
 static GstCaps *
-gst_faac_enc_generate_sink_caps (void)
+gst_faac_getcaps (GstAudioEncoder * enc, GstCaps * filter)
 {
-  GstCaps *caps = gst_caps_new_empty ();
-  GstStructure *s, *t;
-  gint i, c;
-  static const int rates[] = {
-    8000, 11025, 12000, 16000, 22050, 24000,
-    32000, 44100, 48000, 64000, 88200, 96000
-  };
-  GValue rates_arr = { 0, };
-  GValue tmp_v = { 0, };
+  static volatile gsize sinkcaps = 0;
 
-  g_value_init (&rates_arr, GST_TYPE_LIST);
-  g_value_init (&tmp_v, G_TYPE_INT);
-  for (i = 0; i < G_N_ELEMENTS (rates); i++) {
-    g_value_set_int (&tmp_v, rates[i]);
-    gst_value_list_append_value (&rates_arr, &tmp_v);
+  if (g_once_init_enter (&sinkcaps)) {
+    GstCaps *tmp = gst_caps_new_empty ();
+    GstStructure *s, *t;
+    gint i, c;
+    static const int rates[] = {
+      8000, 11025, 12000, 16000, 22050, 24000,
+      32000, 44100, 48000, 64000, 88200, 96000
+    };
+    GValue rates_arr = { 0, };
+    GValue tmp_v = { 0, };
+
+    g_value_init (&rates_arr, GST_TYPE_LIST);
+    g_value_init (&tmp_v, G_TYPE_INT);
+    for (i = 0; i < G_N_ELEMENTS (rates); i++) {
+      g_value_set_int (&tmp_v, rates[i]);
+      gst_value_list_append_value (&rates_arr, &tmp_v);
+    }
+    g_value_unset (&tmp_v);
+
+    s = gst_structure_new ("audio/x-raw",
+        "format", G_TYPE_STRING, GST_AUDIO_NE (S16),
+        "layout", G_TYPE_STRING, "interleaved", NULL);
+    gst_structure_set_value (s, "rate", &rates_arr);
+
+    for (i = 1; i <= 6; i++) {
+      guint64 channel_mask = 0;
+      t = gst_structure_copy (s);
+
+      gst_structure_set (t, "channels", G_TYPE_INT, i, NULL);
+      if (i > 1) {
+        for (c = 0; c < i; c++)
+          channel_mask |=
+              G_GUINT64_CONSTANT (1) << aac_channel_positions[i - 1][c];
+
+        gst_structure_set (t, "channel-mask", GST_TYPE_BITMASK, channel_mask,
+            NULL);
+      }
+      gst_caps_append_structure (tmp, t);
+    }
+    gst_structure_free (s);
+    g_value_unset (&rates_arr);
+
+    GST_DEBUG_OBJECT (enc, "Generated sinkcaps: %" GST_PTR_FORMAT, tmp);
+
+    g_once_init_leave (&sinkcaps, (gsize) tmp);
   }
-  g_value_unset (&tmp_v);
 
-  s = gst_structure_new ("audio/x-raw",
-      "format", G_TYPE_STRING, GST_AUDIO_NE (S16),
-      "layout", G_TYPE_STRING, "interleaved", NULL);
-  gst_structure_set_value (s, "rate", &rates_arr);
-
-  t = gst_structure_copy (s);
-  gst_structure_set (t, "channels", G_TYPE_INT, 1, NULL);
-  gst_caps_append_structure (caps, t);
-
-  for (i = 2; i <= 6; i++) {
-    guint64 channel_mask = 0;
-    t = gst_structure_copy (s);
-
-    gst_structure_set (t, "channels", G_TYPE_INT, i, NULL);
-    for (c = 0; c < i; c++)
-      channel_mask |= G_GUINT64_CONSTANT (1) << aac_channel_positions[i - 1][c];
-
-    gst_structure_set (t, "channel-mask", GST_TYPE_BITMASK, channel_mask, NULL);
-    gst_caps_append_structure (caps, t);
-  }
-  gst_structure_free (s);
-  g_value_unset (&rates_arr);
-
-  GST_DEBUG ("Generated sinkcaps: %" GST_PTR_FORMAT, caps);
-  return caps;
-}
-
-static void
-gst_faac_set_tags (GstFaac * faac)
-{
-  GstTagList *taglist;
-
-  /* create a taglist and add a bitrate tag to it */
-  taglist = gst_tag_list_new_empty ();
-  gst_tag_list_add (taglist, GST_TAG_MERGE_REPLACE,
-      GST_TAG_BITRATE, faac->bitrate, NULL);
-
-  gst_audio_encoder_merge_tags (GST_AUDIO_ENCODER (faac), taglist,
-      GST_TAG_MERGE_REPLACE);
-
-  gst_tag_list_unref (taglist);
+  return gst_audio_encoder_proxy_getcaps (enc, (GstCaps *) sinkcaps, filter);
 }
 
 static gboolean
@@ -394,8 +387,6 @@ gst_faac_set_format (GstAudioEncoder * enc, GstAudioInfo * info)
   if (!result)
     goto done;
 
-  gst_faac_set_tags (faac);
-
   /* report needs to base class */
   gst_audio_encoder_set_frame_samples_min (enc, faac->samples);
   gst_audio_encoder_set_frame_samples_max (enc, faac->samples);
@@ -782,7 +773,8 @@ gst_faac_get_property (GObject * object,
 static gboolean
 plugin_init (GstPlugin * plugin)
 {
-  return GST_ELEMENT_REGISTER (faac, plugin);
+  return gst_element_register (plugin, "faac", GST_RANK_SECONDARY,
+      GST_TYPE_FAAC);
 }
 
 GST_PLUGIN_DEFINE (GST_VERSION_MAJOR,
