@@ -167,12 +167,9 @@ static GstStaticPadTemplate gst_ca_opengl_layer_sink_template =
     GST_STATIC_PAD_TEMPLATE ("sink",
     GST_PAD_SINK,
     GST_PAD_ALWAYS,
-    GST_STATIC_CAPS ("video/x-raw(" GST_CAPS_FEATURE_MEMORY_GL_MEMORY "), "
-      "format = (string) RGBA, "
-      "width = " GST_VIDEO_SIZE_RANGE ", "
-      "height = " GST_VIDEO_SIZE_RANGE ", "
-      "framerate = " GST_VIDEO_FPS_RANGE ","
-      "texture-target = (string) 2D")
+    GST_STATIC_CAPS (GST_VIDEO_CAPS_MAKE_WITH_FEATURES
+        (GST_CAPS_FEATURE_MEMORY_GL_MEMORY,
+            "RGBA"))
     );
 
 enum
@@ -227,7 +224,8 @@ gst_ca_opengl_layer_sink_class_init (GstCAOpenGLLayerSinkClass * klass)
       "Sink/Video", "A video sink based on CAOpenGLLayer",
       "Matthew Waters <matthew@centricular.com>");
 
-  gst_element_class_add_static_pad_template (element_class, &gst_ca_opengl_layer_sink_template);
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&gst_ca_opengl_layer_sink_template));
 
   gobject_class->finalize = gst_ca_opengl_layer_sink_finalize;
 
@@ -249,6 +247,7 @@ gst_ca_opengl_layer_sink_init (GstCAOpenGLLayerSink * ca_sink)
 {
   ca_sink->display = NULL;
   ca_sink->keep_aspect_ratio = TRUE;
+  ca_sink->pool = NULL;
   ca_sink->stored_buffer = NULL;
   ca_sink->redisplay_texture = 0;
 
@@ -358,8 +357,12 @@ _ensure_gl_setup (GstCAOpenGLLayerSink * ca_sink)
   gst_gl_display_filter_gl_api (ca_sink->display, SUPPORTED_GL_APIS);
 
   if (!ca_sink->context) {
-    if (!gst_gl_display_create_context (ca_sink->display,
-            ca_sink->other_context, &ca_sink->context, &error)) {
+    ca_sink->context = gst_gl_context_new (ca_sink->display);
+    if (!ca_sink->context)
+      goto context_creation_error;
+
+    if (!gst_gl_context_create (ca_sink->context, ca_sink->other_context,
+            &error)) {
       goto context_error;
     }
   }
@@ -369,6 +372,13 @@ _ensure_gl_setup (GstCAOpenGLLayerSink * ca_sink)
 
   return TRUE;
 
+context_creation_error:
+  {
+    GST_ELEMENT_ERROR (ca_sink, RESOURCE, NOT_FOUND,
+        ("Failed to create GL context"), (NULL));
+    return FALSE;
+  }
+
 context_error:
   {
     GST_ELEMENT_ERROR (ca_sink, RESOURCE, NOT_FOUND, ("%s", error->message),
@@ -383,6 +393,7 @@ static gboolean
 gst_ca_opengl_layer_sink_query (GstBaseSink * bsink, GstQuery * query)
 {
   GstCAOpenGLLayerSink *ca_sink = GST_CA_OPENGL_LAYER_SINK (bsink);
+  gboolean res = FALSE;
 
   switch (GST_QUERY_TYPE (query)) {
     case GST_QUERY_CONTEXT:
@@ -439,6 +450,7 @@ gst_ca_opengl_layer_sink_query (GstBaseSink * bsink, GstQuery * query)
       gst_buffer_replace (&ca_sink->next_buffer, NULL);
       gst_buffer_replace (&ca_sink->next_sync, NULL);
 
+      res = GST_BASE_SINK_CLASS (parent_class)->query (bsink, query);
       break;
     }
     default:
@@ -453,6 +465,11 @@ gst_ca_opengl_layer_sink_stop (GstBaseSink * bsink)
 {
   GstCAOpenGLLayerSink *ca_sink = GST_CA_OPENGL_LAYER_SINK (bsink);
 
+  if (ca_sink->pool) {
+    gst_object_unref (ca_sink->pool);
+    ca_sink->pool = NULL;
+  }
+
   if (ca_sink->gl_caps) {
     gst_caps_unref (ca_sink->gl_caps);
     ca_sink->gl_caps = NULL;
@@ -471,8 +488,6 @@ gst_ca_opengl_layer_sink_set_context (GstElement * element, GstContext * context
 
   if (ca_sink->display)
     gst_gl_display_filter_gl_api (ca_sink->display, SUPPORTED_GL_APIS);
-
-  GST_ELEMENT_CLASS (parent_class)->set_context (element, context);
 }
 
 static GstStateChangeReturn
@@ -492,6 +507,7 @@ gst_ca_opengl_layer_sink_change_state (GstElement * element, GstStateChange tran
       _ensure_gl_setup (ca_sink);
       break;
     case GST_STATE_CHANGE_READY_TO_PAUSED:
+      g_atomic_int_set (&ca_sink->to_quit, 0);
       break;
     case GST_STATE_CHANGE_PAUSED_TO_PLAYING:
       break;
@@ -524,6 +540,12 @@ gst_ca_opengl_layer_sink_change_state (GstElement * element, GstStateChange tran
       gst_buffer_replace (&ca_sink->next_sync, NULL);
       GST_CA_OPENGL_LAYER_SINK_UNLOCK (ca_sink);
 
+      if (ca_sink->pool) {
+        gst_buffer_pool_set_active (ca_sink->pool, FALSE);
+        gst_object_unref (ca_sink->pool);
+        ca_sink->pool = NULL;
+      }
+
       GST_VIDEO_SINK_WIDTH (ca_sink) = 1;
       GST_VIDEO_SINK_HEIGHT (ca_sink) = 1;
       if (ca_sink->context) {
@@ -739,6 +761,12 @@ gst_ca_opengl_layer_sink_show_frame (GstVideoSink * vsink, GstBuffer * buf)
   if (old_sync)
     gst_buffer_unref (old_sync);
 
+  if (g_atomic_int_get (&ca_sink->to_quit) != 0) {
+    GST_ELEMENT_ERROR (ca_sink, RESOURCE, NOT_FOUND,
+        ("%s", gst_gl_context_get_error ()), (NULL));
+    return GST_FLOW_ERROR;
+  }
+
   return GST_FLOW_OK;
 }
 
@@ -746,7 +774,6 @@ static gboolean
 gst_ca_opengl_layer_sink_propose_allocation (GstBaseSink * bsink, GstQuery * query)
 {
   GstCAOpenGLLayerSink *ca_sink = GST_CA_OPENGL_LAYER_SINK (bsink);
-  GstBufferPool *pool = NULL;
   GstStructure *config;
   GstCaps *caps;
   guint size;
@@ -769,18 +796,36 @@ gst_ca_opengl_layer_sink_propose_allocation (GstBaseSink * bsink, GstQuery * que
     /* the normal size of a frame */
     size = info.size;
 
-    GST_DEBUG_OBJECT (ca_sink, "create new pool");
+    if (ca_sink->pool) {
+      GstCaps *pcaps;
 
-    pool = gst_gl_buffer_pool_new (ca_sink->context);
-    config = gst_buffer_pool_get_config (pool);
-    gst_buffer_pool_config_set_params (config, caps, size, 0, 0);
+      /* we had a pool, check caps */
+      GST_DEBUG_OBJECT (ca_sink, "check existing pool caps");
+      config = gst_buffer_pool_get_config (ca_sink->pool);
+      gst_buffer_pool_config_get_params (config, &pcaps, &size, NULL, NULL);
 
-    if (!gst_buffer_pool_set_config (pool, config))
+      if (!gst_caps_is_equal (caps, pcaps)) {
+        GST_DEBUG_OBJECT (ca_sink, "pool has different caps");
+        /* different caps, we can't use this pool */
+        gst_object_unref (ca_sink->pool);
+        ca_sink->pool = NULL;
+      }
+      gst_structure_free (config);
+    }
+
+    if (ca_sink->pool == NULL) {
+      GST_DEBUG_OBJECT (ca_sink, "create new pool");
+
+      ca_sink->pool = gst_gl_buffer_pool_new (ca_sink->context);
+      config = gst_buffer_pool_get_config (ca_sink->pool);
+      gst_buffer_pool_config_set_params (config, caps, size, 0, 0);
+
+      if (!gst_buffer_pool_set_config (ca_sink->pool, config))
         goto config_failed;
+    }
 
     /* we need at least 2 buffer because we hold on to the last one */
-    gst_query_add_allocation_pool (query, pool, size, 2, 0);
-    gst_object_unref (pool);
+    gst_query_add_allocation_pool (query, ca_sink->pool, size, 2, 0);
   }
 
   if (ca_sink->context->gl_vtable->FenceSync)
@@ -854,20 +899,13 @@ static void
 gst_ca_opengl_layer_sink_thread_init_redisplay (GstCAOpenGLLayerSink * ca_sink)
 {
   const GstGLFuncs *gl = ca_sink->context->gl_vtable;
-  GError *error = NULL;
 
-  if (!(ca_sink->redisplay_shader = gst_gl_shader_new_default (ca_sink->context, &error))) {
-    GST_ERROR_OBJECT (ca_sink, "Failed to link shader: %s", error->message);
+  ca_sink->redisplay_shader = gst_gl_shader_new (ca_sink->context);
+
+  if (!gst_gl_shader_compile_with_default_vf_and_check
+      (ca_sink->redisplay_shader, &ca_sink->attr_position,
+          &ca_sink->attr_texture))
     gst_ca_opengl_layer_sink_cleanup_glthread (ca_sink);
-    return;
-  }
-
-  ca_sink->attr_position =
-      gst_gl_shader_get_attribute_location (ca_sink->redisplay_shader,
-      "a_position");
-  ca_sink->attr_texture =
-      gst_gl_shader_get_attribute_location (ca_sink->redisplay_shader,
-      "a_texcoord");
 
   if (gl->GenVertexArrays) {
     gl->GenVertexArrays (1, &ca_sink->vao);
@@ -998,7 +1036,7 @@ gst_ca_opengl_layer_sink_on_draw (GstCAOpenGLLayerSink * ca_sink)
     gst_ca_opengl_layer_sink_on_resize (ca_sink, ca_sink->window_width,
         ca_sink->window_height);
     GST_CA_OPENGL_LAYER_SINK_LOCK (ca_sink);
-    ca_sink->caps_change = FALSE;
+    ca_sink->caps_change = TRUE;
   }
 
   sync_meta = gst_buffer_get_gl_sync_meta (ca_sink->stored_sync);
