@@ -34,7 +34,7 @@ gst_gl_effects_sin_callback (gint width, gint height, guint texture,
   GstGLFuncs *gl = context->gl_vtable;
 
   shader = gst_gl_effects_get_fragment_shader (effects, "sin",
-      sin_fragment_source_gles2);
+      sin_fragment_source_gles2, sin_fragment_source_opengl);
 
   if (!shader)
     return;
