@@ -46,7 +46,6 @@ typedef struct {
   gint effect;
   guint supported_properties;
   const gchar *filter_name;
-  const gchar *filter_longname;
 } GstGLEffectsFilterDescriptor;
 
 typedef void (* GstGLEffectProcessFunc) (GstGLEffects *effects);
@@ -90,7 +89,7 @@ struct _GstGLEffectsClass
 GType gst_gl_effects_get_type (void);
 gboolean gst_gl_effects_register_filters (GstPlugin *, GstRank);
 GstGLShader* gst_gl_effects_get_fragment_shader (GstGLEffects *effects,
-    const gchar * shader_name, const gchar * shader_source_gles2);
+    const gchar * shader_name, const gchar * shader_source_gles2, const gchar * shader_source_opengl);
 
 void gst_gl_effects_identity (GstGLEffects *effects);
 void gst_gl_effects_mirror (GstGLEffects *effects);
