@@ -62,14 +62,9 @@ struct _GstGLImageSink
     gint width;
     gint height;
 
-    /* Input info before 3d stereo output conversion, if any */
-    GstVideoInfo in_info;
-    GstCaps *in_caps;
-
-    /* format/caps we actually hand off to the app */
-    GstVideoInfo out_info;
-    GstCaps *out_caps;
-    GstGLTextureTarget texture_target;
+    //caps
+    GstVideoInfo info;
+    GstCaps *caps;
 
     GstGLDisplay *display;
     GstGLContext *context;
@@ -77,51 +72,32 @@ struct _GstGLImageSink
     gboolean handle_events;
     gboolean ignore_alpha;
 
-    GstGLViewConvert *convert_views;
-
-    /* Original input RGBA buffer, ready for display,
-     * or possible reconversion through the views filter */
-    GstBuffer *input_buffer;
-    /* Secondary view buffer - when operating in frame-by-frame mode */
-    GstBuffer *input_buffer2;
-
     guint      next_tex;
     GstBuffer *next_buffer;
-    GstBuffer *next_buffer2; /* frame-by-frame 2nd view */
     GstBuffer *next_sync;
-    GstGLSyncMeta *next_sync_meta;
 
     volatile gint to_quit;
     gboolean keep_aspect_ratio;
     gint par_n, par_d;
 
+    GstBufferPool *pool;
+
     /* avoid replacing the stored_buffer while drawing */
     GMutex drawing_lock;
-    GstBuffer *stored_buffer[2];
+    GstBuffer *stored_buffer;
     GstBuffer *stored_sync;
-    GstGLSyncMeta *stored_sync_meta;
     GLuint redisplay_texture;
 
-    /* protected with drawing_lock */
-    gboolean window_resized;
+    gboolean caps_change;
     guint window_width;
     guint window_height;
 
-    GstVideoRectangle display_rect;
-
     GstGLShader *redisplay_shader;
     GLuint vao;
     GLuint vbo_indices;
     GLuint vertex_buffer;
     GLint  attr_position;
     GLint  attr_texture;
-
-    GstVideoMultiviewMode mview_output_mode;
-    GstVideoMultiviewFlags mview_output_flags;
-    gboolean output_mode_changed;
-    GstGLStereoDownmix mview_downmix_mode;
-
-    GstGLOverlayCompositor *overlay_compositor;
 };
 
 struct _GstGLImageSinkClass
