@@ -60,10 +60,6 @@ struct _GstGLTransformation
     gfloat ytranslation;
     gfloat ztranslation;
 
-    gfloat xpivot;
-    gfloat ypivot;
-    gfloat zpivot;
-
     /* perspective */
     gfloat fov;
     gfloat aspect;
@@ -71,9 +67,6 @@ struct _GstGLTransformation
     gfloat zfar;
     gboolean ortho;
 
-    graphene_matrix_t model_matrix;
-    graphene_matrix_t view_matrix;
-    graphene_matrix_t projection_matrix;
     graphene_matrix_t mvp_matrix;
 
     gboolean caps_change;
