@@ -1,7 +1,7 @@
 /*
  * GStreamer
  * Copyright (C) 2010 Thiago Santos <thiago.sousa.santos@collabora.co.uk>
- *
+ * 
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
  * to deal in the Software without restriction, including without limitation
@@ -44,6 +44,8 @@
 #ifndef __GST_CV_EQUALIZE_HIST_H__
 #define __GST_CV_EQUALIZE_HIST_H__
 
+#include <gst/gst.h>
+
 #include <gstopencvvideofilter.h>
 
 G_BEGIN_DECLS
@@ -68,7 +70,7 @@ struct _GstCvEqualizeHist
   GstOpencvVideoFilter element;
 };
 
-struct _GstCvEqualizeHistClass
+struct _GstCvEqualizeHistClass 
 {
   GstOpencvVideoFilterClass parent_class;
 };
