@@ -4,7 +4,7 @@
  * Copyright (C) 2005 Ronald S. Bultje <rbultje@ronald.bitfreak.net>
  * Copyright (C) 2008 Michael Sheldon <mike@mikeasoft.com>
  * Copyright (C) 2011 Robert Jobbagy <jobbagy.robert@gmail.com>
- *
+ * 
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
  * to deal in the Software without restriction, including without limitation
@@ -48,10 +48,13 @@
 #define __GST_FACE_BLUR_H__
 
 #include <gst/gst.h>
-#include <gst/opencv/gstopencvvideofilter.h>
+#include <opencv2/core/version.hpp>
 
-#include <opencv2/core.hpp>
-#include <opencv2/objdetect.hpp>
+#include "gstopencvvideofilter.h"
+
+#if (CV_MAJOR_VERSION >= 2) && (CV_MINOR_VERSION >= 2)
+#include <opencv2/objdetect/objdetect.hpp>
+#endif
 
 G_BEGIN_DECLS
 /* #defines don't like whitespacey bits */
@@ -83,7 +86,8 @@ struct _GstFaceBlur
   gint min_size_height;
 
   IplImage *cvGray;
-  cv::CascadeClassifier *cvCascade;
+  CvHaarClassifierCascade *cvCascade;
+  CvMemStorage *cvStorage;
 };
 
 struct _GstFaceBlurClass
