@@ -24,9 +24,8 @@
 
 #include <gst/gst.h>
 #include <gst/video/video.h>
-#include <gst/video/gstvideodecoder.h>
 
-#include <openjpeg.h>
+#include "gstopenjpeg.h"
 
 G_BEGIN_DECLS
 
