@@ -33,7 +33,7 @@
  * <refsect2>
  * <title>Example pipelines</title>
  * |[
- * gst-launch-1.0 -v filesrc location=opus.ogg ! oggdemux ! opusdec ! audioconvert ! audioresample ! alsasink
+ * gst-launch -v filesrc location=opus.ogg ! oggdemux ! opusdec ! audioconvert ! audioresample ! alsasink
  * ]| Decode an Ogg/Opus file. To create an Ogg/Opus file refer to the documentation of opusenc.
  * </refsect2>
  */
@@ -47,7 +47,6 @@
 #include "gstopusheader.h"
 #include "gstopuscommon.h"
 #include "gstopusdec.h"
-#include <gst/pbutils/pbutils.h>
 
 GST_DEBUG_CATEGORY_STATIC (opusdec_debug);
 #define GST_CAT_DEFAULT opusdec_debug
@@ -64,15 +63,10 @@ GST_STATIC_PAD_TEMPLATE ("src",
     );
 
 static GstStaticPadTemplate opus_dec_sink_factory =
-    GST_STATIC_PAD_TEMPLATE ("sink",
+GST_STATIC_PAD_TEMPLATE ("sink",
     GST_PAD_SINK,
     GST_PAD_ALWAYS,
-    GST_STATIC_CAPS ("audio/x-opus, "
-        "channel-mapping-family = (int) 0; "
-        "audio/x-opus, "
-        "channel-mapping-family = (int) [1, 255], "
-        "channels = (int) [1, 255], "
-        "stream-count = (int) [1, 255], " "coupled-count = (int) [0, 255]")
+    GST_STATIC_CAPS ("audio/x-opus")
     );
 
 G_DEFINE_TYPE (GstOpusDec, gst_opus_dec, GST_TYPE_AUDIO_DECODER);
@@ -133,8 +127,8 @@ gst_opus_dec_class_init (GstOpusDecClass * klass)
       "Vincent Penquerc'h <vincent.penquerch@collabora.co.uk>");
   g_object_class_install_property (gobject_class, PROP_USE_INBAND_FEC,
       g_param_spec_boolean ("use-inband-fec", "Use in-band FEC",
-          "Use forward error correction if available (needs PLC enabled)",
-          DEFAULT_USE_INBAND_FEC, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+          "Use forward error correction if available", DEFAULT_USE_INBAND_FEC,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   g_object_class_install_property (gobject_class, PROP_APPLY_GAIN,
       g_param_spec_boolean ("apply-gain", "Apply gain",
@@ -163,7 +157,6 @@ gst_opus_dec_reset (GstOpusDec * dec)
   dec->r128_gain = 0;
   dec->sample_rate = 0;
   dec->n_channels = 0;
-  dec->leftover_plc_duration = 0;
 }
 
 static void
@@ -173,9 +166,6 @@ gst_opus_dec_init (GstOpusDec * dec)
   dec->apply_gain = DEFAULT_APPLY_GAIN;
 
   gst_audio_decoder_set_needs_format (GST_AUDIO_DECODER (dec), TRUE);
-  gst_audio_decoder_set_use_default_pad_acceptcaps (GST_AUDIO_DECODER_CAST
-      (dec), TRUE);
-  GST_PAD_SET_ACCEPT_TEMPLATE (GST_AUDIO_DECODER_SINK_PAD (dec));
 
   gst_opus_dec_reset (dec);
 }
@@ -231,8 +221,6 @@ gst_opus_dec_negotiate (GstOpusDec * dec, const GstAudioChannelPosition * pos)
   GstAudioInfo info;
 
   if (caps) {
-    gint rate, channels;
-
     caps = gst_caps_truncate (caps);
     caps = gst_caps_make_writable (caps);
     s = gst_caps_get_structure (caps, 0);
@@ -241,15 +229,13 @@ gst_opus_dec_negotiate (GstOpusDec * dec, const GstAudioChannelPosition * pos)
       gst_structure_fixate_field_nearest_int (s, "rate", dec->sample_rate);
     else
       gst_structure_set (s, "rate", G_TYPE_INT, dec->sample_rate, NULL);
-    gst_structure_get_int (s, "rate", &rate);
-    dec->sample_rate = rate;
+    gst_structure_get_int (s, "rate", &dec->sample_rate);
 
     if (gst_structure_has_field (s, "channels"))
       gst_structure_fixate_field_nearest_int (s, "channels", dec->n_channels);
     else
       gst_structure_set (s, "channels", G_TYPE_INT, dec->n_channels, NULL);
-    gst_structure_get_int (s, "channels", &channels);
-    dec->n_channels = channels;
+    gst_structure_get_int (s, "channels", &dec->n_channels);
 
     gst_caps_unref (caps);
   }
@@ -283,6 +269,7 @@ gst_opus_dec_negotiate (GstOpusDec * dec, const GstAudioChannelPosition * pos)
   /* but we still need the opus order for later reordering */
   if (pos) {
     memcpy (dec->opus_pos, pos, sizeof (pos[0]) * dec->n_channels);
+    gst_audio_channel_positions_to_valid_order (dec->opus_pos, dec->n_channels);
   } else {
     dec->opus_pos[0] = GST_AUDIO_CHANNEL_POSITION_INVALID;
   }
@@ -293,64 +280,83 @@ gst_opus_dec_negotiate (GstOpusDec * dec, const GstAudioChannelPosition * pos)
 static GstFlowReturn
 gst_opus_dec_parse_header (GstOpusDec * dec, GstBuffer * buf)
 {
+  const guint8 *data;
   GstAudioChannelPosition pos[64];
   const GstAudioChannelPosition *posn = NULL;
+  GstMapInfo map;
 
   if (!gst_opus_header_is_id_header (buf)) {
     GST_ERROR_OBJECT (dec, "Header is not an Opus ID header");
     return GST_FLOW_ERROR;
   }
 
-  if (!gst_codec_utils_opus_parse_header (buf,
-          &dec->sample_rate,
-          &dec->n_channels,
-          &dec->channel_mapping_family,
-          &dec->n_streams,
-          &dec->n_stereo_streams,
-          dec->channel_mapping, &dec->pre_skip, &dec->r128_gain)) {
-    GST_ERROR_OBJECT (dec, "Failed to parse Opus ID header");
+  gst_buffer_map (buf, &map, GST_MAP_READ);
+  data = map.data;
+
+  if (!(dec->n_channels == 0 || dec->n_channels == data[9])) {
+    gst_buffer_unmap (buf, &map);
+    GST_ERROR_OBJECT (dec, "Opus ID header has invalid channels");
     return GST_FLOW_ERROR;
   }
-  dec->r128_gain_volume = gst_opus_dec_get_r128_volume (dec->r128_gain);
 
+  dec->n_channels = data[9];
+  dec->sample_rate = GST_READ_UINT32_LE (data + 12);
+  dec->pre_skip = GST_READ_UINT16_LE (data + 10);
+  dec->r128_gain = GST_READ_UINT16_LE (data + 16);
+  dec->r128_gain_volume = gst_opus_dec_get_r128_volume (dec->r128_gain);
   GST_INFO_OBJECT (dec,
       "Found pre-skip of %u samples, R128 gain %d (volume %f)",
       dec->pre_skip, dec->r128_gain, dec->r128_gain_volume);
 
-  if (dec->channel_mapping_family == 1) {
-    GST_INFO_OBJECT (dec, "Channel mapping family 1, Vorbis mapping");
-    switch (dec->n_channels) {
-      case 1:
-      case 2:
-        /* nothing */
-        break;
-      case 3:
-      case 4:
-      case 5:
-      case 6:
-      case 7:
-      case 8:
-        posn = gst_opus_channel_positions[dec->n_channels - 1];
-        break;
-      default:{
-        gint i;
-
-        GST_ELEMENT_WARNING (GST_ELEMENT (dec), STREAM, DECODE,
-            (NULL), ("Using NONE channel layout for more than 8 channels"));
-
-        for (i = 0; i < dec->n_channels; i++)
-          pos[i] = GST_AUDIO_CHANNEL_POSITION_NONE;
-
-        posn = pos;
-      }
-    }
+  dec->channel_mapping_family = data[18];
+  if (dec->channel_mapping_family == 0) {
+    /* implicit mapping */
+    GST_INFO_OBJECT (dec, "Channel mapping family 0, implicit mapping");
+    dec->n_streams = dec->n_stereo_streams = 1;
+    dec->channel_mapping[0] = 0;
+    dec->channel_mapping[1] = 1;
   } else {
-    GST_INFO_OBJECT (dec, "Channel mapping family %d",
-        dec->channel_mapping_family);
+    dec->n_streams = data[19];
+    dec->n_stereo_streams = data[20];
+    memcpy (dec->channel_mapping, data + 21, dec->n_channels);
+
+    if (dec->channel_mapping_family == 1) {
+      GST_INFO_OBJECT (dec, "Channel mapping family 1, Vorbis mapping");
+      switch (dec->n_channels) {
+        case 1:
+        case 2:
+          /* nothing */
+          break;
+        case 3:
+        case 4:
+        case 5:
+        case 6:
+        case 7:
+        case 8:
+          posn = gst_opus_channel_positions[dec->n_channels - 1];
+          break;
+        default:{
+          gint i;
+
+          GST_ELEMENT_WARNING (GST_ELEMENT (dec), STREAM, DECODE,
+              (NULL), ("Using NONE channel layout for more than 8 channels"));
+
+          for (i = 0; i < dec->n_channels; i++)
+            pos[i] = GST_AUDIO_CHANNEL_POSITION_NONE;
+
+          posn = pos;
+        }
+      }
+    } else {
+      GST_INFO_OBJECT (dec, "Channel mapping family %d",
+          dec->channel_mapping_family);
+    }
   }
 
   gst_opus_dec_negotiate (dec, posn);
 
+  gst_buffer_unmap (buf, &map);
+
   return GST_FLOW_OK;
 }
 
@@ -367,14 +373,13 @@ opus_dec_chain_parse_data (GstOpusDec * dec, GstBuffer * buffer)
   GstFlowReturn res = GST_FLOW_OK;
   gsize size;
   guint8 *data;
-  GstBuffer *outbuf, *bufd;
+  GstBuffer *outbuf;
   gint16 *out_data;
   int n, err;
   int samples;
   unsigned int packet_size;
   GstBuffer *buf;
   GstMapInfo map, omap;
-  GstAudioClippingMeta *cmeta = NULL;
 
   if (dec->state == NULL) {
     /* If we did not get any headers, default to 2 channels */
@@ -429,9 +434,6 @@ opus_dec_chain_parse_data (GstOpusDec * dec, GstBuffer * buffer)
       && gst_buffer_get_size (dec->last_buffer) >
       0) ? dec->last_buffer : buffer;
 
-  /* That's the buffer we get duration from */
-  bufd = dec->use_inband_fec ? dec->last_buffer : buffer;
-
   if (buf && gst_buffer_get_size (buf) > 0) {
     gst_buffer_map (buf, &map, GST_MAP_READ);
     data = map.data;
@@ -444,54 +446,9 @@ opus_dec_chain_parse_data (GstOpusDec * dec, GstBuffer * buffer)
     size = 0;
   }
 
-  if (gst_buffer_get_size (bufd) == 0) {
-    GstClockTime const opus_plc_alignment = 2500 * GST_USECOND;
-    GstClockTime aligned_missing_duration;
-    GstClockTime missing_duration = GST_BUFFER_DURATION (bufd);
-
-    GST_DEBUG_OBJECT (dec,
-        "missing buffer, doing PLC duration %" GST_TIME_FORMAT
-        " plus leftover %" GST_TIME_FORMAT, GST_TIME_ARGS (missing_duration),
-        GST_TIME_ARGS (dec->leftover_plc_duration));
-
-    /* add the leftover PLC duration to that of the buffer */
-    missing_duration += dec->leftover_plc_duration;
-
-    /* align the combined buffer and leftover PLC duration to multiples
-     * of 2.5ms, rounding to nearest, and store excess duration for later */
-    aligned_missing_duration =
-        ((missing_duration +
-            opus_plc_alignment / 2) / opus_plc_alignment) * opus_plc_alignment;
-    dec->leftover_plc_duration = missing_duration - aligned_missing_duration;
-
-    /* Opus' PLC cannot operate with less than 2.5ms; skip PLC
-     * and accumulate the missing duration in the leftover_plc_duration
-     * for the next PLC attempt */
-    if (aligned_missing_duration < opus_plc_alignment) {
-      GST_DEBUG_OBJECT (dec,
-          "current duration %" GST_TIME_FORMAT
-          " of missing data not enough for PLC (minimum needed: %"
-          GST_TIME_FORMAT ") - skipping", GST_TIME_ARGS (missing_duration),
-          GST_TIME_ARGS (opus_plc_alignment));
-      goto done;
-    }
-
-    /* convert the duration (in nanoseconds) to sample count */
-    samples =
-        gst_util_uint64_scale_int (aligned_missing_duration, dec->sample_rate,
-        GST_SECOND);
-
-    GST_DEBUG_OBJECT (dec,
-        "calculated PLC frame length: %" GST_TIME_FORMAT
-        " num frame samples: %d new leftover: %" GST_TIME_FORMAT,
-        GST_TIME_ARGS (aligned_missing_duration), samples,
-        GST_TIME_ARGS (dec->leftover_plc_duration));
-  } else {
-    /* use maximum size (120 ms) as the number of returned samples is
-       not constant over the stream. */
-    samples = 120 * dec->sample_rate / 1000;
-  }
-
+  /* use maximum size (120 ms) as the number of returned samples is
+     not constant over the stream. */
+  samples = 120 * dec->sample_rate / 1000;
   packet_size = samples * dec->n_channels * 2;
 
   outbuf =
@@ -505,7 +462,7 @@ opus_dec_chain_parse_data (GstOpusDec * dec, GstBuffer * buffer)
   out_data = (gint16 *) omap.data;
 
   if (dec->use_inband_fec) {
-    if (gst_buffer_get_size (dec->last_buffer) > 0) {
+    if (dec->last_buffer) {
       /* normal delayed decode */
       GST_LOG_OBJECT (dec, "FEC enabled, decoding last delayed buffer");
       n = opus_multistream_decode (dec->state, data, size, out_data, samples,
@@ -533,41 +490,17 @@ opus_dec_chain_parse_data (GstOpusDec * dec, GstBuffer * buffer)
   GST_DEBUG_OBJECT (dec, "decoded %d samples", n);
   gst_buffer_set_size (outbuf, n * 2 * dec->n_channels);
 
-  cmeta = gst_buffer_get_audio_clipping_meta (buf);
-
-  g_assert (!cmeta || cmeta->format == GST_FORMAT_DEFAULT);
-
   /* Skip any samples that need skipping */
-  if (cmeta && cmeta->start) {
-    guint pre_skip = cmeta->start;
-    guint scaled_pre_skip = pre_skip * dec->sample_rate / 48000;
+  if (dec->pre_skip > 0) {
+    guint scaled_pre_skip = dec->pre_skip * dec->sample_rate / 48000;
     guint skip = scaled_pre_skip > n ? n : scaled_pre_skip;
     guint scaled_skip = skip * 48000 / dec->sample_rate;
 
     gst_buffer_resize (outbuf, skip * 2 * dec->n_channels, -1);
-
+    dec->pre_skip -= scaled_skip;
     GST_INFO_OBJECT (dec,
-        "Skipping %u samples at the beginning (%u at 48000 Hz)",
-        skip, scaled_skip);
-  }
-
-  if (cmeta && cmeta->end) {
-    guint post_skip = cmeta->end;
-    guint scaled_post_skip = post_skip * dec->sample_rate / 48000;
-    guint skip = scaled_post_skip > n ? n : scaled_post_skip;
-    guint scaled_skip = skip * 48000 / dec->sample_rate;
-    guint outsize = gst_buffer_get_size (outbuf);
-    guint skip_bytes = skip * 2 * dec->n_channels;
-
-    if (outsize > skip_bytes)
-      outsize -= skip_bytes;
-    else
-      outsize = 0;
-
-    gst_buffer_resize (outbuf, 0, outsize);
-
-    GST_INFO_OBJECT (dec,
-        "Skipping %u samples at the end (%u at 48000 Hz)", skip, scaled_skip);
+        "Skipping %u samples (%u at 48000 Hz, %u left to skip)", skip,
+        scaled_skip, dec->pre_skip);
   }
 
   if (gst_buffer_get_size (outbuf) == 0) {
@@ -658,10 +591,8 @@ gst_opus_dec_set_format (GstAudioDecoder * bdec, GstCaps * caps)
     if (header && G_VALUE_HOLDS (header, GST_TYPE_BUFFER)) {
       buf = gst_value_get_buffer (header);
       res = gst_opus_dec_parse_header (dec, buf);
-      if (res != GST_FLOW_OK) {
-        ret = FALSE;
+      if (res != GST_FLOW_OK)
         goto done;
-      }
       gst_buffer_replace (&dec->streamheader, buf);
     }
 
@@ -669,26 +600,26 @@ gst_opus_dec_set_format (GstAudioDecoder * bdec, GstCaps * caps)
     if (vorbiscomment && G_VALUE_HOLDS (vorbiscomment, GST_TYPE_BUFFER)) {
       buf = gst_value_get_buffer (vorbiscomment);
       res = gst_opus_dec_parse_comments (dec, buf);
-      if (res != GST_FLOW_OK) {
-        ret = FALSE;
+      if (res != GST_FLOW_OK)
         goto done;
-      }
       gst_buffer_replace (&dec->vorbiscomment, buf);
     }
   } else {
-    const GstAudioChannelPosition *posn = NULL;
+    /* defaults if not in the caps */
+    dec->n_channels = 2;
+    dec->sample_rate = 48000;
 
-    if (!gst_codec_utils_opus_parse_caps (caps, &dec->sample_rate,
-            &dec->n_channels, &dec->channel_mapping_family, &dec->n_streams,
-            &dec->n_stereo_streams, dec->channel_mapping)) {
-      ret = FALSE;
-      goto done;
-    }
+    gst_structure_get_int (s, "channels", &dec->n_channels);
+    gst_structure_get_int (s, "rate", &dec->sample_rate);
 
-    if (dec->channel_mapping_family == 1 && dec->n_channels <= 8)
-      posn = gst_opus_channel_positions[dec->n_channels - 1];
+    /* default stereo mapping */
+    dec->channel_mapping_family = 0;
+    dec->channel_mapping[0] = 0;
+    dec->channel_mapping[1] = 1;
+    dec->n_streams = 1;
+    dec->n_stereo_streams = 1;
 
-    gst_opus_dec_negotiate (dec, posn);
+    gst_opus_dec_negotiate (dec, NULL);
   }
 
 done:
