@@ -123,12 +123,13 @@ gst_webp_enc_class_init (GstWebpEncClass * klass)
 
   gobject_class->set_property = gst_webp_enc_set_property;
   gobject_class->get_property = gst_webp_enc_get_property;
-  gst_element_class_add_static_pad_template (element_class,
-      &webp_enc_sink_factory);
-  gst_element_class_add_static_pad_template (element_class,
-      &webp_enc_src_factory);
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&webp_enc_sink_factory));
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&webp_enc_src_factory));
   gst_element_class_set_static_metadata (element_class, "WEBP image encoder",
-      "Codec/Encoder/Image", "Encode images in WEBP format",
+      "Codec/Encoder/Image",
+      "Encode images in WEBP format",
       "Sreerenj Balachandran <sreerenjb@gnome.org>");
 
   venc_class->start = gst_webp_enc_start;
@@ -157,15 +158,11 @@ gst_webp_enc_class_init (GstWebpEncClass * klass)
 
   GST_DEBUG_CATEGORY_INIT (webpenc_debug, "webpenc", 0,
       "WEBP encoding element");
-
-  gst_type_mark_as_plugin_api (GST_WEBP_ENC_PRESET_TYPE, 0);
 }
 
 static void
 gst_webp_enc_init (GstWebpEnc * webpenc)
 {
-  GST_PAD_SET_ACCEPT_TEMPLATE (GST_VIDEO_ENCODER_SINK_PAD (webpenc));
-
   webpenc->lossless = DEFAULT_LOSSLESS;
   webpenc->quality = DEFAULT_QUALITY;
   webpenc->speed = DEFAULT_SPEED;
@@ -294,7 +291,6 @@ gst_webp_enc_handle_frame (GstVideoEncoder * encoder,
     out_buffer = gst_buffer_new_allocate (NULL, enc->webp_writer.size, NULL);
     if (!out_buffer) {
       GST_ERROR_OBJECT (enc, "Failed to create output buffer");
-      gst_video_frame_unmap (&vframe);
       return GST_FLOW_ERROR;
     }
     gst_buffer_fill (out_buffer, 0, enc->webp_writer.mem,
@@ -302,7 +298,6 @@ gst_webp_enc_handle_frame (GstVideoEncoder * encoder,
     free (enc->webp_writer.mem);
   } else {
     GST_ERROR_OBJECT (enc, "Failed to encode WebPPicture");
-    gst_video_frame_unmap (&vframe);
     return GST_FLOW_ERROR;
   }
 
