@@ -122,7 +122,6 @@ struct _GstAdaptiveDemuxPrivate
   gint64 next_update;
 
   gboolean exposing;
-  guint32 segment_seqnum;
 };
 
 static GstBinClass *parent_class = NULL;
@@ -152,7 +151,7 @@ static void gst_adaptive_demux_stream_download_loop (GstAdaptiveDemuxStream *
     stream);
 static void gst_adaptive_demux_reset (GstAdaptiveDemux * demux);
 static gboolean gst_adaptive_demux_expose_streams (GstAdaptiveDemux * demux,
-    gboolean first_and_live);
+    gboolean first_segment);
 static gboolean gst_adaptive_demux_is_live (GstAdaptiveDemux * demux);
 static GstFlowReturn gst_adaptive_demux_stream_seek (GstAdaptiveDemux * demux,
     GstAdaptiveDemuxStream * stream, GstClockTime ts);
@@ -333,9 +332,6 @@ gst_adaptive_demux_init (GstAdaptiveDemux * demux,
   demux->priv->input_adapter = gst_adapter_new ();
   demux->downloader = gst_uri_downloader_new ();
   demux->stream_struct_size = sizeof (GstAdaptiveDemuxStream);
-  demux->priv->segment_seqnum = gst_util_seqnum_next ();
-  demux->have_group_id = FALSE;
-  demux->group_id = G_MAXUINT;
 
   gst_segment_init (&demux->segment, GST_FORMAT_TIME);
 
@@ -484,7 +480,7 @@ gst_adaptive_demux_sink_event (GstPad * pad, GstObject * parent,
 
       gst_element_post_message (GST_ELEMENT_CAST (demux),
           gst_message_new_element (GST_OBJECT_CAST (demux),
-              gst_structure_new (GST_ADAPTIVE_DEMUX_STATISTICS_MESSAGE_NAME,
+              gst_structure_new (STATISTICS_MESSAGE_NAME,
                   "manifest-uri", G_TYPE_STRING,
                   demux->manifest_uri, "uri", G_TYPE_STRING,
                   demux->manifest_uri,
@@ -511,8 +507,7 @@ gst_adaptive_demux_sink_event (GstPad * pad, GstObject * parent,
         }
 
         if (demux->next_streams) {
-          gst_adaptive_demux_expose_streams (demux,
-              gst_adaptive_demux_is_live (demux));
+          gst_adaptive_demux_expose_streams (demux, TRUE);
           gst_adaptive_demux_start_tasks (demux);
           if (gst_adaptive_demux_is_live (demux)) {
             /* Task to periodically update the manifest */
@@ -597,7 +592,6 @@ gst_adaptive_demux_reset (GstAdaptiveDemux * demux)
   demux->have_group_id = FALSE;
   demux->group_id = G_MAXUINT;
   demux->priv->exposing = FALSE;
-  demux->priv->segment_seqnum = gst_util_seqnum_next ();
 }
 
 static void
@@ -724,26 +718,13 @@ gst_adaptive_demux_stream_get_presentation_offset (GstAdaptiveDemux * demux,
   return klass->get_presentation_offset (demux, stream);
 }
 
-static GstClockTime
-gst_adaptive_demux_get_period_start_time (GstAdaptiveDemux * demux)
-{
-  GstAdaptiveDemuxClass *klass;
-
-  klass = GST_ADAPTIVE_DEMUX_GET_CLASS (demux);
-
-  if (klass->get_period_start_time == NULL)
-    return 0;
-
-  return klass->get_period_start_time (demux);
-}
-
 static gboolean
 gst_adaptive_demux_expose_streams (GstAdaptiveDemux * demux,
-    gboolean first_and_live)
+    gboolean first_segment)
 {
   GList *iter;
   GList *old_streams;
-  GstClockTime period_start, min_pts = GST_CLOCK_TIME_NONE;
+  GstClockTime min_pts = GST_CLOCK_TIME_NONE;
 
   g_return_val_if_fail (demux->next_streams != NULL, FALSE);
 
@@ -760,7 +741,7 @@ gst_adaptive_demux_expose_streams (GstAdaptiveDemux * demux,
       /* TODO act on error */
     }
 
-    if (first_and_live) {
+    if (first_segment) {
       /* TODO we only need the first timestamp, maybe create a simple function */
       gst_adaptive_demux_stream_update_fragment_info (demux, stream);
 
@@ -772,94 +753,18 @@ gst_adaptive_demux_expose_streams (GstAdaptiveDemux * demux,
     }
   }
 
-  /* For live streams, the subclass is supposed to seek to the current
-   * fragment and then tell us its timestamp in stream->fragment.timestamp.
-   * We now also have to seek our demuxer segment to reflect this.
-   *
-   * FIXME: This needs some refactoring at some point.
-   */
-  if (first_and_live) {
-    gst_segment_do_seek (&demux->segment, demux->segment.rate, GST_FORMAT_TIME,
-        GST_SEEK_FLAG_FLUSH, GST_SEEK_TYPE_SET, min_pts, GST_SEEK_TYPE_NONE, -1,
-        NULL);
-  }
-
-  period_start = gst_adaptive_demux_get_period_start_time (demux);
-
+  if (first_segment)
+    demux->segment.start = demux->segment.position = demux->segment.time =
+        min_pts;
   for (iter = demux->streams; iter; iter = g_list_next (iter)) {
     GstAdaptiveDemuxStream *stream = iter->data;
     GstClockTime offset;
 
     offset = gst_adaptive_demux_stream_get_presentation_offset (demux, stream);
     stream->segment = demux->segment;
-
-    /* The demuxer segment is just built from seek events, but for each stream
-     * we have to adjust segments according to the current period and the
-     * stream specific presentation time offset.
-     *
-     * For each period, buffer timestamps start again from 0. Additionally the
-     * buffer timestamps are shifted by the stream specific presentation time
-     * offset, so the first buffer timestamp of a period is 0 + presentation
-     * time offset. If the stream contains timestamps itself, this is also
-     * supposed to be the presentation time stored inside the stream.
-     *
-     * The stream time over periods is supposed to be continuous, that is the
-     * buffer timestamp 0 + presentation time offset should map to the start
-     * time of the current period.
-     *
-     *
-     * The adjustment of the stream segments as such works the following.
-     *
-     * If the demuxer segment start is bigger than the period start, this
-     * means that we have to drop some media at the beginning of the current
-     * period, e.g. because a seek into the middle of the period has
-     * happened. The amount of media to drop is the difference between the
-     * period start and the demuxer segment start, and as each period starts
-     * again from 0, this difference is going to be the actual stream's
-     * segment start. As all timestamps of the stream are shifted by the
-     * presentation time offset, we will also have to move the segment start
-     * by that offset.
-     *
-     * Now the running time and stream time at the stream's segment start has to
-     * be the one that is stored inside the demuxer's segment, which means
-     * that segment.base and segment.time have to be copied over.
-     *
-     *
-     * If the demuxer segment start is smaller than the period start time,
-     * this means that the whole period is inside the segment. As each period
-     * starts timestamps from 0, and additionally timestamps are shifted by
-     * the presentation time offset, the stream's first timestamp (and as such
-     * the stream's segment start) has to be the presentation time offset.
-     * The stream time at the segment start is supposed to be the stream time
-     * of the period start according to the demuxer segment, so the stream
-     * segment's time would be set to that. The same goes for the stream
-     * segment's base, which is supposed to be the running time of the period
-     * start according to the demuxer's segment.
-     *
-     *
-     * For the first case where not the complete period is inside the segment,
-     * the segment time and base as calculated by the second case would be
-     * equivalent.
-     */
-
-    if (demux->segment.start > period_start) {
-      stream->segment.start = demux->segment.start - period_start + offset;
-      stream->segment.position = offset;
-      stream->segment.time = demux->segment.time;
-      stream->segment.base = demux->segment.base;
-    } else {
-      stream->segment.start = offset;
-      stream->segment.position = offset;
-      stream->segment.time =
-          gst_segment_to_stream_time (&demux->segment, GST_FORMAT_TIME,
-          period_start);
-      stream->segment.base =
-          gst_segment_to_running_time (&demux->segment, GST_FORMAT_TIME,
-          period_start);
-    }
-
+    stream->segment.start = stream->segment.position =
+        stream->fragment.timestamp + offset;
     stream->pending_segment = gst_event_new_segment (&stream->segment);
-    gst_event_set_seqnum (stream->pending_segment, demux->priv->segment_seqnum);
   }
 
   gst_element_no_more_pads (GST_ELEMENT_CAST (demux));
@@ -957,11 +862,6 @@ gst_adaptive_demux_stream_free (GstAdaptiveDemuxStream * stream)
     stream->pending_segment = NULL;
   }
 
-  if (stream->pending_events) {
-    g_list_free_full (stream->pending_events, (GDestroyNotify) gst_event_unref);
-    stream->pending_events = NULL;
-  }
-
   if (stream->src_srcpad) {
     gst_object_unref (stream->src_srcpad);
     stream->src_srcpad = NULL;
@@ -1035,6 +935,7 @@ gst_adaptive_demux_src_event (GstPad * pad, GstObject * parent,
       GstSeekType start_type, stop_type;
       gint64 start, stop;
       guint32 seqnum;
+      GList *iter;
       gboolean update;
       gboolean ret = TRUE;
       GstSegment oldsegment;
@@ -1099,11 +1000,20 @@ gst_adaptive_demux_src_event (GstPad * pad, GstObject * parent,
       GST_MANIFEST_LOCK (demux);
       ret = demux_class->seek (demux, event);
 
-      if (!ret) {
+      if (ret) {
+        GstEvent *seg_evt;
+
+        seg_evt = gst_event_new_segment (&demux->segment);
+        gst_event_set_seqnum (seg_evt, seqnum);
+        for (iter = demux->streams; iter; iter = g_list_next (iter)) {
+          GstAdaptiveDemuxStream *stream = iter->data;
+
+          gst_event_replace (&stream->pending_segment, seg_evt);
+        }
+        gst_event_unref (seg_evt);
+      } else {
         /* Is there anything else we can do if it fails? */
         gst_segment_copy_into (&oldsegment, &demux->segment);
-      } else {
-        demux->priv->segment_seqnum = seqnum;
       }
 
       if (flags & GST_SEEK_FLAG_FLUSH) {
@@ -1114,31 +1024,8 @@ gst_adaptive_demux_src_event (GstPad * pad, GstObject * parent,
         gst_event_set_seqnum (fevent, seqnum);
         gst_adaptive_demux_push_src_event (demux, fevent);
       }
-
       if (demux->next_streams) {
-        gst_adaptive_demux_expose_streams (demux, FALSE);
-      } else {
-        GList *iter;
-        GstClockTime period_start =
-            gst_adaptive_demux_get_period_start_time (demux);
-
-        for (iter = demux->streams; iter; iter = g_list_next (iter)) {
-          GstAdaptiveDemuxStream *stream = iter->data;
-          GstEvent *seg_evt;
-          GstClockTime offset;
-
-          /* See comments in gst_adaptive_demux_get_period_start_time() for
-           * an explanation of the segment modifications */
-          stream->segment = demux->segment;
-          offset =
-              gst_adaptive_demux_stream_get_presentation_offset (demux, stream);
-          stream->segment.start += offset - period_start;
-          stream->segment.position = stream->segment.start;
-          seg_evt = gst_event_new_segment (&stream->segment);
-          gst_event_set_seqnum (seg_evt, demux->priv->segment_seqnum);
-          gst_event_replace (&stream->pending_segment, seg_evt);
-          gst_event_unref (seg_evt);
-        }
+        gst_adaptive_demux_expose_streams (demux, TRUE);
       }
 
       /* Restart the demux */
@@ -1228,7 +1115,7 @@ gst_adaptive_demux_src_query (GstPad * pad, GstObject * parent,
         }
       }
       GST_MANIFEST_UNLOCK (demux);
-      GST_LOG_OBJECT (demux, "GST_QUERY_DURATION returns %s with duration %"
+      GST_DEBUG_OBJECT (demux, "GST_QUERY_DURATION returns %s with duration %"
           GST_TIME_FORMAT, ret ? "TRUE" : "FALSE", GST_TIME_ARGS (duration));
       break;
     }
@@ -1386,13 +1273,6 @@ gst_adaptive_demux_stream_set_tags (GstAdaptiveDemuxStream * stream,
   stream->pending_tags = tags;
 }
 
-void
-gst_adaptive_demux_stream_queue_event (GstAdaptiveDemuxStream * stream,
-    GstEvent * event)
-{
-  stream->pending_events = g_list_append (stream->pending_events, event);
-}
-
 static guint64
 _update_average_bitrate (GstAdaptiveDemux * demux,
     GstAdaptiveDemuxStream * stream, guint64 new_bitrate)
@@ -1485,13 +1365,10 @@ gst_adaptive_demux_stream_push_buffer (GstAdaptiveDemuxStream * stream,
   GstAdaptiveDemux *demux = stream->demux;
   GstFlowReturn ret = GST_FLOW_OK;
   gboolean discont = FALSE;
+  GstClockTime offset =
+      gst_adaptive_demux_stream_get_presentation_offset (demux, stream);
 
   if (stream->first_fragment_buffer) {
-    GstClockTime offset =
-        gst_adaptive_demux_stream_get_presentation_offset (demux, stream);
-    GstClockTime period_start =
-        gst_adaptive_demux_get_period_start_time (demux);
-
     if (demux->segment.rate < 0)
       /* Set DISCONT flag for every first buffer in reverse playback mode
        * as each fragment for its own has to be reversed */
@@ -1499,18 +1376,7 @@ gst_adaptive_demux_stream_push_buffer (GstAdaptiveDemuxStream * stream,
 
     GST_BUFFER_PTS (buffer) = stream->fragment.timestamp;
     if (GST_BUFFER_PTS_IS_VALID (buffer))
-      GST_BUFFER_PTS (buffer) += offset;
-
-    if (GST_BUFFER_PTS_IS_VALID (buffer)) {
       stream->segment.position = GST_BUFFER_PTS (buffer);
-
-      /* Convert from position inside the stream's segment to the demuxer's
-       * segment, they are not necessarily the same */
-      if (stream->segment.position - offset + period_start >
-          demux->segment.position)
-        demux->segment.position =
-            stream->segment.position - offset + period_start;
-    }
   } else {
     GST_BUFFER_PTS (buffer) = GST_CLOCK_TIME_NONE;
   }
@@ -1529,6 +1395,8 @@ gst_adaptive_demux_stream_push_buffer (GstAdaptiveDemuxStream * stream,
 
   stream->first_fragment_buffer = FALSE;
 
+  if (GST_BUFFER_PTS (buffer) != GST_CLOCK_TIME_NONE)
+    GST_BUFFER_PTS (buffer) += offset;
   GST_BUFFER_DURATION (buffer) = GST_CLOCK_TIME_NONE;
   GST_BUFFER_DTS (buffer) = GST_CLOCK_TIME_NONE;
 
@@ -1551,15 +1419,6 @@ gst_adaptive_demux_stream_push_buffer (GstAdaptiveDemuxStream * stream,
     gst_pad_push_event (stream->pad, gst_event_new_tag (stream->pending_tags));
     stream->pending_tags = NULL;
   }
-  while (stream->pending_events != NULL) {
-    GstEvent *event = stream->pending_events->data;
-
-    if (!gst_pad_push_event (stream->pad, event))
-      GST_ERROR_OBJECT (stream->pad, "Failed to send pending event");
-
-    stream->pending_events =
-        g_list_delete_link (stream->pending_events, stream->pending_events);
-  }
 
   ret = gst_pad_push (stream->pad, buffer);
   GST_LOG_OBJECT (stream->pad, "Push result: %d %s", ret,
@@ -1601,34 +1460,19 @@ _src_chain (GstPad * pad, GstObject * parent, GstBuffer * buffer)
   GstFlowReturn ret = GST_FLOW_OK;
 
   if (stream->starting_fragment) {
-    GstClockTime offset =
-        gst_adaptive_demux_stream_get_presentation_offset (demux, stream);
-    GstClockTime period_start =
-        gst_adaptive_demux_get_period_start_time (demux);
-
     stream->starting_fragment = FALSE;
     if (klass->start_fragment) {
       klass->start_fragment (demux, stream);
     }
 
     GST_BUFFER_PTS (buffer) = stream->fragment.timestamp;
-    if (GST_BUFFER_PTS_IS_VALID (buffer))
-      GST_BUFFER_PTS (buffer) += offset;
 
     GST_LOG_OBJECT (stream->pad, "set fragment pts=%" GST_TIME_FORMAT,
         GST_TIME_ARGS (GST_BUFFER_PTS (buffer)));
 
-    if (GST_BUFFER_PTS_IS_VALID (buffer)) {
+    if (GST_BUFFER_PTS_IS_VALID (buffer))
       stream->segment.position = GST_BUFFER_PTS (buffer);
 
-      /* Convert from position inside the stream's segment to the demuxer's
-       * segment, they are not necessarily the same */
-      if (stream->segment.position - offset + period_start >
-          demux->segment.position)
-        demux->segment.position =
-            stream->segment.position - offset + period_start;
-    }
-
   } else {
     GST_BUFFER_PTS (buffer) = GST_CLOCK_TIME_NONE;
   }
@@ -1675,17 +1519,16 @@ gst_adaptive_demux_stream_fragment_download_finish (GstAdaptiveDemuxStream *
   g_mutex_lock (&stream->fragment_download_lock);
   stream->download_finished = TRUE;
 
-  GST_DEBUG_OBJECT (stream->pad, "Download finish: %d %s - err: %p", ret,
-      gst_flow_get_name (ret), err);
-
   /* if we have an error, only replace last_ret if it was OK before to avoid
    * overwriting the first error we got */
-  if (stream->last_ret == GST_FLOW_OK) {
-    stream->last_ret = ret;
-    if (err) {
+  if (err) {
+    if (stream->last_ret == GST_FLOW_OK) {
+      stream->last_ret = ret;
       g_clear_error (&stream->last_error);
       stream->last_error = g_error_copy (err);
     }
+  } else {
+    stream->last_ret = ret;
   }
   g_cond_signal (&stream->fragment_download_cond);
   g_mutex_unlock (&stream->fragment_download_lock);
@@ -1745,9 +1588,6 @@ gst_adaptive_demux_stream_wait_manifest_update (GstAdaptiveDemux * demux,
       break;
     }
 
-    GST_DEBUG_OBJECT (demux, "No fragment left but live playlist, wait a bit");
-    g_cond_wait (&demux->manifest_cond, GST_MANIFEST_GET_LOCK (demux));
-
     /* Got a new fragment or not live anymore? */
     if (gst_adaptive_demux_stream_has_next_fragment (demux, stream)) {
       GST_DEBUG_OBJECT (demux, "new fragment available, "
@@ -1762,6 +1602,9 @@ gst_adaptive_demux_stream_wait_manifest_update (GstAdaptiveDemux * demux,
       ret = FALSE;
       break;
     }
+
+    GST_DEBUG_OBJECT (demux, "No fragment left but live playlist, wait a bit");
+    g_cond_wait (&demux->manifest_cond, GST_MANIFEST_GET_LOCK (demux));
   }
   GST_DEBUG_OBJECT (demux, "Retrying now");
   return ret;
@@ -1958,8 +1801,7 @@ gst_adaptive_demux_stream_download_uri (GstAdaptiveDemux * demux,
       }
       ret = stream->last_ret;
 
-      GST_DEBUG_OBJECT (stream->pad, "Fragment download finished: %s %d %s",
-          uri, stream->last_ret, gst_flow_get_name (stream->last_ret));
+      GST_DEBUG_OBJECT (stream->pad, "Fragment download finished: %s", uri);
     }
     g_mutex_unlock (&stream->fragment_download_lock);
   } else {
@@ -2053,22 +1895,12 @@ gst_adaptive_demux_stream_download_fragment (GstAdaptiveDemuxStream * stream)
         stream->last_ret, gst_flow_get_name (stream->last_ret));
     if (ret != GST_FLOW_OK) {
       /* TODO check if we are truly stoping */
-      if (ret == GST_FLOW_CUSTOM_ERROR && gst_adaptive_demux_is_live (demux)) {
+      if (ret != GST_FLOW_ERROR && gst_adaptive_demux_is_live (demux)) {
         if (++stream->download_error_count <= MAX_DOWNLOAD_ERROR_COUNT) {
           /* looks like there is no way of knowing when a live stream has ended
            * Have to assume we are falling behind and cause a manifest reload */
-          GST_DEBUG_OBJECT (stream->pad,
-              "Converting error of live stream to EOS");
           return GST_FLOW_EOS;
         }
-      } else if (ret == GST_FLOW_CUSTOM_ERROR
-          && !gst_adaptive_demux_stream_has_next_fragment (demux, stream)) {
-        /* If this is the last fragment, consider failures EOS and not actual
-         * errors. Due to rounding errors in the durations, the last fragment
-         * might not actually exist */
-        GST_DEBUG_OBJECT (stream->pad,
-            "Converting error for last fragment to EOS");
-        return GST_FLOW_EOS;
       }
     }
   }
@@ -2098,14 +1930,14 @@ gst_adaptive_demux_stream_download_loop (GstAdaptiveDemuxStream * stream)
   /* Check if we're done with our segment */
   if (demux->segment.rate > 0) {
     if (GST_CLOCK_TIME_IS_VALID (demux->segment.stop)
-        && stream->segment.position >= stream->segment.stop) {
+        && stream->segment.position >= demux->segment.stop) {
       ret = GST_FLOW_EOS;
       gst_task_stop (stream->download_task);
       goto end_of_manifest;
     }
   } else {
     if (GST_CLOCK_TIME_IS_VALID (demux->segment.start)
-        && stream->segment.position < stream->segment.start) {
+        && stream->segment.position < demux->segment.start) {
       ret = GST_FLOW_EOS;
       gst_task_stop (stream->download_task);
       goto end_of_manifest;
@@ -2143,9 +1975,7 @@ gst_adaptive_demux_stream_download_loop (GstAdaptiveDemuxStream * stream)
     GST_DEBUG_OBJECT (stream->pad,
         "Activating stream due to reconfigure event");
 
-    cur = ts =
-        gst_segment_to_stream_time (&stream->segment, GST_FORMAT_TIME,
-        stream->segment.position);
+    cur = ts = stream->segment.position;
 
     if (gst_pad_peer_query_position (stream->pad, GST_FORMAT_TIME, &pos)) {
       ts = (GstClockTime) pos;
@@ -2179,23 +2009,14 @@ gst_adaptive_demux_stream_download_loop (GstAdaptiveDemuxStream * stream)
     gst_segment_copy_into (&demux->segment, &segment);
 
     if (GST_CLOCK_TIME_IS_VALID (ts)) {
-      GstClockTime offset, period_start;
-
-      offset =
-          gst_adaptive_demux_stream_get_presentation_offset (demux, stream);
-      period_start = gst_adaptive_demux_get_period_start_time (demux);
-
       /* TODO check return */
       gst_adaptive_demux_stream_seek (demux, stream, ts);
 
-      segment.position = ts - period_start + offset;
+      if (cur < ts) {
+        segment.position = ts;
+      }
     }
-
-    /* The stream's segment is still correct except for
-     * the position, so let's send a new one with the
-     * updated position */
-    seg_event = gst_event_new_segment (&stream->segment);
-    gst_event_set_seqnum (seg_event, demux->priv->segment_seqnum);
+    seg_event = gst_event_new_segment (&segment);
     GST_DEBUG_OBJECT (stream->pad, "Sending restart segment: %"
         GST_PTR_FORMAT, seg_event);
     gst_pad_push_event (stream->pad, seg_event);
@@ -2247,8 +2068,6 @@ gst_adaptive_demux_stream_download_loop (GstAdaptiveDemuxStream * stream)
     GST_OBJECT_UNLOCK (demux);
 
     GST_MANIFEST_LOCK (demux);
-  } else {
-    stream->last_ret = ret;
   }
 
   switch (ret) {
@@ -2558,7 +2377,7 @@ gst_adaptive_demux_stream_advance_fragment_unlocked (GstAdaptiveDemux * demux,
   /* FIXME - url has no indication of byte ranges for subsegments */
   gst_element_post_message (GST_ELEMENT_CAST (demux),
       gst_message_new_element (GST_OBJECT_CAST (demux),
-          gst_structure_new (GST_ADAPTIVE_DEMUX_STATISTICS_MESSAGE_NAME,
+          gst_structure_new (STATISTICS_MESSAGE_NAME,
               "manifest-uri", G_TYPE_STRING,
               demux->manifest_uri, "uri", G_TYPE_STRING,
               stream->fragment.uri, "fragment-start-time",
@@ -2569,22 +2388,9 @@ gst_adaptive_demux_stream_advance_fragment_unlocked (GstAdaptiveDemux * demux,
               GST_TYPE_CLOCK_TIME,
               stream->download_total_time * GST_USECOND, NULL)));
 
-  if (GST_CLOCK_TIME_IS_VALID (duration)) {
-    GstClockTime offset =
-        gst_adaptive_demux_stream_get_presentation_offset (demux, stream);
-    GstClockTime period_start =
-        gst_adaptive_demux_get_period_start_time (demux);
-
+  if (GST_CLOCK_TIME_IS_VALID (duration))
     stream->segment.position += duration;
 
-    /* Convert from position inside the stream's segment to the demuxer's
-     * segment, they are not necessarily the same */
-    if (stream->segment.position - offset + period_start >
-        demux->segment.position)
-      demux->segment.position =
-          stream->segment.position - offset + period_start;
-  }
-
   if (gst_adaptive_demux_is_live (demux)
       || gst_adaptive_demux_stream_has_next_fragment (demux, stream)) {
     ret = klass->stream_advance_fragment (stream);
