@@ -117,7 +117,7 @@ _context_ready (gpointer data)
   display = gst_gl_context_get_display (GST_GL_CONTEXT (self->gst_gl_context));
   self->draw_context = gst_gl_context_new_wrapped (display,
       (guintptr) self->gl_context, GST_GL_PLATFORM_CGL,
-      gst_gl_context_get_current_gl_api (GST_GL_PLATFORM_CGL, NULL, NULL));
+      gst_gl_context_get_current_gl_api (NULL, NULL));
   gst_object_unref (display);
 
   if (!self->draw_context) {
@@ -126,7 +126,6 @@ _context_ready (gpointer data)
   }
 
   gst_gl_context_activate (self->draw_context, TRUE);
-  gst_gl_context_set_shared_with (self->draw_context, self->gst_gl_context);
   if (!gst_gl_context_fill_info (self->draw_context, &error)) {
     GST_ERROR ("failed to fill wrapped context information: %s", error->message);
     return NULL;
@@ -136,10 +135,6 @@ _context_ready (gpointer data)
   return self->gl_context;
 }
 
-- (void)queueResize {
-  self->queue_resize = TRUE;
-}
-
 - (void)releaseCGLContext:(CGLContextObj)glContext {
   CGLReleaseContext (glContext);
 }
@@ -189,12 +184,11 @@ _context_ready (gpointer data)
   gl->GetIntegerv (GL_VIEWPORT, ca_viewport);
 
   gst_gl_context_activate (self->draw_context, TRUE);
-  if (self->queue_resize || self->last_bounds.size.width != self.bounds.size.width
+  if (self->last_bounds.size.width != self.bounds.size.width
       || self->last_bounds.size.height != self.bounds.size.height) {
     if (self->resize_cb) {
-      self->resize_cb (self->resize_data, 
-          self.bounds.size.width*self.contentsScale,
-          self.bounds.size.height*self.contentsScale);
+      self->resize_cb (self->resize_data, self.bounds.size.width,
+          self.bounds.size.height);
 
       gl->GetIntegerv (GL_VIEWPORT, self->expected_dims);
     } else {
@@ -206,7 +200,6 @@ _context_ready (gpointer data)
     }
 
     self->last_bounds = self.bounds;
-    self->queue_resize = FALSE;
   }
 
   src.x = self->expected_dims[0];
