@@ -72,13 +72,11 @@ gst_gl_context_cocoa_init (GstGLContextCocoa * context)
 
 /* Must be called in the gl thread */
 GstGLContextCocoa *
-gst_gl_context_cocoa_new (GstGLDisplay * display)
+gst_gl_context_cocoa_new (void)
 {
-  if ((gst_gl_display_get_handle_type (display) & GST_GL_DISPLAY_TYPE_COCOA) == 0)
-    /* we require an cocoa display to create CGL contexts */
-    return NULL;
+  GstGLContextCocoa *context = g_object_new (GST_GL_TYPE_CONTEXT_COCOA, NULL);
 
-  return g_object_new (GST_GL_TYPE_CONTEXT_COCOA, NULL);
+  return context;
 }
 
 struct pixel_attr
@@ -90,6 +88,7 @@ struct pixel_attr
 static struct pixel_attr pixel_attrs[] = {
   {kCGLPFAAllRenderers, "All Renderers"},
   {kCGLPFADoubleBuffer, "Double Buffered"},
+  {kCGLPFAStereo, "Stereo"},
   {kCGLPFAAuxBuffers, "Aux Buffers"},
   {kCGLPFAColorSize, "Color Size"},
   {kCGLPFAAlphaSize, "Alpha Size"},
@@ -114,23 +113,20 @@ static struct pixel_attr pixel_attrs[] = {
   {kCGLPFAAcceleratedCompute, "Accelerated Compute"},
   {kCGLPFAOpenGLProfile, "OpenGL Profile"},
   {kCGLPFAVirtualScreenCount, "Virtual Screen Count"},
-#if MAC_OS_X_VERSION_MAX_ALLOWED < MAC_OS_X_VERSION_10_11
-  {kCGLPFAStereo, "Stereo"},
-#endif
-#if MAC_OS_X_VERSION_MAX_ALLOWED < MAC_OS_X_VERSION_10_9
+#if MAC_OS_X_VERSION_MAX_ALLOWED < 1090
   {kCGLPFACompliant, "Compliant"},
   {kCGLPFARemotePBuffer, "Remote PBuffer"},
   {kCGLPFASingleRenderer, "Single Renderer"},
   {kCGLPFAWindow, "Window"},
 #endif
-#if MAC_OS_X_VERSION_MAX_ALLOWED < MAC_OS_X_VERSION_10_7
+#if MAC_OS_X_VERSION_MAX_ALLOWED < 1070
 //  {kCGLPFAOffScreen, "Off Screen"},
 //  {kCGLPFAPBuffer, "PBuffer"},
 #endif
-#if MAC_OS_X_VERSION_MAX_ALLOWED < MAC_OS_X_VERSION_10_6
+#if MAC_OS_X_VERSION_MAX_ALLOWED < 1060
 //  {kCGLPFAFullScreen, "Full Screen"},
 #endif
-#if MAC_OS_X_VERSION_MAX_ALLOWED < MAC_OS_X_VERSION_10_5
+#if MAC_OS_X_VERSION_MAX_ALLOWED < 1050
 //  {kCGLPFAMPSafe, "MP Safe"},
 //  {kCGLPFAMultiScreen, "Multi Screen"},
 //  {kCGLPFARobust, "Robust"},
