@@ -29,9 +29,6 @@
  */
 
 #include "gstglcontext_egl.h"
-#include <gst/gl/egl/gstegl.h>
-#include "../utils/opengl_versions.h"
-#include "../utils/gles_versions.h"
 
 #if GST_GL_HAVE_WINDOW_X11
 #include "../x11/gstglwindow_x11.h"
@@ -102,17 +99,19 @@ gst_gl_context_egl_init (GstGLContextEGL * context)
 
 /* Must be called in the gl thread */
 GstGLContextEGL *
-gst_gl_context_egl_new (GstGLDisplay * display)
+gst_gl_context_egl_new (void)
 {
-  /* XXX: display type could theoretically be anything, as long as
-   * eglGetDisplay supports it. */
-  return g_object_new (GST_GL_TYPE_CONTEXT_EGL, NULL);
+  GstGLContextEGL *window = g_object_new (GST_GL_TYPE_CONTEXT_EGL, NULL);
+
+  return window;
 }
 
-const gchar *
-gst_gl_context_egl_get_error_string (EGLint err)
+static const gchar *
+gst_gl_context_egl_get_error_string (void)
 {
-  switch (err) {
+  EGLint nErr = eglGetError ();
+
+  switch (nErr) {
     case EGL_SUCCESS:
       return "EGL_SUCCESS";
     case EGL_BAD_DISPLAY:
@@ -150,7 +149,7 @@ static gboolean
 gst_gl_context_egl_choose_format (GstGLContext * context, GError ** error)
 {
 #if GST_GL_HAVE_WINDOW_X11
-  if (GST_IS_GL_WINDOW_X11 (context->window)) {
+  if (GST_GL_IS_WINDOW_X11 (context->window)) {
     GstGLWindow *window = gst_gl_context_get_window (context);
     GstGLWindowX11 *window_x11 = GST_GL_WINDOW_X11 (window);
     gint ret;
@@ -173,36 +172,19 @@ gst_gl_context_egl_choose_format (GstGLContext * context, GError ** error)
 }
 
 static gboolean
-gst_gl_context_egl_choose_config (GstGLContextEGL * egl, GstGLAPI gl_api,
-    gint major, GError ** error)
+gst_gl_context_egl_choose_config (GstGLContextEGL * egl,
+    GstGLContext * other_context, GError ** error)
 {
-  gboolean create_context;
   EGLint numConfigs;
   gint i = 0;
   EGLint config_attrib[20];
 
-  create_context =
-      gst_gl_check_extension ("EGL_KHR_create_context", egl->egl_exts);
-  /* silence unused warnings */
-  (void) create_context;
-
   config_attrib[i++] = EGL_SURFACE_TYPE;
   config_attrib[i++] = EGL_WINDOW_BIT;
   config_attrib[i++] = EGL_RENDERABLE_TYPE;
-  if (gl_api & GST_GL_API_GLES2) {
-    if (major == 3) {
-#if defined(EGL_KHR_create_context)
-      if (create_context) {
-        config_attrib[i++] = EGL_OPENGL_ES3_BIT_KHR;
-      } else
-#endif
-      {
-        return FALSE;
-      }
-    } else {
-      config_attrib[i++] = EGL_OPENGL_ES2_BIT;
-    }
-  } else
+  if (egl->gl_api & GST_GL_API_GLES2)
+    config_attrib[i++] = EGL_OPENGL_ES2_BIT;
+  else
     config_attrib[i++] = EGL_OPENGL_BIT;
 #if defined(USE_EGL_RPI) && GST_GL_HAVE_WINDOW_WAYLAND
   /* The configurations with a=0 seems to be buggy whereas
@@ -227,7 +209,7 @@ gst_gl_context_egl_choose_config (GstGLContextEGL * egl, GstGLAPI gl_api,
   } else {
     g_set_error (error, GST_GL_CONTEXT_ERROR, GST_GL_CONTEXT_ERROR_WRONG_CONFIG,
         "Failed to set window configuration: %s",
-        gst_gl_context_egl_get_error_string (eglGetError ()));
+        gst_gl_context_egl_get_error_string ());
     goto failure;
   }
 
@@ -237,71 +219,6 @@ failure:
   return FALSE;
 }
 
-static EGLContext
-_create_context_with_flags (GstGLContextEGL * egl, EGLContext share_context,
-    GstGLAPI gl_api, gint major, gint minor, gint contextFlags,
-    gint profileMask)
-{
-  gboolean create_context;
-#define N_ATTRIBS 20
-  gint attribs[N_ATTRIBS];
-  gint n = 0;
-
-  /* fail creation of apis/versions/flags that require EGL_KHR_create_context
-   * if the extension doesn't exist, namely:0
-   *
-   * - profile mask
-   * - context flags
-   * - GL3 > 3.1
-   * - GLES2 && minor > 0
-   */
-  create_context =
-      gst_gl_check_extension ("EGL_KHR_create_context", egl->egl_exts);
-  (void) create_context;
-  if (!create_context && (profileMask || contextFlags
-          || ((gl_api & GST_GL_API_OPENGL3)
-              && GST_GL_CHECK_GL_VERSION (major, minor, 3, 2))
-          || ((gl_api & GST_GL_API_GLES2) && minor > 0))) {
-    return 0;
-  }
-
-  GST_DEBUG_OBJECT (egl, "attempting to create OpenGL%s context version %d.%d "
-      "flags %x profile %x", gl_api & GST_GL_API_GLES2 ? " ES" : "", major,
-      minor, contextFlags, profileMask);
-
-#if defined(EGL_KHR_create_context)
-  if (create_context) {
-    if (major) {
-      attribs[n++] = EGL_CONTEXT_MAJOR_VERSION_KHR;
-      attribs[n++] = major;
-    }
-    if (minor) {
-      attribs[n++] = EGL_CONTEXT_MINOR_VERSION_KHR;
-      attribs[n++] = minor;
-    }
-    if (contextFlags) {
-      attribs[n++] = EGL_CONTEXT_FLAGS_KHR;
-      attribs[n++] = contextFlags;
-    }
-    if (profileMask) {
-      attribs[n++] = EGL_CONTEXT_OPENGL_PROFILE_MASK_KHR;
-      attribs[n++] = profileMask;
-    }
-  } else
-#endif
-  {
-    attribs[n++] = EGL_CONTEXT_CLIENT_VERSION;
-    attribs[n++] = major;
-  }
-  attribs[n++] = EGL_NONE;
-
-  g_assert (n < N_ATTRIBS);
-#undef N_ATTRIBS
-
-  return eglCreateContext (egl->egl_display, egl->egl_config, share_context,
-      attribs);
-}
-
 static gboolean
 gst_gl_context_egl_create_context (GstGLContext * context,
     GstGLAPI gl_api, GstGLContext * other_context, GError ** error)
@@ -309,8 +226,11 @@ gst_gl_context_egl_create_context (GstGLContext * context,
   GstGLContextEGL *egl;
   GstGLWindow *window = NULL;
   EGLNativeWindowType window_handle = (EGLNativeWindowType) 0;
-  EGLint egl_major;
-  EGLint egl_minor;
+  gint i = 0;
+  EGLint context_attrib[5];
+  EGLint majorVersion;
+  EGLint minorVersion;
+  const gchar *egl_exts;
   gboolean need_surface = TRUE;
   guintptr external_gl_context = 0;
   GstGLDisplay *display;
@@ -330,8 +250,7 @@ gst_gl_context_egl_create_context (GstGLContext * context,
     external_gl_context = gst_gl_context_get_gl_context (other_context);
   }
 
-  if ((gl_api & (GST_GL_API_OPENGL | GST_GL_API_OPENGL3 | GST_GL_API_GLES2)) ==
-      GST_GL_API_NONE) {
+  if ((gl_api & (GST_GL_API_OPENGL | GST_GL_API_GLES2)) == GST_GL_API_NONE) {
     g_set_error (error, GST_GL_CONTEXT_ERROR, GST_GL_CONTEXT_ERROR_WRONG_API,
         "EGL supports opengl or gles2");
     goto failure;
@@ -363,33 +282,27 @@ gst_gl_context_egl_create_context (GstGLContext * context,
   }
   gst_object_unref (display);
 
-  if (eglInitialize (egl->egl_display, &egl_major, &egl_minor)) {
-    GST_INFO ("egl initialized, version: %d.%d", egl_major, egl_minor);
+  if (eglInitialize (egl->egl_display, &majorVersion, &minorVersion)) {
+    GST_INFO ("egl initialized, version: %d.%d", majorVersion, minorVersion);
   } else {
     g_set_error (error, GST_GL_CONTEXT_ERROR,
         GST_GL_CONTEXT_ERROR_RESOURCE_UNAVAILABLE,
-        "Failed to initialize egl: %s",
-        gst_gl_context_egl_get_error_string (eglGetError ()));
+        "Failed to initialize egl: %s", gst_gl_context_egl_get_error_string ());
     goto failure;
   }
 
-  egl->egl_exts = eglQueryString (egl->egl_display, EGL_EXTENSIONS);
-
-  if (gl_api & (GST_GL_API_OPENGL | GST_GL_API_OPENGL3)) {
-    GstGLAPI chosen_gl_api = 0;
-    gint i;
-
+  if (gl_api & GST_GL_API_OPENGL) {
     /* egl + opengl only available with EGL 1.4+ */
-    if (egl_major == 1 && egl_minor <= 3) {
+    if (majorVersion == 1 && minorVersion <= 3) {
       if ((gl_api & ~GST_GL_API_OPENGL) == GST_GL_API_NONE) {
         g_set_error (error, GST_GL_CONTEXT_ERROR, GST_GL_CONTEXT_ERROR_OLD_LIBS,
             "EGL version (%i.%i) too old for OpenGL support, (needed at least 1.4)",
-            egl_major, egl_minor);
+            majorVersion, minorVersion);
         goto failure;
       } else {
         GST_WARNING
             ("EGL version (%i.%i) too old for OpenGL support, (needed at least 1.4)",
-            egl_major, egl_minor);
+            majorVersion, minorVersion);
         if (gl_api & GST_GL_API_GLES2) {
           goto try_gles2;
         } else {
@@ -404,117 +317,67 @@ gst_gl_context_egl_create_context (GstGLContext * context,
     if (!eglBindAPI (EGL_OPENGL_API)) {
       g_set_error (error, GST_GL_CONTEXT_ERROR, GST_GL_CONTEXT_ERROR_FAILED,
           "Failed to bind OpenGL API: %s",
-          gst_gl_context_egl_get_error_string (eglGetError ()));
+          gst_gl_context_egl_get_error_string ());
       goto failure;
     }
 
-    GST_INFO ("Bound OpenGL");
-
-    /* api, version only matters for gles */
-    if (!gst_gl_context_egl_choose_config (egl, GST_GL_API_OPENGL, 0, error)) {
-      g_assert (error == NULL || *error != NULL);
-      goto failure;
-    }
-
-    for (i = 0; i < G_N_ELEMENTS (opengl_versions); i++) {
-      gint profileMask = 0;
-      gint contextFlags = 0;
-
-      if (GST_GL_CHECK_GL_VERSION (opengl_versions[i].major,
-              opengl_versions[i].minor, 3, 2)) {
-        /* skip gl3 contexts if requested */
-        if ((gl_api & GST_GL_API_OPENGL3) == 0)
-          continue;
-
-        chosen_gl_api = GST_GL_API_OPENGL3;
-#if defined(EGL_KHR_create_context)
-        profileMask |= EGL_CONTEXT_OPENGL_CORE_PROFILE_BIT_KHR;
-        contextFlags |= EGL_CONTEXT_OPENGL_DEBUG_BIT_KHR;
-#endif
-      } else if (opengl_versions[i].major == 3 && opengl_versions[i].minor == 1) {
-        /* skip 3.1, the implementation is free to give us either a core or a
-         * compatibility context (we have no say) */
-        continue;
-      } else {
-        /* skip legacy contexts if requested */
-        if ((gl_api & GST_GL_API_OPENGL) == 0)
-          continue;
-
-        chosen_gl_api = GST_GL_API_OPENGL;
-      }
-
-      egl->egl_context =
-          _create_context_with_flags (egl, (EGLContext) external_gl_context,
-          chosen_gl_api, opengl_versions[i].major,
-          opengl_versions[i].minor, contextFlags, profileMask);
-
-      if (egl->egl_context)
-        break;
-
-#if defined(EGL_KHR_create_context)
-      profileMask &= ~EGL_CONTEXT_OPENGL_DEBUG_BIT_KHR;
-
-      egl->egl_context =
-          _create_context_with_flags (egl, (EGLContext) external_gl_context,
-          chosen_gl_api, opengl_versions[i].major,
-          opengl_versions[i].minor, contextFlags, profileMask);
-
-      if (egl->egl_context)
-        break;
-#endif
-    }
-
-    egl->gl_api = chosen_gl_api;
+    GST_INFO ("Using OpenGL");
+    egl->gl_api = GST_GL_API_OPENGL;
   } else if (gl_api & GST_GL_API_GLES2) {
-    gint i;
-
   try_gles2:
     if (!eglBindAPI (EGL_OPENGL_ES_API)) {
       g_set_error (error, GST_GL_CONTEXT_ERROR, GST_GL_CONTEXT_ERROR_FAILED,
           "Failed to bind OpenGL|ES API: %s",
-          gst_gl_context_egl_get_error_string (eglGetError ()));
+          gst_gl_context_egl_get_error_string ());
       goto failure;
     }
 
-    GST_INFO ("Bound OpenGL|ES");
-
-    for (i = 0; i < G_N_ELEMENTS (gles2_versions); i++) {
-      gint profileMask = 0;
-      gint contextFlags = 0;
-      guint maj = gles2_versions[i].major;
-      guint min = gles2_versions[i].minor;
-
-      if (!gst_gl_context_egl_choose_config (egl, GST_GL_API_GLES2, maj, error)) {
-        GST_DEBUG_OBJECT (context, "Failed to choose a GLES%d config: %s",
-            maj, error && *error ? (*error)->message : "Unknown");
-        g_clear_error (error);
-        continue;
-      }
-#if defined(EGL_KHR_create_context)
-      /* try a debug context */
-      contextFlags |= EGL_CONTEXT_OPENGL_DEBUG_BIT_KHR;
-
-      egl->egl_context =
-          _create_context_with_flags (egl, (EGLContext) external_gl_context,
-          GST_GL_API_GLES2, maj, min, contextFlags, profileMask);
-
-      if (egl->egl_context)
-        break;
-
-      /* try without a debug context */
-      contextFlags &= ~EGL_CONTEXT_OPENGL_DEBUG_BIT_KHR;
-#endif
-
-      egl->egl_context =
-          _create_context_with_flags (egl, (EGLContext) external_gl_context,
-          GST_GL_API_GLES2, maj, min, contextFlags, profileMask);
-
-      if (egl->egl_context)
-        break;
-    }
+    GST_INFO ("Using OpenGL|ES 2.0");
     egl->gl_api = GST_GL_API_GLES2;
   }
 
+  if (!gst_gl_context_egl_choose_config (egl, other_context, error)) {
+    g_assert (error == NULL || *error != NULL);
+    goto failure;
+  }
+
+  egl_exts = eglQueryString (egl->egl_display, EGL_EXTENSIONS);
+
+  GST_DEBUG ("about to create gl context");
+
+  if (egl->gl_api & GST_GL_API_GLES2) {
+    context_attrib[i++] = EGL_CONTEXT_CLIENT_VERSION;
+    context_attrib[i++] = 2;
+  }
+#if !defined(GST_DISABLE_GST_DEBUG) && defined(EGL_KHR_create_context)
+  if (gst_gl_check_extension ("EGL_KHR_create_context", egl_exts)) {
+    context_attrib[i++] = EGL_CONTEXT_FLAGS_KHR;
+    context_attrib[i++] = EGL_CONTEXT_OPENGL_DEBUG_BIT_KHR;
+  }
+#endif
+  context_attrib[i++] = EGL_NONE;
+
+  egl->egl_context =
+      eglCreateContext (egl->egl_display, egl->egl_config,
+      (EGLContext) external_gl_context, context_attrib);
+
+#ifdef EGL_KHR_create_context
+  if (egl->egl_context == EGL_NO_CONTEXT && egl->gl_api & GST_GL_API_GLES2
+      && eglGetError () != EGL_SUCCESS) {
+    /* try without EGL_CONTEXT_FLAGS flags as it was added to
+     * EGL_KHR_create_context for gles contexts */
+    int i;
+    for (i = 0; i < G_N_ELEMENTS (context_attrib); i++) {
+      if (context_attrib[i] == EGL_CONTEXT_FLAGS_KHR ||
+          context_attrib[i] == EGL_CONTEXT_OPENGL_DEBUG_BIT_KHR)
+        context_attrib[i] = EGL_NONE;
+    }
+    egl->egl_context =
+        eglCreateContext (egl->egl_display, egl->egl_config,
+        (EGLContext) external_gl_context, context_attrib);
+  }
+#endif
+
   if (egl->egl_context != EGL_NO_CONTEXT) {
     GST_INFO ("gl context created: %" G_GUINTPTR_FORMAT,
         (guintptr) egl->egl_context);
@@ -522,30 +385,30 @@ gst_gl_context_egl_create_context (GstGLContext * context,
     g_set_error (error, GST_GL_CONTEXT_ERROR,
         GST_GL_CONTEXT_ERROR_CREATE_CONTEXT,
         "Failed to create a OpenGL context: %s",
-        gst_gl_context_egl_get_error_string (eglGetError ()));
+        gst_gl_context_egl_get_error_string ());
     goto failure;
   }
-  /* FIXME do we want a window vfunc ? */
-#if GST_GL_HAVE_WINDOW_X11
-  if (GST_IS_GL_WINDOW_X11 (context->window)) {
-    gst_gl_window_x11_create_window ((GstGLWindowX11 *) context->window);
-  }
-#endif
 
   if (other_context == NULL) {
+    /* FIXME do we want a window vfunc ? */
+#if GST_GL_HAVE_WINDOW_X11
+    if (GST_GL_IS_WINDOW_X11 (context->window)) {
+      gst_gl_window_x11_create_window ((GstGLWindowX11 *) context->window);
+    }
+#endif
 #if GST_GL_HAVE_WINDOW_WAYLAND
-    if (GST_IS_GL_WINDOW_WAYLAND_EGL (context->window)) {
+    if (GST_GL_IS_WINDOW_WAYLAND_EGL (context->window)) {
       gst_gl_window_wayland_egl_create_window ((GstGLWindowWaylandEGL *)
           context->window);
     }
 #endif
 #if GST_GL_HAVE_WINDOW_WIN32
-    if (GST_IS_GL_WINDOW_WIN32 (context->window)) {
+    if (GST_GL_IS_WINDOW_WIN32 (context->window)) {
       gst_gl_window_win32_create_window ((GstGLWindowWin32 *) context->window);
     }
 #endif
 #if GST_GL_HAVE_WINDOW_DISPMANX
-    if (GST_IS_GL_WINDOW_DISPMANX_EGL (context->window)) {
+    if (GST_GL_IS_WINDOW_DISPMANX_EGL (context->window)) {
       gst_gl_window_dispmanx_egl_create_window ((GstGLWindowDispmanxEGL *)
           context->window);
     }
@@ -564,8 +427,7 @@ gst_gl_context_egl_create_context (GstGLContext * context,
         window_handle, NULL);
     /* Store window handle for later comparision */
     egl->window_handle = window_handle;
-  } else if (!gst_gl_check_extension ("EGL_KHR_surfaceless_context",
-          egl->egl_exts)) {
+  } else if (!gst_gl_check_extension ("EGL_KHR_surfaceless_context", egl_exts)) {
     EGLint surface_attrib[7];
     gint j = 0;
 
@@ -595,34 +457,28 @@ gst_gl_context_egl_create_context (GstGLContext * context,
     } else {
       g_set_error (error, GST_GL_CONTEXT_ERROR, GST_GL_CONTEXT_ERROR_FAILED,
           "Failed to create window surface: %s",
-          gst_gl_context_egl_get_error_string (eglGetError ()));
+          gst_gl_context_egl_get_error_string ());
       goto failure;
     }
   }
 
   /* EGLImage functions */
-  if (GST_GL_CHECK_GL_VERSION (egl_major, egl_minor, 1, 5)) {
+  if (GST_GL_CHECK_GL_VERSION (majorVersion, minorVersion, 1, 5)) {
     egl->eglCreateImage = gst_gl_context_get_proc_address (context,
         "eglCreateImage");
     egl->eglDestroyImage = gst_gl_context_get_proc_address (context,
         "eglDestroyImage");
-    if (egl->eglCreateImage == NULL || egl->eglDestroyImage == NULL) {
-      egl->eglCreateImage = NULL;
-      egl->eglDestroyImage = NULL;
-    }
-  } else if (gst_gl_check_extension ("EGL_KHR_image_base", egl->egl_exts)) {
-    egl->eglCreateImageKHR = gst_gl_context_get_proc_address (context,
+  } else if (gst_gl_check_extension ("EGL_KHR_image_base", egl_exts)) {
+    egl->eglCreateImage = gst_gl_context_get_proc_address (context,
         "eglCreateImageKHR");
     egl->eglDestroyImage = gst_gl_context_get_proc_address (context,
         "eglDestroyImageKHR");
-    if (egl->eglCreateImageKHR == NULL || egl->eglDestroyImage == NULL) {
-      egl->eglCreateImageKHR = NULL;
-      egl->eglDestroyImage = NULL;
-    }
   }
-  egl->egl_major = egl_major;
-  egl->egl_minor = egl_minor;
- 
+  if (egl->eglCreateImage == NULL || egl->eglDestroyImage == NULL) {
+    egl->eglCreateImage = NULL;
+    egl->eglDestroyImage = NULL;
+  }
+
   if (window)
     gst_object_unref (window);
 
@@ -683,7 +539,7 @@ gst_gl_context_egl_activate (GstGLContext * context, gboolean activate)
         egl->egl_surface = EGL_NO_SURFACE;
         if (!result) {
           GST_ERROR_OBJECT (context, "Failed to destroy old window surface: %s",
-              gst_gl_context_egl_get_error_string (eglGetError ()));
+              gst_gl_context_egl_get_error_string ());
           goto done;
         }
       }
@@ -694,22 +550,21 @@ gst_gl_context_egl_activate (GstGLContext * context, gboolean activate)
 
       if (egl->egl_surface == EGL_NO_SURFACE) {
         GST_ERROR_OBJECT (context, "Failed to create window surface: %s",
-            gst_gl_context_egl_get_error_string (eglGetError ()));
+            gst_gl_context_egl_get_error_string ());
         result = FALSE;
         goto done;
       }
     }
     result = eglMakeCurrent (egl->egl_display, egl->egl_surface,
         egl->egl_surface, egl->egl_context);
-  } else {
+  } else
     result = eglMakeCurrent (egl->egl_display, EGL_NO_SURFACE,
         EGL_NO_SURFACE, EGL_NO_CONTEXT);
-  }
 
   if (!result) {
     GST_ERROR_OBJECT (context,
         "Failed to bind context to the current rendering thread: %s",
-        gst_gl_context_egl_get_error_string (eglGetError ()));
+        gst_gl_context_egl_get_error_string ());
   }
 
 done:
@@ -754,7 +609,7 @@ load_egl_module (gpointer user_data)
 #else
   /* On Linux the .so is only in -dev packages, try with a real soname
    * Proper compilers will optimize away the strcmp */
-  if (g_strcmp0 (G_MODULE_SUFFIX, "so") == 0)
+  if (strcmp (G_MODULE_SUFFIX, "so") == 0)
     module_egl = g_module_open ("libEGL.so.1", G_MODULE_BIND_LAZY);
 
   /* This automatically handles the suffix and even .la files */
@@ -766,40 +621,13 @@ load_egl_module (gpointer user_data)
 }
 
 gpointer
-gst_gl_context_egl_get_proc_address (GstGLAPI gl_api, const gchar * name)
+gst_gl_context_egl_get_proc_address (GstGLContext * context, const gchar * name)
 {
   gpointer result = NULL;
   static GOnce g_once = G_ONCE_INIT;
+  GstGLAPI gl_api = gst_gl_context_get_gl_api (context);
 
-#ifdef __APPLE__
-#if GST_GL_HAVE_OPENGL && !defined(GST_GL_LIBGL_MODULE_NAME)
-  if (!result && (gl_api & (GST_GL_API_OPENGL | GST_GL_API_OPENGL3))) {
-    static GModule *module_opengl = NULL;
-    if (g_once_init_enter (&module_opengl)) {
-      GModule *setup_module_opengl =
-          g_module_open ("libGL.dylib", G_MODULE_BIND_LAZY);
-      g_once_init_leave (&module_opengl, setup_module_opengl);
-    }
-    if (module_opengl)
-      g_module_symbol (module_opengl, name, &result);
-  }
-#endif
-#if GST_GL_HAVE_GLES2 && !defined(GST_GL_LIBGLESV2_MODULE_NAME)
-  if (!result && (gl_api & (GST_GL_API_GLES2))) {
-    static GModule *module_gles2 = NULL;
-    if (g_once_init_enter (&module_gles2)) {
-      GModule *setup_module_gles2 =
-          g_module_open ("libGLESv2.dylib", G_MODULE_BIND_LAZY);
-      g_once_init_leave (&module_gles2, setup_module_gles2);
-    }
-    if (module_gles2)
-      g_module_symbol (module_gles2, name, &result);
-  }
-#endif
-#endif // __APPLE__
-
-  if (!result)
-    result = gst_gl_context_default_get_proc_address (gl_api, name);
+  result = gst_gl_context_default_get_proc_address (gl_api, name);
 
   g_once (&g_once, load_egl_module, NULL);
 
@@ -825,11 +653,7 @@ gst_gl_context_egl_check_feature (GstGLContext * context, const gchar * feature)
   GstGLContextEGL *context_egl = GST_GL_CONTEXT_EGL (context);
 
   if (g_strcmp0 (feature, "EGL_KHR_image_base") == 0) {
-    if (GST_GL_CHECK_GL_VERSION (context_egl->egl_major, context_egl->egl_minor, 1, 5))
-      return context_egl->eglCreateImage != NULL &&
-        context_egl->eglDestroyImage != NULL;
-    else
-      return context_egl->eglCreateImageKHR != NULL &&
+    return context_egl->eglCreateImage != NULL &&
         context_egl->eglDestroyImage != NULL;
   }
 
