@@ -31,28 +31,20 @@
 #include <gst/gl/gstglapi.h>
 #include <gst/gl/gstgldisplay.h>
 #include <gst/gl/gstglcontext.h>
-#include <gst/gl/gstgldebug.h>
 #include <gst/gl/gstglfeature.h>
-#include <gst/gl/gstglformat.h>
 #include <gst/gl/gstglutils.h>
 #include <gst/gl/gstglwindow.h>
-#include <gst/gl/gstglslstage.h>
 #include <gst/gl/gstglshader.h>
-#include <gst/gl/gstglshaderstrings.h>
 #include <gst/gl/gstglcolorconvert.h>
 #include <gst/gl/gstglupload.h>
-#include <gst/gl/gstglbasememory.h>
-#include <gst/gl/gstglbuffer.h>
+#include <gst/gl/gstgluploadmeta.h>
+#include <gst/gl/gstgldownload.h>
 #include <gst/gl/gstglmemory.h>
-#include <gst/gl/gstglmemorypbo.h>
 #include <gst/gl/gstglbufferpool.h>
 #include <gst/gl/gstglframebuffer.h>
 #include <gst/gl/gstglbasefilter.h>
-#include <gst/gl/gstglviewconvert.h>
 #include <gst/gl/gstglfilter.h>
+#include <gst/gl/gstglshadervariables.h>
 #include <gst/gl/gstglsyncmeta.h>
-#include <gst/gl/gstgloverlaycompositor.h>
-#include <gst/gl/gstglquery.h>
-#include <gst/gl/gstglcontrolbindingproxy.h>
 
 #endif /* __GST_GL_H__ */
