@@ -24,8 +24,7 @@
 #include "gstglapi.h"
 
 /**
- * gst_gl_api_to_string():
- *
+ * gst_gl_api_to_string:
  * @api: a #GstGLAPI to stringify
  *
  * Returns: A space seperated string of the OpenGL api's enabled in @api
@@ -78,11 +77,10 @@ out:
 }
 
 /**
- * gst_gl_api_from_string():
+ * gst_gl_api_from_string:
+ * @api_s: a space seperated string of OpenGL apis
  *
- * @apis_s: a space seperated string of OpenGL apis
- *
- * Returns: The #GstGLAPI represented by @apis_s
+ * Returns: The #GstGLAPI represented by @api_s
  */
 GstGLAPI
 gst_gl_api_from_string (const gchar * apis_s)
@@ -121,8 +119,7 @@ gst_gl_api_from_string (const gchar * apis_s)
 }
 
 /**
- * gst_gl_platform_to_string():
- *
+ * gst_gl_platform_to_string:
  * @api: a #GstGLPlatform to stringify
  *
  * Returns: A space seperated string of the OpenGL platforms enabled in @api
@@ -155,6 +152,9 @@ gst_gl_platform_to_string (GstGLPlatform platform)
   if (platform & GST_GL_PLATFORM_CGL) {
     str = g_string_append (str, "cgl ");
   }
+  if (platform & GST_GL_PLATFORM_GPU_PROCESS) {
+    str = g_string_append (str, "gpu_process ");
+  }
 
 out:
   if (!str)
@@ -165,8 +165,7 @@ out:
 }
 
 /**
- * gst_gl_platform_from_string():
- *
+ * gst_gl_platform_from_string:
  * @platform_s: a space seperated string of OpenGL platformss
  *
  * Returns: The #GstGLPlatform represented by @platform_s
@@ -197,6 +196,9 @@ gst_gl_platform_from_string (const gchar * platform_s)
       } else if (g_strstr_len (platform, 3, "cgl")) {
         ret |= GST_GL_PLATFORM_CGL;
         platform = &platform[3];
+      } else if (g_strstr_len (platform, 11, "gpu_process")) {
+        ret |= GST_GL_PLATFORM_GPU_PROCESS;
+        platform = &platform[11];
       } else {
         GST_ERROR ("Error parsing \'%s\'", platform);
         break;
