@@ -26,30 +26,37 @@
 #include "gstglbufferpool.h"
 #include "gstglutils.h"
 
+#if GST_GL_HAVE_PLATFORM_EGL
+#include <gst/gl/egl/gsteglimagememory.h>
+#endif
+
 /**
  * SECTION:gstglbufferpool
- * @title: GstGlBufferPool
- * @short_description: buffer pool for #GstGLBaseMemory objects
- * @see_also: #GstBufferPool, #GstGLBaseMemory, #GstGLMemory
+ * @short_description: buffer pool for #GstGLMemory objects
+ * @see_also: #GstBufferPool, #GstGLMemory
  *
- * a #GstGLBufferPool is an object that allocates buffers with #GstGLBaseMemory
+ * a #GstGLBufferPool is an object that allocates buffers with #GstGLMemory
  *
  * A #GstGLBufferPool is created with gst_gl_buffer_pool_new()
  *
- * #GstGLBufferPool implements the VideoMeta buffer pool option
- * %GST_BUFFER_POOL_OPTION_VIDEO_META, the VideoAligment buffer pool option
- * %GST_BUFFER_POOL_OPTION_VIDEO_ALIGNMENT as well as the OpenGL specific
- * %GST_BUFFER_POOL_OPTION_GL_SYNC_META buffer pool option.
+ * #GstGLBufferPool implements the VideoMeta buffer pool option 
+ * #GST_BUFFER_POOL_OPTION_VIDEO_META
  */
 
 /* bufferpool */
 struct _GstGLBufferPoolPrivate
 {
   GstAllocator *allocator;
-  GstGLVideoAllocationParams *gl_params;
+  GstAllocationParams params;
   GstCaps *caps;
+  gint im_format;
+  GstVideoInfo info;
+  GstVideoAlignment valign;
   gboolean add_videometa;
+  gboolean add_uploadmeta;
   gboolean add_glsyncmeta;
+  gboolean want_eglimage;
+  GstBuffer *last_buffer;
 };
 
 static void gst_gl_buffer_pool_finalize (GObject * object);
@@ -69,10 +76,9 @@ static const gchar **
 gst_gl_buffer_pool_get_options (GstBufferPool * pool)
 {
   static const gchar *options[] = { GST_BUFFER_POOL_OPTION_VIDEO_META,
+    GST_BUFFER_POOL_OPTION_VIDEO_GL_TEXTURE_UPLOAD_META,
     GST_BUFFER_POOL_OPTION_GL_SYNC_META,
     GST_BUFFER_POOL_OPTION_VIDEO_ALIGNMENT,
-    GST_BUFFER_POOL_OPTION_GL_TEXTURE_TARGET_2D,
-    GST_BUFFER_POOL_OPTION_GL_TEXTURE_TARGET_RECTANGLE,
     NULL
   };
 
@@ -87,11 +93,9 @@ gst_gl_buffer_pool_set_config (GstBufferPool * pool, GstStructure * config)
   GstVideoInfo info;
   GstCaps *caps = NULL;
   guint min_buffers, max_buffers;
-  guint max_align, n;
   GstAllocator *allocator = NULL;
   GstAllocationParams alloc_params;
-  GstGLTextureTarget tex_target;
-  gboolean ret = TRUE;
+  gboolean reset = TRUE;
   gint p;
 
   if (!gst_buffer_pool_config_get_params (config, &caps, NULL, &min_buffers,
@@ -111,124 +115,75 @@ gst_gl_buffer_pool_set_config (GstBufferPool * pool, GstStructure * config)
   if (!gst_buffer_pool_config_get_allocator (config, &allocator, &alloc_params))
     goto wrong_config;
 
-  gst_caps_replace (&priv->caps, caps);
-
   if (priv->allocator)
     gst_object_unref (priv->allocator);
 
-  if (allocator) {
-    if (!GST_IS_GL_MEMORY_ALLOCATOR (allocator)) {
-      gst_object_unref (allocator);
-      goto wrong_allocator;
-    } else {
-      priv->allocator = gst_object_ref (allocator);
-    }
+  if (!allocator) {
+    gst_gl_memory_init ();
+    priv->allocator = gst_allocator_find (GST_GL_MEMORY_ALLOCATOR);
   } else {
-    priv->allocator =
-        GST_ALLOCATOR (gst_gl_memory_allocator_get_default (glpool->context));
-    g_assert (priv->allocator);
+    priv->allocator = gst_object_ref (allocator);
   }
 
+  priv->params = alloc_params;
+
+  priv->im_format = GST_VIDEO_INFO_FORMAT (&info);
+  if (priv->im_format == -1)
+    goto unknown_format;
+
+  if (priv->caps)
+    reset = !gst_caps_is_equal (priv->caps, caps);
+
+  gst_caps_replace (&priv->caps, caps);
+  priv->info = info;
+
   priv->add_videometa = gst_buffer_pool_config_has_option (config,
       GST_BUFFER_POOL_OPTION_VIDEO_META);
+  priv->add_uploadmeta = gst_buffer_pool_config_has_option (config,
+      GST_BUFFER_POOL_OPTION_VIDEO_GL_TEXTURE_UPLOAD_META);
   priv->add_glsyncmeta = gst_buffer_pool_config_has_option (config,
       GST_BUFFER_POOL_OPTION_GL_SYNC_META);
 
-  if (priv->gl_params)
-    gst_gl_allocation_params_free ((GstGLAllocationParams *) priv->gl_params);
-  priv->gl_params = (GstGLVideoAllocationParams *)
-      gst_buffer_pool_config_get_gl_allocation_params (config);
-  if (!priv->gl_params)
-    priv->gl_params = gst_gl_video_allocation_params_new (glpool->context,
-        &alloc_params, &info, -1, NULL, 0, 0);
-
-  max_align = alloc_params.align;
+#if GST_GL_HAVE_PLATFORM_EGL
+  g_assert (priv->allocator != NULL);
+  priv->want_eglimage =
+      (g_strcmp0 (priv->allocator->mem_type, GST_EGL_IMAGE_MEMORY_TYPE) == 0);
+#else
+  priv->want_eglimage = FALSE;
+#endif
 
   if (gst_buffer_pool_config_has_option (config,
           GST_BUFFER_POOL_OPTION_VIDEO_ALIGNMENT)) {
+
     priv->add_videometa = TRUE;
 
-    gst_buffer_pool_config_get_video_alignment (config,
-        priv->gl_params->valign);
+    gst_buffer_pool_config_get_video_alignment (config, &priv->valign);
+    gst_video_info_align (&priv->info, &priv->valign);
 
-    for (n = 0; n < GST_VIDEO_MAX_PLANES; ++n)
-      max_align |= priv->gl_params->valign->stride_align[n];
-
-    for (n = 0; n < GST_VIDEO_MAX_PLANES; ++n)
-      priv->gl_params->valign->stride_align[n] = max_align;
-
-    gst_video_info_align (priv->gl_params->v_info, priv->gl_params->valign);
-
-    gst_buffer_pool_config_set_video_alignment (config,
-        priv->gl_params->valign);
+    gst_buffer_pool_config_set_video_alignment (config, &priv->valign);
+  } else {
+    gst_video_alignment_reset (&priv->valign);
   }
 
-  if (alloc_params.align < max_align) {
-    GST_WARNING_OBJECT (pool, "allocation params alignment %u is smaller "
-        "than the max specified video stride alignment %u, fixing",
-        (guint) alloc_params.align, max_align);
+  if (reset) {
+    if (glpool->upload)
+      gst_object_unref (glpool->upload);
 
-    alloc_params.align = max_align;
-    gst_buffer_pool_config_set_allocator (config, allocator, &alloc_params);
-    if (priv->gl_params->parent.alloc_params)
-      gst_allocation_params_free (priv->gl_params->parent.alloc_params);
-    priv->gl_params->parent.alloc_params =
-        gst_allocation_params_copy (&alloc_params);
-  }
-
-  {
-    GstStructure *s = gst_caps_get_structure (caps, 0);
-    const gchar *target_str = gst_structure_get_string (s, "texture-target");
-    gboolean multiple_texture_targets = FALSE;
-
-    tex_target = priv->gl_params->target;
-    if (target_str)
-      tex_target = gst_gl_texture_target_from_string (target_str);
-
-    if (gst_buffer_pool_config_has_option (config,
-            GST_BUFFER_POOL_OPTION_GL_TEXTURE_TARGET_2D)) {
-      if (tex_target && tex_target != GST_GL_TEXTURE_TARGET_2D)
-        multiple_texture_targets = TRUE;
-      tex_target = GST_GL_TEXTURE_TARGET_2D;
-    }
-    if (gst_buffer_pool_config_has_option (config,
-            GST_BUFFER_POOL_OPTION_GL_TEXTURE_TARGET_RECTANGLE)) {
-      if (tex_target && tex_target != GST_GL_TEXTURE_TARGET_RECTANGLE)
-        multiple_texture_targets = TRUE;
-      tex_target = GST_GL_TEXTURE_TARGET_RECTANGLE;
-    }
-    if (gst_buffer_pool_config_has_option (config,
-            GST_BUFFER_POOL_OPTION_GL_TEXTURE_TARGET_EXTERNAL_OES)) {
-      if (tex_target && tex_target != GST_GL_TEXTURE_TARGET_EXTERNAL_OES)
-        multiple_texture_targets = TRUE;
-      tex_target = GST_GL_TEXTURE_TARGET_EXTERNAL_OES;
-    }
-
-    if (!tex_target)
-      tex_target = GST_GL_TEXTURE_TARGET_2D;
-
-    if (multiple_texture_targets) {
-      GST_WARNING_OBJECT (pool, "Multiple texture targets configured either "
-          "through caps or buffer pool options");
-      ret = FALSE;
-    }
-
-    priv->gl_params->target = tex_target;
+    glpool->upload = gst_gl_upload_meta_new (glpool->context);
   }
 
   /* Recalulate the size and offset as we don't add padding between planes. */
-  priv->gl_params->v_info->size = 0;
-  for (p = 0; p < GST_VIDEO_INFO_N_PLANES (priv->gl_params->v_info); p++) {
-    priv->gl_params->v_info->offset[p] = priv->gl_params->v_info->size;
-    priv->gl_params->v_info->size +=
-        gst_gl_get_plane_data_size (priv->gl_params->v_info,
-        priv->gl_params->valign, p);
+  priv->info.size = 0;
+  for (p = 0; p < GST_VIDEO_INFO_N_PLANES (&priv->info); p++) {
+    priv->info.offset[p] = priv->info.size;
+    priv->info.size +=
+        gst_gl_get_plane_data_size (&priv->info, &priv->valign, p);
   }
 
-  gst_buffer_pool_config_set_params (config, caps,
-      priv->gl_params->v_info->size, min_buffers, max_buffers);
+  gst_buffer_pool_config_set_params (config, caps, priv->info.size,
+      min_buffers, max_buffers);
 
-  return GST_BUFFER_POOL_CLASS (parent_class)->set_config (pool, config) && ret;
+  return GST_BUFFER_POOL_CLASS (parent_class)->set_config (pool, config);
 
   /* ERRORS */
 wrong_config:
@@ -247,9 +202,14 @@ wrong_caps:
         "failed getting geometry from caps %" GST_PTR_FORMAT, caps);
     return FALSE;
   }
-wrong_allocator:
+unknown_format:
   {
-    GST_WARNING_OBJECT (pool, "Incorrect allocator type for this pool");
+    GST_WARNING_OBJECT (glpool, "failed to get format from caps %"
+        GST_PTR_FORMAT, caps);
+    GST_ELEMENT_ERROR (glpool, RESOURCE, WRITE,
+        ("Failed to create output image buffer of %dx%d pixels",
+            priv->info.width, priv->info.height),
+        ("Invalid input caps %" GST_PTR_FORMAT, caps));
     return FALSE;
   }
 }
@@ -257,6 +217,11 @@ wrong_allocator:
 static gboolean
 gst_gl_buffer_pool_start (GstBufferPool * pool)
 {
+  GstGLBufferPool *glpool = GST_GL_BUFFER_POOL_CAST (pool);
+  GstGLBufferPoolPrivate *priv = glpool->priv;
+
+  gst_gl_upload_meta_set_format (glpool->upload, &priv->info);
+
   return GST_BUFFER_POOL_CLASS (parent_class)->start (pool);
 }
 
@@ -265,19 +230,38 @@ static GstFlowReturn
 gst_gl_buffer_pool_alloc (GstBufferPool * pool, GstBuffer ** buffer,
     GstBufferPoolAcquireParams * params)
 {
-  GstGLMemoryAllocator *alloc;
   GstGLBufferPool *glpool = GST_GL_BUFFER_POOL_CAST (pool);
   GstGLBufferPoolPrivate *priv = glpool->priv;
+  GstVideoInfo *info;
+  GstVideoAlignment *valign;
   GstBuffer *buf;
 
+  info = &priv->info;
+  valign = &priv->valign;
+
   if (!(buf = gst_buffer_new ())) {
     goto no_buffer;
   }
+#if GST_GL_HAVE_PLATFORM_EGL
+  if (priv->want_eglimage) {
+    /* alloc and append memories, also add video_meta and
+     * texture_upload_meta */
+    if (!gst_egl_image_memory_setup_buffer (glpool->context, info, buf))
+      goto egl_image_mem_create_failed;
 
-  alloc = GST_GL_MEMORY_ALLOCATOR (priv->allocator);
-  if (!gst_gl_memory_setup_buffer (alloc, buf, priv->gl_params, NULL, NULL, 0))
+    *buffer = buf;
+
+    return GST_FLOW_OK;
+  }
+#endif
+
+  if (!gst_gl_memory_setup_buffer (glpool->context, &priv->params, info,
+          valign, buf))
     goto mem_create_failed;
 
+  if (priv->add_uploadmeta)
+    gst_gl_upload_meta_add_to_buffer (glpool->upload, buf);
+
   if (priv->add_glsyncmeta)
     gst_buffer_add_gl_sync_meta (glpool->context, buf);
 
@@ -296,6 +280,48 @@ mem_create_failed:
     GST_WARNING_OBJECT (pool, "Could not create GL Memory");
     return GST_FLOW_ERROR;
   }
+
+#if GST_GL_HAVE_PLATFORM_EGL
+egl_image_mem_create_failed:
+  {
+    GST_WARNING_OBJECT (pool, "Could not create EGLImage Memory");
+    return GST_FLOW_ERROR;
+  }
+#endif
+}
+
+
+static GstFlowReturn
+gst_gl_buffer_pool_acquire_buffer (GstBufferPool * bpool,
+    GstBuffer ** buffer, GstBufferPoolAcquireParams * params)
+{
+  GstFlowReturn ret = GST_FLOW_OK;
+  GstGLBufferPool *glpool = NULL;
+
+  ret =
+      GST_BUFFER_POOL_CLASS
+      (gst_gl_buffer_pool_parent_class)->acquire_buffer (bpool, buffer, params);
+  if (ret != GST_FLOW_OK || !*buffer)
+    return ret;
+
+  glpool = GST_GL_BUFFER_POOL (bpool);
+
+  /* XXX: Don't return the memory we just rendered, glEGLImageTargetTexture2DOES()
+   * keeps the EGLImage unmappable until the next one is uploaded
+   */
+  if (glpool->priv->want_eglimage && *buffer
+      && *buffer == glpool->priv->last_buffer) {
+    GstBuffer *oldbuf = *buffer;
+
+    ret =
+        GST_BUFFER_POOL_CLASS
+        (gst_gl_buffer_pool_parent_class)->acquire_buffer (bpool,
+        buffer, params);
+    gst_object_replace ((GstObject **) & oldbuf->pool, (GstObject *) glpool);
+    gst_buffer_unref (oldbuf);
+  }
+
+  return ret;
 }
 
 /**
@@ -318,6 +344,24 @@ gst_gl_buffer_pool_new (GstGLContext * context)
   return GST_BUFFER_POOL_CAST (pool);
 }
 
+/**
+ * gst_gl_buffer_pool_replace_last_buffer:
+ * @pool: a #GstGLBufferPool
+ * @buffer: a #GstBuffer
+ *
+ * Set @pool<--  -->s last buffer to @buffer for #GstGLPlatform<--  -->s that
+ * require it.
+ */
+void
+gst_gl_buffer_pool_replace_last_buffer (GstGLBufferPool * pool,
+    GstBuffer * buffer)
+{
+  g_return_if_fail (pool != NULL);
+  g_return_if_fail (buffer != NULL);
+
+  gst_buffer_replace (&pool->priv->last_buffer, buffer);
+}
+
 static void
 gst_gl_buffer_pool_class_init (GstGLBufferPoolClass * klass)
 {
@@ -331,6 +375,7 @@ gst_gl_buffer_pool_class_init (GstGLBufferPoolClass * klass)
   gstbufferpool_class->get_options = gst_gl_buffer_pool_get_options;
   gstbufferpool_class->set_config = gst_gl_buffer_pool_set_config;
   gstbufferpool_class->alloc_buffer = gst_gl_buffer_pool_alloc;
+  gstbufferpool_class->acquire_buffer = gst_gl_buffer_pool_acquire_buffer;
   gstbufferpool_class->start = gst_gl_buffer_pool_start;
 }
 
@@ -344,8 +389,14 @@ gst_gl_buffer_pool_init (GstGLBufferPool * pool)
 
   priv->allocator = NULL;
   priv->caps = NULL;
+  priv->im_format = GST_VIDEO_FORMAT_UNKNOWN;
   priv->add_videometa = TRUE;
   priv->add_glsyncmeta = FALSE;
+  priv->want_eglimage = FALSE;
+  priv->last_buffer = FALSE;
+
+  gst_video_info_init (&priv->info);
+  gst_allocation_params_init (&priv->params);
 }
 
 static void
@@ -356,9 +407,14 @@ gst_gl_buffer_pool_finalize (GObject * object)
 
   GST_LOG_OBJECT (pool, "finalize GL buffer pool %p", pool);
 
+  gst_buffer_replace (&pool->priv->last_buffer, NULL);
+
   if (priv->caps)
     gst_caps_unref (priv->caps);
 
+  if (pool->upload)
+    gst_object_unref (pool->upload);
+
   G_OBJECT_CLASS (gst_gl_buffer_pool_parent_class)->finalize (object);
 
   /* only release the context once all our memory have been deleted */
@@ -371,44 +427,4 @@ gst_gl_buffer_pool_finalize (GObject * object)
     gst_object_unref (priv->allocator);
     priv->allocator = NULL;
   }
-
-  if (priv->gl_params)
-    gst_gl_allocation_params_free ((GstGLAllocationParams *) priv->gl_params);
-  priv->gl_params = NULL;
-}
-
-/**
- * gst_buffer_pool_config_get_gl_allocation_params:
- * @config: a buffer pool config
- *
- * Returns: (transfer full): the currently set #GstGLAllocationParams or %NULL
- */
-GstGLAllocationParams *
-gst_buffer_pool_config_get_gl_allocation_params (GstStructure * config)
-{
-  GstGLAllocationParams *ret;
-
-  if (!gst_structure_get (config, "gl-allocation-params",
-          GST_TYPE_GL_ALLOCATION_PARAMS, &ret, NULL))
-    ret = NULL;
-
-  return ret;
-}
-
-/**
- * gst_buffer_pool_config_set_gl_allocation_params:
- * @config: a buffer pool config
- * @params: (transfer none): a #GstGLAllocationParams
- *
- * Sets @params on @config
- */
-void
-gst_buffer_pool_config_set_gl_allocation_params (GstStructure * config,
-    GstGLAllocationParams * params)
-{
-  g_return_if_fail (config != NULL);
-  g_return_if_fail (params != NULL);
-
-  gst_structure_set (config, "gl-allocation-params",
-      GST_TYPE_GL_ALLOCATION_PARAMS, params, NULL);
 }
