@@ -58,9 +58,6 @@
 #if GST_GL_HAVE_WINDOW_EAGL
 #include "eagl/gstglwindow_eagl.h"
 #endif
-#if GST_GL_HAVE_WINDOW_VIV_FB
-#include "viv-fb/gstglwindow_viv_fb_egl.h"
-#endif
 #if GST_GL_HAVE_WINDOW_DISPMANX
 #include "dispmanx/gstglwindow_dispmanx_egl.h"
 #endif
@@ -78,7 +75,7 @@ GST_DEBUG_CATEGORY (GST_CAT_DEFAULT);
 G_DEFINE_ABSTRACT_TYPE (GstGLWindow, gst_gl_window, GST_TYPE_OBJECT);
 
 #define GST_GL_WINDOW_GET_PRIVATE(o) \
-  (G_TYPE_INSTANCE_GET_PRIVATE((o), GST_TYPE_GL_WINDOW, GstGLWindowPrivate))
+  (G_TYPE_INSTANCE_GET_PRIVATE((o), GST_GL_TYPE_WINDOW, GstGLWindowPrivate))
 
 static void gst_gl_window_default_draw (GstGLWindow * window);
 static void gst_gl_window_default_run (GstGLWindow * window);
@@ -87,18 +84,19 @@ static void gst_gl_window_default_send_message (GstGLWindow * window,
     GstGLWindowCB callback, gpointer data);
 static void gst_gl_window_default_send_message_async (GstGLWindow * window,
     GstGLWindowCB callback, gpointer data, GDestroyNotify destroy);
+static gpointer gst_gl_window_navigation_thread (GstGLWindow * window);
+void gst_gl_window_run_navigation (GstGLWindow * window);
+void gst_gl_window_open_navigation (GstGLWindow * window);
+void gst_gl_window_close_navigation (GstGLWindow * window);
+void gst_gl_window_quit_navigation (GstGLWindow * window);
 
 struct _GstGLWindowPrivate
 {
+  GMainContext *main_context;
   GMainLoop *loop;
-
-  guint surface_width;
-  guint surface_height;
+  GThread *navigation_thread;
 
   gboolean alive;
-
-  GMutex sync_message_lock;
-  GCond sync_message_cond;
 };
 
 static void gst_gl_window_finalize (GObject * object);
@@ -136,12 +134,35 @@ gst_gl_window_error_quark (void)
 static gboolean
 gst_gl_window_default_open (GstGLWindow * window, GError ** error)
 {
+  GstGLWindowPrivate *priv = window->priv;
+
+  if (g_main_context_get_thread_default ()) {
+    if (priv->main_context)
+      g_main_context_unref (priv->main_context);
+    if (priv->loop)
+      g_main_loop_unref (priv->loop);
+    priv->main_context = g_main_context_ref_thread_default ();
+    priv->loop = NULL;
+    priv->alive = TRUE;
+  } else {
+    g_main_context_push_thread_default (priv->main_context);
+  }
+
   return TRUE;
 }
 
 static void
 gst_gl_window_default_close (GstGLWindow * window)
 {
+  GstGLWindowPrivate *priv = window->priv;
+
+  if (!priv->loop) {
+    priv->alive = FALSE;
+    g_main_context_unref (priv->main_context);
+    priv->main_context = NULL;
+  } else {
+    g_main_context_pop_thread_default (priv->main_context);
+  }
 }
 
 static void
@@ -163,15 +184,17 @@ gst_gl_window_init (GstGLWindow * window)
   window->priv = priv;
 
   g_mutex_init (&window->lock);
+  g_mutex_init (&window->nav_lock);
+  g_cond_init (&window->nav_create_cond);
+  g_cond_init (&window->nav_destroy_cond);
+  window->nav_created = FALSE;
+  window->nav_alive = FALSE;
   window->is_drawing = FALSE;
 
   g_weak_ref_init (&window->context_ref, NULL);
 
-  g_mutex_init (&window->priv->sync_message_lock);
-  g_cond_init (&window->priv->sync_message_cond);
-
-  window->main_context = g_main_context_new ();
-  priv->loop = g_main_loop_new (window->main_context, FALSE);
+  priv->main_context = g_main_context_new ();
+  priv->loop = g_main_loop_new (priv->main_context, FALSE);
 }
 
 static void
@@ -183,6 +206,7 @@ gst_gl_window_class_init (GstGLWindowClass * klass)
   klass->close = GST_DEBUG_FUNCPTR (gst_gl_window_default_close);
   klass->run = GST_DEBUG_FUNCPTR (gst_gl_window_default_run);
   klass->quit = GST_DEBUG_FUNCPTR (gst_gl_window_default_quit);
+  klass->draw_unlocked = GST_DEBUG_FUNCPTR (gst_gl_window_default_draw);
   klass->draw = GST_DEBUG_FUNCPTR (gst_gl_window_default_draw);
   klass->send_message = GST_DEBUG_FUNCPTR (gst_gl_window_default_send_message);
   klass->send_message_async =
@@ -248,7 +272,7 @@ gst_gl_window_new (GstGLDisplay * display)
 
 #if GST_GL_HAVE_WINDOW_COCOA
   if (!window && (!user_choice || g_strstr_len (user_choice, 5, "cocoa")))
-    window = GST_GL_WINDOW (gst_gl_window_cocoa_new (display));
+    window = GST_GL_WINDOW (gst_gl_window_cocoa_new ());
 #endif
 #if GST_GL_HAVE_WINDOW_X11
   if (!window && (!user_choice || g_strstr_len (user_choice, 3, "x11")))
@@ -256,29 +280,24 @@ gst_gl_window_new (GstGLDisplay * display)
 #endif
 #if GST_GL_HAVE_WINDOW_WIN32
   if (!window && (!user_choice || g_strstr_len (user_choice, 5, "win32")))
-    window = GST_GL_WINDOW (gst_gl_window_win32_new (display));
+    window = GST_GL_WINDOW (gst_gl_window_win32_new ());
 #endif
 #if GST_GL_HAVE_WINDOW_WAYLAND
   if (!window && (!user_choice || g_strstr_len (user_choice, 7, "wayland")))
-    window = GST_GL_WINDOW (gst_gl_window_wayland_egl_new (display));
+    window = GST_GL_WINDOW (gst_gl_window_wayland_egl_new ());
 #endif
 #if GST_GL_HAVE_WINDOW_DISPMANX
   if (!window && (!user_choice || g_strstr_len (user_choice, 8, "dispmanx")))
-    window = GST_GL_WINDOW (gst_gl_window_dispmanx_egl_new (display));
+    window = GST_GL_WINDOW (gst_gl_window_dispmanx_egl_new ());
 #endif
 #if GST_GL_HAVE_WINDOW_ANDROID
   if (!window && (!user_choice || g_strstr_len (user_choice, 7, "android")))
-    window = GST_GL_WINDOW (gst_gl_window_android_egl_new (display));
+    window = GST_GL_WINDOW (gst_gl_window_android_egl_new ());
 #endif
 #if GST_GL_HAVE_WINDOW_EAGL
   if (!window && (!user_choice || g_strstr_len (user_choice, 4, "eagl")))
-    window = GST_GL_WINDOW (gst_gl_window_eagl_new (display));
+    window = GST_GL_WINDOW (gst_gl_window_eagl_new ());
 #endif
-#if GST_GL_HAVE_WINDOW_VIV_FB
-  if (!window && (!user_choice || g_strstr_len (user_choice, 2, "viv-fb")))
-    window = GST_GL_WINDOW (gst_gl_window_viv_fb_egl_new (display));
-#endif
-
   if (!window) {
     /* subclass returned a NULL window */
     GST_WARNING ("Could not create window. user specified %s, creating dummy"
@@ -289,6 +308,17 @@ gst_gl_window_new (GstGLDisplay * display)
 
   window->display = gst_object_ref (display);
 
+  g_mutex_lock (&window->nav_lock);
+
+  if (!window->nav_created) {
+    window->priv->navigation_thread = g_thread_new ("gstglnavigation",
+        (GThreadFunc) gst_gl_window_navigation_thread, window);
+
+    g_cond_wait (&window->nav_create_cond, &window->nav_lock);
+    window->nav_created = TRUE;
+  }
+  g_mutex_unlock (&window->nav_lock);
+
   return window;
 }
 
@@ -298,18 +328,28 @@ gst_gl_window_finalize (GObject * object)
   GstGLWindow *window = GST_GL_WINDOW (object);
   GstGLWindowPrivate *priv = window->priv;
 
+  if (window->nav_alive) {
+    g_mutex_lock (&window->nav_lock);
+    GST_INFO ("send quit navigation loop");
+    gst_gl_window_quit_navigation (window);
+    while (window->nav_alive) {
+      g_cond_wait (&window->nav_destroy_cond, &window->nav_lock);
+    }
+    g_mutex_unlock (&window->nav_lock);
+  }
+
   if (priv->loop)
     g_main_loop_unref (priv->loop);
 
-  if (window->main_context)
-    g_main_context_unref (window->main_context);
-  window->main_context = NULL;
+  if (priv->main_context)
+    g_main_context_unref (priv->main_context);
 
   g_weak_ref_clear (&window->context_ref);
 
   g_mutex_clear (&window->lock);
-  g_mutex_clear (&window->priv->sync_message_lock);
-  g_cond_clear (&window->priv->sync_message_cond);
+  g_mutex_clear (&window->nav_lock);
+  g_cond_clear (&window->nav_create_cond);
+  g_cond_clear (&window->nav_destroy_cond);
   gst_object_unref (window->display);
 
   G_OBJECT_CLASS (gst_gl_window_parent_class)->finalize (object);
@@ -373,7 +413,7 @@ gst_gl_window_set_window_handle (GstGLWindow * window, guintptr handle)
   GstGLWindowClass *window_class;
   GstSetWindowHandleCb *data;
 
-  g_return_if_fail (GST_IS_GL_WINDOW (window));
+  g_return_if_fail (GST_GL_IS_WINDOW (window));
   g_return_if_fail (handle != 0);
   window_class = GST_GL_WINDOW_GET_CLASS (window);
   g_return_if_fail (window_class->set_window_handle != NULL);
@@ -397,13 +437,6 @@ draw_cb (gpointer data)
   GstGLContext *context = gst_gl_window_get_context (window);
   GstGLContextClass *context_class = GST_GL_CONTEXT_GET_CLASS (context);
 
-  if (window->queue_resize) {
-    guint width, height;
-
-    gst_gl_window_get_surface_dimensions (window, &width, &height);
-    gst_gl_window_resize (window, width, height);
-  }
-
   if (window->draw)
     window->draw (window->draw_data);
 
@@ -419,6 +452,27 @@ gst_gl_window_default_draw (GstGLWindow * window)
   gst_gl_window_send_message (window, (GstGLWindowCB) draw_cb, window);
 }
 
+
+/**
+ * gst_gl_window_draw_unlocked:
+ * @window: a #GstGLWindow
+ *
+ * Redraw the window contents.  Implementations should invoke the draw callback.
+ *
+ * Since: 1.4
+ */
+void
+gst_gl_window_draw_unlocked (GstGLWindow * window)
+{
+  GstGLWindowClass *window_class;
+
+  g_return_if_fail (GST_GL_IS_WINDOW (window));
+  window_class = GST_GL_WINDOW_GET_CLASS (window);
+  g_return_if_fail (window_class->draw_unlocked != NULL);
+
+  window_class->draw_unlocked (window);
+}
+
 /**
  * gst_gl_window_draw:
  * @window: a #GstGLWindow
@@ -432,7 +486,7 @@ gst_gl_window_draw (GstGLWindow * window)
 {
   GstGLWindowClass *window_class;
 
-  g_return_if_fail (GST_IS_GL_WINDOW (window));
+  g_return_if_fail (GST_GL_IS_WINDOW (window));
   window_class = GST_GL_WINDOW_GET_CLASS (window);
   g_return_if_fail (window_class->draw != NULL);
 
@@ -460,7 +514,7 @@ gst_gl_window_set_preferred_size (GstGLWindow * window, gint width, gint height)
 {
   GstGLWindowClass *window_class;
 
-  g_return_if_fail (GST_IS_GL_WINDOW (window));
+  g_return_if_fail (GST_GL_IS_WINDOW (window));
   window_class = GST_GL_WINDOW_GET_CLASS (window);
 
   if (window_class->set_preferred_size)
@@ -480,7 +534,7 @@ gst_gl_window_show (GstGLWindow * window)
 {
   GstGLWindowClass *window_class;
 
-  g_return_if_fail (GST_IS_GL_WINDOW (window));
+  g_return_if_fail (GST_GL_IS_WINDOW (window));
   window_class = GST_GL_WINDOW_GET_CLASS (window);
 
   if (window_class->show)
@@ -490,13 +544,7 @@ gst_gl_window_show (GstGLWindow * window)
 static void
 gst_gl_window_default_run (GstGLWindow * window)
 {
-  GstGLWindowPrivate *priv = window->priv;
-
-  g_main_context_push_thread_default (window->main_context);
-
-  g_main_loop_run (priv->loop);
-
-  g_main_context_pop_thread_default (window->main_context);
+  g_main_loop_run (window->priv->loop);
 }
 
 /**
@@ -512,7 +560,7 @@ gst_gl_window_run (GstGLWindow * window)
 {
   GstGLWindowClass *window_class;
 
-  g_return_if_fail (GST_IS_GL_WINDOW (window));
+  g_return_if_fail (GST_GL_IS_WINDOW (window));
   window_class = GST_GL_WINDOW_GET_CLASS (window);
   g_return_if_fail (window_class->run != NULL);
 
@@ -520,6 +568,23 @@ gst_gl_window_run (GstGLWindow * window)
   window_class->run (window);
 }
 
+/**
+ * gst_gl_window_run_navigation:
+ * @window: a #GstGLWindow
+ *
+ * Start the execution of the navigation runloop.
+ *
+ * Since: 1.6
+ */
+void
+gst_gl_window_run_navigation (GstGLWindow * window)
+{
+  g_return_if_fail (GST_GL_IS_WINDOW (window));
+  g_return_if_fail (window->navigation_context != NULL);
+  g_return_if_fail (window->navigation_loop != NULL);
+  g_main_loop_run (window->navigation_loop);
+}
+
 static void
 gst_gl_window_default_quit (GstGLWindow * window)
 {
@@ -539,7 +604,7 @@ gst_gl_window_quit (GstGLWindow * window)
 {
   GstGLWindowClass *window_class;
 
-  g_return_if_fail (GST_IS_GL_WINDOW (window));
+  g_return_if_fail (GST_GL_IS_WINDOW (window));
   window_class = GST_GL_WINDOW_GET_CLASS (window);
   g_return_if_fail (window_class->quit != NULL);
 
@@ -556,7 +621,8 @@ gst_gl_window_quit (GstGLWindow * window)
 
 typedef struct _GstGLSyncMessage
 {
-  GstGLWindow *window;
+  GMutex lock;
+  GCond cond;
   gboolean fired;
 
   GstGLWindowCB callback;
@@ -566,14 +632,14 @@ typedef struct _GstGLSyncMessage
 static void
 _run_message_sync (GstGLSyncMessage * message)
 {
+  g_mutex_lock (&message->lock);
 
   if (message->callback)
     message->callback (message->data);
 
-  g_mutex_lock (&message->window->priv->sync_message_lock);
   message->fired = TRUE;
-  g_cond_broadcast (&message->window->priv->sync_message_cond);
-  g_mutex_unlock (&message->window->priv->sync_message_lock);
+  g_cond_signal (&message->cond);
+  g_mutex_unlock (&message->lock);
 }
 
 void
@@ -582,21 +648,24 @@ gst_gl_window_default_send_message (GstGLWindow * window,
 {
   GstGLSyncMessage message;
 
-  message.window = window;
   message.callback = callback;
   message.data = data;
   message.fired = FALSE;
+  g_mutex_init (&message.lock);
+  g_cond_init (&message.cond);
 
   gst_gl_window_send_message_async (window, (GstGLWindowCB) _run_message_sync,
       &message, NULL);
 
-  g_mutex_lock (&window->priv->sync_message_lock);
+  g_mutex_lock (&message.lock);
 
   /* block until opengl calls have been executed in the gl thread */
   while (!message.fired)
-    g_cond_wait (&window->priv->sync_message_cond,
-        &window->priv->sync_message_lock);
-  g_mutex_unlock (&window->priv->sync_message_lock);
+    g_cond_wait (&message.cond, &message.lock);
+  g_mutex_unlock (&message.lock);
+
+  g_mutex_clear (&message.lock);
+  g_cond_clear (&message.cond);
 }
 
 /**
@@ -616,7 +685,7 @@ gst_gl_window_send_message (GstGLWindow * window, GstGLWindowCB callback,
 {
   GstGLWindowClass *window_class;
 
-  g_return_if_fail (GST_IS_GL_WINDOW (window));
+  g_return_if_fail (GST_GL_IS_WINDOW (window));
   g_return_if_fail (callback != NULL);
   window_class = GST_GL_WINDOW_GET_CLASS (window);
   g_return_if_fail (window_class->send_message != NULL);
@@ -649,13 +718,14 @@ static void
 gst_gl_window_default_send_message_async (GstGLWindow * window,
     GstGLWindowCB callback, gpointer data, GDestroyNotify destroy)
 {
+  GstGLWindowPrivate *priv = window->priv;
   GstGLAsyncMessage *message = g_slice_new (GstGLAsyncMessage);
 
   message->callback = callback;
   message->data = data;
   message->destroy = destroy;
 
-  g_main_context_invoke (window->main_context, (GSourceFunc) _run_message_async,
+  g_main_context_invoke (priv->main_context, (GSourceFunc) _run_message_async,
       message);
 }
 
@@ -664,7 +734,7 @@ gst_gl_window_default_send_message_async (GstGLWindow * window,
  * @window: a #GstGLWindow
  * @callback: (scope async): function to invoke
  * @data: (closure): data to invoke @callback with
- * @destroy: called when @data is not needed anymore
+ * @destroy: (destroy): called when @data is not needed anymore
  *
  * Invoke @callback with @data on the window thread.  The callback may not
  * have been executed when this function returns.
@@ -677,7 +747,7 @@ gst_gl_window_send_message_async (GstGLWindow * window, GstGLWindowCB callback,
 {
   GstGLWindowClass *window_class;
 
-  g_return_if_fail (GST_IS_GL_WINDOW (window));
+  g_return_if_fail (GST_GL_IS_WINDOW (window));
   g_return_if_fail (callback != NULL);
   window_class = GST_GL_WINDOW_GET_CLASS (window);
   g_return_if_fail (window_class->send_message_async != NULL);
@@ -690,7 +760,7 @@ gst_gl_window_send_message_async (GstGLWindow * window, GstGLWindowCB callback,
  * @window: a #GstGLWindow
  * @callback: (scope notified): function to invoke
  * @data: (closure): data to invoke @callback with
- * @destroy_notify: called when @data is not needed any more
+ * @destroy_notify: (destroy): called when @data is not needed any more
  *
  * Sets the draw callback called everytime gst_gl_window_draw() is called
  *
@@ -700,7 +770,7 @@ void
 gst_gl_window_set_draw_callback (GstGLWindow * window, GstGLWindowCB callback,
     gpointer data, GDestroyNotify destroy_notify)
 {
-  g_return_if_fail (GST_IS_GL_WINDOW (window));
+  g_return_if_fail (GST_GL_IS_WINDOW (window));
 
   GST_GL_WINDOW_LOCK (window);
 
@@ -719,7 +789,7 @@ gst_gl_window_set_draw_callback (GstGLWindow * window, GstGLWindowCB callback,
  * @window: a #GstGLWindow
  * @callback: (scope notified): function to invoke
  * @data: (closure): data to invoke @callback with
- * @destroy_notify: called when @data is not needed any more
+ * @destroy_notify: (destroy): called when @data is not needed any more
  *
  * Sets the resize callback called everytime a resize of the window occurs.
  *
@@ -729,7 +799,7 @@ void
 gst_gl_window_set_resize_callback (GstGLWindow * window,
     GstGLWindowResizeCB callback, gpointer data, GDestroyNotify destroy_notify)
 {
-  g_return_if_fail (GST_IS_GL_WINDOW (window));
+  g_return_if_fail (GST_GL_IS_WINDOW (window));
 
   GST_GL_WINDOW_LOCK (window);
 
@@ -748,7 +818,7 @@ gst_gl_window_set_resize_callback (GstGLWindow * window,
  * @window: a #GstGLWindow
  * @callback: (scope notified): function to invoke
  * @data: (closure): data to invoke @callback with
- * @destroy_notify: called when @data is not needed any more
+ * @destroy_notify: (destroy): called when @data is not needed any more
  *
  * Sets the callback called when the window is about to close.
  *
@@ -758,7 +828,7 @@ void
 gst_gl_window_set_close_callback (GstGLWindow * window, GstGLWindowCB callback,
     gpointer data, GDestroyNotify destroy_notify)
 {
-  g_return_if_fail (GST_IS_GL_WINDOW (window));
+  g_return_if_fail (GST_GL_IS_WINDOW (window));
 
   GST_GL_WINDOW_LOCK (window);
 
@@ -772,6 +842,22 @@ gst_gl_window_set_close_callback (GstGLWindow * window, GstGLWindowCB callback,
   GST_GL_WINDOW_UNLOCK (window);
 }
 
+/**
+ * gst_gl_window_is_running:
+ * @window: a #GstGLWindow
+ *
+ * Whether the runloop is running
+ *
+ * Since: 1.4
+ */
+gboolean
+gst_gl_window_is_running (GstGLWindow * window)
+{
+  GstGLWindowPrivate *priv = window->priv;
+  return priv->alive && (!priv->loop
+      || g_main_loop_is_running (window->priv->loop));
+}
+
 /**
  * gst_gl_window_get_display:
  * @window: a #GstGLWindow
@@ -785,7 +871,7 @@ gst_gl_window_get_display (GstGLWindow * window)
 {
   GstGLWindowClass *window_class;
 
-  g_return_val_if_fail (GST_IS_GL_WINDOW (window), 0);
+  g_return_val_if_fail (GST_GL_IS_WINDOW (window), 0);
   window_class = GST_GL_WINDOW_GET_CLASS (window);
   g_return_val_if_fail (window_class->get_display != NULL, 0);
 
@@ -805,7 +891,7 @@ gst_gl_window_get_window_handle (GstGLWindow * window)
 {
   GstGLWindowClass *window_class;
 
-  g_return_val_if_fail (GST_IS_GL_WINDOW (window), 0);
+  g_return_val_if_fail (GST_GL_IS_WINDOW (window), 0);
   window_class = GST_GL_WINDOW_GET_CLASS (window);
   g_return_val_if_fail (window_class->get_window_handle != NULL, 0);
 
@@ -823,7 +909,7 @@ gst_gl_window_get_window_handle (GstGLWindow * window)
 GstGLContext *
 gst_gl_window_get_context (GstGLWindow * window)
 {
-  g_return_val_if_fail (GST_IS_GL_WINDOW (window), NULL);
+  g_return_val_if_fail (GST_GL_IS_WINDOW (window), NULL);
 
   return (GstGLContext *) g_weak_ref_get (&window->context_ref);
 }
@@ -840,10 +926,132 @@ void
 gst_gl_window_get_surface_dimensions (GstGLWindow * window, guint * width,
     guint * height)
 {
-  if (width)
-    *width = window->priv->surface_width;
-  if (height)
-    *height = window->priv->surface_height;
+  GstGLWindowClass *window_class;
+  g_return_if_fail (GST_GL_IS_WINDOW (window));
+  window_class = GST_GL_WINDOW_GET_CLASS (window);
+  g_return_if_fail (window_class->get_surface_dimensions != NULL);
+  window_class->get_surface_dimensions (window, width, height);
+}
+
+GType gst_gl_dummy_window_get_type (void);
+
+G_DEFINE_TYPE (GstGLDummyWindow, gst_gl_dummy_window, GST_GL_TYPE_WINDOW);
+
+void
+gst_gl_window_open_navigation (GstGLWindow * window)
+{
+  g_return_if_fail (GST_GL_IS_WINDOW (window));
+  g_mutex_lock (&window->nav_lock);
+  window->navigation_context = g_main_context_new ();
+  window->navigation_loop = g_main_loop_new (window->navigation_context, FALSE);
+  g_main_context_push_thread_default (window->navigation_context);
+  window->nav_alive = TRUE;
+  g_cond_signal (&window->nav_create_cond);
+  g_mutex_unlock (&window->nav_lock);
+}
+
+void
+gst_gl_window_close_navigation (GstGLWindow * window)
+{
+  g_return_if_fail (GST_GL_IS_WINDOW (window));
+  g_return_if_fail (window->navigation_context != NULL);
+  g_return_if_fail (window->navigation_loop != NULL);
+
+  g_mutex_lock (&window->nav_lock);
+  window->nav_alive = FALSE;
+  g_main_context_pop_thread_default (window->navigation_context);
+  g_main_loop_unref (window->navigation_loop);
+  g_main_context_unref (window->navigation_context);
+  g_cond_signal (&window->nav_destroy_cond);
+  g_mutex_unlock (&window->nav_lock);
+}
+
+void
+gst_gl_window_quit_navigation (GstGLWindow * window)
+{
+  g_return_if_fail (GST_GL_IS_WINDOW (window));
+
+  g_main_loop_quit (window->navigation_loop);
+}
+
+static gpointer
+gst_gl_window_navigation_thread (GstGLWindow * window)
+{
+  gst_gl_window_open_navigation (window);
+  gst_gl_window_run_navigation (window);
+  GST_INFO ("navigation loop exited\n");
+  gst_gl_window_close_navigation (window);
+
+  return NULL;
+}
+
+static void
+gst_gl_dummy_window_set_window_handle (GstGLWindow * window, guintptr handle)
+{
+  GstGLDummyWindow *dummy = (GstGLDummyWindow *) window;
+
+  dummy->handle = handle;
+}
+
+static guintptr
+gst_gl_dummy_window_get_window_handle (GstGLWindow * window)
+{
+  GstGLDummyWindow *dummy = (GstGLDummyWindow *) window;
+
+  return (guintptr) dummy->handle;
+}
+
+static guintptr
+gst_gl_dummy_window_get_display (GstGLWindow * window)
+{
+  return 0;
+}
+
+static void
+gst_gl_dummy_window_get_surface_dimensions (GstGLWindow * window, guint * width,
+    guint * height)
+{
+}
+
+static void
+gst_gl_dummy_window_class_init (GstGLDummyWindowClass * klass)
+{
+  GstGLWindowClass *window_class = (GstGLWindowClass *) klass;
+
+  window_class->get_display =
+      GST_DEBUG_FUNCPTR (gst_gl_dummy_window_get_display);
+  window_class->get_window_handle =
+      GST_DEBUG_FUNCPTR (gst_gl_dummy_window_get_window_handle);
+  window_class->set_window_handle =
+      GST_DEBUG_FUNCPTR (gst_gl_dummy_window_set_window_handle);
+  window_class->get_surface_dimensions =
+      GST_DEBUG_FUNCPTR (gst_gl_dummy_window_get_surface_dimensions);
+}
+
+static void
+gst_gl_dummy_window_init (GstGLDummyWindow * dummy)
+{
+  dummy->handle = 0;
+}
+
+GstGLDummyWindow *
+gst_gl_dummy_window_new (void)
+{
+  return g_object_new (gst_gl_dummy_window_get_type (), NULL);
+}
+
+gboolean
+gst_gl_window_key_event_cb (gpointer data)
+{
+  struct key_event *key_data = (struct key_event *) data;
+  GST_DEBUG
+      ("%s called data struct %p window %p key %s event %s ",
+      __func__, key_data, key_data->window, key_data->key_str,
+      key_data->event_type);
+  gst_gl_window_send_key_event (GST_GL_WINDOW (key_data->window),
+      key_data->event_type, key_data->key_str);
+  g_slice_free (struct key_event, key_data);
+  return G_SOURCE_REMOVE;
 }
 
 void
@@ -854,6 +1062,20 @@ gst_gl_window_send_key_event (GstGLWindow * window, const char *event_type,
       event_type, key_str);
 }
 
+gboolean
+gst_gl_window_mouse_event_cb (gpointer data)
+{
+  struct mouse_event *mouse_data = (struct mouse_event *) data;
+  GST_DEBUG ("%s called data struct %p mouse event %s button %d at %g, %g",
+      __func__, mouse_data, mouse_data->event_type, mouse_data->button,
+      mouse_data->posx, mouse_data->posy);
+  gst_gl_window_send_mouse_event (GST_GL_WINDOW (mouse_data->window),
+      mouse_data->event_type, mouse_data->button, mouse_data->posx,
+      mouse_data->posy);
+  g_slice_free (struct mouse_event, mouse_data);
+  return G_SOURCE_REMOVE;
+}
+
 void
 gst_gl_window_send_mouse_event (GstGLWindow * window, const char *event_type,
     int button, double posx, double posy)
@@ -878,7 +1100,7 @@ gst_gl_window_handle_events (GstGLWindow * window, gboolean handle_events)
 {
   GstGLWindowClass *window_class;
 
-  g_return_if_fail (GST_IS_GL_WINDOW (window));
+  g_return_if_fail (GST_GL_IS_WINDOW (window));
   window_class = GST_GL_WINDOW_GET_CLASS (window);
 
   if (window_class->handle_events)
@@ -905,7 +1127,7 @@ gst_gl_window_set_render_rectangle (GstGLWindow * window, gint x, gint y,
   GstGLWindowClass *window_class;
   gboolean ret = FALSE;
 
-  g_return_val_if_fail (GST_IS_GL_WINDOW (window), FALSE);
+  g_return_val_if_fail (GST_GL_IS_WINDOW (window), FALSE);
   window_class = GST_GL_WINDOW_GET_CLASS (window);
 
   if (x < 0 || y < 0 || width <= 0 || height <= 0)
@@ -916,103 +1138,3 @@ gst_gl_window_set_render_rectangle (GstGLWindow * window, gint x, gint y,
 
   return ret;
 }
-
-void
-gst_gl_window_queue_resize (GstGLWindow * window)
-{
-  GstGLWindowClass *window_class;
-
-  g_return_if_fail (GST_IS_GL_WINDOW (window));
-  window_class = GST_GL_WINDOW_GET_CLASS (window);
-
-  window->queue_resize = TRUE;
-  if (window_class->queue_resize)
-    window_class->queue_resize (window);
-}
-
-struct resize_data
-{
-  GstGLWindow *window;
-  guint width, height;
-};
-
-static void
-_on_resize (gpointer data)
-{
-  struct resize_data *resize = data;
-
-  resize->window->resize (resize->window->resize_data, resize->width,
-      resize->height);
-}
-
-void
-gst_gl_window_resize (GstGLWindow * window, guint width, guint height)
-{
-  g_return_if_fail (GST_IS_GL_WINDOW (window));
-
-  if (window->resize) {
-    struct resize_data resize = { 0, };
-
-    resize.window = window;
-    resize.width = width;
-    resize.height = height;
-
-    gst_gl_window_send_message (window, (GstGLWindowCB) _on_resize, &resize);
-  }
-
-  window->priv->surface_width = width;
-  window->priv->surface_height = height;
-
-  window->queue_resize = FALSE;
-}
-
-GType gst_gl_dummy_window_get_type (void);
-
-G_DEFINE_TYPE (GstGLDummyWindow, gst_gl_dummy_window, GST_TYPE_GL_WINDOW);
-
-static void
-gst_gl_dummy_window_set_window_handle (GstGLWindow * window, guintptr handle)
-{
-  GstGLDummyWindow *dummy = (GstGLDummyWindow *) window;
-
-  dummy->handle = handle;
-}
-
-static guintptr
-gst_gl_dummy_window_get_window_handle (GstGLWindow * window)
-{
-  GstGLDummyWindow *dummy = (GstGLDummyWindow *) window;
-
-  return (guintptr) dummy->handle;
-}
-
-static guintptr
-gst_gl_dummy_window_get_display (GstGLWindow * window)
-{
-  return 0;
-}
-
-static void
-gst_gl_dummy_window_class_init (GstGLDummyWindowClass * klass)
-{
-  GstGLWindowClass *window_class = (GstGLWindowClass *) klass;
-
-  window_class->get_display =
-      GST_DEBUG_FUNCPTR (gst_gl_dummy_window_get_display);
-  window_class->get_window_handle =
-      GST_DEBUG_FUNCPTR (gst_gl_dummy_window_get_window_handle);
-  window_class->set_window_handle =
-      GST_DEBUG_FUNCPTR (gst_gl_dummy_window_set_window_handle);
-}
-
-static void
-gst_gl_dummy_window_init (GstGLDummyWindow * dummy)
-{
-  dummy->handle = 0;
-}
-
-GstGLDummyWindow *
-gst_gl_dummy_window_new (void)
-{
-  return g_object_new (gst_gl_dummy_window_get_type (), NULL);
-}
