@@ -55,17 +55,24 @@ struct display {
   struct wl_surface      *cursor_surface;
   struct window          *window;
   guint32                 serial;
+
+  gdouble pointer_x;
+  gdouble pointer_y;
 };
 
 struct window {
   struct display *display;
 
-  struct wl_egl_window      *native;
+  struct wl_event_queue     *queue;
   struct wl_surface         *surface;
   struct wl_shell_surface   *shell_surface;
+  struct wl_egl_window      *native;
+  struct wl_surface         *foreign_surface;
+  struct wl_subsurface      *subsurface;
   struct wl_callback        *callback;
   int fullscreen, configured;
   int window_width, window_height;
+  int window_x, window_y;
 };
 
 struct _GstGLWindowWaylandEGL {
@@ -76,8 +83,6 @@ struct _GstGLWindowWaylandEGL {
   struct window  window;
 
   GSource *wl_source;
-  GMainContext *main_context;
-  GMainLoop *loop;
 
   gpointer _reserved[GST_PADDING];
 };
@@ -94,6 +99,8 @@ GType gst_gl_window_wayland_egl_get_type     (void);
 
 GstGLWindowWaylandEGL * gst_gl_window_wayland_egl_new  (void);
 
+void gst_gl_window_wayland_egl_create_window (GstGLWindowWaylandEGL * window_egl);
+
 G_END_DECLS
 
 #endif /* __GST_GL_WINDOW_X11_H__ */
