@@ -58,10 +58,6 @@ static gboolean gst_uri_downloader_sink_event (GstPad * pad, GstObject * parent,
 static GstBusSyncReply gst_uri_downloader_bus_handler (GstBus * bus,
     GstMessage * message, gpointer data);
 
-static gboolean gst_uri_downloader_ensure_src (GstUriDownloader * downloader,
-    const gchar * uri);
-static void gst_uri_downloader_destroy_src (GstUriDownloader * downloader);
-
 static GstStaticPadTemplate sinkpadtemplate = GST_STATIC_PAD_TEMPLATE ("sink",
     GST_PAD_SINK,
     GST_PAD_ALWAYS,
@@ -116,7 +112,11 @@ gst_uri_downloader_dispose (GObject * object)
 {
   GstUriDownloader *downloader = GST_URI_DOWNLOADER (object);
 
-  gst_uri_downloader_destroy_src (downloader);
+  if (downloader->priv->urisrc != NULL) {
+    gst_element_set_state (downloader->priv->urisrc, GST_STATE_NULL);
+    gst_object_unref (downloader->priv->urisrc);
+    downloader->priv->urisrc = NULL;
+  }
 
   if (downloader->priv->bus != NULL) {
     gst_object_unref (downloader->priv->bus);
@@ -178,20 +178,6 @@ gst_uri_downloader_sink_event (GstPad * pad, GstObject * parent,
       gst_event_unref (event);
       break;
     }
-    case GST_EVENT_CUSTOM_DOWNSTREAM_STICKY:{
-      const GstStructure *str;
-      str = gst_event_get_structure (event);
-      if (gst_structure_has_name (str, "http-headers")) {
-        GST_OBJECT_LOCK (downloader);
-        if (downloader->priv->download != NULL) {
-          if (downloader->priv->download->headers)
-            gst_structure_free (downloader->priv->download->headers);
-          downloader->priv->download->headers = gst_structure_copy (str);
-        }
-        GST_OBJECT_UNLOCK (downloader);
-      }
-    }
-      /* falls through */
     default:
       ret = gst_pad_event_default (pad, parent, event);
       break;
@@ -347,8 +333,16 @@ gst_uri_downloader_set_range (GstUriDownloader * downloader,
 }
 
 static gboolean
-gst_uri_downloader_ensure_src (GstUriDownloader * downloader, const gchar * uri)
+gst_uri_downloader_set_uri (GstUriDownloader * downloader, const gchar * uri,
+    const gchar * referer, gboolean compress, gboolean refresh,
+    gboolean allow_cache)
 {
+  GstPad *pad;
+  GObjectClass *gobject_class;
+
+  if (!gst_uri_is_valid (uri))
+    return FALSE;
+
   if (downloader->priv->urisrc) {
     gchar *old_protocol, *new_protocol;
     gchar *old_uri;
@@ -359,18 +353,22 @@ gst_uri_downloader_ensure_src (GstUriDownloader * downloader, const gchar * uri)
     new_protocol = gst_uri_get_protocol (uri);
 
     if (!g_str_equal (old_protocol, new_protocol)) {
-      gst_uri_downloader_destroy_src (downloader);
+      gst_element_set_state (downloader->priv->urisrc, GST_STATE_NULL);
+      gst_object_unref (downloader->priv->urisrc);
+      downloader->priv->urisrc = NULL;
       GST_DEBUG_OBJECT (downloader, "Can't re-use old source element");
     } else {
       GError *err = NULL;
 
       GST_DEBUG_OBJECT (downloader, "Re-using old source element");
-      if (!gst_uri_handler_set_uri
-          (GST_URI_HANDLER (downloader->priv->urisrc), uri, &err)) {
-        GST_DEBUG_OBJECT (downloader,
-            "Failed to re-use old source element: %s", err->message);
+      if (!gst_uri_handler_set_uri (GST_URI_HANDLER (downloader->priv->urisrc),
+              uri, &err)) {
+        GST_DEBUG_OBJECT (downloader, "Failed to re-use old source element: %s",
+            err->message);
         g_clear_error (&err);
-        gst_uri_downloader_destroy_src (downloader);
+        gst_element_set_state (downloader->priv->urisrc, GST_STATE_NULL);
+        gst_object_unref (downloader->priv->urisrc);
+        downloader->priv->urisrc = NULL;
       }
     }
     g_free (old_uri);
@@ -383,43 +381,10 @@ gst_uri_downloader_ensure_src (GstUriDownloader * downloader, const gchar * uri)
         uri);
     downloader->priv->urisrc =
         gst_element_make_from_uri (GST_URI_SRC, uri, NULL, NULL);
-    if (downloader->priv->urisrc) {
-      /* gst_element_make_from_uri returns a floating reference
-       * and we are not going to transfer the ownership, so we
-       * should take it.
-       */
-      gst_object_ref_sink (downloader->priv->urisrc);
-    }
+    if (!downloader->priv->urisrc)
+      return FALSE;
   }
 
-  return downloader->priv->urisrc != NULL;
-}
-
-static void
-gst_uri_downloader_destroy_src (GstUriDownloader * downloader)
-{
-  if (!downloader->priv->urisrc)
-    return;
-
-  gst_element_set_state (downloader->priv->urisrc, GST_STATE_NULL);
-  gst_object_unref (downloader->priv->urisrc);
-  downloader->priv->urisrc = NULL;
-}
-
-static gboolean
-gst_uri_downloader_set_uri (GstUriDownloader * downloader, const gchar * uri,
-    const gchar * referer, gboolean compress,
-    gboolean refresh, gboolean allow_cache)
-{
-  GstPad *pad;
-  GObjectClass *gobject_class;
-
-  if (!gst_uri_is_valid (uri))
-    return FALSE;
-
-  if (!gst_uri_downloader_ensure_src (downloader, uri))
-    return FALSE;
-
   gobject_class = G_OBJECT_GET_CLASS (downloader->priv->urisrc);
   if (g_object_class_find_property (gobject_class, "compress"))
     g_object_set (downloader->priv->urisrc, "compress", compress, NULL);
@@ -463,23 +428,6 @@ gst_uri_downloader_set_uri (GstUriDownloader * downloader, const gchar * uri,
   return TRUE;
 }
 
-static gboolean
-gst_uri_downloader_set_method (GstUriDownloader * downloader,
-    const gchar * method)
-{
-  GObjectClass *gobject_class;
-
-  if (!downloader->priv->urisrc)
-    return FALSE;
-
-  gobject_class = G_OBJECT_GET_CLASS (downloader->priv->urisrc);
-  if (g_object_class_find_property (gobject_class, "method")) {
-    g_object_set (downloader->priv->urisrc, "method", method, NULL);
-    return TRUE;
-  }
-  return FALSE;
-}
-
 GstFragment *
 gst_uri_downloader_fetch_uri (GstUriDownloader * downloader,
     const gchar * uri, const gchar * referer, gboolean compress,
@@ -529,8 +477,6 @@ gst_uri_downloader_fetch_uri_with_range (GstUriDownloader *
   if (downloader->priv->download)
     g_object_unref (downloader->priv->download);
   downloader->priv->download = gst_fragment_new ();
-  downloader->priv->download->range_start = range_start;
-  downloader->priv->download->range_end = range_end;
   GST_OBJECT_UNLOCK (downloader);
   ret = gst_element_set_state (downloader->priv->urisrc, GST_STATE_READY);
   GST_OBJECT_LOCK (downloader);
@@ -544,16 +490,9 @@ gst_uri_downloader_fetch_uri_with_range (GstUriDownloader *
     goto quit;
   }
 
-  if (range_start < 0 && range_end < 0) {
-    if (!gst_uri_downloader_set_method (downloader, "HEAD")) {
-      GST_WARNING_OBJECT (downloader, "Failed to set HTTP method");
-      goto quit;
-    }
-  } else {
-    if (!gst_uri_downloader_set_range (downloader, range_start, range_end)) {
-      GST_WARNING_OBJECT (downloader, "Failed to set range");
-      goto quit;
-    }
+  if (!gst_uri_downloader_set_range (downloader, range_start, range_end)) {
+    GST_WARNING_OBJECT (downloader, "Failed to set range");
+    goto quit;
   }
 
   GST_OBJECT_UNLOCK (downloader);
@@ -592,13 +531,9 @@ gst_uri_downloader_fetch_uri_with_range (GstUriDownloader *
   download = downloader->priv->download;
   downloader->priv->download = NULL;
   if (!downloader->priv->got_buffer) {
-    if (download->range_start < 0 && download->range_end < 0) {
-      /* HEAD request, so we don't expect a response */
-    } else {
-      g_object_unref (download);
-      download = NULL;
-      GST_ERROR_OBJECT (downloader, "Didn't retrieve a buffer before EOS");
-    }
+    g_object_unref (download);
+    download = NULL;
+    GST_ERROR_OBJECT (downloader, "Didn't retrieve a buffer before EOS");
   }
 
   if (download != NULL)
