@@ -61,7 +61,7 @@
  * Client (receiver)
  * |[
  * gst-launch udpsrc port=3333 ! "caps_from_rtpasfpay_at_sender" \
- * ! rtpasfdepay ! decodebin2 name=d ! queue \
+ * ! rtpasfdepay ! decodebin name=d ! queue \
  * ! videoconvert ! autovideosink \
  * d. ! queue ! audioconvert ! autoaudiosink
  * ]|
@@ -373,12 +373,15 @@ gst_asf_mux_sink_event (GstCollectPads * pads, GstCollectData * cdata,
  * Returns: the result of #gst_pad_push on the buffer
  */
 static GstFlowReturn
-gst_asf_mux_push_buffer (GstAsfMux * asfmux, GstBuffer * buf)
+gst_asf_mux_push_buffer (GstAsfMux * asfmux, GstBuffer * buf, gsize bufsize)
 {
   GstFlowReturn ret;
+
   ret = gst_pad_push (asfmux->srcpad, buf);
+
   if (ret == GST_FLOW_OK)
-    asfmux->file_size += gst_buffer_get_size (buf);
+    asfmux->file_size += bufsize;
+
   return ret;
 }
 
@@ -705,11 +708,7 @@ gst_asf_mux_write_stream_properties (GstAsfMux * asfmux, guint8 ** buf,
   else
     gst_asf_put_guid (*buf + 24, guids[ASF_VIDEO_MEDIA_INDEX]);
   /* error correction */
-  if (asfpad->is_audio) {
-    gst_asf_put_guid (*buf + 40, guids[ASF_NO_ERROR_CORRECTION_INDEX]);
-  } else {
-    gst_asf_put_guid (*buf + 40, guids[ASF_NO_ERROR_CORRECTION_INDEX]);
-  }
+  gst_asf_put_guid (*buf + 40, guids[ASF_NO_ERROR_CORRECTION_INDEX]);
   GST_WRITE_UINT64_LE (*buf + 56, 0);   /* time offset */
 
   GST_WRITE_UINT32_LE (*buf + 64, codec_data_length + media_specific_data_length);      /* type specific data length */
@@ -1260,6 +1259,7 @@ gst_asf_mux_start_file (GstAsfMux * asfmux)
   guint64 padding = asfmux->prop_padding;
   GstSegment segment;
   GstMapInfo map;
+  gsize bufsize;
   gchar s_id[32];
 
   if (padding < ASF_PADDING_OBJECT_SIZE)
@@ -1305,6 +1305,7 @@ gst_asf_mux_start_file (GstAsfMux * asfmux)
 
   gst_buffer_map (buf, &map, GST_MAP_WRITE);
   bufdata = map.data;
+  bufsize = map.size;
 
   gst_asf_mux_write_header_object (asfmux, &bufdata, map.size -
       ASF_DATA_OBJECT_SIZE, 2 + stream_num);
@@ -1326,11 +1327,9 @@ gst_asf_mux_start_file (GstAsfMux * asfmux)
     gst_asf_mux_write_ext_content_description (asfmux, &bufdata, asftags->tags);
   }
 
-  if (asftags) {
-    if (asftags->tags)
-      gst_tag_list_unref (asftags->tags);
-    g_free (asftags);
-  }
+  if (asftags->tags)
+    gst_tag_list_unref (asftags->tags);
+  g_free (asftags);
 
   /* writing header extension objects */
   gst_asf_mux_write_header_extension (asfmux, &bufdata, stream_num *
@@ -1367,7 +1366,7 @@ gst_asf_mux_start_file (GstAsfMux * asfmux)
 
   g_assert (bufdata - map.data == map.size);
   gst_buffer_unmap (buf, &map);
-  return gst_asf_mux_push_buffer (asfmux, buf);
+  return gst_asf_mux_push_buffer (asfmux, buf, bufsize);
 }
 
 /**
@@ -1408,16 +1407,16 @@ gst_asf_mux_add_simple_index_entry (GstAsfMux * asfmux,
  * Returns: the result of pushing the buffer downstream
  */
 static GstFlowReturn
-gst_asf_mux_send_packet (GstAsfMux * asfmux, GstBuffer * buf)
+gst_asf_mux_send_packet (GstAsfMux * asfmux, GstBuffer * buf, gsize bufsize)
 {
-  g_assert (gst_buffer_get_size (buf) == asfmux->packet_size);
+  g_assert (bufsize == asfmux->packet_size);
   asfmux->total_data_packets++;
   GST_LOG_OBJECT (asfmux,
       "Pushing a packet of size %" G_GSIZE_FORMAT " and timestamp %"
-      G_GUINT64_FORMAT, gst_buffer_get_size (buf), GST_BUFFER_TIMESTAMP (buf));
+      G_GUINT64_FORMAT, bufsize, GST_BUFFER_TIMESTAMP (buf));
   GST_LOG_OBJECT (asfmux, "Total data packets: %" G_GUINT64_FORMAT,
       asfmux->total_data_packets);
-  return gst_asf_mux_push_buffer (asfmux, buf);
+  return gst_asf_mux_push_buffer (asfmux, buf, bufsize);
 }
 
 /**
@@ -1438,6 +1437,7 @@ gst_asf_mux_flush_payloads (GstAsfMux * asfmux)
   GstClockTime send_ts = GST_CLOCK_TIME_NONE;
   guint64 size_left;
   guint8 *data;
+  gsize size;
   GSList *walk;
   GstAsfPad *pad;
   gboolean has_keyframe;
@@ -1577,6 +1577,8 @@ gst_asf_mux_flush_payloads (GstAsfMux * asfmux)
 
   /* fill payload parsing info */
   data = map.data;
+  size = map.size;
+
   /* flags */
   GST_WRITE_UINT8 (data, (0x0 << 7) |   /* no error correction */
       (ASF_FIELD_TYPE_DWORD << 5) |     /* packet length type */
@@ -1593,7 +1595,7 @@ gst_asf_mux_flush_payloads (GstAsfMux * asfmux)
   offset++;
 
   /* Due to a limitation in WMP while streaming through WMSP we reduce the
-   * packet & padding size to 16bit if theay are <= 65535 bytes 
+   * packet & padding size to 16bit if they are <= 65535 bytes
    */
   if (asfmux->packet_size > 65535) {
     GST_WRITE_UINT32_LE (data + offset, asfmux->packet_size - size_left);
@@ -1632,10 +1634,12 @@ gst_asf_mux_flush_payloads (GstAsfMux * asfmux)
   if (payloads_count == 0) {
     GST_WARNING_OBJECT (asfmux, "Sending packet without any payload");
   }
-  asfmux->data_object_size += gst_buffer_get_size (buf);
+  asfmux->data_object_size += size;
+
   if (!has_keyframe)
     GST_BUFFER_FLAG_SET (buf, GST_BUFFER_FLAG_DELTA_UNIT);
-  return gst_asf_mux_send_packet (asfmux, buf);
+
+  return gst_asf_mux_send_packet (asfmux, buf, size);
 }
 
 /**
@@ -1660,11 +1664,15 @@ gst_asf_mux_push_simple_index (GstAsfMux * asfmux, GstAsfVideoPad * pad)
 {
   guint64 object_size = ASF_SIMPLE_INDEX_OBJECT_SIZE +
       g_slist_length (pad->simple_index) * ASF_SIMPLE_INDEX_ENTRY_SIZE;
-  GstBuffer *buf = gst_buffer_new_and_alloc (object_size);
+  GstBuffer *buf;
   GSList *walk;
   guint8 *data;
   guint32 entries_count = g_slist_length (pad->simple_index);
   GstMapInfo map;
+  gsize bufsize;
+
+  buf = gst_buffer_new_and_alloc (object_size);
+  bufsize = object_size;
 
   gst_buffer_map (buf, &map, GST_MAP_WRITE);
   data = map.data;
@@ -1696,7 +1704,7 @@ gst_asf_mux_push_simple_index (GstAsfMux * asfmux, GstAsfVideoPad * pad)
   GST_DEBUG_OBJECT (asfmux, "Pushing the simple index");
   g_assert (data - map.data == object_size);
   gst_buffer_unmap (buf, &map);
-  return gst_asf_mux_push_buffer (asfmux, buf);
+  return gst_asf_mux_push_buffer (asfmux, buf, bufsize);
 }
 
 static GstFlowReturn
