@@ -53,12 +53,11 @@ struct _GstAudioAggregatorPadPrivate
                                    cached values. */
   guint position, size;
 
-  guint64 output_offset;        /* Sample offset in output segment relative to
-                                   segment.start that collect.pos refers to in the
+  guint64 output_offset;        /* Offset in output segment that
+                                   collect.pos refers to in the
                                    current buffer. */
 
-  guint64 next_offset;          /* Next expected sample offset in the input segment
-                                   relative to segment.start */
+  guint64 next_offset;          /* Next expected offset in the input segment */
 
   /* Last time we noticed a discont */
   GstClockTime discont_time;
@@ -146,8 +145,7 @@ struct _GstAudioAggregatorPrivate
 
   /* counters to keep track of timestamps */
   /* Readable with object lock, writable with both aag lock and object lock */
-
-  gint64 offset;                /* Sample offset starting from 0 at segment.start */
+  gint64 offset;
 };
 
 #define GST_AUDIO_AGGREGATOR_LOCK(self)   g_mutex_lock (&(self)->priv->mutex);
@@ -197,16 +195,10 @@ gst_audio_aggregator_get_next_time (GstAggregator * agg)
   GstClockTime next_time;
 
   GST_OBJECT_LOCK (agg);
-  if (agg->segment.position == -1 || agg->segment.position < agg->segment.start)
+  if (agg->segment.position == -1)
     next_time = agg->segment.start;
   else
     next_time = agg->segment.position;
-
-  if (agg->segment.stop != -1 && next_time > agg->segment.stop)
-    next_time = agg->segment.stop;
-
-  next_time =
-      gst_segment_to_running_time (&agg->segment, GST_FORMAT_TIME, next_time);
   GST_OBJECT_UNLOCK (agg);
 
   return next_time;
@@ -586,9 +578,8 @@ gst_audio_aggregator_src_query (GstAggregator * agg, GstQuery * query)
 
       switch (format) {
         case GST_FORMAT_TIME:
-          gst_query_set_position (query, format,
-              gst_segment_to_stream_time (&agg->segment, GST_FORMAT_TIME,
-                  agg->segment.position));
+          /* FIXME, bring to stream time, might be tricky */
+          gst_query_set_position (query, format, agg->segment.position);
           res = TRUE;
           break;
         case GST_FORMAT_BYTES:
@@ -625,18 +616,13 @@ void
 gst_audio_aggregator_set_sink_caps (GstAudioAggregator * aagg,
     GstAudioAggregatorPad * pad, GstCaps * caps)
 {
-#ifndef G_DISABLE_ASSERT
   gboolean valid;
 
   GST_OBJECT_LOCK (pad);
   valid = gst_audio_info_from_caps (&pad->info, caps);
-  g_assert (valid);
-
-#else
-  GST_OBJECT_LOCK (pad);
-  (void) gst_audio_info_from_caps (&pad->info, caps);
   GST_OBJECT_UNLOCK (pad);
-#endif
+
+  g_assert (valid);
 }
 
 
@@ -737,7 +723,7 @@ gst_audio_aggregator_do_clip (GstAggregator * agg,
   bpf = GST_AUDIO_INFO_BPF (&pad->info);
 
   GST_OBJECT_LOCK (bpad);
-  *out = gst_audio_buffer_clip (buffer, &bpad->clip_segment, rate, bpf);
+  *out = gst_audio_buffer_clip (buffer, &bpad->segment, rate, bpf);
   GST_OBJECT_UNLOCK (bpad);
 
   return GST_FLOW_OK;
@@ -753,9 +739,9 @@ gst_audio_aggregator_fill_buffer (GstAudioAggregator * aagg,
   GstClockTime start_time, end_time;
   gboolean discont = FALSE;
   guint64 start_offset, end_offset;
+  GstClockTime timestamp, stream_time = GST_CLOCK_TIME_NONE;
   gint rate, bpf;
 
-  GstAggregator *agg = GST_AGGREGATOR (aagg);
   GstAggregatorPad *aggpad = GST_AGGREGATOR_PAD (pad);
 
   g_assert (pad->priv->buffer == NULL);
@@ -776,17 +762,21 @@ gst_audio_aggregator_fill_buffer (GstAudioAggregator * aagg,
     goto done;
   }
 
+  timestamp = GST_BUFFER_PTS (inbuf);
+  stream_time = gst_segment_to_stream_time (&aggpad->segment, GST_FORMAT_TIME,
+      timestamp);
+
+  /* sync object properties on stream time */
+  /* TODO: Ideally we would want to do that on every sample */
+  if (GST_CLOCK_TIME_IS_VALID (stream_time))
+    gst_object_sync_values (GST_OBJECT (pad), stream_time);
+
   start_time = GST_BUFFER_PTS (inbuf);
   end_time =
       start_time + gst_util_uint64_scale_ceil (pad->priv->size, GST_SECOND,
       rate);
 
-  /* Clipping should've ensured this */
-  g_assert (start_time >= aggpad->segment.start);
-
-  start_offset =
-      gst_util_uint64_scale (start_time - aggpad->segment.start, rate,
-      GST_SECOND);
+  start_offset = gst_util_uint64_scale (start_time, rate, GST_SECOND);
   end_offset = start_offset + pad->priv->size;
 
   if (GST_BUFFER_IS_DISCONT (inbuf)
@@ -841,8 +831,8 @@ gst_audio_aggregator_fill_buffer (GstAudioAggregator * aagg,
   if (pad->priv->output_offset == -1) {
     GstClockTime start_running_time;
     GstClockTime end_running_time;
-    guint64 start_output_offset;
-    guint64 end_output_offset;
+    guint64 start_running_time_offset;
+    guint64 end_running_time_offset;
 
     start_running_time =
         gst_segment_to_running_time (&aggpad->segment,
@@ -850,40 +840,12 @@ gst_audio_aggregator_fill_buffer (GstAudioAggregator * aagg,
     end_running_time =
         gst_segment_to_running_time (&aggpad->segment,
         GST_FORMAT_TIME, end_time);
+    start_running_time_offset =
+        gst_util_uint64_scale (start_running_time, rate, GST_SECOND);
+    end_running_time_offset =
+        gst_util_uint64_scale (end_running_time, rate, GST_SECOND);
 
-    /* Convert to position in the output segment */
-    start_output_offset =
-        gst_segment_to_position (&agg->segment, GST_FORMAT_TIME,
-        start_running_time);
-    if (start_output_offset != -1)
-      start_output_offset =
-          gst_util_uint64_scale (start_output_offset - agg->segment.start, rate,
-          GST_SECOND);
-
-    end_output_offset =
-        gst_segment_to_position (&agg->segment, GST_FORMAT_TIME,
-        end_running_time);
-    if (end_output_offset != -1)
-      end_output_offset =
-          gst_util_uint64_scale (end_output_offset - agg->segment.start, rate,
-          GST_SECOND);
-
-    if (start_output_offset == -1 && end_output_offset == -1) {
-      /* Outside output segment, drop */
-      gst_buffer_unref (inbuf);
-      pad->priv->buffer = NULL;
-      pad->priv->position = 0;
-      pad->priv->size = 0;
-      pad->priv->output_offset = -1;
-      GST_DEBUG_OBJECT (pad, "Buffer outside output segment");
-      return FALSE;
-    }
-
-    /* Calculate end_output_offset if it was outside the output segment */
-    if (end_output_offset == -1)
-      end_output_offset = start_output_offset + pad->priv->size;
-
-    if (end_output_offset < aagg->priv->offset) {
+    if (end_running_time_offset < aagg->priv->offset) {
       /* Before output segment, drop */
       gst_buffer_unref (inbuf);
       pad->priv->buffer = NULL;
@@ -892,25 +854,12 @@ gst_audio_aggregator_fill_buffer (GstAudioAggregator * aagg,
       pad->priv->output_offset = -1;
       GST_DEBUG_OBJECT (pad,
           "Buffer before segment or current position: %" G_GUINT64_FORMAT " < %"
-          G_GUINT64_FORMAT, end_output_offset, aagg->priv->offset);
+          G_GUINT64_FORMAT, end_running_time_offset, aagg->priv->offset);
       return FALSE;
     }
 
-    if (start_output_offset == -1 || start_output_offset < aagg->priv->offset) {
-      guint diff;
-
-      if (start_output_offset == -1 && end_output_offset < pad->priv->size) {
-        diff = pad->priv->size - end_output_offset + aagg->priv->offset;
-      } else if (start_output_offset == -1) {
-        start_output_offset = end_output_offset - pad->priv->size;
-
-        if (start_output_offset < aagg->priv->offset)
-          diff = aagg->priv->offset - start_output_offset;
-        else
-          diff = 0;
-      } else {
-        diff = aagg->priv->offset - start_output_offset;
-      }
+    if (start_running_time_offset < aagg->priv->offset) {
+      guint diff = aagg->priv->offset - start_running_time_offset;
 
       pad->priv->position += diff;
       if (pad->priv->position >= pad->priv->size) {
@@ -922,16 +871,14 @@ gst_audio_aggregator_fill_buffer (GstAudioAggregator * aagg,
         pad->priv->output_offset = -1;
         GST_DEBUG_OBJECT (pad,
             "Buffer before segment or current position: %" G_GUINT64_FORMAT
-            " < %" G_GUINT64_FORMAT, end_output_offset, aagg->priv->offset);
+            " < %" G_GUINT64_FORMAT, end_running_time_offset,
+            aagg->priv->offset);
         return FALSE;
       }
     }
 
-    if (start_output_offset == -1 || start_output_offset < aagg->priv->offset)
-      pad->priv->output_offset = aagg->priv->offset;
-    else
-      pad->priv->output_offset = start_output_offset;
-
+    pad->priv->output_offset =
+        MAX (start_running_time_offset, aagg->priv->offset);
     GST_DEBUG_OBJECT (pad,
         "Buffer resynced: Pad offset %" G_GUINT64_FORMAT
         ", current audio aggregator offset %" G_GUINT64_FORMAT,
@@ -976,7 +923,7 @@ gst_audio_aggregator_mix_buffer (GstAudioAggregator * aagg,
   if (GST_BUFFER_FLAG_IS_SET (inbuf, GST_BUFFER_FLAG_GAP)) {
     /* skip gap buffer */
     GST_LOG_OBJECT (pad, "skipping GAP buffer");
-    pad->priv->output_offset += pad->priv->size - pad->priv->position;
+    pad->priv->output_offset += pad->priv->size;
     pad->priv->position = pad->priv->size;
 
     gst_buffer_replace (&pad->priv->buffer, NULL);
@@ -1006,8 +953,8 @@ static GstBuffer *
 gst_audio_aggregator_create_output_buffer (GstAudioAggregator * aagg,
     guint num_frames)
 {
-  GstBuffer *outbuf = gst_buffer_new_allocate (NULL, num_frames *
-      GST_AUDIO_INFO_BPF (&aagg->info), NULL);
+  GstBuffer *outbuf = gst_buffer_new_and_alloc (num_frames *
+      GST_AUDIO_INFO_BPF (&aagg->info));
   GstMapInfo outmap;
 
   gst_buffer_map (outbuf, &outmap, GST_MAP_WRITE);
@@ -1017,29 +964,6 @@ gst_audio_aggregator_create_output_buffer (GstAudioAggregator * aagg,
   return outbuf;
 }
 
-static gboolean
-sync_pad_values (GstAudioAggregator * aagg, GstAudioAggregatorPad * pad)
-{
-  GstAggregatorPad *bpad = GST_AGGREGATOR_PAD (pad);
-  GstClockTime timestamp, stream_time;
-
-  if (pad->priv->buffer == NULL)
-    return TRUE;
-
-  timestamp = GST_BUFFER_PTS (pad->priv->buffer);
-  GST_OBJECT_LOCK (bpad);
-  stream_time = gst_segment_to_stream_time (&bpad->segment, GST_FORMAT_TIME,
-      timestamp);
-  GST_OBJECT_UNLOCK (bpad);
-
-  /* sync object properties on stream time */
-  /* TODO: Ideally we would want to do that on every sample */
-  if (GST_CLOCK_TIME_IS_VALID (stream_time))
-    gst_object_sync_values (GST_OBJECT (pad), stream_time);
-
-  return TRUE;
-}
-
 static GstFlowReturn
 gst_audio_aggregator_aggregate (GstAggregator * agg, gboolean timeout)
 {
@@ -1087,9 +1011,9 @@ gst_audio_aggregator_aggregate (GstAggregator * agg, gboolean timeout)
   element = GST_ELEMENT (agg);
   aagg = GST_AUDIO_AGGREGATOR (agg);
 
-  /* Sync pad properties to the stream time */
-  gst_aggregator_iterate_sinkpads (agg,
-      (GstAggregatorPadForeachFunc) GST_DEBUG_FUNCPTR (sync_pad_values), NULL);
+  blocksize = gst_util_uint64_scale (aagg->priv->output_buffer_duration,
+      GST_AUDIO_INFO_RATE (&aagg->info), GST_SECOND);
+  blocksize = MAX (1, blocksize);
 
   GST_AUDIO_AGGREGATOR_LOCK (aagg);
   GST_OBJECT_LOCK (agg);
@@ -1128,16 +1052,16 @@ gst_audio_aggregator_aggregate (GstAggregator * agg, gboolean timeout)
     GST_OBJECT_UNLOCK (agg);
     gst_aggregator_set_src_caps (agg, aagg->current_caps);
     GST_OBJECT_LOCK (agg);
+    aagg->priv->offset = gst_util_uint64_scale (agg->segment.position,
+        GST_AUDIO_INFO_RATE (&aagg->info), GST_SECOND);
 
     aagg->priv->send_caps = FALSE;
   }
 
+
   rate = GST_AUDIO_INFO_RATE (&aagg->info);
   bpf = GST_AUDIO_INFO_BPF (&aagg->info);
 
-  blocksize = gst_util_uint64_scale (aagg->priv->output_buffer_duration,
-      GST_AUDIO_INFO_RATE (&aagg->info), GST_SECOND);
-  blocksize = MAX (1, blocksize);
 
   /* for the next timestamp, use the sample counter, which will
    * never accumulate rounding errors */
@@ -1149,9 +1073,7 @@ gst_audio_aggregator_aggregate (GstAggregator * agg, gboolean timeout)
     next_offset = aagg->priv->offset - blocksize;
   }
 
-  next_timestamp =
-      agg->segment.start + gst_util_uint64_scale (next_offset, GST_SECOND,
-      rate);
+  next_timestamp = gst_util_uint64_scale (next_offset, GST_SECOND, rate);
 
   if (aagg->priv->current_buffer == NULL) {
     GST_OBJECT_UNLOCK (agg);
@@ -1309,9 +1231,7 @@ gst_audio_aggregator_aggregate (GstAggregator * agg, gboolean timeout)
           "Last buffer is incomplete: %" G_GUINT64_FORMAT " <= %"
           G_GUINT64_FORMAT, max_offset, next_offset);
       next_offset = max_offset;
-      next_timestamp =
-          agg->segment.start + gst_util_uint64_scale (next_offset, GST_SECOND,
-          rate);
+      next_timestamp = gst_util_uint64_scale (next_offset, GST_SECOND, rate);
 
       if (next_offset > aagg->priv->offset)
         gst_buffer_resize (outbuf, 0, (next_offset - aagg->priv->offset) * bpf);
@@ -1332,6 +1252,9 @@ gst_audio_aggregator_aggregate (GstAggregator * agg, gboolean timeout)
     GST_BUFFER_DURATION (outbuf) = agg->segment.position - next_timestamp;
   }
 
+  aagg->priv->offset = next_offset;
+  agg->segment.position = next_timestamp;
+
   GST_OBJECT_UNLOCK (agg);
 
   /* send it out */
@@ -1347,27 +1270,6 @@ gst_audio_aggregator_aggregate (GstAggregator * agg, gboolean timeout)
 
   GST_LOG_OBJECT (aagg, "pushed outbuf, result = %s", gst_flow_get_name (ret));
 
-  GST_AUDIO_AGGREGATOR_LOCK (aagg);
-  GST_OBJECT_LOCK (agg);
-  aagg->priv->offset = next_offset;
-  agg->segment.position = next_timestamp;
-
-  /* If there was a timeout and there was a gap in data in out of the streams,
-   * then it's a very good time to for a resync with the timestamps.
-   */
-  if (timeout) {
-    for (iter = element->sinkpads; iter; iter = iter->next) {
-      GstAudioAggregatorPad *pad = GST_AUDIO_AGGREGATOR_PAD (iter->data);
-
-      GST_OBJECT_LOCK (pad);
-      if (pad->priv->output_offset < aagg->priv->offset)
-        pad->priv->output_offset = -1;
-      GST_OBJECT_UNLOCK (pad);
-    }
-  }
-  GST_OBJECT_UNLOCK (agg);
-  GST_AUDIO_AGGREGATOR_UNLOCK (aagg);
-
   return ret;
   /* ERRORS */
 not_negotiated:
