@@ -20,17 +20,17 @@
  */
 /**
  * SECTION:element-spacescope
- * @title: spacescope
  * @see_also: goom
  *
  * Spacescope is a simple audio visualisation element. It maps the left and
  * right channel to x and y coordinates.
  *
- * ## Example launch line
+ * <refsect2>
+ * <title>Example launch line</title>
  * |[
- * gst-launch-1.0 audiotestsrc ! audioconvert ! spacescope ! ximagesink
+ * gst-launch audiotestsrc ! audioconvert ! spacescope ! ximagesink
  * ]|
- *
+ * </refsect2>
  */
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -39,17 +39,15 @@
 #include <stdlib.h>
 #include "gstspacescope.h"
 
-#if G_BYTE_ORDER == G_BIG_ENDIAN
-#define RGB_ORDER "xRGB"
-#else
-#define RGB_ORDER "BGRx"
-#endif
-
 static GstStaticPadTemplate gst_space_scope_src_template =
 GST_STATIC_PAD_TEMPLATE ("src",
     GST_PAD_SRC,
     GST_PAD_ALWAYS,
-    GST_STATIC_CAPS (GST_VIDEO_CAPS_MAKE (RGB_ORDER))
+#if G_BYTE_ORDER == G_BIG_ENDIAN
+    GST_STATIC_CAPS (GST_VIDEO_CAPS_MAKE ("xRGB"))
+#else
+    GST_STATIC_CAPS (GST_VIDEO_CAPS_MAKE ("BGRx"))
+#endif
     );
 
 static GstStaticPadTemplate gst_space_scope_sink_template =
@@ -133,10 +131,10 @@ gst_space_scope_class_init (GstSpaceScopeClass * g_class)
       "Visualization",
       "Simple stereo visualizer", "Stefan Kost <ensonic@users.sf.net>");
 
-  gst_element_class_add_static_pad_template (element_class,
-      &gst_space_scope_src_template);
-  gst_element_class_add_static_pad_template (element_class,
-      &gst_space_scope_sink_template);
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&gst_space_scope_src_template));
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&gst_space_scope_sink_template));
 
   gobject_class->set_property = gst_space_scope_set_property;
   gobject_class->get_property = gst_space_scope_get_property;
@@ -148,8 +146,6 @@ gst_space_scope_class_init (GstSpaceScopeClass * g_class)
           "Drawing styles for the space scope display.",
           GST_TYPE_SPACE_SCOPE_STYLE, STYLE_DOTS,
           G_PARAM_CONSTRUCT | G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
-
-  gst_type_mark_as_plugin_api (GST_TYPE_SPACE_SCOPE_STYLE, 0);
 }
 
 static void
