@@ -22,7 +22,6 @@
 
 /**
  * SECTION:element-compositor
- * @title: compositor
  *
  * Compositor can accept AYUV, ARGB and BGRA video streams. For each of the requested
  * sink pads it will compare the incoming geometry and framerate to define the
@@ -30,19 +29,40 @@
  * biggest incoming video stream and the framerate of the fastest incoming one.
  *
  * Compositor will do colorspace conversion.
- *
+ * 
  * Individual parameters for each input stream can be configured on the
  * #GstCompositorPad:
  *
- * * "xpos": The x-coordinate position of the top-left corner of the picture (#gint)
- * * "ypos": The y-coordinate position of the top-left corner of the picture (#gint)
- * * "width": The width of the picture; the input will be scaled if necessary (#gint)
- * * "height": The height of the picture; the input will be scaled if necessary (#gint)
- * * "alpha": The transparency of the picture; between 0.0 and 1.0. The blending
- *   is a simple copy when fully-transparent (0.0) and fully-opaque (1.0). (#gdouble)
- * * "zorder": The z-order position of the picture in the composition (#guint)
+ * <itemizedlist>
+ * <listitem>
+ * "xpos": The x-coordinate position of the top-left corner of the picture
+ * (#gint)
+ * </listitem>
+ * <listitem>
+ * "ypos": The y-coordinate position of the top-left corner of the picture
+ * (#gint)
+ * </listitem>
+ * <listitem>
+ * "width": The width of the picture; the input will be scaled if necessary
+ * (#gint)
+ * </listitem>
+ * <listitem>
+ * "height": The height of the picture; the input will be scaled if necessary
+ * (#gint)
+ * </listitem>
+ * <listitem>
+ * "alpha": The transparency of the picture; between 0.0 and 1.0. The blending
+ * is a simple copy when fully-transparent (0.0) and fully-opaque (1.0).
+ * (#gdouble)
+ * </listitem>
+ * <listitem>
+ * "zorder": The z-order position of the picture in the composition; between
+ * 0 and 10000. (#guint)
+ * </listitem>
+ * </itemizedlist>
  *
- * ## Sample pipelines
+ * <refsect2>
+ * <title>Sample pipelines</title>
  * |[
  * gst-launch-1.0 \
  *   videotestsrc pattern=1 ! \
@@ -55,7 +75,7 @@
  * ]| A pipeline to demonstrate compositor used together with videobox.
  * This should show a 320x240 pixels video test source with some transparency
  * showing the background checker pattern. Another video test source with just
- * the snow pattern of 100x100 pixels is overlaid on top of the first one on
+ * the snow pattern of 100x100 pixels is overlayed on top of the first one on
  * the left vertically centered with a small transparency showing the first
  * video test source behind and the checker pattern under it. Note that the
  * framerate of the output video is 10 frames per second.
@@ -65,7 +85,7 @@
  *   compositor name=comp ! videoconvert ! ximagesink \
  *   videotestsrc !  \
  *   video/x-raw, framerate=\(fraction\)5/1, width=320, height=240 ! comp.
- * ]| A pipeline to demostrate bgra comping. (This does not demonstrate alpha blending).
+ * ]| A pipeline to demostrate bgra comping. (This does not demonstrate alpha blending). 
  * |[
  * gst-launch-1.0 videotestsrc pattern=1 ! \
  *   video/x-raw,format =I420, framerate=\(fraction\)10/1, width=100, height=100 ! \
@@ -83,7 +103,7 @@
  *   "video/x-raw,format=AYUV,width=800,height=600,framerate=(fraction)10/1" ! \
  *   timeoverlay ! queue2 ! comp.
  * ]| A pipeline to demonstrate synchronized compositing (the second stream starts after 3 seconds)
- *
+ * </refsect2>
  */
 
 #ifdef HAVE_CONFIG_H
@@ -125,7 +145,6 @@ static GstStaticPadTemplate sink_factory = GST_STATIC_PAD_TEMPLATE ("sink_%u",
 #define DEFAULT_PAD_WIDTH  0
 #define DEFAULT_PAD_HEIGHT 0
 #define DEFAULT_PAD_ALPHA  1.0
-#define DEFAULT_PAD_CROSSFADE_RATIO  -1.0
 enum
 {
   PROP_PAD_0,
@@ -133,8 +152,7 @@ enum
   PROP_PAD_YPOS,
   PROP_PAD_WIDTH,
   PROP_PAD_HEIGHT,
-  PROP_PAD_ALPHA,
-  PROP_PAD_CROSSFADE_RATIO,
+  PROP_PAD_ALPHA
 };
 
 G_DEFINE_TYPE (GstCompositorPad, gst_compositor_pad,
@@ -162,9 +180,6 @@ gst_compositor_pad_get_property (GObject * object, guint prop_id,
     case PROP_PAD_ALPHA:
       g_value_set_double (value, pad->alpha);
       break;
-    case PROP_PAD_CROSSFADE_RATIO:
-      g_value_set_double (value, pad->crossfade);
-      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -193,10 +208,6 @@ gst_compositor_pad_set_property (GObject * object, guint prop_id,
     case PROP_PAD_ALPHA:
       pad->alpha = g_value_get_double (value);
       break;
-    case PROP_PAD_CROSSFADE_RATIO:
-      pad->crossfade = g_value_get_double (value);
-      GST_VIDEO_AGGREGATOR_PAD (pad)->needs_alpha = pad->crossfade >= 0.0f;
-      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -205,9 +216,9 @@ gst_compositor_pad_set_property (GObject * object, guint prop_id,
 
 static void
 _mixer_pad_get_output_size (GstCompositor * comp,
-    GstCompositorPad * comp_pad, gint out_par_n, gint out_par_d, gint * width,
-    gint * height)
+    GstCompositorPad * comp_pad, gint * width, gint * height)
 {
+  GstVideoAggregator *vagg = GST_VIDEO_AGGREGATOR (comp);
   GstVideoAggregatorPad *vagg_pad = GST_VIDEO_AGGREGATOR_PAD (comp_pad);
   gint pad_width, pad_height;
   guint dar_n, dar_d;
@@ -228,16 +239,15 @@ _mixer_pad_get_output_size (GstCompositor * comp,
       comp_pad->height <=
       0 ? GST_VIDEO_INFO_HEIGHT (&vagg_pad->info) : comp_pad->height;
 
-  if (!gst_video_calculate_display_ratio (&dar_n, &dar_d, pad_width, pad_height,
-          GST_VIDEO_INFO_PAR_N (&vagg_pad->info),
-          GST_VIDEO_INFO_PAR_D (&vagg_pad->info), out_par_n, out_par_d)) {
-    GST_WARNING_OBJECT (comp_pad, "Cannot calculate display aspect ratio");
-    *width = *height = 0;
-    return;
-  }
+  gst_video_calculate_display_ratio (&dar_n, &dar_d, pad_width, pad_height,
+      GST_VIDEO_INFO_PAR_N (&vagg_pad->info),
+      GST_VIDEO_INFO_PAR_D (&vagg_pad->info),
+      GST_VIDEO_INFO_PAR_N (&vagg->info), GST_VIDEO_INFO_PAR_D (&vagg->info)
+      );
   GST_LOG_OBJECT (comp_pad, "scaling %ux%u by %u/%u (%u/%u / %u/%u)", pad_width,
       pad_height, dar_n, dar_d, GST_VIDEO_INFO_PAR_N (&vagg_pad->info),
-      GST_VIDEO_INFO_PAR_D (&vagg_pad->info), out_par_n, out_par_d);
+      GST_VIDEO_INFO_PAR_D (&vagg_pad->info),
+      GST_VIDEO_INFO_PAR_N (&vagg->info), GST_VIDEO_INFO_PAR_D (&vagg->info));
 
   if (pad_height % dar_n == 0) {
     pad_width = gst_util_uint64_scale_int (pad_height, dar_n, dar_d);
@@ -247,8 +257,10 @@ _mixer_pad_get_output_size (GstCompositor * comp,
     pad_width = gst_util_uint64_scale_int (pad_height, dar_n, dar_d);
   }
 
-  *width = pad_width;
-  *height = pad_height;
+  if (width)
+    *width = pad_width;
+  if (height)
+    *height = pad_height;
 }
 
 static gboolean
@@ -273,14 +285,6 @@ gst_compositor_pad_set_info (GstVideoAggregatorPad * pad,
 
   cpad->convert = NULL;
 
-  if (GST_VIDEO_INFO_MULTIVIEW_MODE (current_info) !=
-      GST_VIDEO_MULTIVIEW_MODE_NONE
-      && GST_VIDEO_INFO_MULTIVIEW_MODE (current_info) !=
-      GST_VIDEO_MULTIVIEW_MODE_MONO) {
-    GST_FIXME_OBJECT (pad, "Multiview support is not implemented yet");
-    return FALSE;
-  }
-
   colorimetry = gst_video_colorimetry_to_string (&(current_info->colorimetry));
   chroma = gst_video_chroma_to_string (current_info->chroma_site);
 
@@ -288,8 +292,7 @@ gst_compositor_pad_set_info (GstVideoAggregatorPad * pad,
       gst_video_colorimetry_to_string (&(wanted_info->colorimetry));
   best_chroma = gst_video_chroma_to_string (wanted_info->chroma_site);
 
-  _mixer_pad_get_output_size (comp, cpad, GST_VIDEO_INFO_PAR_N (&vagg->info),
-      GST_VIDEO_INFO_PAR_D (&vagg->info), &width, &height);
+  _mixer_pad_get_output_size (comp, cpad, &width, &height);
 
   if (GST_VIDEO_INFO_FORMAT (wanted_info) !=
       GST_VIDEO_INFO_FORMAT (current_info)
@@ -307,20 +310,16 @@ gst_compositor_pad_set_info (GstVideoAggregatorPad * pad,
         width, height);
     tmp_info.chroma_site = wanted_info->chroma_site;
     tmp_info.colorimetry = wanted_info->colorimetry;
-    tmp_info.par_n = wanted_info->par_n;
-    tmp_info.par_d = wanted_info->par_d;
+    tmp_info.par_n = vagg->info.par_n;
+    tmp_info.par_d = vagg->info.par_d;
     tmp_info.fps_n = current_info->fps_n;
     tmp_info.fps_d = current_info->fps_d;
     tmp_info.flags = current_info->flags;
     tmp_info.interlace_mode = current_info->interlace_mode;
 
-    GST_DEBUG_OBJECT (pad, "This pad will be converted from format %s to %s, "
-        "colorimetry %s to %s, chroma-site %s to %s, "
-        "width/height %d/%d to %d/%d",
-        current_info->finfo->name, tmp_info.finfo->name,
-        colorimetry, best_colorimetry,
-        chroma, best_chroma,
-        current_info->width, current_info->height, width, height);
+    GST_DEBUG_OBJECT (pad, "This pad will be converted from %d to %d",
+        GST_VIDEO_INFO_FORMAT (current_info),
+        GST_VIDEO_INFO_FORMAT (&tmp_info));
 
     cpad->convert = gst_video_converter_new (current_info, &tmp_info, NULL);
     cpad->conversion_info = tmp_info;
@@ -351,26 +350,6 @@ is_rectangle_contained (GstVideoRectangle rect1, GstVideoRectangle rect2)
   return FALSE;
 }
 
-static GstVideoRectangle
-clamp_rectangle (gint x, gint y, gint w, gint h, gint outer_width,
-    gint outer_height)
-{
-  gint x2 = x + w;
-  gint y2 = y + h;
-  GstVideoRectangle clamped;
-
-  /* Clamp the x/y coordinates of this frame to the output boundaries to cover
-   * the case where (say, with negative xpos/ypos or w/h greater than the output
-   * size) the non-obscured portion of the frame could be outside the bounds of
-   * the video itself and hence not visible at all */
-  clamped.x = CLAMP (x, 0, outer_width);
-  clamped.y = CLAMP (y, 0, outer_height);
-  clamped.w = CLAMP (x2, 0, outer_width) - clamped.x;
-  clamped.h = CLAMP (y2, 0, outer_height) - clamped.y;
-
-  return clamped;
-}
-
 static gboolean
 gst_compositor_pad_prepare_frame (GstVideoAggregatorPad * pad,
     GstVideoAggregator * vagg)
@@ -394,7 +373,7 @@ gst_compositor_pad_prepare_frame (GstVideoAggregatorPad * pad,
 
   /* There's three types of width/height here:
    * 1. GST_VIDEO_FRAME_WIDTH/HEIGHT:
-   *     The frame width/height (same as pad->info.height/width;
+   *     The frame width/height (same as pad->buffer_vinfo.height/width;
    *     see gst_video_frame_map())
    * 2. cpad->width/height:
    *     The optional pad property for scaling the frame (if zero, the video is
@@ -404,8 +383,7 @@ gst_compositor_pad_prepare_frame (GstVideoAggregatorPad * pad,
    *     width/height. See ->set_info()
    * */
 
-  _mixer_pad_get_output_size (comp, cpad, GST_VIDEO_INFO_PAR_N (&vagg->info),
-      GST_VIDEO_INFO_PAR_D (&vagg->info), &width, &height);
+  _mixer_pad_get_output_size (comp, cpad, &width, &height);
 
   /* The only thing that can change here is the width
    * and height, otherwise set_info would've been called */
@@ -422,20 +400,21 @@ gst_compositor_pad_prepare_frame (GstVideoAggregatorPad * pad,
       gst_video_converter_free (cpad->convert);
     cpad->convert = NULL;
 
-    colorimetry = gst_video_colorimetry_to_string (&pad->info.colorimetry);
-    chroma = gst_video_chroma_to_string (pad->info.chroma_site);
+    colorimetry =
+        gst_video_colorimetry_to_string (&pad->buffer_vinfo.colorimetry);
+    chroma = gst_video_chroma_to_string (pad->buffer_vinfo.chroma_site);
 
     wanted_colorimetry =
         gst_video_colorimetry_to_string (&cpad->conversion_info.colorimetry);
     wanted_chroma =
         gst_video_chroma_to_string (cpad->conversion_info.chroma_site);
 
-    if (GST_VIDEO_INFO_FORMAT (&pad->info) !=
+    if (GST_VIDEO_INFO_FORMAT (&pad->buffer_vinfo) !=
         GST_VIDEO_INFO_FORMAT (&cpad->conversion_info)
         || g_strcmp0 (colorimetry, wanted_colorimetry)
         || g_strcmp0 (chroma, wanted_chroma)
-        || width != GST_VIDEO_INFO_WIDTH (&pad->info)
-        || height != GST_VIDEO_INFO_HEIGHT (&pad->info)) {
+        || width != GST_VIDEO_INFO_WIDTH (&pad->buffer_vinfo)
+        || height != GST_VIDEO_INFO_HEIGHT (&pad->buffer_vinfo)) {
       GstVideoInfo tmp_info;
 
       gst_video_info_set_format (&tmp_info, cpad->conversion_info.finfo->format,
@@ -450,10 +429,11 @@ gst_compositor_pad_prepare_frame (GstVideoAggregatorPad * pad,
       tmp_info.interlace_mode = cpad->conversion_info.interlace_mode;
 
       GST_DEBUG_OBJECT (pad, "This pad will be converted from %d to %d",
-          GST_VIDEO_INFO_FORMAT (&pad->info),
+          GST_VIDEO_INFO_FORMAT (&pad->buffer_vinfo),
           GST_VIDEO_INFO_FORMAT (&tmp_info));
 
-      cpad->convert = gst_video_converter_new (&pad->info, &tmp_info, NULL);
+      cpad->convert =
+          gst_video_converter_new (&pad->buffer_vinfo, &tmp_info, NULL);
       cpad->conversion_info = tmp_info;
 
       if (!cpad->convert) {
@@ -471,44 +451,28 @@ gst_compositor_pad_prepare_frame (GstVideoAggregatorPad * pad,
     g_free (wanted_colorimetry);
   }
 
-  if (cpad->alpha == 0.0) {
-    GST_DEBUG_OBJECT (vagg, "Pad has alpha 0.0, not converting frame");
-    converted_frame = NULL;
-    goto done;
-  }
-
-  frame_rect = clamp_rectangle (cpad->xpos, cpad->ypos, width, height,
-      GST_VIDEO_INFO_WIDTH (&vagg->info), GST_VIDEO_INFO_HEIGHT (&vagg->info));
-
-  if (frame_rect.w == 0 || frame_rect.h == 0) {
-    GST_DEBUG_OBJECT (vagg, "Resulting frame is zero-width or zero-height "
-        "(w: %i, h: %i), skipping", frame_rect.w, frame_rect.h);
-    converted_frame = NULL;
-    goto done;
-  }
+  /* Clamp the x/y coordinates of this frame to the video boundaries to cover
+   * the case where (say, with negative xpos/ypos) the non-obscured portion of
+   * the frame could be outside the bounds of the video itself and hence not
+   * visible at all */
+  frame_rect.x = CLAMP (cpad->xpos, 0, GST_VIDEO_INFO_WIDTH (&vagg->info));
+  frame_rect.y = CLAMP (cpad->ypos, 0, GST_VIDEO_INFO_HEIGHT (&vagg->info));
+  /* Clamp the width/height to the frame boundaries as well */
+  frame_rect.w = MAX (width - frame_rect.x, 0);
+  frame_rect.h = MAX (height - frame_rect.y, 0);
 
   GST_OBJECT_LOCK (vagg);
-  /* Check if we are crossfading the pad one way or another */
-  l = g_list_find (GST_ELEMENT (vagg)->sinkpads, pad);
-  if ((l->prev && GST_COMPOSITOR_PAD (l->prev->data)->crossfade >= 0.0) ||
-      (GST_COMPOSITOR_PAD (pad)->crossfade >= 0.0)) {
-    GST_DEBUG_OBJECT (pad, "Is being crossfaded with previous pad");
-    l = NULL;
-  } else {
-    l = l->next;
-  }
-
   /* Check if this frame is obscured by a higher-zorder frame
    * TODO: Also skip a frame if it's obscured by a combination of
    * higher-zorder frames */
-  for (; l; l = l->next) {
+  for (l = g_list_find (GST_ELEMENT (vagg)->sinkpads, pad)->next; l;
+      l = l->next) {
     GstVideoRectangle frame2_rect;
     GstVideoAggregatorPad *pad2 = l->data;
     GstCompositorPad *cpad2 = GST_COMPOSITOR_PAD (pad2);
     gint pad2_width, pad2_height;
 
-    _mixer_pad_get_output_size (comp, cpad2, GST_VIDEO_INFO_PAR_N (&vagg->info),
-        GST_VIDEO_INFO_PAR_D (&vagg->info), &pad2_width, &pad2_height);
+    _mixer_pad_get_output_size (comp, cpad2, &pad2_width, &pad2_height);
 
     /* We don't need to clamp the coords of the second rectangle */
     frame2_rect.x = cpad2->xpos;
@@ -524,12 +488,8 @@ gst_compositor_pad_prepare_frame (GstVideoAggregatorPad * pad,
         !GST_VIDEO_INFO_HAS_ALPHA (&pad2->info) &&
         is_rectangle_contained (frame_rect, frame2_rect)) {
       frame_obscured = TRUE;
-      GST_DEBUG_OBJECT (pad, "%ix%i@(%i,%i) obscured by %s %ix%i@(%i,%i) "
-          "in output of size %ix%i; skipping frame", frame_rect.w, frame_rect.h,
-          frame_rect.x, frame_rect.y, GST_PAD_NAME (pad2), frame2_rect.w,
-          frame2_rect.h, frame2_rect.x, frame2_rect.y,
-          GST_VIDEO_INFO_WIDTH (&vagg->info),
-          GST_VIDEO_INFO_HEIGHT (&vagg->info));
+      GST_DEBUG_OBJECT (pad, "Obscured by %s, skipping frame",
+          GST_PAD_NAME (pad2));
       break;
     }
   }
@@ -540,9 +500,16 @@ gst_compositor_pad_prepare_frame (GstVideoAggregatorPad * pad,
     goto done;
   }
 
+  if (cpad->alpha == 0.0) {
+    GST_DEBUG_OBJECT (vagg, "Pad has alpha 0.0, not converting frame");
+    converted_frame = NULL;
+    goto done;
+  }
+
   frame = g_slice_new0 (GstVideoFrame);
 
-  if (!gst_video_frame_map (frame, &pad->info, pad->buffer, GST_MAP_READ)) {
+  if (!gst_video_frame_map (frame, &pad->buffer_vinfo, pad->buffer,
+          GST_MAP_READ)) {
     GST_WARNING_OBJECT (vagg, "Could not map input buffer");
     return FALSE;
   }
@@ -643,12 +610,6 @@ gst_compositor_pad_class_init (GstCompositorPadClass * klass)
       g_param_spec_double ("alpha", "Alpha", "Alpha of the picture", 0.0, 1.0,
           DEFAULT_PAD_ALPHA,
           G_PARAM_READWRITE | GST_PARAM_CONTROLLABLE | G_PARAM_STATIC_STRINGS));
-  g_object_class_install_property (gobject_class, PROP_PAD_CROSSFADE_RATIO,
-      g_param_spec_double ("crossfade-ratio", "Crossfade ratio",
-          "The crossfade ratio to use while crossfading with the following pad."
-          "A value inferior to 0 means no crossfading.",
-          -1.0, 1.0, DEFAULT_PAD_CROSSFADE_RATIO,
-          G_PARAM_READWRITE | GST_PARAM_CONTROLLABLE | G_PARAM_STATIC_STRINGS));
 
   vaggpadclass->set_info = GST_DEBUG_FUNCPTR (gst_compositor_pad_set_info);
   vaggpadclass->prepare_frame =
@@ -663,7 +624,6 @@ gst_compositor_pad_init (GstCompositorPad * compo_pad)
   compo_pad->xpos = DEFAULT_PAD_XPOS;
   compo_pad->ypos = DEFAULT_PAD_YPOS;
   compo_pad->alpha = DEFAULT_PAD_ALPHA;
-  compo_pad->crossfade = DEFAULT_PAD_CROSSFADE_RATIO;
 }
 
 
@@ -672,7 +632,7 @@ gst_compositor_pad_init (GstCompositorPad * compo_pad)
 enum
 {
   PROP_0,
-  PROP_BACKGROUND,
+  PROP_BACKGROUND
 };
 
 #define GST_TYPE_COMPOSITOR_BACKGROUND (gst_compositor_background_get_type())
@@ -899,27 +859,17 @@ set_functions (GstCompositor * self, GstVideoInfo * info)
 }
 
 static GstCaps *
-_fixate_caps (GstAggregator * agg, GstCaps * caps)
+_update_caps (GstVideoAggregator * vagg, GstCaps * caps)
 {
-  GstVideoAggregator *vagg = GST_VIDEO_AGGREGATOR (agg);
   GList *l;
   gint best_width = -1, best_height = -1;
-  gint best_fps_n = -1, best_fps_d = -1;
-  gint par_n, par_d;
-  gdouble best_fps = 0.;
+  GstVideoInfo info;
   GstCaps *ret = NULL;
-  GstStructure *s;
 
-  ret = gst_caps_make_writable (caps);
+  gst_video_info_from_caps (&info, caps);
 
-  /* we need this to calculate how large to make the output frame */
-  s = gst_caps_get_structure (ret, 0);
-  if (gst_structure_has_field (s, "pixel-aspect-ratio")) {
-    gst_structure_fixate_field_nearest_fraction (s, "pixel-aspect-ratio", 1, 1);
-    gst_structure_get_fraction (s, "pixel-aspect-ratio", &par_n, &par_d);
-  } else {
-    par_n = par_d = 1;
-  }
+  /* FIXME: this doesn't work for non 1/1 output par's as we don't have that
+   * information available at this time */
 
   GST_OBJECT_LOCK (vagg);
   for (l = GST_ELEMENT (vagg)->sinkpads; l; l = l->next) {
@@ -927,13 +877,9 @@ _fixate_caps (GstAggregator * agg, GstCaps * caps)
     GstCompositorPad *compositor_pad = GST_COMPOSITOR_PAD (vaggpad);
     gint this_width, this_height;
     gint width, height;
-    gint fps_n, fps_d;
-    gdouble cur_fps;
 
-    fps_n = GST_VIDEO_INFO_FPS_N (&vaggpad->info);
-    fps_d = GST_VIDEO_INFO_FPS_D (&vaggpad->info);
-    _mixer_pad_get_output_size (GST_COMPOSITOR (vagg), compositor_pad, par_n,
-        par_d, &width, &height);
+    _mixer_pad_get_output_size (GST_COMPOSITOR (vagg), compositor_pad, &width,
+        &height);
 
     if (width == 0 || height == 0)
       continue;
@@ -945,176 +891,22 @@ _fixate_caps (GstAggregator * agg, GstCaps * caps)
       best_width = this_width;
     if (best_height < this_height)
       best_height = this_height;
-
-    if (fps_d == 0)
-      cur_fps = 0.0;
-    else
-      gst_util_fraction_to_double (fps_n, fps_d, &cur_fps);
-
-    if (best_fps < cur_fps) {
-      best_fps = cur_fps;
-      best_fps_n = fps_n;
-      best_fps_d = fps_d;
-    }
   }
   GST_OBJECT_UNLOCK (vagg);
 
-  if (best_fps_n <= 0 || best_fps_d <= 0 || best_fps == 0.0) {
-    best_fps_n = 25;
-    best_fps_d = 1;
-    best_fps = 25.0;
-  }
+  if (best_width > 0 && best_height > 0) {
+    info.width = best_width;
+    info.height = best_height;
+    if (set_functions (GST_COMPOSITOR (vagg), &info))
+      ret = gst_video_info_to_caps (&info);
 
-  gst_structure_fixate_field_nearest_int (s, "width", best_width);
-  gst_structure_fixate_field_nearest_int (s, "height", best_height);
-  gst_structure_fixate_field_nearest_fraction (s, "framerate", best_fps_n,
-      best_fps_d);
-  ret = gst_caps_fixate (ret);
+    gst_caps_set_simple (ret, "pixel-aspect-ratio", GST_TYPE_FRACTION_RANGE,
+        1, G_MAXINT, G_MAXINT, 1, NULL);
+  }
 
   return ret;
 }
 
-static gboolean
-_negotiated_caps (GstAggregator * agg, GstCaps * caps)
-{
-  GstVideoInfo v_info;
-
-  GST_DEBUG_OBJECT (agg, "Negotiated caps %" GST_PTR_FORMAT, caps);
-
-  if (!gst_video_info_from_caps (&v_info, caps))
-    return FALSE;
-
-  if (!set_functions (GST_COMPOSITOR (agg), &v_info)) {
-    GST_ERROR_OBJECT (agg, "Failed to setup vfuncs");
-    return FALSE;
-  }
-
-  return GST_AGGREGATOR_CLASS (parent_class)->negotiated_src_caps (agg, caps);
-}
-
-/* Fills frame with transparent pixels if @nframe is NULL otherwise copy @frame
- * properties and fill @nframes with transparent pixels */
-static GstFlowReturn
-gst_compositor_fill_transparent (GstCompositor * self, GstVideoFrame * frame,
-    GstVideoFrame * nframe)
-{
-  guint plane, num_planes, height, i;
-
-  if (nframe) {
-    GstBuffer *cbuffer = gst_buffer_copy_deep (frame->buffer);
-
-    if (!gst_video_frame_map (nframe, &frame->info, cbuffer, GST_MAP_WRITE)) {
-      GST_WARNING_OBJECT (self, "Could not map output buffer");
-      gst_buffer_unref (cbuffer);
-      return GST_FLOW_ERROR;
-    }
-
-    /* the last reference is owned by the frame and released once the frame
-     * is unmapped. We leak it if we don't unref here */
-    gst_buffer_unref (cbuffer);
-  } else {
-    nframe = frame;
-  }
-
-  num_planes = GST_VIDEO_FRAME_N_PLANES (nframe);
-  for (plane = 0; plane < num_planes; ++plane) {
-    guint8 *pdata;
-    gsize rowsize, plane_stride;
-
-    pdata = GST_VIDEO_FRAME_PLANE_DATA (nframe, plane);
-    plane_stride = GST_VIDEO_FRAME_PLANE_STRIDE (nframe, plane);
-    rowsize = GST_VIDEO_FRAME_COMP_WIDTH (nframe, plane)
-        * GST_VIDEO_FRAME_COMP_PSTRIDE (nframe, plane);
-    height = GST_VIDEO_FRAME_COMP_HEIGHT (nframe, plane);
-    for (i = 0; i < height; ++i) {
-      memset (pdata, 0, rowsize);
-      pdata += plane_stride;
-    }
-  }
-
-  return GST_FLOW_OK;
-}
-
-/* WITH GST_OBJECT_LOCK !!
- * Returns: %TRUE if outframe is allready ready to be used as we are using
- * a transparent background and all pads have already been crossfaded
- * %FALSE otherwise
- */
-static gboolean
-gst_compositor_crossfade_frames (GstCompositor * self, GstVideoFrame * outframe)
-{
-  GList *l;
-  gboolean all_crossfading = FALSE;
-  GstVideoAggregator *vagg = GST_VIDEO_AGGREGATOR (self);
-
-  if (self->background == COMPOSITOR_BACKGROUND_TRANSPARENT) {
-
-    all_crossfading = TRUE;
-    for (l = GST_ELEMENT (self)->sinkpads; l; l = l->next) {
-      GstCompositorPad *compo_pad = GST_COMPOSITOR_PAD (l->data);
-
-      if (compo_pad->crossfade < 0.0 && l->next &&
-          GST_COMPOSITOR_PAD (l->next->data)->crossfade < 0) {
-        all_crossfading = FALSE;
-
-        break;
-      }
-    }
-  }
-
-  for (l = GST_ELEMENT (self)->sinkpads; l; l = l->next) {
-    GstVideoAggregatorPad *pad = l->data;
-    GstCompositorPad *compo_pad = GST_COMPOSITOR_PAD (pad);
-
-    if (compo_pad->crossfade >= 0.0f && pad->aggregated_frame) {
-      gfloat alpha = compo_pad->crossfade * compo_pad->alpha;
-      GstVideoAggregatorPad *npad = l->next ? l->next->data : NULL;
-      GstVideoFrame *nframe;
-
-      if (!all_crossfading) {
-        nframe = g_slice_new0 (GstVideoFrame);
-        gst_compositor_fill_transparent (self, outframe, nframe);
-      } else {
-        nframe = outframe;
-      }
-
-      self->overlay (pad->aggregated_frame,
-          compo_pad->crossfaded ? 0 : compo_pad->xpos,
-          compo_pad->crossfaded ? 0 : compo_pad->ypos,
-          alpha, nframe, COMPOSITOR_BLEND_MODE_ADDITIVE);
-
-      if (npad && npad->aggregated_frame) {
-        GstCompositorPad *next_compo_pad = GST_COMPOSITOR_PAD (npad);
-
-        alpha = (1.0 - compo_pad->crossfade) * next_compo_pad->alpha;
-        self->overlay (npad->aggregated_frame, next_compo_pad->xpos,
-            next_compo_pad->ypos, alpha, nframe,
-            COMPOSITOR_BLEND_MODE_ADDITIVE);
-
-        /* Replace frame with current frame */
-        gst_compositor_pad_clean_frame (npad, vagg);
-        npad->aggregated_frame = !all_crossfading ? nframe : NULL;
-        next_compo_pad->crossfaded = TRUE;
-
-        /* Frame is now consumed, clean it up */
-        gst_compositor_pad_clean_frame (pad, vagg);
-        pad->aggregated_frame = NULL;
-      } else {
-        GST_LOG_OBJECT (self, "Simply fading out as no following pad found");
-        gst_compositor_pad_clean_frame (pad, vagg);
-        pad->aggregated_frame = !all_crossfading ? nframe : NULL;
-        compo_pad->crossfaded = TRUE;
-      }
-    }
-  }
-
-  if (all_crossfading)
-    for (l = GST_ELEMENT (self)->sinkpads; l; l = l->next)
-      GST_COMPOSITOR_PAD (l->data)->crossfaded = FALSE;
-
-  return all_crossfading;
-}
-
 static GstFlowReturn
 gst_compositor_aggregate_frames (GstVideoAggregator * vagg, GstBuffer * outbuf)
 {
@@ -1144,26 +936,39 @@ gst_compositor_aggregate_frames (GstVideoAggregator * vagg, GstBuffer * outbuf)
       self->fill_color (outframe, 240, 128, 128);
       break;
     case COMPOSITOR_BACKGROUND_TRANSPARENT:
-      gst_compositor_fill_transparent (self, outframe, NULL);
+    {
+      guint i, plane, num_planes, height;
+
+      num_planes = GST_VIDEO_FRAME_N_PLANES (outframe);
+      for (plane = 0; plane < num_planes; ++plane) {
+        guint8 *pdata;
+        gsize rowsize, plane_stride;
+
+        pdata = GST_VIDEO_FRAME_PLANE_DATA (outframe, plane);
+        plane_stride = GST_VIDEO_FRAME_PLANE_STRIDE (outframe, plane);
+        rowsize = GST_VIDEO_FRAME_COMP_WIDTH (outframe, plane)
+            * GST_VIDEO_FRAME_COMP_PSTRIDE (outframe, plane);
+        height = GST_VIDEO_FRAME_COMP_HEIGHT (outframe, plane);
+        for (i = 0; i < height; ++i) {
+          memset (pdata, 0, rowsize);
+          pdata += plane_stride;
+        }
+      }
+
       /* use overlay to keep background transparent */
       composite = self->overlay;
       break;
+    }
   }
 
   GST_OBJECT_LOCK (vagg);
-  /* First mix the crossfade frames as required */
-  if (!gst_compositor_crossfade_frames (self, outframe)) {
-    for (l = GST_ELEMENT (vagg)->sinkpads; l; l = l->next) {
-      GstVideoAggregatorPad *pad = l->data;
-      GstCompositorPad *compo_pad = GST_COMPOSITOR_PAD (pad);
+  for (l = GST_ELEMENT (vagg)->sinkpads; l; l = l->next) {
+    GstVideoAggregatorPad *pad = l->data;
+    GstCompositorPad *compo_pad = GST_COMPOSITOR_PAD (pad);
 
-      if (pad->aggregated_frame != NULL) {
-        composite (pad->aggregated_frame,
-            compo_pad->crossfaded ? 0 : compo_pad->xpos,
-            compo_pad->crossfaded ? 0 : compo_pad->ypos, compo_pad->alpha,
-            outframe, COMPOSITOR_BLEND_MODE_NORMAL);
-        compo_pad->crossfaded = FALSE;
-      }
+    if (pad->aggregated_frame != NULL) {
+      composite (pad->aggregated_frame, compo_pad->xpos, compo_pad->ypos,
+          compo_pad->alpha, outframe);
     }
   }
   GST_OBJECT_UNLOCK (vagg);
@@ -1228,9 +1033,9 @@ gst_compositor_class_init (GstCompositorClass * klass)
   gobject_class->get_property = gst_compositor_get_property;
   gobject_class->set_property = gst_compositor_set_property;
 
+  agg_class->sinkpads_type = GST_TYPE_COMPOSITOR_PAD;
   agg_class->sink_query = _sink_query;
-  agg_class->fixate_src_caps = _fixate_caps;
-  agg_class->negotiated_src_caps = _negotiated_caps;
+  videoaggregator_class->update_caps = _update_caps;
   videoaggregator_class->aggregate_frames = gst_compositor_aggregate_frames;
 
   g_object_class_install_property (gobject_class, PROP_BACKGROUND,
@@ -1238,10 +1043,10 @@ gst_compositor_class_init (GstCompositorClass * klass)
           GST_TYPE_COMPOSITOR_BACKGROUND,
           DEFAULT_BACKGROUND, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
-  gst_element_class_add_static_pad_template_with_gtype (gstelement_class,
-      &src_factory, GST_TYPE_AGGREGATOR_PAD);
-  gst_element_class_add_static_pad_template_with_gtype (gstelement_class,
-      &sink_factory, GST_TYPE_COMPOSITOR_PAD);
+  gst_element_class_add_pad_template (gstelement_class,
+      gst_static_pad_template_get (&src_factory));
+  gst_element_class_add_pad_template (gstelement_class,
+      gst_static_pad_template_get (&sink_factory));
 
   gst_element_class_set_static_metadata (gstelement_class, "Compositor",
       "Filter/Editor/Video/Compositor",
@@ -1252,8 +1057,8 @@ gst_compositor_class_init (GstCompositorClass * klass)
 static void
 gst_compositor_init (GstCompositor * self)
 {
-  /* initialize variables */
   self->background = DEFAULT_BACKGROUND;
+  /* initialize variables */
 }
 
 /* Element registration */
