@@ -14,7 +14,7 @@
  *
  * You should have received a copy of the GNU Lesser General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
 /**
@@ -27,7 +27,7 @@
  * |[
  * gst-launch videotestsrc ! fpsdisplaysink
  * gst-launch videotestsrc ! fpsdisplaysink text-overlay=false
- * gst-launch filesrc location=video.avi ! decodebin2 name=d ! queue ! fpsdisplaysink d. ! queue ! fakesink sync=true
+ * gst-launch filesrc location=video.avi ! decodebin name=d ! queue ! fpsdisplaysink d. ! queue ! fakesink sync=true
  * gst-launch playbin uri=file:///path/to/video.avi video-sink="fpsdisplaysink" audio-sink=fakesink
  * ]|
  * </refsect2>
@@ -435,11 +435,7 @@ display_current_fps (gpointer data)
     g_free (self->last_message);
     self->last_message = g_strdup (fps_message);
     GST_OBJECT_UNLOCK (self);
-#if !GLIB_CHECK_VERSION(2,26,0)
-    g_object_notify ((GObject *) self, "last-message");
-#else
     g_object_notify_by_pspec ((GObject *) self, pspec_last_message);
-#endif
   }
 
   self->last_frames_rendered = frames_rendered;
@@ -521,11 +517,7 @@ fps_display_sink_stop (GstFPSDisplaySink * self)
     g_free (self->last_message);
     self->last_message = str;
     GST_OBJECT_UNLOCK (self);
-#if !GLIB_CHECK_VERSION(2,26,0)
-    g_object_notify ((GObject *) self, "last-message");
-#else
     g_object_notify_by_pspec ((GObject *) self, pspec_last_message);
-#endif
   }
 
   GST_OBJECT_LOCK (self);
