@@ -45,7 +45,6 @@ static void dvb_sub_init (void);
 
 /**
  * SECTION:dvb-sub
- * @title: GstDvbSub
  * @short_description: a DVB subtitle parsing class
  * @stability: Unstable
  *
@@ -54,7 +53,7 @@ static void dvb_sub_init (void);
  */
 
 #define AYUV(y,u,v,a) (((a) << 24) | ((y) << 16) | ((u) << 8) | (v))
-#define RGBA_TO_AYUV(r,g,b,a) ((((guint32)(a)) << 24) | ((rgb_to_y(r,g,b)) << 16) | ((rgb_to_u(r,g,b)) << 8) | (rgb_to_v(r,g,b)))
+#define RGBA_TO_AYUV(r,g,b,a) (((a) << 24) | ((rgb_to_y(r,g,b)) << 16) | ((rgb_to_u(r,g,b)) << 8) | (rgb_to_v(r,g,b)))
 
 
 typedef struct DVBSubCLUT
@@ -82,7 +81,7 @@ typedef struct DVBSubObjectDisplay
   int fgcolor;
   int bgcolor;
 
-  /* FIXME: Should we use GSList? The relating interaction and pointer assignment is quite complex and perhaps unsuited for a plain GSList anyway */
+  /* FIXME: Should we use GSList? The relating interaction and pointer assigment is quite complex and perhaps unsuited for a plain GSList anyway */
   struct DVBSubObjectDisplay *region_list_next;
   struct DVBSubObjectDisplay *object_list_next;
 } DVBSubObjectDisplay;
@@ -256,26 +255,14 @@ delete_region_display_list (DvbSub * dvb_sub, DVBSubRegion * region)
 
           *obj2_ptr = obj2->next;
 
-          g_free (obj2);
+          g_slice_free (DVBSubObject, obj2);
         }
       }
     }
 
     region->display_list = display->region_list_next;
 
-    g_free (display);
-  }
-}
-
-static void
-dvb_sub_free_clut_list (DVBSubCLUT * list)
-{
-  DVBSubCLUT *next;
-
-  while (list != NULL) {
-    next = list->next;
-    g_free (list);
-    list = next;
+    g_slice_free (DVBSubObjectDisplay, display);
   }
 }
 
@@ -290,12 +277,13 @@ delete_state (DvbSub * dvb_sub)
     dvb_sub->region_list = region->next;
 
     delete_region_display_list (dvb_sub, region);
-    g_free (region->pbuf);
+    if (region->pbuf)
+      g_free (region->pbuf);
 
-    g_free (region);
+    g_slice_free (DVBSubRegion, region);
   }
 
-  dvb_sub_free_clut_list (dvb_sub->clut_list);
+  g_slice_free_chain (DVBSubCLUT, dvb_sub->clut_list, next);
   dvb_sub->clut_list = NULL;
 
   /* Should already be NULL */
@@ -422,7 +410,7 @@ _dvb_sub_parse_page_segment (DvbSub * dvb_sub, guint16 page_id, guint8 * buf,
     }
 
     if (!display)
-      display = g_new0 (DVBSubRegionDisplay, 1);
+      display = g_slice_new0 (DVBSubRegionDisplay);
 
     display->region_id = region_id;
 
@@ -446,7 +434,7 @@ _dvb_sub_parse_page_segment (DvbSub * dvb_sub, guint16 page_id, guint8 * buf,
 
     tmp_display_list = display->next;
 
-    g_free (display);
+    g_slice_free (DVBSubRegionDisplay, display);
   }
 }
 
@@ -470,7 +458,7 @@ _dvb_sub_parse_region_segment (DvbSub * dvb_sub, guint16 page_id, guint8 * buf,
   region = get_region (dvb_sub, region_id);
 
   if (!region) {                /* Create a new region */
-    region = g_new0 (DVBSubRegion, 1);
+    region = g_slice_new0 (DVBSubRegion);
     region->id = region_id;
     region->next = dvb_sub->region_list;
     dvb_sub->region_list = region;
@@ -484,11 +472,12 @@ _dvb_sub_parse_region_segment (DvbSub * dvb_sub, guint16 page_id, guint8 * buf,
   buf += 2;
 
   if (region->width * region->height != region->buf_size) {     /* FIXME: Read closer from spec what happens when dimensions change */
-    g_free (region->pbuf);
+    if (region->pbuf)
+      g_free (region->pbuf);
 
     region->buf_size = region->width * region->height;
 
-    region->pbuf = g_malloc (region->buf_size);
+    region->pbuf = g_malloc (region->buf_size); /* TODO: We can probably use GSlice here if careful about freeing while buf_size still records the correct size */
 
     fill = 1;                   /* FIXME: Validate from spec that fill is forced on (in the following codes context) when dimensions change */
   }
@@ -531,7 +520,7 @@ _dvb_sub_parse_region_segment (DvbSub * dvb_sub, guint16 page_id, guint8 * buf,
     object = get_object (dvb_sub, object_id);
 
     if (!object) {
-      object = g_new0 (DVBSubObject, 1);
+      object = g_slice_new0 (DVBSubObject);
 
       object->id = object_id;
 
@@ -541,7 +530,7 @@ _dvb_sub_parse_region_segment (DvbSub * dvb_sub, guint16 page_id, guint8 * buf,
 
     object->type = (*buf) >> 6;
 
-    object_display = g_new0 (DVBSubObjectDisplay, 1);
+    object_display = g_slice_new0 (DVBSubObjectDisplay);
 
     object_display->object_id = object_id;
     object_display->region_id = region_id;
@@ -591,7 +580,7 @@ _dvb_sub_parse_clut_segment (DvbSub * dvb_sub, guint16 page_id, guint8 * buf,
   clut = get_clut (dvb_sub, clut_id);
 
   if (!clut) {
-    clut = g_new (DVBSubCLUT, 1);
+    clut = g_slice_new (DVBSubCLUT);
 
     memcpy (clut, &default_clut, sizeof (DVBSubCLUT));
 
@@ -1151,7 +1140,6 @@ _dvb_sub_parse_display_definition_segment (DvbSub * dvb_sub, guint8 * buf,
     gint buf_size)
 {
   int dds_version, info_byte;
-  int display_width, display_height;
 
   if (buf_size < 5)
     return -1;
@@ -1159,23 +1147,14 @@ _dvb_sub_parse_display_definition_segment (DvbSub * dvb_sub, guint8 * buf,
   info_byte = *buf++;
   dds_version = info_byte >> 4;
 
-  display_width = GST_READ_UINT16_BE (buf) + 1;
-  buf += 2;
-  display_height = GST_READ_UINT16_BE (buf) + 1;
-  buf += 2;
-
-  if ((display_width != dvb_sub->display_def.display_width)
-      || (display_height != dvb_sub->display_def.display_height)) {
-    dvb_sub->display_def.display_width = display_width;
-    dvb_sub->display_def.display_height = display_height;
-
-    dvb_sub->display_def.version = -1;
-  }
-
   if (dvb_sub->display_def.version == dds_version)
     return 0;                   /* already have this display definition version */
 
   dvb_sub->display_def.version = dds_version;
+  dvb_sub->display_def.display_width = GST_READ_UINT16_BE (buf) + 1;
+  buf += 2;
+  dvb_sub->display_def.display_height = GST_READ_UINT16_BE (buf) + 1;
+  buf += 2;
 
   dvb_sub->display_def.window_flag = info_byte & 1 << 3;
 
@@ -1206,7 +1185,7 @@ _dvb_sub_parse_end_of_display_set (DvbSub * dvb_sub, guint16 page_id,
 
   GST_DEBUG ("DISPLAY SET END: page_id = %u", page_id);
 
-  sub = g_new0 (DVBSubtitles, 1);
+  sub = g_slice_new0 (DVBSubtitles);
 
 #if 0                           /* FIXME: PTS stuff not figured out yet */
   sub->start_display_time = 0;
@@ -1266,14 +1245,14 @@ _dvb_sub_parse_end_of_display_set (DvbSub * dvb_sub, guint16 page_id,
 
     /* FIXME: Tweak this to be saved in a format most suitable for Qt and GStreamer instead.
      * Currently kept in AVPicture for quick save_display_set testing */
-    rect->pict.palette = g_new (guint32, (1 << region->depth));
+    rect->pict.palette = g_malloc ((1 << region->depth) * sizeof (guint32));    /* FIXME: Can we use GSlice here? */
     memcpy (rect->pict.palette, clut_table,
         (1 << region->depth) * sizeof (guint32));
 
     GST_MEMDUMP ("rect->pict.data.palette content",
         (guint8 *) rect->pict.palette, (1 << region->depth) * sizeof (guint32));
 
-    rect->pict.data = g_malloc (region->buf_size);
+    rect->pict.data = g_malloc (region->buf_size);      /* FIXME: Can we use GSlice here? */
     memcpy (rect->pict.data, region->pbuf, region->buf_size);
 
     GST_DEBUG ("DISPLAY: an object rect created: iteration %u, "
@@ -1313,7 +1292,7 @@ dvb_subtitles_free (DVBSubtitles * sub)
     g_free (sub->rects[i].pict.data);
   }
   g_free (sub->rects);
-  g_free (sub);
+  g_slice_free (DVBSubtitles, sub);
 }
 
 DvbSub *
@@ -1327,7 +1306,7 @@ dvb_sub_new (void)
     g_once_init_leave (&inited, TRUE);
   }
 
-  sub = g_new0 (DvbSub, 1);
+  sub = g_slice_new0 (DvbSub);
 
   /* TODO: Add initialization code here */
   /* FIXME: Do we have a reason to initiate the members to zero, or are we guaranteed that anyway? */
@@ -1353,11 +1332,11 @@ dvb_sub_free (DvbSub * sub)
   delete_state (sub);
   while (sub->display_list) {
     DVBSubRegionDisplay *tmp = sub->display_list->next;
-    g_free (sub->display_list);
+    g_slice_free (DVBSubRegionDisplay, sub->display_list);
     sub->display_list = tmp;
   }
   g_string_free (sub->pes_buffer, TRUE);
-  g_free (sub);
+  g_slice_free (DvbSub, sub);
 }
 
 #define DVB_SUB_SEGMENT_PAGE_COMPOSITION 0x10
@@ -1382,7 +1361,7 @@ dvb_sub_free (DvbSub * sub)
  * the PTS information).
  *
  * Return value: -1 if data was unhandled (e.g, not a subtitle packet),
- *				 -2 if data parsing was unsuccessful (e.g, length was invalid),
+ *				 -2 if data parsing was unsuccesful (e.g, length was invalid),
  *				  0 or positive if data was handled. If positive, then amount of data consumed on success. FIXME: List the positive return values.
  */
 gint
