@@ -13,8 +13,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 /**
  * SECTION:element-dvdspu
@@ -32,10 +32,6 @@
 #  include <config.h>
 #endif
 
-/* FIXME 0.11: suppress warnings for deprecated API such as GStaticRecMutex
- * with newer GLib versions (>= 2.31.0) */
-#define GLIB_DISABLE_DEPRECATION_WARNINGS
-
 #include <gst/gst-i18n-plugin.h>
 #include <gst/video/video.h>
 
@@ -79,13 +75,6 @@ static GstStaticPadTemplate subpic_sink_factory =
     GST_STATIC_CAPS ("subpicture/x-dvd; subpicture/x-pgs")
     );
 
-static const guint32 default_clut[16] = {
-  0xb48080, 0x248080, 0x628080, 0xd78080,
-  0x808080, 0x808080, 0x808080, 0x808080,
-  0x808080, 0x808080, 0x808080, 0x808080,
-  0x808080, 0x808080, 0x808080, 0x808080
-};
-
 #define gst_dvd_spu_parent_class parent_class
 G_DEFINE_TYPE (GstDVDSpu, gst_dvd_spu, GST_TYPE_ELEMENT);
 
@@ -172,6 +161,8 @@ gst_dvd_spu_init (GstDVDSpu * dvdspu)
   gst_pad_set_chain_function (dvdspu->subpic_sinkpad, gst_dvd_spu_subpic_chain);
   gst_pad_set_event_function (dvdspu->subpic_sinkpad, gst_dvd_spu_subpic_event);
 
+  GST_PAD_SET_PROXY_ALLOCATION (dvdspu->videosinkpad);
+
   gst_element_add_pad (GST_ELEMENT (dvdspu), dvdspu->videosinkpad);
   gst_element_add_pad (GST_ELEMENT (dvdspu), dvdspu->subpic_sinkpad);
   gst_element_add_pad (GST_ELEMENT (dvdspu), dvdspu->srcpad);
@@ -788,22 +779,19 @@ gst_dvd_spu_advance_spu (GstDVDSpu * dvdspu, GstClockTime new_ts)
         GST_TIME_ARGS (state->next_ts), GST_TIME_ARGS (new_ts));
 
     if (!gstspu_execute_event (dvdspu)) {
-      GstClockTime vid_run_ts;
-
       /* No current command buffer, try and get one */
       SpuPacket *packet = (SpuPacket *) g_queue_pop_head (dvdspu->pending_spus);
 
       if (packet == NULL)
         return;                 /* No SPU packets available */
 
-      vid_run_ts =
-          gst_segment_to_running_time (&dvdspu->video_seg, GST_FORMAT_TIME,
-          dvdspu->video_seg.position);
       GST_LOG_OBJECT (dvdspu,
           "Popped new SPU packet with TS %" GST_TIME_FORMAT
           ". Video position=%" GST_TIME_FORMAT " (%" GST_TIME_FORMAT
           ") type %s",
-          GST_TIME_ARGS (packet->event_ts), GST_TIME_ARGS (vid_run_ts),
+          GST_TIME_ARGS (packet->event_ts),
+          GST_TIME_ARGS (gst_segment_to_running_time (&dvdspu->video_seg,
+                  GST_FORMAT_TIME, dvdspu->video_seg.position)),
           GST_TIME_ARGS (dvdspu->video_seg.position),
           packet->buf ? "buffer" : "event");
 
@@ -838,6 +826,9 @@ gst_dvd_spu_check_still_updates (GstDVDSpu * dvdspu)
 
   if (dvdspu->spu_state.flags & SPU_STATE_STILL_FRAME) {
 
+    if (dvdspu->video_seg.format != GST_FORMAT_TIME)
+      return;                   /* No video segment or frames yet */
+
     vid_ts = gst_segment_to_running_time (&dvdspu->video_seg,
         GST_FORMAT_TIME, dvdspu->video_seg.position);
     sub_ts = gst_segment_to_running_time (&dvdspu->subp_seg,
@@ -953,7 +944,7 @@ gst_dvd_spu_subpic_chain (GstPad * pad, GstObject * parent, GstBuffer * buf)
 
   switch (dvdspu->spu_input_type) {
     case SPU_INPUT_TYPE_VOBSUB:
-      if (size > 4) {
+      if (size >= 2) {
         guint8 header[2];
         guint16 packet_size;
 
@@ -962,6 +953,10 @@ gst_dvd_spu_subpic_chain (GstPad * pad, GstObject * parent, GstBuffer * buf)
         if (packet_size == size) {
           submit_new_spu_packet (dvdspu, dvdspu->partial_spu);
           dvdspu->partial_spu = NULL;
+        } else if (packet_size == 0) {
+          GST_LOG_OBJECT (dvdspu, "Discarding empty SPU buffer");
+          gst_buffer_unref (dvdspu->partial_spu);
+          dvdspu->partial_spu = NULL;
         } else if (packet_size < size) {
           /* Somehow we collected too much - something is wrong. Drop the
            * packet entirely and wait for a new one */
@@ -1063,6 +1058,7 @@ gst_dvd_spu_subpic_event (GstPad * pad, GstObject * parent, GstEvent * event)
       break;
     }
     case GST_EVENT_CUSTOM_DOWNSTREAM:
+    case GST_EVENT_CUSTOM_DOWNSTREAM_STICKY:
     case GST_EVENT_CUSTOM_DOWNSTREAM_OOB:
     {
       const GstStructure *structure = gst_event_get_structure (event);
@@ -1173,7 +1169,6 @@ gst_dvd_spu_subpic_event (GstPad * pad, GstObject * parent, GstEvent * event)
        * video might still continue, though */
       gst_event_unref (event);
       goto done;
-      break;
     default:
       res = gst_pad_event_default (pad, parent, event);
       break;
