@@ -88,11 +88,9 @@
 #include <stdio.h>
 #include <string.h>
 
-/* FIXME 0.11: suppress warnings for deprecated API such as GStaticRecMutex
- * with newer GLib versions (>= 2.31.0) */
-#define GLIB_DISABLE_DEPRECATION_WARNINGS
-
+#include <gst/tag/tag.h>
 #include <gst/video/video.h>
+#include <gst/mpegts/mpegts.h>
 
 #include "mpegtsmux.h"
 
@@ -104,12 +102,13 @@ GST_DEBUG_CATEGORY (mpegtsmux_debug);
 
 enum
 {
-  ARG_0,
-  ARG_PROG_MAP,
-  ARG_M2TS_MODE,
-  ARG_PAT_INTERVAL,
-  ARG_PMT_INTERVAL,
-  ARG_ALIGNMENT
+  PROP_0,
+  PROP_PROG_MAP,
+  PROP_M2TS_MODE,
+  PROP_PAT_INTERVAL,
+  PROP_PMT_INTERVAL,
+  PROP_ALIGNMENT,
+  PROP_SI_INTERVAL
 };
 
 #define MPEGTSMUX_DEFAULT_ALIGNMENT    -1
@@ -120,13 +119,17 @@ static GstStaticPadTemplate mpegtsmux_sink_factory =
     GST_PAD_SINK,
     GST_PAD_REQUEST,
     GST_STATIC_CAPS ("video/mpeg, "
+        "parsed = (boolean) TRUE, "
         "mpegversion = (int) { 1, 2, 4 }, "
         "systemstream = (boolean) false; "
         "video/x-dirac;"
-        "video/x-h264,stream-format=(string)byte-stream;"
+        "video/x-h264,stream-format=(string)byte-stream,"
+        "alignment=(string){au, nal}; "
         "audio/mpeg, "
+        "parsed = (boolean) TRUE, "
         "mpegversion = (int) { 1, 2 };"
         "audio/mpeg, "
+        "framed = (boolean) TRUE, "
         "mpegversion = (int) 4, stream-format = (string) { raw, adts };"
         "audio/x-lpcm, "
         "width = (int) { 16, 20, 24 }, "
@@ -134,7 +137,9 @@ static GstStaticPadTemplate mpegtsmux_sink_factory =
         "channels = (int) [ 1, 8 ], "
         "dynamic_range = (int) [ 0, 255 ], "
         "emphasis = (boolean) { FALSE, TRUE }, "
-        "mute = (boolean) { FALSE, TRUE }; " "audio/x-ac3;" "audio/x-dts;"
+        "mute = (boolean) { FALSE, TRUE }; "
+        "audio/x-ac3, framed = (boolean) TRUE;"
+        "audio/x-dts, framed = (boolean) TRUE;"
         "subpicture/x-dvb;" "application/x-teletext"));
 
 static GstStaticPadTemplate mpegtsmux_src_factory =
@@ -162,7 +167,7 @@ static GstFlowReturn mpegtsmux_push_packets (MpegTsMux * mux, gboolean force);
 static gboolean new_packet_m2ts (MpegTsMux * mux, GstBuffer * buf,
     gint64 new_pcr);
 
-static void mpegtsdemux_prepare_srcpad (MpegTsMux * mux);
+static void mpegtsmux_prepare_srcpad (MpegTsMux * mux);
 GstFlowReturn mpegtsmux_clip_inc_running_time (GstCollectPads * pads,
     GstCollectData * cdata, GstBuffer * buf, GstBuffer ** outbuf,
     gpointer user_data);
@@ -176,7 +181,8 @@ static GstPad *mpegtsmux_request_new_pad (GstElement * element,
 static void mpegtsmux_release_pad (GstElement * element, GstPad * pad);
 static GstStateChangeReturn mpegtsmux_change_state (GstElement * element,
     GstStateChange transition);
-static void mpegtsdemux_set_header_on_caps (MpegTsMux * mux);
+static gboolean mpegtsmux_send_event (GstElement * element, GstEvent * event);
+static void mpegtsmux_set_header_on_caps (MpegTsMux * mux);
 static gboolean mpegtsmux_src_event (GstPad * pad, GstObject * parent,
     GstEvent * event);
 
@@ -241,41 +247,48 @@ mpegtsmux_class_init (MpegTsMuxClass * klass)
   gstelement_class->request_new_pad = mpegtsmux_request_new_pad;
   gstelement_class->release_pad = mpegtsmux_release_pad;
   gstelement_class->change_state = mpegtsmux_change_state;
+  gstelement_class->send_event = mpegtsmux_send_event;
 
 #if 0
   gstelement_class->set_index = GST_DEBUG_FUNCPTR (mpegtsmux_set_index);
   gstelement_class->get_index = GST_DEBUG_FUNCPTR (mpegtsmux_get_index);
 #endif
 
-  g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_PROG_MAP,
+  g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_PROG_MAP,
       g_param_spec_boxed ("prog-map", "Program map",
           "A GstStructure specifies the mapping from elementary streams to programs",
           GST_TYPE_STRUCTURE, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
-  g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_M2TS_MODE,
+  g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_M2TS_MODE,
       g_param_spec_boolean ("m2ts-mode", "M2TS(192 bytes) Mode",
           "Set to TRUE to output Blu-Ray disc format with 192 byte packets. "
           "FALSE for standard TS format with 188 byte packets.",
           MPEGTSMUX_DEFAULT_M2TS, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
-  g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_PAT_INTERVAL,
+  g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_PAT_INTERVAL,
       g_param_spec_uint ("pat-interval", "PAT interval",
           "Set the interval (in ticks of the 90kHz clock) for writing out the PAT table",
           1, G_MAXUINT, TSMUX_DEFAULT_PAT_INTERVAL,
           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
-  g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_PMT_INTERVAL,
+  g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_PMT_INTERVAL,
       g_param_spec_uint ("pmt-interval", "PMT interval",
           "Set the interval (in ticks of the 90kHz clock) for writing out the PMT table",
           1, G_MAXUINT, TSMUX_DEFAULT_PMT_INTERVAL,
           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
-  g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_ALIGNMENT,
+  g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_ALIGNMENT,
       g_param_spec_int ("alignment", "packet alignment",
           "Number of packets per buffer (padded with dummy packets on EOS) "
-          "(-1 = auto, 0 = all available packets)",
+          "(-1 = auto, 0 = all available packets, 7 for UDP streaming)",
           -1, G_MAXINT, MPEGTSMUX_DEFAULT_ALIGNMENT,
           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+
+  g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_SI_INTERVAL,
+      g_param_spec_uint ("si-interval", "SI interval",
+          "Set the interval (in ticks of the 90kHz clock) for writing out the Service"
+          "Information tables", 1, G_MAXUINT, TSMUX_DEFAULT_SI_INTERVAL,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 }
 
 static void
@@ -299,9 +312,6 @@ mpegtsmux_init (MpegTsMux * mux)
   gst_collect_pads_set_clip_function (mux->collect, (GstCollectPadsClipFunction)
       GST_DEBUG_FUNCPTR (mpegtsmux_clip_inc_running_time), mux);
 
-  mux->tsmux = tsmux_new ();
-  tsmux_set_write_func (mux->tsmux, new_packet_cb, mux);
-
   mux->adapter = gst_adapter_new ();
   mux->out_adapter = gst_adapter_new ();
 
@@ -309,6 +319,7 @@ mpegtsmux_init (MpegTsMux * mux)
   mux->m2ts_mode = MPEGTSMUX_DEFAULT_M2TS;
   mux->pat_interval = TSMUX_DEFAULT_PAT_INTERVAL;
   mux->pmt_interval = TSMUX_DEFAULT_PMT_INTERVAL;
+  mux->si_interval = TSMUX_DEFAULT_SI_INTERVAL;
   mux->prog_map = NULL;
   mux->alignment = MPEGTSMUX_DEFAULT_ALIGNMENT;
 
@@ -320,8 +331,7 @@ static void
 mpegtsmux_pad_reset (MpegTsPadData * pad_data)
 {
   pad_data->pid = 0;
-  pad_data->last_pts = GST_CLOCK_TIME_NONE;
-  pad_data->last_dts = GST_CLOCK_TIME_NONE;
+  pad_data->min_dts = GST_CLOCK_TIME_NONE;
   pad_data->prog_id = -1;
 #if 0
   pad_data->prog_id = -1;
@@ -340,6 +350,12 @@ mpegtsmux_pad_reset (MpegTsPadData * pad_data)
   /* reference owned elsewhere */
   pad_data->stream = NULL;
   pad_data->prog = NULL;
+
+  if (pad_data->language) {
+    g_free (pad_data->language);
+    pad_data->language = NULL;
+  }
+
 }
 
 static void
@@ -355,7 +371,6 @@ mpegtsmux_reset (MpegTsMux * mux, gboolean alloc)
   mux->is_delta = TRUE;
 
   mux->streamheader_sent = FALSE;
-  mux->force_key_unit_event = NULL;
   mux->pending_key_unit_ts = GST_CLOCK_TIME_NONE;
 #if 0
   mux->spn_count = 0;
@@ -365,8 +380,10 @@ mpegtsmux_reset (MpegTsMux * mux, gboolean alloc)
     mux->element_index = NULL;
   }
 #endif
-  gst_adapter_clear (mux->adapter);
-  gst_adapter_clear (mux->out_adapter);
+  if (mux->adapter)
+    gst_adapter_clear (mux->adapter);
+  if (mux->out_adapter)
+    gst_adapter_clear (mux->out_adapter);
 
   if (mux->tsmux) {
     tsmux_free (mux->tsmux);
@@ -391,10 +408,12 @@ mpegtsmux_reset (MpegTsMux * mux, gboolean alloc)
   gst_event_replace (&mux->force_key_unit_event, NULL);
   gst_buffer_replace (&mux->out_buffer, NULL);
 
-  GST_COLLECT_PADS_STREAM_LOCK (mux->collect);
-  for (walk = mux->collect->data; walk != NULL; walk = g_slist_next (walk))
-    mpegtsmux_pad_reset ((MpegTsPadData *) walk->data);
-  GST_COLLECT_PADS_STREAM_UNLOCK (mux->collect);
+  if (mux->collect) {
+    GST_COLLECT_PADS_STREAM_LOCK (mux->collect);
+    for (walk = mux->collect->data; walk != NULL; walk = g_slist_next (walk))
+      mpegtsmux_pad_reset ((MpegTsPadData *) walk->data);
+    GST_COLLECT_PADS_STREAM_UNLOCK (mux->collect);
+  }
 
   if (alloc) {
     mux->tsmux = tsmux_new ();
@@ -437,11 +456,11 @@ gst_mpegtsmux_set_property (GObject * object, guint prop_id,
   GSList *walk;
 
   switch (prop_id) {
-    case ARG_M2TS_MODE:
+    case PROP_M2TS_MODE:
       /*set incase if the output stream need to be of 192 bytes */
       mux->m2ts_mode = g_value_get_boolean (value);
       break;
-    case ARG_PROG_MAP:
+    case PROP_PROG_MAP:
     {
       const GstStructure *s = gst_value_get_structure (value);
       if (mux->prog_map) {
@@ -453,12 +472,12 @@ gst_mpegtsmux_set_property (GObject * object, guint prop_id,
         mux->prog_map = NULL;
       break;
     }
-    case ARG_PAT_INTERVAL:
+    case PROP_PAT_INTERVAL:
       mux->pat_interval = g_value_get_uint (value);
       if (mux->tsmux)
         tsmux_set_pat_interval (mux->tsmux, mux->pat_interval);
       break;
-    case ARG_PMT_INTERVAL:
+    case PROP_PMT_INTERVAL:
       walk = mux->collect->data;
       mux->pmt_interval = g_value_get_uint (value);
 
@@ -469,9 +488,13 @@ gst_mpegtsmux_set_property (GObject * object, guint prop_id,
         walk = g_slist_next (walk);
       }
       break;
-    case ARG_ALIGNMENT:
+    case PROP_ALIGNMENT:
       mux->alignment = g_value_get_int (value);
       break;
+    case PROP_SI_INTERVAL:
+      mux->si_interval = g_value_get_uint (value);
+      tsmux_set_si_interval (mux->tsmux, mux->si_interval);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -485,21 +508,24 @@ gst_mpegtsmux_get_property (GObject * object, guint prop_id,
   MpegTsMux *mux = GST_MPEG_TSMUX (object);
 
   switch (prop_id) {
-    case ARG_M2TS_MODE:
+    case PROP_M2TS_MODE:
       g_value_set_boolean (value, mux->m2ts_mode);
       break;
-    case ARG_PROG_MAP:
+    case PROP_PROG_MAP:
       gst_value_set_structure (value, mux->prog_map);
       break;
-    case ARG_PAT_INTERVAL:
+    case PROP_PAT_INTERVAL:
       g_value_set_uint (value, mux->pat_interval);
       break;
-    case ARG_PMT_INTERVAL:
+    case PROP_PMT_INTERVAL:
       g_value_set_uint (value, mux->pmt_interval);
       break;
-    case ARG_ALIGNMENT:
+    case PROP_ALIGNMENT:
       g_value_set_int (value, mux->alignment);
       break;
+    case PROP_SI_INTERVAL:
+      g_value_set_uint (value, mux->si_interval);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -565,7 +591,6 @@ mpegtsmux_create_stream (MpegTsMux * mux, MpegTsPadData * ts_data)
       GST_PTR_FORMAT, ts_data->pid, caps);
 
   s = gst_caps_get_structure (caps, 0);
-  g_return_val_if_fail (s != NULL, FALSE);
 
   mt = gst_structure_get_name (s);
   value = gst_structure_get_value (s, "codec_data");
@@ -646,7 +671,8 @@ mpegtsmux_create_stream (MpegTsMux * mux, MpegTsPadData * ts_data)
   }
 
   if (st != TSMUX_ST_RESERVED) {
-    ts_data->stream = tsmux_create_stream (mux->tsmux, st, ts_data->pid);
+    ts_data->stream = tsmux_create_stream (mux->tsmux, st, ts_data->pid,
+        ts_data->language);
   } else {
     GST_DEBUG_OBJECT (pad, "Failed to determine stream type");
   }
@@ -779,6 +805,8 @@ mpegtsmux_sink_event (GstCollectPads * pads, GstCollectData * data,
   MpegTsMux *mux = GST_MPEG_TSMUX (user_data);
   gboolean res = FALSE;
   gboolean forward = TRUE;
+  MpegTsPadData *pad_data = (MpegTsPadData *) data;
+
 #ifndef GST_DISABLE_GST_DEBUG
   GstPad *pad;
 
@@ -800,10 +828,9 @@ mpegtsmux_sink_event (GstCollectPads * pads, GstCollectData * data,
 
       gst_video_event_parse_downstream_force_key_unit (event,
           &timestamp, &stream_time, &running_time, &all_headers, &count);
-      GST_INFO_OBJECT (mux, "have downstream force-key-unit event on pad %s, "
+      GST_INFO_OBJECT (pad, "have downstream force-key-unit event, "
           "seqnum %d, running-time %" GST_TIME_FORMAT " count %d",
-          gst_pad_get_name (pad), gst_event_get_seqnum (event),
-          GST_TIME_ARGS (running_time), count);
+          gst_event_get_seqnum (event), GST_TIME_ARGS (running_time), count);
 
       if (mux->force_key_unit_event != NULL) {
         GST_INFO_OBJECT (mux, "skipping downstream force key unit event "
@@ -818,6 +845,32 @@ mpegtsmux_sink_event (GstCollectPads * pads, GstCollectData * data,
       gst_event_replace (&mux->force_key_unit_event, event);
       break;
     }
+    case GST_EVENT_TAG:{
+      GstTagList *list;
+      gchar *lang = NULL;
+
+      GST_DEBUG_OBJECT (mux, "received tag event");
+      gst_event_parse_tag (event, &list);
+
+      /* Matroska wants ISO 639-2B code, taglist most likely contains 639-1 */
+      if (gst_tag_list_get_string (list, GST_TAG_LANGUAGE_CODE, &lang)) {
+        const gchar *lang_code;
+
+        lang_code = gst_tag_get_language_code_iso_639_2B (lang);
+        if (lang_code) {
+          GST_DEBUG_OBJECT (pad, "Setting language to '%s'", lang_code);
+          pad_data->language = g_strdup (lang_code);
+        } else {
+          GST_WARNING_OBJECT (pad, "Did not get language code for '%s'", lang);
+        }
+        g_free (lang);
+      }
+
+      /* handled this, don't want collectpads to forward it downstream */
+      res = TRUE;
+      forward = gst_tag_list_get_scope (list) == GST_TAG_SCOPE_GLOBAL;
+      break;
+    }
     default:
       break;
   }
@@ -834,7 +887,7 @@ out:
 static gboolean
 mpegtsmux_src_event (GstPad * pad, GstObject * parent, GstEvent * event)
 {
-  MpegTsMux *mux = GST_MPEG_TSMUX (gst_pad_get_parent (pad));
+  MpegTsMux *mux = GST_MPEG_TSMUX (parent);
   gboolean res = TRUE, forward = TRUE;
 
   switch (GST_EVENT_TYPE (event)) {
@@ -845,7 +898,7 @@ mpegtsmux_src_event (GstPad * pad, GstObject * parent, GstEvent * event)
       GstPad *sinkpad;
       GValue sinkpad_value = G_VALUE_INIT;
       GstClockTime running_time;
-      gboolean all_headers, done;
+      gboolean all_headers, done, res = FALSE;
       guint count;
 
       if (!gst_video_event_is_force_key_unit (event))
@@ -870,7 +923,8 @@ mpegtsmux_src_event (GstPad * pad, GstObject * parent, GstEvent * event)
       iter = gst_element_iterate_sink_pads (GST_ELEMENT_CAST (mux));
       done = FALSE;
       while (!done) {
-        gboolean res = FALSE, tmp;
+        gboolean tmp;
+
         iter_ret = gst_iterator_next (iter, &sinkpad_value);
         sinkpad = g_value_get_object (&sinkpad_value);
 
@@ -879,8 +933,7 @@ mpegtsmux_src_event (GstPad * pad, GstObject * parent, GstEvent * event)
             done = TRUE;
             break;
           case GST_ITERATOR_OK:
-            GST_INFO_OBJECT (mux, "forwarding to %s",
-                gst_pad_get_name (sinkpad));
+            GST_INFO_OBJECT (pad, "forwarding");
             tmp = gst_pad_push_event (sinkpad, gst_event_ref (event));
             GST_INFO_OBJECT (mux, "result %d", tmp);
             /* succeed if at least one pad succeeds */
@@ -907,7 +960,6 @@ mpegtsmux_src_event (GstPad * pad, GstObject * parent, GstEvent * event)
   else
     gst_event_unref (event);
 
-  gst_object_unref (mux);
   return res;
 }
 
@@ -920,8 +972,7 @@ check_pending_key_unit_event (GstEvent * pending_event, GstSegment * segment,
   guint count;
   GstEvent *event = NULL;
 
-  g_return_val_if_fail (pending_event != NULL, NULL);
-  g_return_val_if_fail (segment != NULL, NULL);
+  g_assert (segment != NULL);
 
   if (pending_event == NULL)
     goto out;
@@ -947,8 +998,13 @@ check_pending_key_unit_event (GstEvent * pending_event, GstSegment * segment,
   stream_time = gst_segment_to_stream_time (segment,
       GST_FORMAT_TIME, timestamp);
 
-  gst_video_event_parse_upstream_force_key_unit (pending_event,
-      NULL, &all_headers, &count);
+  if (GST_EVENT_TYPE (pending_event) == GST_EVENT_CUSTOM_DOWNSTREAM) {
+    gst_video_event_parse_downstream_force_key_unit (pending_event,
+        NULL, NULL, NULL, &all_headers, &count);
+  } else {
+    gst_video_event_parse_upstream_force_key_unit (pending_event, NULL,
+        &all_headers, &count);
+  }
 
   event =
       gst_video_event_new_downstream_force_key_unit (timestamp, stream_time,
@@ -984,7 +1040,6 @@ mpegtsmux_clip_inc_running_time (GstCollectPads * pads,
       GST_LOG_OBJECT (cdata->pad, "buffer pts %" GST_TIME_FORMAT " -> %"
           GST_TIME_FORMAT " running time",
           GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (buf)), GST_TIME_ARGS (time));
-      pad_data->last_pts = time;
       buf = *outbuf = gst_buffer_make_writable (buf);
       GST_BUFFER_TIMESTAMP (*outbuf) = time;
     }
@@ -1003,16 +1058,13 @@ mpegtsmux_clip_inc_running_time (GstCollectPads * pads,
       GST_LOG_OBJECT (cdata->pad, "buffer dts %" GST_TIME_FORMAT " -> %"
           GST_TIME_FORMAT " running time",
           GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (buf)), GST_TIME_ARGS (time));
-      if (GST_CLOCK_TIME_IS_VALID (pad_data->last_dts) &&
-          time < pad_data->last_dts) {
-        /* FIXME DTS/PTS mess again;
-         * probably needs a whole lot more subtle handling (cf qtmux) */
+      if (GST_CLOCK_TIME_IS_VALID (pad_data->min_dts) &&
+          time < pad_data->min_dts) {
+        /* Ignore DTS going backward */
         GST_WARNING_OBJECT (cdata->pad, "ignoring DTS going backward");
-        time = pad_data->last_dts;
-      } else {
-        pad_data->last_dts = time;
+        time = pad_data->min_dts;
       }
-      buf = *outbuf = gst_buffer_make_writable (buf);
+      *outbuf = gst_buffer_make_writable (buf);
       GST_BUFFER_DTS (*outbuf) = time;
     }
   }
@@ -1039,7 +1091,7 @@ mpegtsmux_collected_buffer (GstCollectPads * pads, GstCollectData * data,
   TsMuxProgram *prog;
   gint64 pts = -1;
   guint64 dts = -1;
-  gboolean delta = TRUE;
+  gboolean delta = TRUE, header = FALSE;
   StreamData *stream_data;
 
   GST_DEBUG_OBJECT (mux, "Pads collected");
@@ -1049,7 +1101,7 @@ mpegtsmux_collected_buffer (GstCollectPads * pads, GstCollectData * data,
     if (G_UNLIKELY (ret != GST_FLOW_OK))
       return ret;
 
-    mpegtsdemux_prepare_srcpad (mux);
+    mpegtsmux_prepare_srcpad (mux);
 
     mux->first = FALSE;
   }
@@ -1118,18 +1170,16 @@ mpegtsmux_collected_buffer (GstCollectPads * pads, GstCollectData * data,
   GST_DEBUG_OBJECT (COLLECT_DATA_PAD (best),
       "Chose stream for output (PID: 0x%04x)", best->pid);
 
-  if (GST_CLOCK_TIME_IS_VALID (GST_BUFFER_PTS (buf)) &&
-      GST_CLOCK_TIME_IS_VALID (best->last_pts)) {
-    pts = GSTTIME_TO_MPEGTIME (best->last_pts);
+  if (GST_CLOCK_TIME_IS_VALID (GST_BUFFER_PTS (buf))) {
+    pts = GSTTIME_TO_MPEGTIME (GST_BUFFER_PTS (buf));
     GST_DEBUG_OBJECT (mux, "Buffer has PTS %" GST_TIME_FORMAT " pts %"
-        G_GINT64_FORMAT, GST_TIME_ARGS (best->last_pts), pts);
+        G_GINT64_FORMAT, GST_TIME_ARGS (GST_BUFFER_PTS (buf)), pts);
   }
 
-  if (GST_CLOCK_TIME_IS_VALID (GST_BUFFER_DTS (buf)) &&
-      GST_CLOCK_TIME_IS_VALID (best->last_dts)) {
-    dts = GSTTIME_TO_MPEGTIME (best->last_dts);
+  if (GST_CLOCK_TIME_IS_VALID (GST_BUFFER_DTS (buf))) {
+    dts = GSTTIME_TO_MPEGTIME (GST_BUFFER_DTS (buf));
     GST_DEBUG_OBJECT (mux, "Buffer has DTS %" GST_TIME_FORMAT " dts %"
-        G_GINT64_FORMAT, GST_TIME_ARGS (best->last_dts), dts);
+        G_GINT64_FORMAT, GST_TIME_ARGS (GST_BUFFER_DTS (buf)), dts);
   }
 
   /* should not have a DTS without PTS */
@@ -1140,6 +1190,7 @@ mpegtsmux_collected_buffer (GstCollectPads * pads, GstCollectData * data,
 
   if (best->stream->is_video_stream) {
     delta = GST_BUFFER_FLAG_IS_SET (buf, GST_BUFFER_FLAG_DELTA_UNIT);
+    header = GST_BUFFER_FLAG_IS_SET (buf, GST_BUFFER_FLAG_HEADER);
 #if 0
     GST_OBJECT_LOCK (mux);
     if (mux->element_index && !delta && best->element_index_writer_id != -1) {
@@ -1161,11 +1212,12 @@ mpegtsmux_collected_buffer (GstCollectPads * pads, GstCollectData * data,
   if (prog->pcr_stream == best->stream) {
     /* prefer DTS if present for PCR as it should be monotone */
     mux->last_ts =
-        GST_CLOCK_TIME_IS_VALID (best->last_dts) ? best->last_dts : best->
-        last_pts;
+        GST_CLOCK_TIME_IS_VALID (GST_BUFFER_DTS (buf)) ?
+        GST_BUFFER_DTS (buf) : GST_BUFFER_PTS (buf);
   }
 
   mux->is_delta = delta;
+  mux->is_header = header;
   while (tsmux_stream_bytes_in_buffer (best->stream) > 0) {
     if (!tsmux_write_stream_packet (mux->tsmux, best->stream)) {
       /* Failed writing data for some reason. Set appropriate error */
@@ -1273,7 +1325,7 @@ new_packet_common_init (MpegTsMux * mux, GstBuffer * buf, guint8 * data,
     guint len)
 {
   /* Packets should be at least 188 bytes, but check anyway */
-  g_return_if_fail (len >= 2 || !data);
+  g_assert (len >= 2 || !data);
 
   if (!mux->streamheader_sent && data) {
     guint pid = ((data[1] & 0x1f) << 8) | data[2];
@@ -1289,12 +1341,16 @@ new_packet_common_init (MpegTsMux * mux, GstBuffer * buf, guint8 * data,
       }
       mux->streamheader = g_list_append (mux->streamheader, hbuf);
     } else if (mux->streamheader) {
-      mpegtsdemux_set_header_on_caps (mux);
+      mpegtsmux_set_header_on_caps (mux);
       mux->streamheader_sent = TRUE;
     }
   }
 
   if (buf) {
+    if (mux->is_header) {
+      GST_LOG_OBJECT (mux, "marking as header buffer");
+      GST_BUFFER_FLAG_SET (buf, GST_BUFFER_FLAG_HEADER);
+    }
     if (mux->is_delta) {
       GST_LOG_OBJECT (mux, "marking as delta unit");
       GST_BUFFER_FLAG_SET (buf, GST_BUFFER_FLAG_DELTA_UNIT);
@@ -1308,11 +1364,9 @@ new_packet_common_init (MpegTsMux * mux, GstBuffer * buf, guint8 * data,
 static GstFlowReturn
 mpegtsmux_push_packets (MpegTsMux * mux, gboolean force)
 {
+  GstBufferList *buffer_list;
   gint align = mux->alignment;
   gint av, packet_size;
-  GstBuffer *buf;
-  GstFlowReturn ret = GST_FLOW_OK;
-  GstClockTime ts;
 
   if (mux->m2ts_mode) {
     packet_size = M2TS_PACKET_LENGTH;
@@ -1327,25 +1381,31 @@ mpegtsmux_push_packets (MpegTsMux * mux, gboolean force)
   av = gst_adapter_available (mux->out_adapter);
   GST_LOG_OBJECT (mux, "align %d, av %d", align, av);
 
-  if (!align)
-    align = av;
-  else
-    align *= packet_size;
+  if (av == 0)
+    return GST_FLOW_OK;
 
-  /* FIXME: what about DTS here? */
-  GST_LOG_OBJECT (mux, "aligning to %d bytes", align);
-  if (G_LIKELY ((align <= av) && av)) {
-    GST_LOG_OBJECT (mux, "pushing %d aligned bytes", av - (av % align));
-    ts = gst_adapter_prev_pts (mux->out_adapter, NULL);
-    buf = gst_adapter_take_buffer (mux->out_adapter, av - (av % align));
-    g_assert (buf);
-    GST_BUFFER_PTS (buf) = ts;
-
-    ret = gst_pad_push (mux->srcpad, buf);
-    av = av % align;
+  /* no alignment, just push all available data */
+  if (align == 0) {
+    buffer_list = gst_adapter_take_buffer_list (mux->out_adapter, av);
+    return gst_pad_push_list (mux->srcpad, buffer_list);
   }
 
-  if (av && force) {
+  align *= packet_size;
+
+  if (!force && align > av)
+    return GST_FLOW_OK;
+
+  buffer_list = gst_buffer_list_new_sized ((av / align) + 1);
+
+  GST_LOG_OBJECT (mux, "aligning to %d bytes", align);
+  while (align <= av) {
+    gst_buffer_list_add (buffer_list,
+        gst_adapter_take_buffer (mux->out_adapter, align));
+    av -= align;
+  }
+
+  if (av > 0 && force) {
+    GstBuffer *buf;
     guint8 *data;
     guint32 header;
     gint dummy;
@@ -1355,11 +1415,9 @@ mpegtsmux_push_packets (MpegTsMux * mux, gboolean force)
     buf = gst_buffer_new_and_alloc (align);
     gst_buffer_map (buf, &map, GST_MAP_READ);
     data = map.data;
-    ts = gst_adapter_prev_pts (mux->out_adapter, NULL);
 
     gst_adapter_copy (mux->out_adapter, data, 0, av);
     gst_adapter_clear (mux->out_adapter);
-    GST_BUFFER_PTS (buf) = ts;
 
     data += av;
     header = GST_READ_UINT32_BE (data - packet_size);
@@ -1389,11 +1447,10 @@ mpegtsmux_push_packets (MpegTsMux * mux, gboolean force)
     }
 
     gst_buffer_unmap (buf, &map);
-
-    ret = gst_pad_push (mux->srcpad, buf);
+    gst_buffer_list_add (buffer_list, buf);
   }
 
-  return ret;
+  return gst_pad_push_list (mux->srcpad, buffer_list);
 }
 
 static GstFlowReturn
@@ -1577,7 +1634,7 @@ alloc_packet_cb (GstBuffer ** _buf, void *user_data)
 }
 
 static void
-mpegtsdemux_set_header_on_caps (MpegTsMux * mux)
+mpegtsmux_set_header_on_caps (MpegTsMux * mux)
 {
   GstBuffer *buf;
   GstStructure *structure;
@@ -1611,7 +1668,7 @@ mpegtsdemux_set_header_on_caps (MpegTsMux * mux)
 }
 
 static void
-mpegtsdemux_prepare_srcpad (MpegTsMux * mux)
+mpegtsmux_prepare_srcpad (MpegTsMux * mux)
 {
   GstSegment seg;
   /* we are not going to seek */
@@ -1679,9 +1736,31 @@ mpegtsmux_change_state (GstElement * element, GstStateChange transition)
   return ret;
 }
 
+static gboolean
+mpegtsmux_send_event (GstElement * element, GstEvent * event)
+{
+  GstMpegtsSection *section;
+  MpegTsMux *mux = GST_MPEG_TSMUX (element);
+
+  section = gst_event_parse_mpegts_section (event);
+  gst_event_unref (event);
+
+  if (section) {
+    GST_DEBUG ("Received event with mpegts section");
+
+    /* TODO: Check that the section type is supported */
+    tsmux_add_mpegts_si_section (mux->tsmux, section);
+
+    return TRUE;
+  }
+
+  return FALSE;
+}
+
 static gboolean
 plugin_init (GstPlugin * plugin)
 {
+  gst_mpegts_initialize ();
   if (!gst_element_register (plugin, "mpegtsmux", GST_RANK_PRIMARY,
           mpegtsmux_get_type ()))
     return FALSE;
