@@ -13,8 +13,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
 /*
@@ -42,11 +42,11 @@ static const char mve_preamble[] = MVE_PREAMBLE;
 
 enum
 {
-  ARG_0,
-  ARG_AUDIO_COMPRESSION,
-  ARG_VIDEO_QUICK_ENCODING,
-  ARG_VIDEO_SCREEN_WIDTH,
-  ARG_VIDEO_SCREEN_HEIGHT
+  PROP_0,
+  PROP_AUDIO_COMPRESSION,
+  PROP_VIDEO_QUICK_ENCODING,
+  PROP_VIDEO_SCREEN_WIDTH,
+  PROP_VIDEO_SCREEN_HEIGHT
 };
 
 #define MVE_MUX_DEFAULT_COMPRESSION    FALSE
@@ -226,16 +226,16 @@ gst_mve_mux_get_property (GObject * object,
   mvemux = GST_MVE_MUX (object);
 
   switch (prop_id) {
-    case ARG_AUDIO_COMPRESSION:
+    case PROP_AUDIO_COMPRESSION:
       g_value_set_boolean (value, mvemux->compression);
       break;
-    case ARG_VIDEO_QUICK_ENCODING:
+    case PROP_VIDEO_QUICK_ENCODING:
       g_value_set_boolean (value, mvemux->quick_encoding);
       break;
-    case ARG_VIDEO_SCREEN_WIDTH:
+    case PROP_VIDEO_SCREEN_WIDTH:
       g_value_set_uint (value, mvemux->screen_width);
       break;
-    case ARG_VIDEO_SCREEN_HEIGHT:
+    case PROP_VIDEO_SCREEN_HEIGHT:
       g_value_set_uint (value, mvemux->screen_height);
       break;
     default:
@@ -254,16 +254,16 @@ gst_mve_mux_set_property (GObject * object,
   mvemux = GST_MVE_MUX (object);
 
   switch (prop_id) {
-    case ARG_AUDIO_COMPRESSION:
+    case PROP_AUDIO_COMPRESSION:
       mvemux->compression = g_value_get_boolean (value);
       break;
-    case ARG_VIDEO_QUICK_ENCODING:
+    case PROP_VIDEO_QUICK_ENCODING:
       mvemux->quick_encoding = g_value_get_boolean (value);
       break;
-    case ARG_VIDEO_SCREEN_WIDTH:
+    case PROP_VIDEO_SCREEN_WIDTH:
       mvemux->screen_width = g_value_get_uint (value);
       break;
-    case ARG_VIDEO_SCREEN_HEIGHT:
+    case PROP_VIDEO_SCREEN_HEIGHT:
       mvemux->screen_height = g_value_get_uint (value);
       break;
     default:
@@ -1402,23 +1402,23 @@ gst_mve_mux_class_init (GstMveMuxClass * klass)
   gobject_class->get_property = gst_mve_mux_get_property;
   gobject_class->set_property = gst_mve_mux_set_property;
 
-  g_object_class_install_property (gobject_class, ARG_AUDIO_COMPRESSION,
+  g_object_class_install_property (gobject_class, PROP_AUDIO_COMPRESSION,
       g_param_spec_boolean ("compression", "Audio compression",
           "Whether to compress audio data", MVE_MUX_DEFAULT_COMPRESSION,
           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
-  g_object_class_install_property (gobject_class, ARG_VIDEO_QUICK_ENCODING,
+  g_object_class_install_property (gobject_class, PROP_VIDEO_QUICK_ENCODING,
       g_param_spec_boolean ("quick", "Quick encoding",
           "Whether to disable expensive encoding operations", TRUE,
           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
-  g_object_class_install_property (gobject_class, ARG_VIDEO_SCREEN_WIDTH,
+  g_object_class_install_property (gobject_class, PROP_VIDEO_SCREEN_WIDTH,
       g_param_spec_uint ("screen-width", "Screen width",
           "Suggested screen width", 320, 1600,
           MVE_MUX_DEFAULT_SCREEN_WIDTH,
           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
-  g_object_class_install_property (gobject_class, ARG_VIDEO_SCREEN_HEIGHT,
+  g_object_class_install_property (gobject_class, PROP_VIDEO_SCREEN_HEIGHT,
       g_param_spec_uint ("screen-height", "Screen height",
           "Suggested screen height", 200, 1200,
           MVE_MUX_DEFAULT_SCREEN_HEIGHT,
