@@ -66,6 +66,8 @@ typedef struct {
   MXFUL ul;
   guint16 size;
   guint8 *data;
+  
+  gboolean g_slice; /* TRUE if data was allocated by GSlice */
 } MXFLocalTag;
 
 /* SMPTE 377M 11.1 */
@@ -146,7 +148,7 @@ typedef struct {
 
   guint8 flags;
   guint64 stream_offset;
-
+  
   guint32 *slice_offset;
   MXFFraction *pos_table;
 } MXFIndexEntry;
@@ -168,10 +170,8 @@ typedef struct {
   guint32 n_index_entries;
   MXFIndexEntry *index_entries;
 
-  /* Computed fields (i.e. not present in file) */
-
-  /* Initial essence offset covered by this segment */
-  guint64 segment_start_offset;
+  GHashTable *other_tags;
+  guint64 stream_offset;
 } MXFIndexTableSegment;
 
 #define GST_TAG_MXF_UMID "mxf-umid"
@@ -254,9 +254,8 @@ GstBuffer * mxf_primer_pack_to_buffer (const MXFPrimerPack *pack);
 gboolean mxf_random_index_pack_parse (const MXFUL *ul, const guint8 *data, guint size, GArray **array);
 GstBuffer * mxf_random_index_pack_to_buffer (const GArray *array);
 
-gboolean mxf_index_table_segment_parse (const MXFUL *ul, MXFIndexTableSegment *segment, const guint8 *data, guint size);
+gboolean mxf_index_table_segment_parse (const MXFUL *ul, MXFIndexTableSegment *segment, const MXFPrimerPack *primer, const guint8 *data, guint size);
 void mxf_index_table_segment_reset (MXFIndexTableSegment *segment);
-GstBuffer * mxf_index_table_segment_to_buffer (const MXFIndexTableSegment *segment);
 
 gboolean mxf_local_tag_parse (const guint8 * data, guint size, guint16 * tag,
     guint16 * tag_size, const guint8 ** tag_data);
@@ -265,7 +264,6 @@ void mxf_local_tag_free (MXFLocalTag *tag);
 gboolean mxf_local_tag_add_to_hash_table (const MXFPrimerPack *primer,
   guint16 tag, const guint8 *tag_data, guint16 tag_size,
   GHashTable **hash_table);
-MXFUL *mxf_primer_tag_to_ul (const MXFPrimerPack *primer, guint16 tag);
 gboolean mxf_local_tag_insert (MXFLocalTag *tag, GHashTable **hash_table);
 
 #endif /* __MXF_TYPES_H__ */
