@@ -57,7 +57,7 @@ static gboolean gst_siren_dec_start (GstAudioDecoder * dec);
 static gboolean gst_siren_dec_stop (GstAudioDecoder * dec);
 static gboolean gst_siren_dec_set_format (GstAudioDecoder * dec,
     GstCaps * caps);
-static GstFlowReturn gst_siren_dec_parse (GstAudioDecoder * dec,
+static gboolean gst_siren_dec_parse (GstAudioDecoder * dec,
     GstAdapter * adapter, gint * offset, gint * length);
 static GstFlowReturn gst_siren_dec_handle_frame (GstAudioDecoder * dec,
     GstBuffer * buffer);
@@ -73,8 +73,10 @@ gst_siren_dec_class_init (GstSirenDecClass * klass)
 
   GST_DEBUG_CATEGORY_INIT (sirendec_debug, "sirendec", 0, "sirendec");
 
-  gst_element_class_add_static_pad_template (element_class, &srctemplate);
-  gst_element_class_add_static_pad_template (element_class, &sinktemplate);
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&srctemplate));
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&sinktemplate));
 
   gst_element_class_set_static_metadata (element_class, "Siren Decoder element",
       "Codec/Decoder/Audio ",
@@ -94,9 +96,6 @@ static void
 gst_siren_dec_init (GstSirenDec * dec)
 {
   gst_audio_decoder_set_needs_format (GST_AUDIO_DECODER (dec), TRUE);
-  gst_audio_decoder_set_use_default_pad_acceptcaps (GST_AUDIO_DECODER_CAST
-      (dec), TRUE);
-  GST_PAD_SET_ACCEPT_TEMPLATE (GST_AUDIO_DECODER_SINK_PAD (dec));
 }
 
 static gboolean
