@@ -28,8 +28,6 @@
 #include <gst/gst.h>
 #include <gst/base/gstbaseparse.h>
 #include <gst/codecparsers/gsth264parser.h>
-#include <gst/video/video.h>
-#include "gstvideoparseutils.h"
 
 G_BEGIN_DECLS
 
@@ -60,9 +58,6 @@ struct _GstH264Parse
   gint fps_num, fps_den;
   gint upstream_par_n, upstream_par_d;
   gint parsed_par_n, parsed_par_d;
-  gint parsed_fps_n, parsed_fps_d;
-  GstVideoColorimetry parsed_colorimetry;
-  gboolean lcevc;
   /* current codec_data in output caps, if any */
   GstBuffer *codec_data;
   /* input codec_data, if any */
@@ -75,12 +70,9 @@ struct _GstH264Parse
   /* state */
   GstH264NalParser *nalparser;
   guint state;
-  guint in_align;
   guint align;
   guint format;
   gint current_off;
-  /* True if input format and alignment match negotiated output */
-  gboolean can_passthrough;
 
   GstClockTime last_report;
   gboolean push_codec;
@@ -89,36 +81,17 @@ struct _GstH264Parse
   gboolean have_sps;
   gboolean have_pps;
 
-  /* per frame sps/pps check for periodic push codec decision */
-  gboolean have_sps_in_frame;
-  gboolean have_pps_in_frame;
-
-  /* per frame AU Delimiter check used when in_format == avc or avc3 */
-  gboolean have_aud_in_frame;
-
-  /* tracing state whether h264parse needs to insert AUD or not.
-   * Used when in_format == byte-stream */
-  gboolean aud_needed;
-
-  /* For insertion of AU Delimiter */
-  gboolean aud_insert;
-
-  gboolean first_frame;
+  gboolean sent_codec_tag;
 
   /* collected SPS and PPS NALUs */
   GstBuffer *sps_nals[GST_H264_MAX_SPS_COUNT];
   GstBuffer *pps_nals[GST_H264_MAX_PPS_COUNT];
 
-  /* collected SEI timestamps */
-  guint num_clock_timestamp;
-  GstH264PicTiming pic_timing_sei;
-
   /* Infos we need to keep track of */
   guint32 sei_cpb_removal_delay;
   guint8 sei_pic_struct;
   guint8 sei_pic_struct_pres_flag;
   guint field_pic_flag;
-  gboolean ignore_vui_fps;
 
   /* cached timestamps */
   /* (trying to) track upstream dts and interpolate */
@@ -128,47 +101,24 @@ struct _GstH264Parse
   gboolean do_ts;
 
   gboolean discont;
-  gboolean marker;
 
   /* frame parsing */
   /*guint last_nal_pos;*/
   /*guint next_sc_pos;*/
   gint idr_pos, sei_pos;
-  gint pic_timing_sei_pos;
-  gint pic_timing_sei_end;
   gboolean update_caps;
   GstAdapter *frame_out;
   gboolean keyframe;
-  gboolean predicted;
-  gboolean bidirectional;
   gboolean header;
   gboolean frame_start;
   /* AU state */
   gboolean picture_start;
 
   /* props */
-  gint interval;
-  gboolean update_timecode;
+  guint interval;
 
   GstClockTime pending_key_unit_ts;
   GstEvent *force_key_unit_event;
-
-  /* Stereo / multiview info */
-  GstVideoMultiviewMode multiview_mode;
-  GstVideoMultiviewFlags multiview_flags;
-  gboolean first_in_bundle;
-
-  GstVideoParseUserData user_data;
-  GstVideoParseUserDataUnregistered user_data_unregistered;
-
-  GstVideoMasteringDisplayInfo mastering_display_info;
-  guint mastering_display_info_state;
-
-  GstVideoContentLightLevel content_light_level;
-  guint content_light_level_state;
-
-  /* For forward predicted trickmode */
-  gboolean discard_bidirectional;
 };
 
 struct _GstH264ParseClass
