@@ -27,7 +27,8 @@
 #endif
 
 #include <string.h>
-#include <gst/base/gstbytereader.h>
+#include <gst/base/base.h>
+#include <gst/pbutils/pbutils.h>
 #include <gst/codecparsers/gstmpegvideometa.h>
 
 #include "gstmpegvideoparse.h"
@@ -58,8 +59,7 @@ enum
 {
   PROP_0,
   PROP_DROP,
-  PROP_GOP_SPLIT,
-  PROP_LAST
+  PROP_GOP_SPLIT
 };
 
 #define parent_class gst_mpegv_parse_parent_class
@@ -176,6 +176,7 @@ gst_mpegv_parse_init (GstMpegvParse * parse)
   parse->config_flags = FLAG_NONE;
 
   gst_base_parse_set_pts_interpolation (GST_BASE_PARSE (parse), FALSE);
+  GST_PAD_SET_ACCEPT_INTERSECT (GST_BASE_PARSE_SINK_PAD (parse));
 }
 
 static void
@@ -461,18 +462,21 @@ parse_packet_extension (GstMpegvParse * mpvparse, GstMapInfo * info, guint off)
   }
 }
 
-/* caller guarantees at least start code in @buf at @off */
+/* caller guarantees at least start code in @buf at @off ( - 4)*/
 /* for off == 4 initial code; returns TRUE if code starts a frame
  * otherwise returns TRUE if code terminates preceding frame */
 static gboolean
 gst_mpegv_parse_process_sc (GstMpegvParse * mpvparse,
-    GstMapInfo * info, gint off, GstMpegVideoPacket * packet)
+    GstMapInfo * info, gint off, GstMpegVideoPacket * packet,
+    gboolean * need_more)
 {
   gboolean ret = FALSE, checkconfig = TRUE;
 
   GST_LOG_OBJECT (mpvparse, "process startcode %x (%s) offset:%d", packet->type,
       picture_start_code_name (packet->type), off);
 
+  *need_more = FALSE;
+
   switch (packet->type) {
     case GST_MPEG_VIDEO_PACKET_PICTURE:
       GST_LOG_OBJECT (mpvparse, "startcode is PICTURE");
@@ -500,10 +504,16 @@ gst_mpegv_parse_process_sc (GstMpegvParse * mpvparse,
         ret = TRUE;
       break;
     case GST_MPEG_VIDEO_PACKET_EXTENSION:
+      mpvparse->config_flags |= FLAG_MPEG2;
       GST_LOG_OBJECT (mpvparse, "startcode is VIDEO PACKET EXTENSION");
-      parse_packet_extension (mpvparse, info, off);
-      if (mpvparse->ext_count < G_N_ELEMENTS (mpvparse->ext_offsets))
-        mpvparse->ext_offsets[mpvparse->ext_count++] = off;
+      if (mpvparse->pic_offset >= 0) {
+        GST_LOG_OBJECT (mpvparse, "... considered PICTURE EXTENSION");
+        parse_packet_extension (mpvparse, info, off);
+      } else {
+        GST_LOG_OBJECT (mpvparse, "... considered SEQUENCE EXTENSION");
+        if (mpvparse->ext_count < G_N_ELEMENTS (mpvparse->ext_offsets))
+          mpvparse->ext_offsets[mpvparse->ext_count++] = off;
+      }
       checkconfig = FALSE;
       break;
     default:
@@ -548,6 +558,7 @@ gst_mpegv_parse_process_sc (GstMpegvParse * mpvparse,
         && (mpvparse->config_flags & FLAG_SEQUENCE_EXT)
         && !mpvparse->sequenceext.progressive) {
       if (info->size - off < 2) {       /* we need at least two bytes to read the TSN */
+        *need_more = TRUE;
         ret = FALSE;
       } else {
         /* TSN is stored in first 10 bits */
@@ -563,8 +574,8 @@ gst_mpegv_parse_process_sc (GstMpegvParse * mpvparse,
 }
 
 /* FIXME move into baseparse, or anything equivalent;
- * see https://bugzilla.gnome.org/show_bug.cgi?id=650093 */
-#define GST_BASE_PARSE_FRAME_FLAG_PARSING   0x10000
+ * see https://bugzilla.gnome.org/show_bug.cgi?id=650093
+ * #define GST_BASE_PARSE_FRAME_FLAG_PARSING   0x100000 */
 
 static inline void
 update_frame_parsing_status (GstMpegvParse * mpvparse,
@@ -591,6 +602,7 @@ gst_mpegv_parse_handle_frame (GstBaseParse * parse,
   GstMpegVideoPacket packet;
   guint8 *data;
   gint size;
+  gboolean need_more = FALSE;
   GstMapInfo map;
 
   update_frame_parsing_status (mpvparse, frame);
@@ -606,7 +618,7 @@ retry:
 
   /* if already found a previous start code, e.g. start of frame, go for next */
   if (mpvparse->last_sc >= 0) {
-    off = packet.offset = mpvparse->last_sc;
+    packet.offset = mpvparse->last_sc;
     packet.size = 0;
     goto next;
   }
@@ -630,7 +642,7 @@ retry:
   /* note: initial start code is assumed at offset 0 by subsequent code */
 
   /* examine start code, see if it looks like an initial start code */
-  if (gst_mpegv_parse_process_sc (mpvparse, &map, 4, &packet)) {
+  if (gst_mpegv_parse_process_sc (mpvparse, &map, 4, &packet, &need_more)) {
     /* found sc */
     GST_LOG_OBJECT (mpvparse, "valid start code found");
     mpvparse->last_sc = 0;
@@ -660,22 +672,18 @@ next:
 
   GST_LOG_OBJECT (mpvparse, "next start code at %d", off);
   if (off < 0) {
-    /* if draining, take all */
-    if (GST_BASE_PARSE_DRAINING (parse)) {
-      GST_LOG_OBJECT (mpvparse, "draining, accepting all data");
-      off = size;
-      ret = TRUE;
-    } else {
-      GST_LOG_OBJECT (mpvparse, "need more data");
-      /* resume scan where we left it */
-      mpvparse->last_sc = size - 3;
-      /* request best next available */
-      off = G_MAXUINT;
-      goto exit;
-    }
+    off = size - 3;
+    goto need_more;
   } else {
     /* decide whether this startcode ends a frame */
-    ret = gst_mpegv_parse_process_sc (mpvparse, &map, off + 4, &packet);
+    ret = gst_mpegv_parse_process_sc (mpvparse, &map, off + 4, &packet,
+        &need_more);
+    /* in rare cases, might need to peek past start code */
+    if (need_more) {
+      GST_LOG_OBJECT (mpvparse, "need more data (past start code");
+      ret = FALSE;
+      goto need_more;
+    }
   }
 
   if (!ret)
@@ -702,6 +710,21 @@ exit:
   }
 
   return flowret;
+
+need_more:
+  /* if draining, take all */
+  if (GST_BASE_PARSE_DRAINING (parse)) {
+    GST_LOG_OBJECT (mpvparse, "draining, accepting all data");
+    off = size;
+    ret = TRUE;
+  } else {
+    GST_LOG_OBJECT (mpvparse, "need more data");
+    /* resume scan where we left it */
+    mpvparse->last_sc = off;
+    /* request best next available */
+    off = G_MAXUINT;
+  }
+  goto exit;
 }
 
 static void
@@ -921,16 +944,16 @@ gst_mpegv_parse_pre_push_frame (GstBaseParse * parse, GstBaseParseFrame * frame)
    * have already been sent */
 
   if (G_UNLIKELY (mpvparse->send_codec_tag)) {
-    gchar *codec;
+    GstCaps *caps;
 
     /* codec tag */
-    codec =
-        g_strdup_printf ("MPEG %d Video",
-        (mpvparse->config_flags & FLAG_MPEG2) ? 2 : 1);
-    taglist = gst_tag_list_new (GST_TAG_VIDEO_CODEC, codec, NULL);
-    g_free (codec);
+    taglist = gst_tag_list_new_empty ();
+    caps = gst_pad_get_current_caps (GST_BASE_PARSE_SRC_PAD (parse));
+    gst_pb_utils_add_codec_description_to_tag_list (taglist,
+        GST_TAG_VIDEO_CODEC, caps);
+    gst_caps_unref (caps);
 
-    gst_pad_push_event (GST_BASE_PARSE_SRC_PAD (mpvparse),
+    gst_pad_push_event (GST_BASE_PARSE_SRC_PAD (parse),
         gst_event_new_tag (taglist));
 
     mpvparse->send_codec_tag = FALSE;
@@ -1002,6 +1025,19 @@ gst_mpegv_parse_set_caps (GstBaseParse * parse, GstCaps * caps)
   return TRUE;
 }
 
+static void
+remove_fields (GstCaps * caps)
+{
+  guint i, n;
+
+  n = gst_caps_get_size (caps);
+  for (i = 0; i < n; i++) {
+    GstStructure *s = gst_caps_get_structure (caps, i);
+
+    gst_structure_remove_field (s, "parsed");
+  }
+}
+
 static GstCaps *
 gst_mpegv_parse_get_caps (GstBaseParse * parse, GstCaps * filter)
 {
@@ -1009,27 +1045,23 @@ gst_mpegv_parse_get_caps (GstBaseParse * parse, GstCaps * filter)
   GstCaps *res;
 
   templ = gst_pad_get_pad_template_caps (GST_BASE_PARSE_SINK_PAD (parse));
-  peercaps = gst_pad_peer_query_caps (GST_BASE_PARSE_SRC_PAD (parse), filter);
-  if (peercaps) {
-    guint i, n;
+  if (filter) {
+    GstCaps *fcopy = gst_caps_copy (filter);
+    /* Remove the fields we convert */
+    remove_fields (fcopy);
+    peercaps = gst_pad_peer_query_caps (GST_BASE_PARSE_SRC_PAD (parse), fcopy);
+    gst_caps_unref (fcopy);
+  } else
+    peercaps = gst_pad_peer_query_caps (GST_BASE_PARSE_SRC_PAD (parse), NULL);
 
+  if (peercaps) {
     /* Remove the parsed field */
     peercaps = gst_caps_make_writable (peercaps);
-    n = gst_caps_get_size (peercaps);
-    for (i = 0; i < n; i++) {
-      GstStructure *s = gst_caps_get_structure (peercaps, i);
-      gst_structure_remove_field (s, "parsed");
-    }
+    remove_fields (peercaps);
 
     res = gst_caps_intersect_full (peercaps, templ, GST_CAPS_INTERSECT_FIRST);
     gst_caps_unref (peercaps);
-    res = gst_caps_make_writable (res);
-
-    /* Append the template caps because we still want to accept
-     * caps without any fields in the case upstream does not
-     * know anything.
-     */
-    gst_caps_append (res, templ);
+    gst_caps_unref (templ);
   } else {
     res = templ;
   }
