@@ -41,29 +41,14 @@ static jint (*create_java_vm) (JavaVM ** p_vm, JNIEnv ** p_env, void *vm_args);
 static JavaVM *java_vm;
 static gboolean started_java_vm = FALSE;
 static pthread_key_t current_jni_env;
-static jobject (*get_class_loader) (void);
 
-gint
-gst_amc_jni_get_android_level ()
+static struct
 {
-  JNIEnv *env;
-  gint ret = __ANDROID_API__;
-  jfieldID sdkIntFieldID = NULL;
-
-  env = gst_amc_jni_get_env ();
-
-  jclass versionClass = (*env)->FindClass (env, "android/os/Build$VERSION");
-  if (versionClass == NULL)
-    goto done;
-
-  sdkIntFieldID = (*env)->GetStaticFieldID (env, versionClass, "SDK_INT", "I");
-  if (sdkIntFieldID == NULL)
-    goto done;
-
-  ret = (*env)->GetStaticIntField (env, versionClass, sdkIntFieldID);
-done:
-  return ret;
-}
+  jclass klass;
+  jmethodID get_limit, get_position;
+  jmethodID set_limit, set_position;
+  jmethodID clear;
+} java_nio_buffer;
 
 jclass
 gst_amc_jni_get_class (JNIEnv * env, GError ** err, const gchar * name)
@@ -201,8 +186,6 @@ jobject
 gst_amc_jni_object_make_global (JNIEnv * env, jobject object)
 {
   jobject ret;
-  g_return_val_if_fail (env != NULL, NULL);
-  g_return_val_if_fail (object != NULL, NULL);
 
   ret = (*env)->NewGlobalRef (env, object);
   if (!ret) {
@@ -217,8 +200,6 @@ jobject
 gst_amc_jni_object_ref (JNIEnv * env, jobject object)
 {
   jobject ret;
-  g_return_val_if_fail (env != NULL, NULL);
-  g_return_val_if_fail (object != NULL, NULL);
 
   ret = (*env)->NewGlobalRef (env, object);
   if (!ret) {
@@ -230,18 +211,12 @@ gst_amc_jni_object_ref (JNIEnv * env, jobject object)
 void
 gst_amc_jni_object_unref (JNIEnv * env, jobject object)
 {
-  g_return_if_fail (env != NULL);
-  g_return_if_fail (object != NULL);
-
   (*env)->DeleteGlobalRef (env, object);
 }
 
 void
 gst_amc_jni_object_local_unref (JNIEnv * env, jobject object)
 {
-  g_return_if_fail (env != NULL);
-  g_return_if_fail (object != NULL);
-
   (*env)->DeleteLocalRef (env, object);
 }
 
@@ -513,33 +488,10 @@ gst_amc_jni_detach_current_thread (void *env)
 
   GST_DEBUG ("Detaching thread %p", g_thread_self ());
   if ((ret = (*java_vm)->DetachCurrentThread (java_vm)) != JNI_OK) {
-    GST_DEBUG ("Failed to detach current thread: %d", ret);
+    GST_ERROR ("Failed to detach current thread: %d", ret);
   }
 }
 
-static JavaVM *
-get_application_java_vm (void)
-{
-  GModule *module = NULL;
-  JavaVM *(*get_java_vm) (void);
-  JavaVM *vm = NULL;
-
-  module = g_module_open (NULL, G_MODULE_BIND_LOCAL);
-
-  if (!module) {
-    return NULL;
-  }
-
-  if (g_module_symbol (module, "gst_android_get_java_vm",
-          (gpointer *) & get_java_vm) && get_java_vm) {
-    vm = get_java_vm ();
-  }
-
-  g_module_close (module);
-
-  return vm;
-}
-
 static gboolean
 check_nativehelper (void)
 {
@@ -579,11 +531,8 @@ load_java_module (const gchar * name)
     goto load_failed;
 
   if (!g_module_symbol (java_module, "JNI_CreateJavaVM",
-          (gpointer *) & create_java_vm)) {
-    GST_ERROR ("Could not find 'JNI_CreateJavaVM' in '%s': %s",
-        GST_STR_NULL (name), g_module_error ());
-    create_java_vm = NULL;
-  }
+          (gpointer *) & create_java_vm))
+    goto symbol_error;
 
   if (!g_module_symbol (java_module, "JNI_GetCreatedJavaVMs",
           (gpointer *) & get_created_java_vms))
@@ -607,31 +556,63 @@ symbol_error:
   }
 }
 
-static gboolean
-check_application_class_loader (void)
-{
-  gboolean ret = TRUE;
-  GModule *module = NULL;
-
-  module = g_module_open (NULL, G_MODULE_BIND_LOCAL);
-  if (!module) {
-    return FALSE;
-  }
-  if (!g_module_symbol (module, "gst_android_get_application_class_loader",
-          (gpointer *) & get_class_loader)) {
-    ret = FALSE;
-  }
-
-  g_module_close (module);
-
-  return ret;
-}
-
 static gboolean
 initialize_classes (void)
 {
-  if (!check_application_class_loader ()) {
-    GST_ERROR ("Could not find application class loader provider");
+  JNIEnv *env;
+  GError *err = NULL;
+
+  env = gst_amc_jni_get_env ();
+
+  java_nio_buffer.klass = gst_amc_jni_get_class (env, &err, "java/nio/Buffer");
+  if (!java_nio_buffer.klass) {
+    GST_ERROR ("Failed to get java.nio.Buffer class: %s", err->message);
+    g_clear_error (&err);
+    return FALSE;
+  }
+
+  java_nio_buffer.get_limit =
+      gst_amc_jni_get_method_id (env, &err, java_nio_buffer.klass, "limit",
+      "()I");
+  if (!java_nio_buffer.get_limit) {
+    GST_ERROR ("Failed to get java.nio.Buffer limit(): %s", err->message);
+    g_clear_error (&err);
+    return FALSE;
+  }
+
+  java_nio_buffer.get_position =
+      gst_amc_jni_get_method_id (env, &err, java_nio_buffer.klass, "position",
+      "()I");
+  if (!java_nio_buffer.get_position) {
+    GST_ERROR ("Failed to get java.nio.Buffer position(): %s", err->message);
+    g_clear_error (&err);
+    return FALSE;
+  }
+
+  java_nio_buffer.set_limit =
+      gst_amc_jni_get_method_id (env, &err, java_nio_buffer.klass, "limit",
+      "(I)Ljava/nio/Buffer;");
+  if (!java_nio_buffer.set_limit) {
+    GST_ERROR ("Failed to get java.nio.Buffer limit(): %s", err->message);
+    g_clear_error (&err);
+    return FALSE;
+  }
+
+  java_nio_buffer.set_position =
+      gst_amc_jni_get_method_id (env, &err, java_nio_buffer.klass, "position",
+      "(I)Ljava/nio/Buffer;");
+  if (!java_nio_buffer.set_position) {
+    GST_ERROR ("Failed to get java.nio.Buffer position(): %s", err->message);
+    g_clear_error (&err);
+    return FALSE;
+  }
+
+  java_nio_buffer.clear =
+      gst_amc_jni_get_method_id (env, &err, java_nio_buffer.klass, "clear",
+      "()Ljava/nio/Buffer;");
+  if (!java_nio_buffer.clear) {
+    GST_ERROR ("Failed to get java.nio.Buffer clear(): %s", err->message);
+    g_clear_error (&err);
     return FALSE;
   }
 
@@ -644,17 +625,6 @@ gst_amc_jni_initialize_java_vm (void)
   jsize n_vms;
   gint ret;
 
-  if (java_vm) {
-    GST_DEBUG ("Java VM already provided by the application");
-    return initialize_classes ();
-  }
-
-  java_vm = get_application_java_vm ();
-  if (java_vm) {
-    GST_DEBUG ("Java VM successfully requested from the application");
-    return initialize_classes ();
-  }
-
   /* Returns TRUE if we can safely
    * a) get the current VMs and
    * b) start a VM if none is started yet
@@ -678,7 +648,7 @@ gst_amc_jni_initialize_java_vm (void)
 
   if (n_vms > 0) {
     GST_DEBUG ("Successfully got existing Java VM %p", java_vm);
-  } else if (create_java_vm) {
+  } else {
     JNIEnv *env;
     JavaVMInitArgs vm_args;
     JavaVMOption options[4];
@@ -699,9 +669,6 @@ gst_amc_jni_initialize_java_vm (void)
     GST_DEBUG ("Successfully created Java VM %p", java_vm);
 
     started_java_vm = TRUE;
-  } else {
-    GST_ERROR ("JNI_CreateJavaVM not available");
-    java_vm = NULL;
   }
 
   if (java_vm == NULL)
@@ -784,14 +751,6 @@ gst_amc_jni_initialize_internal (gpointer data)
   return gst_amc_jni_initialize_java_vm ()? GINT_TO_POINTER (1) : NULL;
 }
 
-/* Allow the application to set the Java VM */
-void
-gst_amc_jni_set_java_vm (JavaVM * vm)
-{
-  GST_DEBUG ("Application provides Java VM %p", vm);
-  java_vm = vm;
-}
-
 gboolean
 gst_amc_jni_initialize (void)
 {
@@ -820,64 +779,6 @@ gst_amc_jni_is_vm_started (void)
   return started_java_vm;
 }
 
-jclass
-gst_amc_jni_get_application_class (JNIEnv * env, const gchar * name,
-    GError ** err)
-{
-  jclass tmp = NULL;
-  jclass class = NULL;
-  jstring name_jstr = NULL;
-
-  jobject class_loader = NULL;
-  jclass class_loader_cls = NULL;
-  jmethodID load_class_id = 0;
-
-  GST_LOG ("attempting to retrieve class %s", name);
-
-  if (!get_class_loader) {
-    g_set_error (err, GST_LIBRARY_ERROR, GST_LIBRARY_ERROR_FAILED,
-        "Could not retrieve application class loader function");
-    goto done;
-  }
-
-  class_loader = get_class_loader ();
-  if (!class_loader) {
-    g_set_error (err, GST_LIBRARY_ERROR, GST_LIBRARY_ERROR_FAILED,
-        "Could not retrieve application class loader");
-    goto done;
-  }
-
-  class_loader_cls = (*env)->GetObjectClass (env, class_loader);
-  if (!class_loader_cls) {
-    g_set_error (err, GST_LIBRARY_ERROR, GST_LIBRARY_ERROR_FAILED,
-        "Could not retrieve application class loader java class");
-    goto done;
-  }
-
-  load_class_id =
-      gst_amc_jni_get_method_id (env, err, class_loader_cls, "loadClass",
-      "(Ljava/lang/String;)Ljava/lang/Class;");
-  if (!load_class_id) {
-    goto done;
-  }
-
-  name_jstr = gst_amc_jni_string_from_gchar (env, err, FALSE, name);
-  if (!name_jstr) {
-    goto done;
-  }
-
-  if (gst_amc_jni_call_object_method (env, err, class_loader,
-          load_class_id, &tmp, name_jstr)) {
-    class = gst_amc_jni_object_make_global (env, tmp);
-  }
-
-done:
-  gst_amc_jni_object_local_unref (env, name_jstr);
-  gst_amc_jni_object_local_unref (env, class_loader_cls);
-
-  return class;
-}
-
 #define CALL_STATIC_TYPE_METHOD(_type, _name,  _jname)                                                     \
 gboolean gst_amc_jni_call_static_##_name##_method (JNIEnv *env, GError ** err, jclass klass, jmethodID methodID, _type * value, ...)   \
   {                                                                                                          \
@@ -1013,3 +914,168 @@ GET_STATIC_TYPE_FIELD (gint64, long, Long);
 GET_STATIC_TYPE_FIELD (gfloat, float, Float);
 GET_STATIC_TYPE_FIELD (gdouble, double, Double);
 GET_STATIC_TYPE_FIELD (jobject, object, Object);
+
+gboolean
+gst_amc_jni_get_buffer_array (JNIEnv * env, GError ** err, jobject array,
+    GstAmcBuffer ** buffers, gsize * n_buffers)
+{
+  jsize i;
+
+  *n_buffers = (*env)->GetArrayLength (env, array);
+  *buffers = g_new0 (GstAmcBuffer, *n_buffers);
+
+  for (i = 0; i < *n_buffers; i++) {
+    jobject buffer = NULL;
+
+    buffer = (*env)->GetObjectArrayElement (env, array, i);
+    if ((*env)->ExceptionCheck (env) || !buffer) {
+      gst_amc_jni_set_error (env, err, GST_LIBRARY_ERROR,
+          GST_LIBRARY_ERROR_FAILED, "Failed to get buffer %d", i);
+      goto error;
+    }
+
+    (*buffers)[i].object = gst_amc_jni_object_make_global (env, buffer);
+    if (!(*buffers)[i].object) {
+      gst_amc_jni_set_error (env, err, GST_LIBRARY_ERROR,
+          GST_LIBRARY_ERROR_FAILED,
+          "Failed to create global buffer reference %d", i);
+      goto error;
+    }
+
+    (*buffers)[i].data =
+        (*env)->GetDirectBufferAddress (env, (*buffers)[i].object);
+    if (!(*buffers)[i].data) {
+      gst_amc_jni_set_error (env, err, GST_LIBRARY_ERROR,
+          GST_LIBRARY_ERROR_FAILED, "Failed to get buffer address %d", i);
+      goto error;
+    }
+    (*buffers)[i].size =
+        (*env)->GetDirectBufferCapacity (env, (*buffers)[i].object);
+  }
+
+  return TRUE;
+
+error:
+  if (*buffers)
+    gst_amc_jni_free_buffer_array (env, *buffers, *n_buffers);
+  *buffers = NULL;
+  *n_buffers = 0;
+  return FALSE;
+}
+
+void
+gst_amc_jni_free_buffer_array (JNIEnv * env, GstAmcBuffer * buffers,
+    gsize n_buffers)
+{
+  jsize i;
+
+  g_return_if_fail (buffers != NULL);
+
+  for (i = 0; i < n_buffers; i++) {
+    if (buffers[i].object)
+      gst_amc_jni_object_unref (env, buffers[i].object);
+  }
+  g_free (buffers);
+}
+
+void
+gst_amc_buffer_free (GstAmcBuffer * buffer)
+{
+  JNIEnv *env;
+
+  g_return_if_fail (buffer != NULL);
+
+  env = gst_amc_jni_get_env ();
+
+  if (buffer->object)
+    gst_amc_jni_object_unref (env, buffer->object);
+  g_free (buffer);
+}
+
+GstAmcBuffer *
+gst_amc_buffer_copy (GstAmcBuffer * buffer)
+{
+  JNIEnv *env;
+  GstAmcBuffer *ret;
+
+  g_return_val_if_fail (buffer != NULL, NULL);
+
+  env = gst_amc_jni_get_env ();
+
+  ret = g_new0 (GstAmcBuffer, 1);
+
+  ret->object = gst_amc_jni_object_ref (env, buffer->object);
+  ret->data = buffer->data;
+  ret->size = buffer->size;
+
+  return ret;
+}
+
+gboolean
+gst_amc_buffer_get_position_and_limit (GstAmcBuffer * buffer, GError ** err,
+    gint * position, gint * limit)
+{
+  JNIEnv *env;
+
+  g_return_val_if_fail (buffer != NULL, FALSE);
+  g_return_val_if_fail (buffer->object != NULL, FALSE);
+
+  env = gst_amc_jni_get_env ();
+
+  if (!gst_amc_jni_call_int_method (env, err, buffer->object,
+          java_nio_buffer.get_position, position))
+    return FALSE;
+
+  if (!gst_amc_jni_call_int_method (env, err, buffer->object,
+          java_nio_buffer.get_limit, limit))
+    return FALSE;
+
+  return TRUE;
+}
+
+gboolean
+gst_amc_buffer_set_position_and_limit (GstAmcBuffer * buffer, GError ** err,
+    gint position, gint limit)
+{
+  JNIEnv *env;
+  jobject tmp;
+
+  g_return_val_if_fail (buffer != NULL, FALSE);
+  g_return_val_if_fail (buffer->object != NULL, FALSE);
+
+  env = gst_amc_jni_get_env ();
+
+  if (!gst_amc_jni_call_object_method (env, err, buffer->object,
+          java_nio_buffer.set_limit, &tmp, limit))
+    return FALSE;
+
+  gst_amc_jni_object_local_unref (env, tmp);
+
+  if (!gst_amc_jni_call_object_method (env, err, buffer->object,
+          java_nio_buffer.set_position, &tmp, position))
+    return FALSE;
+
+  gst_amc_jni_object_local_unref (env, tmp);
+
+  return TRUE;
+}
+
+gboolean
+gst_amc_buffer_clear (GstAmcBuffer * buffer, GError ** err)
+{
+  JNIEnv *env;
+  jobject tmp;
+
+  g_return_val_if_fail (buffer != NULL, FALSE);
+  g_return_val_if_fail (buffer->object != NULL, FALSE);
+
+  env = gst_amc_jni_get_env ();
+
+  if (!gst_amc_jni_call_object_method (env, err, buffer->object,
+          java_nio_buffer.clear, &tmp))
+    return FALSE;
+
+  gst_amc_jni_object_local_unref (env, tmp);
+
+  return TRUE;
+}
