@@ -22,15 +22,16 @@
  */
 
 /**
- * SECTION:element-avfassetsrc
+ * SECTION:element-plugin
  *
  * Read and decode samples from AVFoundation assets using the AVFAssetReader API
  *
- * ## Example launch line
- *
+ * <refsect2>
+ * <title>Example launch line</title>
  * |[
- * gst-launch-1.0 -v -m avfassetsrc uri="file://movie.mp4" ! autovideosink
+ * gst-launch -v -m avfassetsrc uri="file://movie.mp4" ! autovideosink
  * ]|
+ * </refsect2>
  */
 
 #ifdef HAVE_CONFIG_H
@@ -44,15 +45,21 @@ GST_DEBUG_CATEGORY_STATIC (gst_avf_asset_src_debug);
 #define GST_CAT_DEFAULT gst_avf_asset_src_debug
 
 #define CMTIME_TO_GST_TIME(x) \
-    (CMTIME_IS_INVALID(x) ? GST_CLOCK_TIME_NONE : (guint64)(x.value * GST_SECOND / x.timescale));
+    (x.value == 0 ? 0 : (guint64)(x.value * GST_SECOND / x.timescale));
 #define GST_AVF_ASSET_SRC_LOCK(x) (g_mutex_lock (&x->lock));
 #define GST_AVF_ASSET_SRC_UNLOCK(x) (g_mutex_unlock (&x->lock));
 #define MEDIA_TYPE_TO_STR(x) \
     (x == GST_AVF_ASSET_READER_MEDIA_TYPE_AUDIO ? "audio" : "video")
 #define AVF_ASSET_READER_HAS_AUDIO(x) \
-    ([GST_AVF_ASSET_SRC_READER(self) hasMediaType:GST_AVF_ASSET_READER_MEDIA_TYPE_AUDIO])
+    ([self->reader hasMediaType:GST_AVF_ASSET_READER_MEDIA_TYPE_AUDIO])
 #define AVF_ASSET_READER_HAS_VIDEO(x) \
-    ([GST_AVF_ASSET_SRC_READER(self) hasMediaType:GST_AVF_ASSET_READER_MEDIA_TYPE_VIDEO])
+    ([self->reader hasMediaType:GST_AVF_ASSET_READER_MEDIA_TYPE_VIDEO])
+#define OBJC_CALLOUT_BEGIN() \
+   NSAutoreleasePool *pool; \
+   \
+   pool = [[NSAutoreleasePool alloc] init]
+#define OBJC_CALLOUT_END() \
+  [pool release]
 
 enum
 {
@@ -87,7 +94,6 @@ static void gst_avf_asset_src_set_property (GObject * object, guint prop_id,
 static void gst_avf_asset_src_get_property (GObject * object, guint prop_id,
     GValue * value, GParamSpec * pspec);
 static void gst_avf_asset_src_dispose (GObject *object);
-static void gst_avf_asset_src_finalize (GObject *object);
 
 static GstStateChangeReturn gst_avf_asset_src_change_state (GstElement * element,
     GstStateChange transition);
@@ -143,14 +149,21 @@ gst_avf_asset_src_class_init (GstAVFAssetSrcClass * klass)
     "Read and decode samples from AVFoundation assets using the AVFAssetReader API",
     "Andoni Morales Alastruey amorales@fluendo.com");
 
-  gst_element_class_add_static_pad_template (gstelement_class, &audio_factory);
-  gst_element_class_add_static_pad_template (gstelement_class, &video_factory);
+  gst_element_class_add_pad_template (gstelement_class,
+      gst_static_pad_template_get (&audio_factory));
+  gst_element_class_add_pad_template (gstelement_class,
+      gst_static_pad_template_get (&video_factory));
 
   gobject_class->set_property = gst_avf_asset_src_set_property;
   gobject_class->get_property = gst_avf_asset_src_get_property;
   gobject_class->dispose = gst_avf_asset_src_dispose;
-  gobject_class->finalize = gst_avf_asset_src_finalize;
 
+  /**
+   * GstAVFAssetSrc:uri
+   *
+   * URI of the asset to read
+   *
+   **/
   g_object_class_install_property (gobject_class, PROP_URI,
       g_param_spec_string ("uri", "Asset URI",
           "URI of the asset to read", NULL,
@@ -171,15 +184,6 @@ gst_avf_asset_src_init (GstAVFAssetSrc * self)
   g_mutex_init (&self->lock);
 }
 
-static void
-gst_avf_asset_src_finalize (GObject *object)
-{
-  GstAVFAssetSrc *self = GST_AVF_ASSET_SRC (object);
-
-  g_mutex_clear (&self->lock);
-  G_OBJECT_CLASS (gst_avf_asset_src_parent_class)->finalize (object);
-}
-
 static void
 gst_avf_asset_src_dispose (GObject *object)
 {
@@ -194,8 +198,6 @@ gst_avf_asset_src_dispose (GObject *object)
     gst_event_unref (self->seek_event);
     self->seek_event = NULL;
   }
-
-  G_OBJECT_CLASS (gst_avf_asset_src_parent_class)->dispose (object);
 }
 
 static void
@@ -206,7 +208,9 @@ gst_avf_asset_src_set_property (GObject * object, guint prop_id,
 
   switch (prop_id) {
     case PROP_URI:
-      g_free (self->uri);
+      if (self->uri) {
+        g_free (self->uri);
+      }
       self->uri = g_value_dup_string (value);
       break;
     default:
@@ -242,16 +246,11 @@ gst_avf_asset_src_change_state (GstElement * element, GstStateChange transition)
       gst_element_state_get_name (GST_STATE_TRANSITION_CURRENT (transition)),
       gst_element_state_get_name (GST_STATE_TRANSITION_NEXT (transition)));
 
+  OBJC_CALLOUT_BEGIN ();
   switch (transition) {
     case GST_STATE_CHANGE_NULL_TO_READY: {
       self->state = GST_AVF_ASSET_SRC_STATE_STOPPED;
-      if (!self->uri) {
-        GST_ELEMENT_ERROR (element, RESOURCE, NOT_FOUND,
-            ("\"uri\" property not set"), (NULL));
-        gst_avf_asset_src_stop_all (self);
-        return GST_STATE_CHANGE_FAILURE;
-      }
-      self->reader = (__bridge_retained gpointer)([[GstAVFAssetReader alloc] initWithURI:self->uri:&error]);
+      self->reader = [[GstAVFAssetReader alloc] initWithURI:self->uri:&error];
       if (error) {
         GST_ELEMENT_ERROR (element, RESOURCE, FAILED, ("AVFAssetReader error"),
             ("%s", error->message));
@@ -281,11 +280,12 @@ gst_avf_asset_src_change_state (GstElement * element, GstStateChange transition)
       gst_avf_asset_src_stop (self);
       break;
     case GST_STATE_CHANGE_READY_TO_NULL:
-      CFBridgingRelease(self->reader);
+      [self->reader release];
       break;
     default:
       break;
   }
+  OBJC_CALLOUT_END ();
   return ret;
 }
 
@@ -321,18 +321,18 @@ gst_avf_asset_src_query (GstPad *pad, GstObject * parent, GstQuery *query)
         ret = TRUE;
         break;
       case GST_QUERY_DURATION:
-        gst_query_set_duration (query, GST_FORMAT_TIME, GST_AVF_ASSET_SRC_READER(self).duration);
+        gst_query_set_duration (query, GST_FORMAT_TIME, self->reader.duration);
         ret = TRUE;
         break;
       case GST_QUERY_POSITION:
-        gst_query_set_position (query, GST_FORMAT_TIME, GST_AVF_ASSET_SRC_READER(self).position);
+        gst_query_set_position (query, GST_FORMAT_TIME, self->reader.position);
         ret = TRUE;
         break;
       case GST_QUERY_SEEKING: {
         GstFormat fmt;
         gst_query_parse_seeking (query, &fmt, NULL, NULL, NULL);
         if (fmt == GST_FORMAT_TIME) {
-          gst_query_set_seeking (query, GST_FORMAT_TIME, TRUE, 0, GST_AVF_ASSET_SRC_READER(self).duration);
+          gst_query_set_seeking (query, GST_FORMAT_TIME, TRUE, 0, self->reader.duration);
           ret = TRUE;
         }
         break;
@@ -360,6 +360,7 @@ gst_avf_asset_src_event (GstPad * pad, GstObject * parent, GstEvent * event)
   gboolean res = TRUE;
   GError *error = NULL;
 
+  OBJC_CALLOUT_BEGIN ();
   self = GST_AVF_ASSET_SRC (gst_pad_get_parent_element (pad));
 
   switch (GST_EVENT_TYPE (event)) {
@@ -406,7 +407,7 @@ gst_avf_asset_src_event (GstPad * pad, GstObject * parent, GstEvent * event)
         stop = GST_CLOCK_TIME_NONE;
       }
       gst_avf_asset_src_send_event (self, gst_event_new_flush_start ());
-      [GST_AVF_ASSET_SRC_READER(self) seekTo: start: stop: &error];
+      [self->reader seekTo: start: stop: &error];
 
       gst_segment_init (&segment, GST_FORMAT_TIME);
       segment.rate = rate;
@@ -436,6 +437,7 @@ gst_avf_asset_src_event (GstPad * pad, GstObject * parent, GstEvent * event)
   }
 
   gst_object_unref (self);
+  OBJC_CALLOUT_END ();
   return res;
 }
 
@@ -497,14 +499,15 @@ gst_avf_asset_src_read_data (GstAVFAssetSrc *self, GstPad *pad,
   GstFlowReturn ret, combined_ret;
   GError *error;
 
+  OBJC_CALLOUT_BEGIN ();
 
   GST_AVF_ASSET_SRC_LOCK (self);
   if (self->state != GST_AVF_ASSET_SRC_STATE_READING) {
     GST_AVF_ASSET_SRC_UNLOCK (self);
-    return;
+    goto exit;
   }
 
-  buf = [GST_AVF_ASSET_SRC_READER(self) nextBuffer:type:&error];
+  buf = [self->reader nextBuffer:type:&error];
   GST_AVF_ASSET_SRC_UNLOCK (self);
 
   if (buf == NULL) {
@@ -515,13 +518,13 @@ gst_avf_asset_src_read_data (GstAVFAssetSrc *self, GstPad *pad,
 
       gst_avf_asset_src_combine_flows (self, type, GST_FLOW_ERROR);
       gst_pad_pause_task (pad);
-        return;
+      goto exit;
     }
 
     gst_pad_push_event (pad, gst_event_new_eos ());
     gst_avf_asset_src_combine_flows (self, type, GST_FLOW_EOS);
     gst_pad_pause_task (pad);
-    return;
+    goto exit;
   }
 
   ret = gst_pad_push (pad, buf);
@@ -536,12 +539,15 @@ gst_avf_asset_src_read_data (GstAVFAssetSrc *self, GstPad *pad,
     }
 
     if (combined_ret != GST_FLOW_OK) {
-      GST_ELEMENT_FLOW_ERROR (self, ret);
+      GST_ELEMENT_ERROR (self, STREAM, FAILED, ("Internal data stream error."),
+          ("stream stopped reason %s", gst_flow_get_name (ret)));
     }
 
     gst_pad_pause_task (pad);
   }
 
+exit:
+  OBJC_CALLOUT_END ();
 }
 
 static void
@@ -564,8 +570,9 @@ gst_avf_asset_src_start_reader (GstAVFAssetSrc * self)
   GError *error = NULL;
   gboolean ret = TRUE;
 
+  OBJC_CALLOUT_BEGIN ();
 
-  [GST_AVF_ASSET_SRC_READER(self) start: &error];
+  [self->reader start: &error];
   if (error != NULL) {
     GST_ELEMENT_ERROR (self, RESOURCE, FAILED,
         ("AVFAssetReader could not start reading"), ("%s", error->message));
@@ -575,6 +582,7 @@ gst_avf_asset_src_start_reader (GstAVFAssetSrc * self)
   }
 
 exit:
+  OBJC_CALLOUT_END ();
   return ret;
 }
 
@@ -583,6 +591,7 @@ gst_avf_asset_src_send_event (GstAVFAssetSrc *self, GstEvent *event)
 {
   gboolean ret = TRUE;
 
+  OBJC_CALLOUT_BEGIN ();
 
   if (AVF_ASSET_READER_HAS_VIDEO (self)) {
     ret |= gst_pad_push_event (self->videopad, gst_event_ref (event));
@@ -592,6 +601,7 @@ gst_avf_asset_src_send_event (GstAVFAssetSrc *self, GstEvent *event)
   }
 
   gst_event_unref (event);
+  OBJC_CALLOUT_END ();
   return ret;
 }
 
@@ -600,24 +610,25 @@ gst_avf_asset_src_start (GstAVFAssetSrc *self)
 {
   GstSegment segment;
 
+  OBJC_CALLOUT_BEGIN ();
   if (self->state == GST_AVF_ASSET_SRC_STATE_STARTED) {
-    return;
+    goto exit;
   }
 
   GST_DEBUG_OBJECT (self, "Creating pads and starting reader");
 
   gst_segment_init (&segment, GST_FORMAT_TIME);
-  segment.duration = GST_AVF_ASSET_SRC_READER(self).duration;
+  segment.duration = self->reader.duration;
 
   /* We call AVFAssetReader's startReading when the pads are linked
    * and no outputs can be added afterwards, so the tracks must be
    * selected before adding any of the new pads */
   if (AVF_ASSET_READER_HAS_AUDIO (self)) {
-    [GST_AVF_ASSET_SRC_READER(self) selectTrack: GST_AVF_ASSET_READER_MEDIA_TYPE_AUDIO:
+    [self->reader selectTrack: GST_AVF_ASSET_READER_MEDIA_TYPE_AUDIO:
         self->selected_audio_track];
   }
   if (AVF_ASSET_READER_HAS_VIDEO (self)) {
-    [GST_AVF_ASSET_SRC_READER(self) selectTrack: GST_AVF_ASSET_READER_MEDIA_TYPE_VIDEO:
+    [self->reader selectTrack: GST_AVF_ASSET_READER_MEDIA_TYPE_VIDEO:
          self->selected_video_track];
   }
 
@@ -631,9 +642,9 @@ gst_avf_asset_src_start (GstAVFAssetSrc *self)
     gst_pad_set_active (self->audiopad, TRUE);
     gst_avf_asset_src_send_start_stream (self, self->audiopad);
     gst_pad_set_caps (self->audiopad,
-        [GST_AVF_ASSET_SRC_READER(self) getCaps: GST_AVF_ASSET_READER_MEDIA_TYPE_AUDIO]);
+        [self->reader getCaps: GST_AVF_ASSET_READER_MEDIA_TYPE_AUDIO]);
     gst_pad_push_event (self->audiopad, gst_event_new_caps (
-        [GST_AVF_ASSET_SRC_READER(self) getCaps: GST_AVF_ASSET_READER_MEDIA_TYPE_AUDIO]));
+        [self->reader getCaps: GST_AVF_ASSET_READER_MEDIA_TYPE_AUDIO]));
     gst_pad_push_event (self->audiopad, gst_event_new_segment (&segment));
     gst_element_add_pad (GST_ELEMENT (self), self->audiopad);
   }
@@ -647,31 +658,35 @@ gst_avf_asset_src_start (GstAVFAssetSrc *self)
     gst_pad_set_active (self->videopad, TRUE);
     gst_avf_asset_src_send_start_stream (self, self->videopad);
     gst_pad_set_caps (self->videopad,
-        [GST_AVF_ASSET_SRC_READER(self) getCaps: GST_AVF_ASSET_READER_MEDIA_TYPE_VIDEO]);
+        [self->reader getCaps: GST_AVF_ASSET_READER_MEDIA_TYPE_VIDEO]);
     gst_pad_push_event (self->videopad, gst_event_new_caps (
-        [GST_AVF_ASSET_SRC_READER(self) getCaps: GST_AVF_ASSET_READER_MEDIA_TYPE_VIDEO]));
+        [self->reader getCaps: GST_AVF_ASSET_READER_MEDIA_TYPE_VIDEO]));
     gst_pad_push_event (self->videopad, gst_event_new_segment (&segment));
     gst_element_add_pad (GST_ELEMENT (self), self->videopad);
   }
   gst_element_no_more_pads (GST_ELEMENT (self));
 
   self->state = GST_AVF_ASSET_SRC_STATE_STARTED;
+
+exit:
+  OBJC_CALLOUT_END ();
 }
 
 static void
 gst_avf_asset_src_stop (GstAVFAssetSrc *self)
 {
   gboolean has_audio, has_video;
+  OBJC_CALLOUT_BEGIN();
 
   if (self->state == GST_AVF_ASSET_SRC_STATE_STOPPED) {
-    return;
+    goto exit;
   }
 
   GST_DEBUG ("Stopping tasks and removing pads");
 
   has_audio = AVF_ASSET_READER_HAS_AUDIO (self);
   has_video = AVF_ASSET_READER_HAS_VIDEO (self);
-  [GST_AVF_ASSET_SRC_READER(self) stop];
+  [self->reader stop];
 
   if (has_audio) {
     gst_pad_stop_task (self->audiopad);
@@ -683,6 +698,9 @@ gst_avf_asset_src_stop (GstAVFAssetSrc *self)
   }
 
   self->state = GST_AVF_ASSET_SRC_STATE_STOPPED;
+
+exit:
+  OBJC_CALLOUT_END ();
 }
 
 static gboolean
@@ -790,20 +808,27 @@ gst_avf_asset_src_uri_set_uri (GstURIHandler * handler, const gchar * uri, GErro
   NSString *str;
   NSURL *url;
   AVAsset *asset;
+  gchar *escaped_uri;
   gboolean ret = FALSE;
 
-  str = [NSString stringWithUTF8String: uri];
+  OBJC_CALLOUT_BEGIN ();
+  escaped_uri = g_uri_escape_string (uri, ":/", TRUE);
+  str = [NSString stringWithUTF8String: escaped_uri];
   url = [[NSURL alloc] initWithString: str];
   asset = [AVAsset assetWithURL: url];
+  g_free (escaped_uri);
 
   if (asset.playable) {
     ret = TRUE;
-    g_free (self->uri);
+    if (self->uri) {
+      g_free (self->uri);
+    }
     self->uri = g_strdup (uri);
   } else {
     g_set_error (error, GST_URI_ERROR, GST_URI_ERROR_BAD_URI,
-        "Invalid URI '%s' for avfassetsrc", uri);
+        "Invalid URI '%s' for avfassetsrc", self->uri);
   }
+  OBJC_CALLOUT_END ();
   return ret;
 }
 
@@ -851,11 +876,11 @@ gst_avf_asset_src_uri_handler_init (gpointer g_iface, gpointer iface_data)
 
 - (void) releaseReader
 {
-  video_track = nil;
-  audio_track = nil;
-  video_tracks = nil;
-  audio_tracks = nil;
-  reader = nil;
+  [video_track release];
+  [audio_track release];
+  [video_tracks release];
+  [audio_tracks release];
+  [reader release];
 }
 
 - (void) initReader: (GError **) error
@@ -868,12 +893,13 @@ gst_avf_asset_src_uri_handler_init (gpointer g_iface, gpointer iface_data)
         [nserror.description UTF8String]);
     *error = g_error_new (GST_AVF_ASSET_SRC_ERROR, GST_AVF_ASSET_ERROR_INIT, "%s",
         [nserror.description UTF8String]);
-
+    [asset release];
+    [reader release];
     return;
   }
 
-  audio_tracks = [asset tracksWithMediaType:AVMediaTypeAudio];
-  video_tracks = [asset tracksWithMediaType:AVMediaTypeVideo];
+  audio_tracks = [[asset tracksWithMediaType:AVMediaTypeAudio] retain];
+  video_tracks = [[asset tracksWithMediaType:AVMediaTypeVideo] retain];
   reader.timeRange = CMTimeRangeMake(kCMTimeZero, asset.duration);
   GST_INFO ("Found %lu video tracks and %lu audio tracks",
       (unsigned long)[video_tracks count], (unsigned long)[audio_tracks count]);
@@ -883,18 +909,21 @@ gst_avf_asset_src_uri_handler_init (gpointer g_iface, gpointer iface_data)
 {
   NSString *str;
   NSURL *url;
+  gchar *escaped_uri;
 
-  GST_INFO ("Initializing AVFAssetReader with uri: %s", uri);
+  GST_INFO ("Initializing AVFAssetReader with uri:%s", uri);
   *error = NULL;
 
-  str = [NSString stringWithUTF8String: uri];
+  escaped_uri = g_uri_escape_string (uri, ":/", TRUE);
+  str = [NSString stringWithUTF8String: escaped_uri];
   url = [[NSURL alloc] initWithString: str];
-  asset = [AVAsset assetWithURL: url];
+  asset = [[AVAsset assetWithURL: url] retain];
+  g_free (escaped_uri);
 
   if (!asset.playable) {
     *error = g_error_new (GST_AVF_ASSET_SRC_ERROR, GST_AVF_ASSET_ERROR_NOT_PLAYABLE,
         "Media is not playable");
-    asset = nil;
+    [asset release];
     return nil;
   }
 
@@ -918,11 +947,11 @@ gst_avf_asset_src_uri_handler_init (gpointer g_iface, gpointer iface_data)
   return self;
 }
 
-- (BOOL) selectTrack: (GstAVFAssetReaderMediaType) type : (gint) index
+- (bool) selectTrack: (GstAVFAssetReaderMediaType) type : (gint) index
 {
   NSArray *tracks;
   AVAssetTrack *track;
-  AVAssetReaderOutput * __strong *output;
+  AVAssetReaderOutput **output;
   NSDictionary *settings;
   NSString *mediaType;
   gint *selected_track;
@@ -956,6 +985,7 @@ gst_avf_asset_src_uri_handler_init (gpointer g_iface, gpointer iface_data)
   *output  = [AVAssetReaderTrackOutput
       assetReaderTrackOutputWithTrack:track
       outputSettings:settings];
+  [*output retain];
   [reader addOutput:*output];
   return TRUE;
 }
@@ -976,11 +1006,11 @@ gst_avf_asset_src_uri_handler_init (gpointer g_iface, gpointer iface_data)
 
 - (void) stop
 {
-  [reader cancelReading];
+  [self->reader cancelReading];
   reading = FALSE;
 }
 
-- (BOOL) hasMediaType: (GstAVFAssetReaderMediaType) type
+- (bool) hasMediaType: (GstAVFAssetReaderMediaType) type
 {
   if (type == GST_AVF_ASSET_READER_MEDIA_TYPE_AUDIO) {
     return [audio_tracks count] != 0;
@@ -1024,15 +1054,18 @@ gst_avf_asset_src_uri_handler_init (gpointer g_iface, gpointer iface_data)
 {
   CMSampleBufferRef cmbuf;
   AVAssetReaderTrackOutput *areader = NULL;
+  GstCaps *caps;
   GstBuffer *buf;
   CMTime dur, ts;
 
   GST_LOG ("Reading %s next buffer", MEDIA_TYPE_TO_STR (type));
   if (type == GST_AVF_ASSET_READER_MEDIA_TYPE_AUDIO && audio_track != NULL) {
     areader = audio_track;
+    caps = audio_caps;
   } else if (type == GST_AVF_ASSET_READER_MEDIA_TYPE_VIDEO &&
       video_track != NULL) {
     areader = video_track;
+    caps = video_caps;
   }
 
   if (areader == NULL) {
@@ -1050,17 +1083,10 @@ gst_avf_asset_src_uri_handler_init (gpointer g_iface, gpointer iface_data)
     return NULL;
   }
 
-  ts = CMSampleBufferGetPresentationTimeStamp (cmbuf);
-  if (!CMTIME_IS_VALID (ts)) {
-    GST_WARNING ("Buffer %p has invalid timestamp", cmbuf);
-  }
+  buf = gst_core_media_buffer_new (cmbuf, FALSE, TRUE);
   dur = CMSampleBufferGetDuration (cmbuf);
-  buf = gst_core_media_buffer_new (cmbuf, FALSE, NULL);
-  CFRelease (cmbuf);
-  if (buf == NULL)
-    return NULL;
-
-  if (CMTIME_IS_VALID (dur) && dur.value != 0) {
+  ts = CMSampleBufferGetPresentationTimeStamp (cmbuf);
+  if (dur.value != 0) {
     GST_BUFFER_DURATION (buf) = CMTIME_TO_GST_TIME (dur);
   }
   GST_BUFFER_TIMESTAMP (buf) = CMTIME_TO_GST_TIME (ts);
@@ -1068,14 +1094,7 @@ gst_avf_asset_src_uri_handler_init (gpointer g_iface, gpointer iface_data)
       GST_TIME_FORMAT, MEDIA_TYPE_TO_STR (type),
       GST_TIME_ARGS(GST_BUFFER_TIMESTAMP (buf)),
       GST_TIME_ARGS(GST_BUFFER_DURATION (buf)));
-
-  /* FIXME: Buffers with invalid timestamp don't contribute to advancing the
-   * position. We have options: 1) try to use the previous buffer duration if
-   * available, 2) advance by one nominal frame duration (though this won't work
-   * with VFR content), or 3) advance by some epsilon value. Not sure yet what's
-   * best.
-   */
-  if (GST_BUFFER_TIMESTAMP_IS_VALID (buf) && GST_BUFFER_TIMESTAMP (buf) > position) {
+  if (GST_BUFFER_TIMESTAMP (buf) > position) {
     position = GST_BUFFER_TIMESTAMP (buf);
   }
   return buf;
@@ -1106,9 +1125,10 @@ gst_avf_asset_src_uri_handler_init (gpointer g_iface, gpointer iface_data)
   return caps;
 }
 
-- (void) dealloc
+- (oneway void) release
 {
-  asset = nil;
+  [asset release];
+
   [self releaseReader];
 
   if (audio_caps != NULL) {
