@@ -51,39 +51,14 @@ struct _GstDecklinkVideoSink
 
   GstDecklinkModeEnum mode;
   gint device_number;
-  gint64 persistent_id;
-  GstDecklinkVideoFormat video_format;
-  GstDecklinkProfileId profile_id;
-  BMDTimecodeFormat timecode_format;
-  BMDKeyerMode keyer_mode;
-  gint keyer_level;
 
   GstVideoInfo info;
 
   GstClockTime internal_base_time;
   GstClockTime external_base_time;
+  GstClockTime last_render_time;
 
   GstDecklinkOutput *output;
-
-  GstVideoVBIEncoder *vbiencoder;
-  GstVideoFormat anc_vformat;
-
-  gint caption_line;
-  guint16 cdp_hdr_sequence_cntr;
-
-  gint afd_bar_line;
-  GstDecklinkMappingFormat mapping_format;
-
-  gboolean initial_sync;
-  GstVecDeque *pending_frames;
-
-  gboolean have_light_level;
-  GstVideoContentLightLevel light_level;
-  gboolean have_mastering_info;
-  GstVideoMasteringDisplayInfo mastering_info;
-
-  gboolean output_vanc;
-  GArray *vanc_cache;
 };
 
 struct _GstDecklinkVideoSinkClass
@@ -93,11 +68,6 @@ struct _GstDecklinkVideoSinkClass
 
 GType gst_decklink_video_sink_get_type (void);
 
-GST_ELEMENT_REGISTER_DECLARE (decklinkvideosink);
-
-void gst_decklink_video_sink_convert_to_internal_clock (GstDecklinkVideoSink * self,
-    GstClockTime * timestamp, GstClockTime * duration);
-
 G_END_DECLS
 
 #endif /* __GST_DECKLINK_VIDEO_SINK_H__ */
