@@ -15,10 +15,12 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
+#include <gst/video/video-format.h>
+
 #include "gstdshow.h"
 #include "gstdshowfakesink.h"
 
@@ -405,9 +407,28 @@ gst_dshow_guid_to_gst_video_format (AM_MEDIA_TYPE *mediatype)
   if (gst_dshow_check_mediatype (mediatype, MEDIASUBTYPE_YUY2, FORMAT_VideoInfo))
     return GST_VIDEO_FORMAT_YUY2;
 
+  if (gst_dshow_check_mediatype (mediatype, MEDIASUBTYPE_UYVY, FORMAT_VideoInfo))
+    return GST_VIDEO_FORMAT_UYVY;
+
   return GST_VIDEO_FORMAT_UNKNOWN;
 }
 
+gboolean
+gst_dshow_is_pin_connected (IPin * pin)
+{
+  IPin *tmp_pin = NULL;
+  gboolean res;
+  HRESULT hres;
+
+  g_assert (pin);
+  hres = pin->ConnectedTo (&tmp_pin);
+  res = (hres != VFW_E_NOT_CONNECTED);
+  if (tmp_pin)
+    tmp_pin->Release ();
+
+  return res;
+}
+
 GstCaps *
 gst_dshow_new_video_caps (GstVideoFormat video_format, const gchar * name,
     GstCapturePinMediaType * pin_mediatype)
@@ -421,13 +442,16 @@ gst_dshow_new_video_caps (GstVideoFormat video_format, const gchar * name,
   /* raw video format */
   switch (video_format) {
     case GST_VIDEO_FORMAT_BGR:
-      video_caps = gst_caps_from_string (GST_VIDEO_CAPS_BGR);
+      video_caps = gst_caps_from_string (GST_VIDEO_CAPS_MAKE ("BGR"));
       break;
     case GST_VIDEO_FORMAT_I420:
-      video_caps = gst_caps_from_string (GST_VIDEO_CAPS_YUV ("I420"));
+      video_caps = gst_caps_from_string (GST_VIDEO_CAPS_MAKE ("I420"));
 	  break;
     case GST_VIDEO_FORMAT_YUY2:
-      video_caps = gst_caps_from_string (GST_VIDEO_CAPS_YUV ("YUY2"));
+      video_caps = gst_caps_from_string (GST_VIDEO_CAPS_MAKE ("YUY2"));
+      break;
+    case GST_VIDEO_FORMAT_UYVY:
+      video_caps = gst_caps_from_string (GST_VIDEO_CAPS_MAKE ("UYVY"));
       break;
     default:
       break;
@@ -438,12 +462,16 @@ gst_dshow_new_video_caps (GstVideoFormat video_format, const gchar * name,
     if (g_ascii_strncasecmp (name, "video/x-dv, systemstream=FALSE", 31) == 0) {
       video_caps = gst_caps_new_simple ("video/x-dv",
           "systemstream", G_TYPE_BOOLEAN, FALSE,
-          "format", GST_TYPE_FOURCC, GST_MAKE_FOURCC ('d', 'v', 's', 'd'),
+          "format", G_TYPE_STRING, "dvsd",
           NULL);
     } else if (g_ascii_strncasecmp (name, "video/x-dv, systemstream=TRUE", 31) == 0) {
       video_caps = gst_caps_new_simple ("video/x-dv",
           "systemstream", G_TYPE_BOOLEAN, TRUE, NULL);
       return video_caps;
+    } else if (g_ascii_strncasecmp (name, "image/jpeg", 10) == 0) {
+      video_caps = gst_caps_new_simple ("image/jpeg", NULL);
+    } else if (g_ascii_strncasecmp (name, "video/x-h264", 12) == 0) {
+      video_caps = gst_caps_new_simple ("video/x-h264", NULL);
     }
   }
 
@@ -490,3 +518,21 @@ gst_dshow_new_video_caps (GstVideoFormat video_format, const gchar * name,
 
   return video_caps;
 }
+
+bool gst_dshow_configure_latency (IPin *pCapturePin, guint bufSizeMS)
+{
+  HRESULT hr;
+  ALLOCATOR_PROPERTIES alloc_prop;
+  IAMBufferNegotiation * pNeg = NULL;
+  hr = pCapturePin->QueryInterface(IID_IAMBufferNegotiation, (void **)&pNeg);
+
+  if(!SUCCEEDED (hr))
+    return FALSE;
+
+  alloc_prop.cbAlign = -1;  // -1 means no preference.
+  alloc_prop.cbBuffer = bufSizeMS;
+  alloc_prop.cbPrefix = -1;
+  alloc_prop.cBuffers = -1;
+  hr = pNeg->SuggestAllocatorProperties (&alloc_prop);
+  return SUCCEEDED (hr);
+}
