@@ -114,7 +114,6 @@ _opensles_recorder_acquire (GstAudioRingBuffer * rb,
   GstOpenSLESRingBuffer *thiz = GST_OPENSLES_RING_BUFFER_CAST (rb);
   SLresult result;
   SLDataFormat_PCM format;
-  SLAndroidConfigurationItf config;
 
   /* Configure audio source */
   SLDataLocator_IODevice loc_dev = {
@@ -130,44 +129,21 @@ _opensles_recorder_acquire (GstAudioRingBuffer * rb,
   SLDataSink audioSink = { &loc_bq, &format };
 
   /* Required optional interfaces */
-  const SLInterfaceID ids[2] = { SL_IID_ANDROIDSIMPLEBUFFERQUEUE,
-    SL_IID_ANDROIDCONFIGURATION
-  };
-  const SLboolean req[2] = { SL_BOOLEAN_TRUE, SL_BOOLEAN_FALSE };
+  const SLInterfaceID id[1] = { SL_IID_ANDROIDSIMPLEBUFFERQUEUE };
+  const SLboolean req[1] = { SL_BOOLEAN_TRUE };
 
   /* Define the audio format in OpenSL ES terminology */
   _opensles_format (spec, &format);
 
   /* Create the audio recorder object (requires the RECORD_AUDIO permission) */
   result = (*thiz->engineEngine)->CreateAudioRecorder (thiz->engineEngine,
-      &thiz->recorderObject, &audioSrc, &audioSink, 2, ids, req);
+      &thiz->recorderObject, &audioSrc, &audioSink, 1, id, req);
   if (result != SL_RESULT_SUCCESS) {
     GST_ERROR_OBJECT (thiz, "engine.CreateAudioRecorder failed(0x%08x)",
         (guint32) result);
     goto failed;
   }
 
-  /* Set the recording preset if we have one */
-  if (thiz->preset != GST_OPENSLES_RECORDING_PRESET_NONE) {
-    SLint32 preset = gst_to_opensles_recording_preset (thiz->preset);
-
-    result = (*thiz->recorderObject)->GetInterface (thiz->recorderObject,
-        SL_IID_ANDROIDCONFIGURATION, &config);
-
-    if (result == SL_RESULT_SUCCESS) {
-      result = (*config)->SetConfiguration (config,
-          SL_ANDROID_KEY_RECORDING_PRESET, &preset, sizeof (preset));
-
-      if (result != SL_RESULT_SUCCESS) {
-        GST_WARNING_OBJECT (thiz, "Failed to set playback stream type (0x%08x)",
-            (guint32) result);
-      }
-    } else {
-      GST_WARNING_OBJECT (thiz,
-          "Could not get configuration interface 0x%08x", (guint32) result);
-    }
-  }
-
   /* Realize the audio recorder object */
   result =
       (*thiz->recorderObject)->Realize (thiz->recorderObject, SL_BOOLEAN_FALSE);
@@ -378,7 +354,6 @@ _opensles_player_acquire (GstAudioRingBuffer * rb,
   GstOpenSLESRingBuffer *thiz = GST_OPENSLES_RING_BUFFER_CAST (rb);
   SLresult result;
   SLDataFormat_PCM format;
-  SLAndroidConfigurationItf config;
 
   /* Configure audio source
    * 4 buffers is the "typical" size as optimized inside Android's
@@ -400,46 +375,21 @@ _opensles_player_acquire (GstAudioRingBuffer * rb,
   SLDataSink audioSink = { &loc_outmix, NULL };
 
   /* Define the required interfaces */
-  const SLInterfaceID ids[3] = { SL_IID_BUFFERQUEUE, SL_IID_VOLUME,
-    SL_IID_ANDROIDCONFIGURATION
-  };
-  const SLboolean req[3] = { SL_BOOLEAN_TRUE, SL_BOOLEAN_TRUE,
-    SL_BOOLEAN_FALSE
-  };
+  const SLInterfaceID ids[2] = { SL_IID_BUFFERQUEUE, SL_IID_VOLUME };
+  const SLboolean req[2] = { SL_BOOLEAN_TRUE, SL_BOOLEAN_TRUE };
 
   /* Define the format in OpenSL ES terminology */
   _opensles_format (spec, &format);
 
   /* Create the player object */
   result = (*thiz->engineEngine)->CreateAudioPlayer (thiz->engineEngine,
-      &thiz->playerObject, &audioSrc, &audioSink, 3, ids, req);
+      &thiz->playerObject, &audioSrc, &audioSink, 2, ids, req);
   if (result != SL_RESULT_SUCCESS) {
     GST_ERROR_OBJECT (thiz, "engine.CreateAudioPlayer failed(0x%08x)",
         (guint32) result);
     goto failed;
   }
 
-  /* Set the stream type if we have one */
-  if (thiz->stream_type != GST_OPENSLES_STREAM_TYPE_NONE) {
-    SLint32 stream_type = gst_to_opensles_stream_type (thiz->stream_type);
-
-    result = (*thiz->playerObject)->GetInterface (thiz->playerObject,
-        SL_IID_ANDROIDCONFIGURATION, &config);
-
-    if (result == SL_RESULT_SUCCESS) {
-      result = (*config)->SetConfiguration (config,
-          SL_ANDROID_KEY_STREAM_TYPE, &stream_type, sizeof (stream_type));
-
-      if (result != SL_RESULT_SUCCESS) {
-        GST_WARNING_OBJECT (thiz, "Failed to set playback stream type (0x%08x)",
-            (guint32) result);
-      }
-    } else {
-      GST_WARNING_OBJECT (thiz,
-          "Could not get configuration interface 0x%08x", (guint32) result);
-    }
-  }
-
   /* Realize the player object */
   result =
       (*thiz->playerObject)->Realize (thiz->playerObject, SL_BOOLEAN_FALSE);
@@ -973,14 +923,7 @@ gst_opensles_ringbuffer_delay (GstAudioRingBuffer * rb)
       playedpos =
           gst_util_uint64_scale_round (position, rb->spec.info.rate, 1000);
       queuedpos = g_atomic_int_get (&thiz->segqueued) * rb->samples_per_seg;
-      if (queuedpos < playedpos) {
-        res = 0;
-        GST_ERROR_OBJECT (thiz,
-            "Queued position smaller than playback position (%" G_GUINT64_FORMAT
-            " < %" G_GUINT64_FORMAT ")", queuedpos, playedpos);
-      } else {
-        res = queuedpos - playedpos;
-      }
+      res = queuedpos - playedpos;
     }
 
     GST_LOG_OBJECT (thiz, "queued samples %" G_GUINT64_FORMAT " position %u ms "
