@@ -19,15 +19,16 @@
 
 /**
  * SECTION:element-openslessink
- * @title: openslessink
  * @see_also: openslessrc
  *
  * This element renders raw audio samples using the OpenSL ES API in Android OS.
  *
- * ## Example pipelines
+ * <refsect2>
+ * <title>Example pipelines</title>
  * |[
- * gst-launch-1.0 -v filesrc location=music.ogg ! oggdemux ! vorbisdec ! audioconvert ! audioresample ! opeslessink
+ * gst-launch -v filesrc location=music.ogg ! oggdemux ! vorbisdec ! audioconvert ! audioresample ! opeslessink
  * ]| Play an Ogg/Vorbis file.
+ * </refsect2>
  *
  */
 
@@ -46,18 +47,18 @@ enum
   PROP_0,
   PROP_VOLUME,
   PROP_MUTE,
-  PROP_STREAM_TYPE,
   PROP_LAST
 };
 
 #define DEFAULT_VOLUME 1.0
 #define DEFAULT_MUTE   FALSE
 
-#define DEFAULT_STREAM_TYPE GST_OPENSLES_STREAM_TYPE_NONE
-
 
 /* According to Android's NDK doc the following are the supported rates */
-#define RATES "8000, 11025, 12000, 16000, 22050, 24000, 32000, 44100, 48000"
+#define RATES "8000, 11025, 12000, 16000, 22050, 24000, 32000, 44100"
+/* 48000 Hz is also claimed to be supported but the AudioFlinger downsampling
+ * doesn't seems to work properly so we relay GStreamer audioresample element
+ * to cope with this samplerate. */
 
 static GstStaticPadTemplate sink_factory = GST_STATIC_PAD_TEMPLATE ("sink",
     GST_PAD_SINK,
@@ -74,8 +75,6 @@ static GstStaticPadTemplate sink_factory = GST_STATIC_PAD_TEMPLATE ("sink",
 #define parent_class gst_opensles_sink_parent_class
 G_DEFINE_TYPE_WITH_CODE (GstOpenSLESSink, gst_opensles_sink,
     GST_TYPE_AUDIO_BASE_SINK, _do_init);
-GST_ELEMENT_REGISTER_DEFINE_WITH_CODE (openslessink, "openslessink",
-    GST_RANK_PRIMARY, GST_TYPE_OPENSLES_SINK, opensles_element_init (plugin));
 
 static GstAudioRingBuffer *
 gst_opensles_sink_create_ringbuffer (GstAudioBaseSink * base)
@@ -86,9 +85,6 @@ gst_opensles_sink_create_ringbuffer (GstAudioBaseSink * base)
   rb = gst_opensles_ringbuffer_new (RB_MODE_SINK_PCM);
   gst_opensles_ringbuffer_set_volume (rb, sink->volume);
   gst_opensles_ringbuffer_set_mute (rb, sink->mute);
-
-  GST_OPENSLES_RING_BUFFER (rb)->stream_type = sink->stream_type;
-
   return rb;
 }
 
@@ -212,9 +208,6 @@ gst_opensles_sink_set_property (GObject * object, guint prop_id,
         gst_opensles_ringbuffer_set_mute (rb, sink->mute);
       }
       break;
-    case PROP_STREAM_TYPE:
-      sink->stream_type = g_value_get_enum (value);
-      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -233,9 +226,6 @@ gst_opensles_sink_get_property (GObject * object, guint prop_id,
     case PROP_MUTE:
       g_value_set_boolean (value, sink->mute);
       break;
-    case PROP_STREAM_TYPE:
-      g_value_set_enum (value, sink->stream_type);
-      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -264,13 +254,8 @@ gst_opensles_sink_class_init (GstOpenSLESSinkClass * klass)
       g_param_spec_boolean ("mute", "Mute", "Mute state of this stream",
           DEFAULT_MUTE, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
-  g_object_class_install_property (gobject_class, PROP_STREAM_TYPE,
-      g_param_spec_enum ("stream-type", "Stream type",
-          "Stream type that this stream should be tagged with",
-          GST_TYPE_OPENSLES_STREAM_TYPE, DEFAULT_STREAM_TYPE,
-          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
-
-  gst_element_class_add_static_pad_template (gstelement_class, &sink_factory);
+  gst_element_class_add_pad_template (gstelement_class,
+      gst_static_pad_template_get (&sink_factory));
 
   gst_element_class_set_static_metadata (gstelement_class, "OpenSL ES Sink",
       "Sink/Audio",
@@ -284,11 +269,14 @@ gst_opensles_sink_class_init (GstOpenSLESSinkClass * klass)
 static void
 gst_opensles_sink_init (GstOpenSLESSink * sink)
 {
-  sink->stream_type = DEFAULT_STREAM_TYPE;
   sink->volume = DEFAULT_VOLUME;
   sink->mute = DEFAULT_MUTE;
 
   _opensles_query_capabilities (sink);
 
   gst_audio_base_sink_set_provide_clock (GST_AUDIO_BASE_SINK (sink), TRUE);
+  /* Override some default values to fit on the AudioFlinger behaviour of
+   * processing 20ms buffers as minimum buffer size. */
+  GST_AUDIO_BASE_SINK (sink)->buffer_time = 200000;
+  GST_AUDIO_BASE_SINK (sink)->latency_time = 20000;
 }
