@@ -13,8 +13,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
 /**
@@ -47,14 +47,15 @@ static GstStaticPadTemplate src_factory = GST_STATIC_PAD_TEMPLATE ("src",
     GST_PAD_ALWAYS,
     GST_STATIC_CAPS ("audio/x-raw, "
         "format = (string) " GST_AUDIO_NE (S16) ", "
-        "rate = (int) { 16000 }, "
-        "channels = (int) 1")
+        "rate = (int) 16000, "
+        "channels = (int) 1, "
+        "layout = (string) interleaved")
     );
 /* *INDENT-ON* */
 
 #define _do_init \
-  GST_DEBUG_CATEGORY_INIT (opensles_src_debug, "opensles_src", 0, \
-      "OpenSL ES Src");
+  GST_DEBUG_CATEGORY_INIT (opensles_src_debug, "openslessrc", 0, \
+      "OpenSLES Source");
 #define parent_class gst_opensles_src_parent_class
 G_DEFINE_TYPE_WITH_CODE (GstOpenSLESSrc, gst_opensles_src,
     GST_TYPE_AUDIO_BASE_SRC, _do_init);
@@ -82,7 +83,7 @@ gst_opensles_src_class_init (GstOpenSLESSrcClass * klass)
       gst_static_pad_template_get (&src_factory));
 
   gst_element_class_set_static_metadata (gstelement_class, "OpenSL ES Src",
-      "Src/Audio",
+      "Source/Audio",
       "Input sound using the OpenSL ES APIs",
       "Josep Torra <support@fluendo.com>");
 
@@ -95,6 +96,6 @@ gst_opensles_src_init (GstOpenSLESSrc * src)
 {
   /* Override some default values to fit on the AudioFlinger behaviour of
    * processing 20ms buffers as minimum buffer size. */
-  GST_AUDIO_BASE_SRC (src)->buffer_time = 400000;
+  GST_AUDIO_BASE_SRC (src)->buffer_time = 200000;
   GST_AUDIO_BASE_SRC (src)->latency_time = 20000;
 }
