@@ -45,24 +45,6 @@
 
 static GMainLoop *main_loop;
 
-static GstCaps *
-_compositor_get_all_supported_caps (void)
-{
-  return gst_caps_from_string (GST_VIDEO_CAPS_MAKE
-      (" { AYUV, BGRA, ARGB, RGBA, ABGR, Y444, Y42B, YUY2, UYVY, "
-          "   YVYU, I420, YV12, NV12, NV21, Y41B, RGB, BGR, xRGB, xBGR, "
-          "   RGBx, BGRx } "));
-}
-
-static GstCaps *
-_compositor_get_non_alpha_supported_caps (void)
-{
-  return gst_caps_from_string (GST_VIDEO_CAPS_MAKE
-      (" { Y444, Y42B, YUY2, UYVY, "
-          "   YVYU, I420, YV12, NV12, NV21, Y41B, RGB, BGR, xRGB, xBGR, "
-          "   RGBx, BGRx } "));
-}
-
 /* make sure downstream gets a CAPS event before buffers are sent */
 GST_START_TEST (test_caps)
 {
@@ -263,378 +245,6 @@ GST_START_TEST (test_event)
 
 GST_END_TEST;
 
-static GstBuffer *
-create_video_buffer (GstCaps * caps, gint ts_in_seconds)
-{
-  GstVideoInfo info;
-  guint size;
-  GstBuffer *buf;
-  GstMapInfo mapinfo;
-
-  fail_unless (gst_video_info_from_caps (&info, caps));
-
-  size = GST_VIDEO_INFO_WIDTH (&info) * GST_VIDEO_INFO_HEIGHT (&info);
-
-  switch (GST_VIDEO_INFO_FORMAT (&info)) {
-    case GST_VIDEO_FORMAT_RGB:
-      size *= 3;
-      break;
-    case GST_VIDEO_FORMAT_RGBA:
-    case GST_VIDEO_FORMAT_ARGB:
-      size *= 4;
-      break;
-    case GST_VIDEO_FORMAT_I420:
-      size *= 2;
-      break;
-    default:
-      fail ("Unsupported test format");
-  }
-
-  buf = gst_buffer_new_and_alloc (size);
-  /* write something to avoid uninitialized error issues (valgrind) */
-  gst_buffer_map (buf, &mapinfo, GST_MAP_WRITE);
-  memset (mapinfo.data, 0, mapinfo.size);
-  gst_buffer_unmap (buf, &mapinfo);
-
-  GST_BUFFER_PTS (buf) = ts_in_seconds * GST_SECOND;
-  GST_BUFFER_DURATION (buf) = GST_SECOND;
-  return buf;
-}
-
-
-GST_START_TEST (test_caps_query)
-{
-  GstElement *compositor, *capsfilter, *sink;
-  GstElement *pipeline;
-  gboolean res;
-  GstStateChangeReturn state_res;
-  GstPad *sinkpad;
-  GstCaps *caps, *restriction_caps;
-  GstCaps *all_caps, *non_alpha_caps;
-
-  /* initial setup */
-  all_caps = _compositor_get_all_supported_caps ();
-  non_alpha_caps = _compositor_get_non_alpha_supported_caps ();
-
-  compositor = gst_element_factory_make ("compositor", "compositor");
-  capsfilter = gst_element_factory_make ("capsfilter", "out-cf");
-  sink = gst_element_factory_make ("fakesink", "sink");
-  pipeline = gst_pipeline_new ("test-pipeline");
-
-  gst_bin_add_many (GST_BIN (pipeline), compositor, capsfilter, sink, NULL);
-  res = gst_element_link (compositor, capsfilter);
-  fail_unless (res == TRUE, NULL);
-  res = gst_element_link (capsfilter, sink);
-  fail_unless (res == TRUE, NULL);
-
-  sinkpad = gst_element_get_request_pad (compositor, "sink_%u");
-
-  state_res = gst_element_set_state (pipeline, GST_STATE_PLAYING);
-  fail_if (state_res == GST_STATE_CHANGE_FAILURE);
-
-  /* try an unrestricted caps query, should return all formats */
-  caps = gst_pad_query_caps (sinkpad, NULL);
-  fail_unless (gst_caps_is_equal (caps, all_caps));
-  gst_caps_unref (caps);
-
-  /* now restrict downstream to a single alpha format, it should still
-   * be able to convert anything else to it */
-  restriction_caps = gst_caps_from_string ("video/x-raw, format=(string)AYUV");
-  g_object_set (capsfilter, "caps", restriction_caps, NULL);
-  caps = gst_pad_query_caps (sinkpad, NULL);
-  fail_unless (gst_caps_is_equal (caps, all_caps));
-  gst_caps_unref (caps);
-  gst_caps_unref (restriction_caps);
-
-  /* now restrict downstream to a non-alpha format, it should
-   * be able to accept non-alpha formats */
-  restriction_caps = gst_caps_from_string ("video/x-raw, format=(string)I420");
-  g_object_set (capsfilter, "caps", restriction_caps, NULL);
-  caps = gst_pad_query_caps (sinkpad, NULL);
-  fail_unless (gst_caps_is_equal (caps, non_alpha_caps));
-  gst_caps_unref (caps);
-  gst_caps_unref (restriction_caps);
-
-  /* check that compositor proxies downstream interlace-mode */
-  restriction_caps =
-      gst_caps_from_string ("video/x-raw, interlace-mode=(string)interleaved");
-  g_object_set (capsfilter, "caps", restriction_caps, NULL);
-  caps = gst_pad_query_caps (sinkpad, NULL);
-  fail_unless (gst_caps_is_subset (caps, restriction_caps));
-  gst_caps_unref (caps);
-  gst_caps_unref (restriction_caps);
-
-  gst_element_set_state (pipeline, GST_STATE_NULL);
-  gst_element_release_request_pad (compositor, sinkpad);
-  gst_object_unref (sinkpad);
-  gst_object_unref (pipeline);
-  gst_caps_unref (all_caps);
-  gst_caps_unref (non_alpha_caps);
-}
-
-GST_END_TEST;
-
-
-GST_START_TEST (test_caps_query_interlaced)
-{
-  GstElement *compositor, *sink;
-  GstElement *pipeline;
-  gboolean res;
-  GstStateChangeReturn state_res;
-  GstPad *sinkpad;
-  GstCaps *caps;
-  GstCaps *caps_mixed, *caps_progressive, *caps_interleaved;
-  GstEvent *caps_event;
-  GstQuery *drain;
-
-  caps_interleaved =
-      gst_caps_from_string ("video/x-raw, interlace-mode=interleaved");
-  caps_mixed = gst_caps_from_string ("video/x-raw, interlace-mode=mixed");
-  caps_progressive =
-      gst_caps_from_string ("video/x-raw, interlace-mode=progressive");
-
-  /* initial setup */
-  compositor = gst_element_factory_make ("compositor", "compositor");
-  sink = gst_element_factory_make ("fakesink", "sink");
-  pipeline = gst_pipeline_new ("test-pipeline");
-
-  gst_bin_add_many (GST_BIN (pipeline), compositor, sink, NULL);
-  res = gst_element_link (compositor, sink);
-  fail_unless (res == TRUE, NULL);
-  sinkpad = gst_element_get_request_pad (compositor, "sink_%u");
-
-  state_res = gst_element_set_state (pipeline, GST_STATE_PLAYING);
-  fail_if (state_res == GST_STATE_CHANGE_FAILURE);
-
-  /* try an unrestricted caps query, should be compatible with all formats */
-  caps = gst_pad_query_caps (sinkpad, NULL);
-  fail_unless (gst_caps_can_intersect (caps, caps_interleaved));
-  fail_unless (gst_caps_can_intersect (caps, caps_progressive));
-  fail_unless (gst_caps_can_intersect (caps, caps_mixed));
-  gst_caps_unref (caps);
-
-  /* now set caps on the pad, it should restrict the interlace-mode for
-   * future caps */
-  caps = gst_caps_from_string ("video/x-raw, width=100, height=100, "
-      "format=RGB, framerate=1/1, interlace-mode=progressive");
-  caps_event = gst_event_new_caps (caps);
-  gst_caps_unref (caps);
-  fail_unless (gst_pad_send_event (sinkpad, caps_event));
-
-  /* Send drain query to make sure this is processed */
-  drain = gst_query_new_drain ();
-  gst_pad_query (sinkpad, drain);
-  gst_query_unref (drain);
-
-  /* now recheck the interlace-mode */
-  gst_object_unref (sinkpad);
-  sinkpad = gst_element_get_request_pad (compositor, "sink_%u");
-  caps = gst_pad_query_caps (sinkpad, NULL);
-  fail_if (gst_caps_can_intersect (caps, caps_interleaved));
-  fail_unless (gst_caps_can_intersect (caps, caps_progressive));
-  fail_if (gst_caps_can_intersect (caps, caps_mixed));
-  gst_object_unref (sinkpad);
-  gst_caps_unref (caps);
-
-  gst_element_set_state (pipeline, GST_STATE_NULL);
-  gst_object_unref (pipeline);
-  gst_caps_unref (caps_interleaved);
-  gst_caps_unref (caps_mixed);
-  gst_caps_unref (caps_progressive);
-}
-
-GST_END_TEST;
-
-static void
-add_interlaced_mode_to_caps (GstCaps * caps, const gchar * mode)
-{
-  GstStructure *s;
-  gint i;
-
-  for (i = 0; i < gst_caps_get_size (caps); i++) {
-    s = gst_caps_get_structure (caps, i);
-    gst_structure_set (s, "interlace-mode", G_TYPE_STRING, mode, NULL);
-  }
-}
-
-#define MODE_ALL 1
-#define MODE_NON_ALPHA 2
-
-static void
-run_late_caps_query_test (GstCaps * input_caps, GstCaps * output_allowed_caps,
-    gint expected_caps_mode)
-{
-  GstElement *compositor, *capsfilter, *sink;
-  GstElement *pipeline;
-  gboolean res;
-  GstStateChangeReturn state_res;
-  GstPad *srcpad1, *srcpad2;
-  GstPad *sinkpad1, *sinkpad2;
-  GstSegment segment;
-  GstCaps *caps, *all_caps, *non_alpha_caps;
-
-  all_caps = _compositor_get_all_supported_caps ();
-  non_alpha_caps = _compositor_get_non_alpha_supported_caps ();
-
-  /* add progressive mode as it is what is used in the test, otherwise
-   * is_equal checks would fail */
-  add_interlaced_mode_to_caps (all_caps, "progressive");
-  add_interlaced_mode_to_caps (non_alpha_caps, "progressive");
-
-  compositor = gst_element_factory_make ("compositor", "compositor");
-  capsfilter = gst_element_factory_make ("capsfilter", "out-cf");
-  sink = gst_element_factory_make ("fakesink", "sink");
-  pipeline = gst_pipeline_new ("test-pipeline");
-
-  gst_bin_add_many (GST_BIN (pipeline), compositor, capsfilter, sink, NULL);
-  res = gst_element_link (compositor, capsfilter);
-  fail_unless (res == TRUE, NULL);
-  res = gst_element_link (capsfilter, sink);
-  fail_unless (res == TRUE, NULL);
-
-  sinkpad1 = gst_element_get_request_pad (compositor, "sink_%u");
-  srcpad1 = gst_pad_new ("src1", GST_PAD_SRC);
-  fail_unless (gst_pad_link (srcpad1, sinkpad1) == GST_PAD_LINK_OK);
-  gst_pad_set_active (srcpad1, TRUE);
-
-  state_res = gst_element_set_state (pipeline, GST_STATE_PLAYING);
-  fail_if (state_res == GST_STATE_CHANGE_FAILURE);
-
-  if (output_allowed_caps)
-    g_object_set (capsfilter, "caps", output_allowed_caps, NULL);
-
-  gst_segment_init (&segment, GST_FORMAT_TIME);
-  fail_unless (gst_pad_push_event (srcpad1,
-          gst_event_new_stream_start ("test-1")));
-  fail_unless (gst_pad_push_event (srcpad1, gst_event_new_caps (input_caps)));
-  fail_unless (gst_pad_push_event (srcpad1, gst_event_new_segment (&segment)));
-  fail_unless (gst_pad_push (srcpad1,
-          create_video_buffer (input_caps, 0)) == GST_FLOW_OK);
-  fail_unless (gst_pad_push (srcpad1,
-          create_video_buffer (input_caps, 1)) == GST_FLOW_OK);
-
-  /* now comes the second pad */
-  sinkpad2 = gst_element_get_request_pad (compositor, "sink_%u");
-  srcpad2 = gst_pad_new ("src2", GST_PAD_SRC);
-  fail_unless (gst_pad_link (srcpad2, sinkpad2) == GST_PAD_LINK_OK);
-  gst_pad_set_active (srcpad2, TRUE);
-  fail_unless (gst_pad_push_event (srcpad2,
-          gst_event_new_stream_start ("test-2")));
-
-  caps = gst_pad_peer_query_caps (srcpad2, NULL);
-  fail_unless (gst_caps_is_equal (caps,
-          expected_caps_mode == MODE_ALL ? all_caps : non_alpha_caps));
-  gst_caps_unref (caps);
-
-  gst_pad_set_active (srcpad1, FALSE);
-  gst_pad_set_active (srcpad2, FALSE);
-  gst_element_set_state (pipeline, GST_STATE_NULL);
-  gst_element_release_request_pad (compositor, sinkpad1);
-  gst_element_release_request_pad (compositor, sinkpad2);
-  gst_object_unref (sinkpad1);
-  gst_object_unref (sinkpad2);
-  gst_object_unref (pipeline);
-  gst_object_unref (srcpad1);
-  gst_object_unref (srcpad2);
-  gst_caps_unref (all_caps);
-  gst_caps_unref (non_alpha_caps);
-}
-
-GST_START_TEST (test_late_caps_query)
-{
-  GstCaps *rgb_caps;
-  GstCaps *non_alpha_caps;
-
-  rgb_caps = gst_caps_from_string ("video/x-raw, format=(string)RGB, "
-      "width=(int)100, height=(int)100, framerate=(fraction)1/1");
-  non_alpha_caps = gst_caps_from_string ("video/x-raw, format=(string)RGB");
-
-  /* check that a 2nd pad that is added late to compositor will be able to
-   * negotiate to formats that depend only on downstream caps and not on what
-   * the other pads have already negotiated */
-  run_late_caps_query_test (rgb_caps, NULL, MODE_ALL);
-  run_late_caps_query_test (rgb_caps, non_alpha_caps, MODE_NON_ALPHA);
-
-  gst_caps_unref (non_alpha_caps);
-  gst_caps_unref (rgb_caps);
-}
-
-GST_END_TEST;
-
-static void
-run_late_caps_set_test (GstCaps * first_caps, GstCaps * expected_query_caps,
-    GstCaps * second_caps, gboolean accept_caps)
-{
-  GstElement *capsfilter_1;
-  GstElement *compositor;
-  GstElement *pipeline;
-  GstStateChangeReturn state_res;
-  GstPad *sinkpad_2;
-  GstCaps *caps;
-  GstBus *bus;
-  GstMessage *msg;
-
-  pipeline =
-      gst_parse_launch ("videotestsrc num-buffers=10 ! capsfilter name=cf1 !"
-      " compositor name=c ! fakesink sync=true", NULL);
-  fail_unless (pipeline != NULL);
-
-  bus = gst_pipeline_get_bus (GST_PIPELINE (pipeline));
-
-  compositor = gst_bin_get_by_name (GST_BIN (pipeline), "c");
-  capsfilter_1 = gst_bin_get_by_name (GST_BIN (pipeline), "cf1");
-
-  g_object_set (capsfilter_1, "caps", first_caps, NULL);
-
-  state_res = gst_element_set_state (pipeline, GST_STATE_PAUSED);
-  fail_if (state_res == GST_STATE_CHANGE_FAILURE);
-
-  /* wait for pipeline to get to paused */
-  msg =
-      gst_bus_timed_pop_filtered (bus, GST_CLOCK_TIME_NONE,
-      GST_MESSAGE_ASYNC_DONE);
-  fail_unless (msg != NULL);
-  fail_unless (GST_MESSAGE_TYPE (msg) == GST_MESSAGE_ASYNC_DONE);
-  gst_message_unref (msg);
-
-  /* try to set the second caps */
-  sinkpad_2 = gst_element_get_request_pad (compositor, "sink_%u");
-  caps = gst_pad_query_caps (sinkpad_2, NULL);
-  fail_unless (gst_caps_is_subset (expected_query_caps, caps));
-  gst_caps_unref (caps);
-  caps = gst_pad_query_caps (sinkpad_2, second_caps);
-  fail_unless (gst_caps_is_empty (caps) != accept_caps);
-  gst_caps_unref (caps);
-  gst_object_unref (sinkpad_2);
-
-  gst_object_unref (bus);
-  gst_object_unref (compositor);
-  gst_object_unref (capsfilter_1);
-  gst_element_set_state (pipeline, GST_STATE_NULL);
-  gst_object_unref (pipeline);
-}
-
-GST_START_TEST (test_late_caps_different_interlaced)
-{
-  GstCaps *non_interlaced_caps;
-  GstCaps *interlaced_caps;
-
-  non_interlaced_caps =
-      gst_caps_from_string ("video/x-raw, interlace-mode=progressive, "
-      "format=RGB, width=100, height=100, framerate=1/1");
-  interlaced_caps =
-      gst_caps_from_string ("video/x-raw, interlace-mode=interleaved, "
-      "format=RGB, width=100, height=100, framerate=1/1");
-
-  run_late_caps_set_test (non_interlaced_caps, non_interlaced_caps,
-      interlaced_caps, FALSE);
-
-  gst_caps_unref (non_interlaced_caps);
-  gst_caps_unref (interlaced_caps);
-}
-
-GST_END_TEST;
-
 static guint play_count = 0;
 static GstEvent *play_seek_event = NULL;
 
@@ -761,7 +371,7 @@ GST_START_TEST (test_play_twice)
   /* cleanup */
   g_main_loop_unref (main_loop);
   gst_consistency_checker_free (consist);
-  gst_event_unref (play_seek_event);
+  gst_event_ref (play_seek_event);
   gst_bus_remove_signal_watch (bus);
   gst_object_unref (bus);
   gst_object_unref (bin);
@@ -864,7 +474,7 @@ GST_START_TEST (test_play_twice_then_add_and_play_again)
 
   /* cleanup */
   g_main_loop_unref (main_loop);
-  gst_event_unref (play_seek_event);
+  gst_event_ref (play_seek_event);
   gst_consistency_checker_free (consist);
   gst_bus_remove_signal_watch (bus);
   gst_object_unref (bus);
@@ -1705,28 +1315,6 @@ GST_START_TEST (test_obscured_skipped)
   xpos0 = ypos0 = xpos1 = ypos1 = 0;
   buffer_mapped = FALSE;
 
-  xpos1 = ypos1 = 0;
-  xpos0 = ypos0 = width0 = height0 = width1 = height1 = 10;
-  out_width = out_height = 20;
-  GST_INFO ("testing bug 754107");
-  _test_obscured (caps_str, xpos0, ypos0, width0, height0, alpha0, xpos1, ypos1,
-      width1, height1, alpha1, out_width, out_height);
-  fail_unless (buffer_mapped == TRUE);
-  xpos0 = ypos0 = xpos1 = ypos1 = width0 = height0 = width1 = height1 = 0;
-  out_width = out_height = 0;
-  buffer_mapped = FALSE;
-
-  xpos1 = -1;
-  xpos0 = ypos0 = width0 = height0 = width1 = height1 = 10;
-  out_width = out_height = 20;
-  GST_INFO ("testing bug 754576");
-  _test_obscured (caps_str, xpos0, ypos0, width0, height0, alpha0, xpos1, ypos1,
-      width1, height1, alpha1, out_width, out_height);
-  fail_unless (buffer_mapped == TRUE);
-  xpos0 = xpos1 = ypos1 = width0 = height0 = width1 = height1 = 0;
-  out_width = out_height = 0;
-  buffer_mapped = FALSE;
-
   xpos0 = ypos0 = 10000;
   out_width = 320;
   out_height = 240;
@@ -1740,341 +1328,6 @@ GST_START_TEST (test_obscured_skipped)
 
 GST_END_TEST;
 
-static void
-_pipeline_eos (GstBus * bus, GstMessage * message, GstPipeline * bin)
-{
-  GST_INFO ("pipeline EOS");
-  g_main_loop_quit (main_loop);
-}
-
-static GstFlowReturn
-_buffer_recvd (GstElement * appsink, gint * buffers_recvd)
-{
-  GstSample *sample;
-
-  g_signal_emit_by_name (appsink, "pull-sample", &sample);
-  ck_assert_msg (sample != NULL, "NULL sample received!");
-
-  (*buffers_recvd)++;
-  GST_INFO ("buffer recvd");
-  gst_sample_unref (sample);
-
-  if (*buffers_recvd > 5)
-    g_main_loop_quit (main_loop);
-
-  return GST_FLOW_OK;
-}
-
-GST_START_TEST (test_ignore_eos)
-{
-  gboolean res;
-  gint buffers_recvd;
-  GstPadLinkReturn link_res;
-  GstStateChangeReturn state_res;
-  GstElement *bin, *src, *compositor, *appsink;
-  GstPad *srcpad, *sinkpad;
-  GstBus *bus;
-
-  GST_INFO ("preparing test");
-
-  /* build pipeline */
-  bin = gst_pipeline_new ("pipeline");
-  bus = gst_element_get_bus (bin);
-  gst_bus_add_signal_watch_full (bus, G_PRIORITY_HIGH);
-
-  src = gst_element_factory_make ("videotestsrc", NULL);
-  g_object_set (src, "num-buffers", 5, NULL);
-  compositor = gst_element_factory_make ("compositor", NULL);
-  appsink = gst_element_factory_make ("appsink", NULL);
-  g_object_set (appsink, "emit-signals", TRUE, NULL);
-  gst_bin_add_many (GST_BIN (bin), src, compositor, appsink, NULL);
-
-  res = gst_element_link (compositor, appsink);
-  ck_assert_msg (res == TRUE, "Could not link compositor with appsink");
-  srcpad = gst_element_get_static_pad (src, "src");
-  sinkpad = gst_element_get_request_pad (compositor, "sink_%u");
-  /* When "ignore-eos" is set, compositor will keep sending the last buffer even
-   * after EOS, so we will receive more buffers than we sent. */
-  g_object_set (sinkpad, "ignore-eos", TRUE, NULL);
-  link_res = gst_pad_link (srcpad, sinkpad);
-  ck_assert_msg (GST_PAD_LINK_SUCCESSFUL (link_res), "videotestsrc -> "
-      "compositor pad  link failed: %i", link_res);
-  gst_object_unref (sinkpad);
-  gst_object_unref (srcpad);
-
-  GST_INFO ("pipeline built, connecting signals");
-
-  buffers_recvd = 0;
-  state_res = gst_element_set_state (bin, GST_STATE_PLAYING);
-  ck_assert_msg (state_res != GST_STATE_CHANGE_FAILURE, "Pipeline didn't play");
-
-  main_loop = g_main_loop_new (NULL, FALSE);
-  g_signal_connect (bus, "message::error", G_CALLBACK (message_received), bin);
-  g_signal_connect (bus, "message::warning", G_CALLBACK (message_received),
-      bin);
-  g_signal_connect (bus, "message::eos", G_CALLBACK (_pipeline_eos), bin);
-  g_signal_connect (appsink, "new-sample", G_CALLBACK (_buffer_recvd),
-      &buffers_recvd);
-
-  GST_INFO ("starting test");
-  g_main_loop_run (main_loop);
-
-  state_res = gst_element_set_state (bin, GST_STATE_NULL);
-  ck_assert_int_ne (state_res, GST_STATE_CHANGE_FAILURE);
-
-  ck_assert_msg (buffers_recvd > 5, "Did not receive more buffers"
-      " than were sent");
-
-  /* cleanup */
-  g_main_loop_unref (main_loop);
-  gst_bus_remove_signal_watch (bus);
-  gst_object_unref (bus);
-  gst_object_unref (bin);
-}
-
-GST_END_TEST;
-
-/* Test that the GST_ELEMENT(vagg)->sinkpads GList is always sorted by zorder */
-GST_START_TEST (test_pad_z_order)
-{
-  GstElement *compositor;
-  GstPad *sinkpad1, *sinkpad2, *sinkpad3;
-  guint zorder1, zorder2;
-  GList *sinkpads;
-
-  GST_INFO ("preparing test");
-
-  compositor = gst_element_factory_make ("compositor", NULL);
-  sinkpad1 = gst_element_get_request_pad (compositor, "sink_%u");
-  sinkpad2 = gst_element_get_request_pad (compositor, "sink_%u");
-
-  /* Pads requested later have a higher z-order than earlier ones by default */
-  g_object_get (sinkpad1, "zorder", &zorder1, NULL);
-  g_object_get (sinkpad2, "zorder", &zorder2, NULL);
-  ck_assert_int_gt (zorder2, zorder1);
-  sinkpads = GST_ELEMENT (compositor)->sinkpads;
-  ck_assert_ptr_eq (sinkpads->data, sinkpad1);
-  ck_assert_ptr_eq (sinkpads->next->data, sinkpad2);
-
-  /* Make sinkpad1's zorder the largest, which should re-sort the sinkpads */
-  g_object_set (sinkpad1, "zorder", zorder2 + 1, NULL);
-  sinkpads = GST_ELEMENT (compositor)->sinkpads;
-  ck_assert_ptr_eq (sinkpads->data, sinkpad2);
-  ck_assert_ptr_eq (sinkpads->next->data, sinkpad1);
-
-  /* Get a new pad, which should be the highest pad now */
-  sinkpad3 = gst_element_get_request_pad (compositor, "sink_%u");
-  sinkpads = GST_ELEMENT (compositor)->sinkpads;
-  ck_assert_ptr_eq (sinkpads->data, sinkpad2);
-  ck_assert_ptr_eq (sinkpads->next->data, sinkpad1);
-  ck_assert_ptr_eq (sinkpads->next->next->data, sinkpad3);
-
-  /* cleanup */
-  gst_object_unref (compositor);
-  gst_object_unref (sinkpad1);
-  gst_object_unref (sinkpad2);
-  gst_object_unref (sinkpad3);
-}
-
-GST_END_TEST;
-
-/* 
- * Test that the pad numbering assigned by aggregator behaves as follows:
- * 1. If a pad number is requested, it must be assigned if it is available
- * 2. When numbering automatically, the largest available pad number is used
- * 3. Pad names must be unique
- */
-GST_START_TEST (test_pad_numbering)
-{
-  GstElement *mixer;
-  GstPad *sinkpad1, *sinkpad2, *sinkpad3, *sinkpad4;
-
-  GST_INFO ("preparing test");
-
-  mixer = gst_element_factory_make ("compositor", NULL);
-  sinkpad1 = gst_element_get_request_pad (mixer, "sink_%u");
-  sinkpad2 = gst_element_get_request_pad (mixer, "sink_7");
-  sinkpad3 = gst_element_get_request_pad (mixer, "sink_1");
-  sinkpad4 = gst_element_get_request_pad (mixer, "sink_%u");
-
-  ck_assert_str_eq (GST_PAD_NAME (sinkpad1), "sink_0");
-  ck_assert_str_eq (GST_PAD_NAME (sinkpad2), "sink_7");
-  ck_assert_str_eq (GST_PAD_NAME (sinkpad3), "sink_1");
-  ck_assert_str_eq (GST_PAD_NAME (sinkpad4), "sink_8");
-
-  /* cleanup */
-  gst_object_unref (mixer);
-  gst_object_unref (sinkpad1);
-  gst_object_unref (sinkpad2);
-  gst_object_unref (sinkpad3);
-  gst_object_unref (sinkpad4);
-}
-
-GST_END_TEST;
-
-typedef struct
-{
-  gint buffers_sent;
-  GstClockTime first_pts;
-  gboolean first;
-  gboolean drop;
-} TestStartTimeSelectionData;
-
-static GstPadProbeReturn
-drop_buffer_cb (GstPad * pad, GstPadProbeInfo * info, gpointer user_data)
-{
-  TestStartTimeSelectionData *data = user_data;
-
-  if (data->drop) {
-    data->buffers_sent = data->buffers_sent + 1;
-    if (data->buffers_sent < 4)
-      return GST_PAD_PROBE_DROP;
-  }
-
-  data->first_pts = GST_BUFFER_PTS (info->data);
-
-  return GST_PAD_PROBE_REMOVE;
-}
-
-static GstFlowReturn
-first_buffer_received_cb (GstElement * appsink, gpointer user_data)
-{
-  TestStartTimeSelectionData *data = user_data;
-  GstSample *sample;
-  GstBuffer *buffer;
-
-  g_signal_emit_by_name (appsink, "pull-sample", &sample);
-  ck_assert_msg (sample != NULL, "NULL sample received!");
-
-  buffer = gst_sample_get_buffer (sample);
-  if (!data->first) {
-    fail_unless_equals_uint64 (GST_BUFFER_PTS (buffer), 0);
-  } else {
-    fail_unless_equals_uint64 (GST_BUFFER_PTS (buffer), data->first_pts);
-  }
-
-  gst_sample_unref (sample);
-
-  g_main_loop_quit (main_loop);
-
-  return GST_FLOW_EOS;
-}
-
-static void
-run_test_start_time (gboolean first, gboolean drop, gboolean unlinked)
-{
-  gboolean res;
-  GstPadLinkReturn link_res;
-  GstStateChangeReturn state_res;
-  GstElement *bin, *src, *compositor, *appsink;
-  GstPad *srcpad, *sinkpad;
-  GstBus *bus;
-  TestStartTimeSelectionData data = { 0, GST_CLOCK_TIME_NONE, first, drop };
-
-  GST_INFO ("preparing test");
-
-  /* build pipeline */
-  bin = gst_pipeline_new ("pipeline");
-  bus = gst_element_get_bus (bin);
-  gst_bus_add_signal_watch_full (bus, G_PRIORITY_HIGH);
-
-  src = gst_element_factory_make ("videotestsrc", NULL);
-
-  srcpad = gst_element_get_static_pad (src, "src");
-  gst_pad_add_probe (srcpad, GST_PAD_PROBE_TYPE_BUFFER, drop_buffer_cb, &data,
-      NULL);
-  gst_object_unref (srcpad);
-
-  g_object_set (src, "is-live", TRUE, NULL);
-  compositor = gst_element_factory_make ("compositor", NULL);
-  g_object_set (compositor, "start-time-selection", (first ? 1 : 0), NULL);
-  appsink = gst_element_factory_make ("appsink", NULL);
-  g_object_set (appsink, "emit-signals", TRUE, NULL);
-  gst_bin_add_many (GST_BIN (bin), src, compositor, appsink, NULL);
-
-  res = gst_element_link (compositor, appsink);
-  ck_assert_msg (res == TRUE, "Could not link compositor with appsink");
-  srcpad = gst_element_get_static_pad (src, "src");
-  sinkpad = gst_element_get_request_pad (compositor, "sink_%u");
-  link_res = gst_pad_link (srcpad, sinkpad);
-  ck_assert_msg (GST_PAD_LINK_SUCCESSFUL (link_res), "videotestsrc -> "
-      "compositor pad  link failed: %i", link_res);
-  gst_object_unref (sinkpad);
-  gst_object_unref (srcpad);
-
-  if (unlinked) {
-    sinkpad = gst_element_get_request_pad (compositor, "sink_%u");
-    gst_object_unref (sinkpad);
-  }
-
-  GST_INFO ("pipeline built, connecting signals");
-
-  state_res = gst_element_set_state (bin, GST_STATE_PLAYING);
-  ck_assert_msg (state_res != GST_STATE_CHANGE_FAILURE, "Pipeline didn't play");
-
-  main_loop = g_main_loop_new (NULL, FALSE);
-  g_signal_connect (bus, "message::error", G_CALLBACK (message_received), bin);
-  g_signal_connect (bus, "message::warning", G_CALLBACK (message_received),
-      bin);
-  g_signal_connect (bus, "message::eos", G_CALLBACK (_pipeline_eos), bin);
-  g_signal_connect (appsink, "new-sample",
-      G_CALLBACK (first_buffer_received_cb), &data);
-
-  GST_INFO ("starting test");
-  g_main_loop_run (main_loop);
-
-  state_res = gst_element_set_state (bin, GST_STATE_NULL);
-  ck_assert_int_ne (state_res, GST_STATE_CHANGE_FAILURE);
-
-  /* cleanup */
-  g_main_loop_unref (main_loop);
-  gst_bus_remove_signal_watch (bus);
-  gst_object_unref (bus);
-  gst_object_unref (bin);
-}
-
-GST_START_TEST (test_start_time_zero_live_drop_0)
-{
-  run_test_start_time (FALSE, FALSE, FALSE);
-}
-
-GST_END_TEST;
-
-GST_START_TEST (test_start_time_zero_live_drop_3)
-{
-  run_test_start_time (FALSE, TRUE, FALSE);
-}
-
-GST_END_TEST;
-
-GST_START_TEST (test_start_time_zero_live_drop_3_unlinked_1)
-{
-  run_test_start_time (FALSE, TRUE, TRUE);
-}
-
-GST_END_TEST;
-
-GST_START_TEST (test_start_time_first_live_drop_0)
-{
-  run_test_start_time (TRUE, FALSE, FALSE);
-}
-
-GST_END_TEST;
-
-GST_START_TEST (test_start_time_first_live_drop_3)
-{
-  run_test_start_time (TRUE, TRUE, FALSE);
-}
-
-GST_END_TEST;
-
-GST_START_TEST (test_start_time_first_live_drop_3_unlinked_1)
-{
-  run_test_start_time (TRUE, TRUE, TRUE);
-}
-
-GST_END_TEST;
-
 static Suite *
 compositor_suite (void)
 {
@@ -2084,10 +1337,6 @@ compositor_suite (void)
   suite_add_tcase (s, tc_chain);
   tcase_add_test (tc_chain, test_caps);
   tcase_add_test (tc_chain, test_event);
-  tcase_add_test (tc_chain, test_caps_query);
-  tcase_add_test (tc_chain, test_caps_query_interlaced);
-  tcase_add_test (tc_chain, test_late_caps_query);
-  tcase_add_test (tc_chain, test_late_caps_different_interlaced);
   tcase_add_test (tc_chain, test_play_twice);
   tcase_add_test (tc_chain, test_play_twice_then_add_and_play_again);
   tcase_add_test (tc_chain, test_add_pad);
@@ -2099,15 +1348,17 @@ compositor_suite (void)
   tcase_add_test (tc_chain, test_flush_start_flush_stop);
   tcase_add_test (tc_chain, test_segment_base_handling);
   tcase_add_test (tc_chain, test_obscured_skipped);
-  tcase_add_test (tc_chain, test_ignore_eos);
-  tcase_add_test (tc_chain, test_pad_z_order);
-  tcase_add_test (tc_chain, test_pad_numbering);
-  tcase_add_test (tc_chain, test_start_time_zero_live_drop_0);
-  tcase_add_test (tc_chain, test_start_time_zero_live_drop_3);
-  tcase_add_test (tc_chain, test_start_time_zero_live_drop_3_unlinked_1);
-  tcase_add_test (tc_chain, test_start_time_first_live_drop_0);
-  tcase_add_test (tc_chain, test_start_time_first_live_drop_3);
-  tcase_add_test (tc_chain, test_start_time_first_live_drop_3_unlinked_1);
+
+  /* Use a longer timeout */
+#ifdef HAVE_VALGRIND
+  if (RUNNING_ON_VALGRIND) {
+    tcase_set_timeout (tc_chain, 5 * 60);
+  } else
+#endif
+  {
+    /* this is shorter than the default 60 seconds?! (tpm) */
+    /* tcase_set_timeout (tc_chain, 6); */
+  }
 
   return s;
 }
