@@ -28,6 +28,29 @@
 
 #include <stdio.h>
 
+#if GST_GL_HAVE_GLES2
+/* *INDENT-OFF* */
+static const gchar *vertex_shader_str_gles2 =
+      "attribute vec4 a_position;   \n"
+      "attribute vec2 a_texCoord;   \n"
+      "varying vec2 v_texCoord;     \n"
+      "void main()                  \n"
+      "{                            \n"
+      "   gl_Position = a_position; \n"
+      "   v_texCoord = a_texCoord;  \n"
+      "}                            \n";
+
+static const gchar *fragment_shader_str_gles2 =
+      "precision mediump float;                            \n"
+      "varying vec2 v_texCoord;                            \n"
+      "uniform sampler2D s_texture;                        \n"
+      "void main()                                         \n"
+      "{                                                   \n"
+      "  gl_FragColor = texture2D( s_texture, v_texCoord );\n"
+      "}                                                   \n";
+/* *INDENT-ON* */
+#endif
+
 static GstGLDisplay *display;
 
 static void
@@ -42,58 +65,48 @@ teardown (void)
   gst_object_unref (display);
 }
 
-static GstGLMemory *gl_tex;
-static GLuint vbo, vbo_indices, vao;
+static GLuint fbo_id, rbo, tex;
 static GstGLFramebuffer *fbo;
+#if GST_GL_HAVE_GLES2
+static GError *error;
 static GstGLShader *shader;
 static GLint shader_attr_position_loc;
 static GLint shader_attr_texture_loc;
-
-static const GLfloat vertices[] = {
-  /* x, y, z, s, t */
-  1.0f, 1.0f, 0.0f, 1.0f, 0.0f,
-  -1.0f, 1.0f, 0.0f, 0.0f, 0.0f,
-  -1.0f, -1.0f, 0.0f, 0.0f, 1.0f,
-  1.0f, -1.0f, 0.0f, 1.0f, 1.0f
-};
-
-static const GLushort indices[] = { 0, 1, 2, 0, 2, 3 };
+#endif
 
 static void
 init (gpointer data)
 {
   GstGLContext *context = data;
-  GError *error = NULL;
-  GstVideoInfo v_info;
-  GstGLMemoryAllocator *allocator;
-  GstGLVideoAllocationParams *params;
-
-  gst_video_info_set_format (&v_info, GST_VIDEO_FORMAT_RGBA, 320, 240);
-  allocator = gst_gl_memory_allocator_get_default (context);
-  params =
-      gst_gl_video_allocation_params_new (context, NULL, &v_info, 0, NULL,
-      GST_GL_TEXTURE_TARGET_2D, GST_VIDEO_GL_TEXTURE_TYPE_RGBA);
 
   /* has to be called in the thread that is going to use the framebuffer */
-  fbo = gst_gl_framebuffer_new_with_default_depth (context, 320, 240);
+  fbo = gst_gl_framebuffer_new (context);
 
-  fail_if (fbo == NULL, "failed to create framebuffer object");
+  gst_gl_framebuffer_generate (fbo, 320, 240, &fbo_id, &rbo);
+  fail_if (fbo == NULL || fbo_id == 0, "failed to create framebuffer object");
 
-  gl_tex =
-      (GstGLMemory *) gst_gl_base_memory_alloc ((GstGLBaseMemoryAllocator *)
-      allocator, (GstGLAllocationParams *) params);
-  gst_object_unref (allocator);
-  gst_gl_allocation_params_free ((GstGLAllocationParams *) params);
-  fail_if (gl_tex == NULL, "failed to create texture");
+  gst_gl_context_gen_texture (context, &tex, GST_VIDEO_FORMAT_RGBA, 320, 240);
+  fail_if (tex == 0, "failed to create texture");
 
-  shader = gst_gl_shader_new_default (context, &error);
-  fail_if (shader == NULL, "failed to create shader object: %s",
-      error->message);
+#if GST_GL_HAVE_GLES2
+  if (gst_gl_context_get_gl_api (context) & GST_GL_API_GLES2) {
+    shader = gst_gl_shader_new (context);
+    fail_if (shader == NULL, "failed to create shader object");
 
-  shader_attr_position_loc =
-      gst_gl_shader_get_attribute_location (shader, "a_position");
-  shader_attr_texture_loc =
-      gst_gl_shader_get_attribute_location (shader, "a_texcoord");
+    gst_gl_shader_set_vertex_source (shader, vertex_shader_str_gles2);
+    gst_gl_shader_set_fragment_source (shader, fragment_shader_str_gles2);
+
+    error = NULL;
+    gst_gl_shader_compile (shader, &error);
+    fail_if (error != NULL, "Error compiling shader %s\n",
+        error ? error->message : "Unknown Error");
+
+    shader_attr_position_loc =
+        gst_gl_shader_get_attribute_location (shader, "a_position");
+    shader_attr_texture_loc =
+        gst_gl_shader_get_attribute_location (shader, "a_texCoord");
+  }
+#endif
 }
 
 static void
@@ -101,14 +114,15 @@ deinit (gpointer data)
 {
   GstGLContext *context = data;
   GstGLFuncs *gl = context->gl_vtable;
-  if (vao)
-    gl->DeleteVertexArrays (1, &vao);
+  gl->DeleteTextures (1, &tex);
   gst_object_unref (fbo);
-  gst_object_unref (shader);
-  gst_memory_unref (GST_MEMORY_CAST (gl_tex));
+#if GST_GL_HAVE_GLES2
+  if (gst_gl_context_get_gl_api (context) & GST_GL_API_GLES2)
+    gst_object_unref (shader);
+#endif
 }
 
-static gboolean
+static void
 clear_tex (gpointer data)
 {
   GstGLContext *context = data;
@@ -121,96 +135,13 @@ clear_tex (gpointer data)
   r = r > 1.0 ? 0.0 : r + 0.03;
   g = g > 1.0 ? 0.0 : g + 0.01;
   b = b > 1.0 ? 0.0 : b + 0.015;
-
-  return TRUE;
 }
 
 static void
 draw_tex (gpointer data)
 {
-  gst_gl_framebuffer_draw_to_texture (fbo, gl_tex,
-      (GstGLFramebufferFunc) clear_tex, data);
-}
-
-static void
-_bind_buffer (GstGLContext * context)
-{
-  const GstGLFuncs *gl = context->gl_vtable;
-
-  gl->BindBuffer (GL_ELEMENT_ARRAY_BUFFER, vbo_indices);
-  gl->BindBuffer (GL_ARRAY_BUFFER, vbo);
-
-  /* Load the vertex position */
-  gl->VertexAttribPointer (shader_attr_position_loc, 3, GL_FLOAT, GL_FALSE,
-      5 * sizeof (GLfloat), (void *) 0);
-
-  /* Load the texture coordinate */
-  gl->VertexAttribPointer (shader_attr_texture_loc, 2, GL_FLOAT, GL_FALSE,
-      5 * sizeof (GLfloat), (void *) (3 * sizeof (GLfloat)));
-
-  gl->EnableVertexAttribArray (shader_attr_position_loc);
-  gl->EnableVertexAttribArray (shader_attr_texture_loc);
-}
-
-static void
-_unbind_buffer (GstGLContext * context)
-{
-  const GstGLFuncs *gl = context->gl_vtable;
-
-  gl->BindBuffer (GL_ELEMENT_ARRAY_BUFFER, 0);
-  gl->BindBuffer (GL_ARRAY_BUFFER, 0);
-
-  gl->DisableVertexAttribArray (shader_attr_position_loc);
-  gl->DisableVertexAttribArray (shader_attr_texture_loc);
-}
-
-static void
-init_blit (gpointer data)
-{
-  GstGLContext *context = data;
-  const GstGLFuncs *gl = context->gl_vtable;
-
-  if (!vbo) {
-    if (gl->GenVertexArrays) {
-      gl->GenVertexArrays (1, &vao);
-      gl->BindVertexArray (vao);
-    }
-
-    gl->GenBuffers (1, &vbo);
-    gl->BindBuffer (GL_ARRAY_BUFFER, vbo);
-    gl->BufferData (GL_ARRAY_BUFFER, 4 * 5 * sizeof (GLfloat), vertices,
-        GL_STATIC_DRAW);
-
-    gl->GenBuffers (1, &vbo_indices);
-    gl->BindBuffer (GL_ELEMENT_ARRAY_BUFFER, vbo_indices);
-    gl->BufferData (GL_ELEMENT_ARRAY_BUFFER, sizeof (indices), indices,
-        GL_STATIC_DRAW);
-
-    if (gl->GenVertexArrays) {
-      _bind_buffer (context);
-      gl->BindVertexArray (0);
-    }
-
-    gl->BindBuffer (GL_ARRAY_BUFFER, 0);
-    gl->BindBuffer (GL_ELEMENT_ARRAY_BUFFER, 0);
-  }
-}
-
-static void
-deinit_blit (gpointer data)
-{
-  GstGLContext *context = data;
-  const GstGLFuncs *gl = context->gl_vtable;
-
-  if (vbo)
-    gl->DeleteBuffers (1, &vbo);
-  vbo = 0;
-  if (vbo_indices)
-    gl->DeleteBuffers (1, &vbo_indices);
-  vbo_indices = 0;
-  if (vao)
-    gl->DeleteVertexArrays (1, &vao);
-  vao = 0;
+  gst_gl_framebuffer_use_v2 (fbo, 320, 240, fbo_id, rbo, tex,
+      (GLCB_V2) clear_tex, data);
 }
 
 static void
@@ -220,25 +151,80 @@ draw_render (gpointer data)
   GstGLContextClass *context_class = GST_GL_CONTEXT_GET_CLASS (context);
   const GstGLFuncs *gl = context->gl_vtable;
 
-  gl->Clear (GL_COLOR_BUFFER_BIT);
+  /* redraw the texture into the system provided framebuffer */
 
-  gst_gl_shader_use (shader);
+#if GST_GL_HAVE_OPENGL
+  if (gst_gl_context_get_gl_api (context) & GST_GL_API_OPENGL) {
+    GLfloat verts[8] = { 1.0f, 1.0f,
+      -1.0f, 1.0f,
+      -1.0f, -1.0f,
+      1.0f, -1.0f
+    };
+    GLfloat texcoords[8] = { 1.0f, 0.0f,
+      0.0f, 0.0f,
+      0.0f, 1.0f,
+      1.0f, 1.0f
+    };
 
-  gl->ActiveTexture (GL_TEXTURE0);
-  gl->BindTexture (GL_TEXTURE_2D, gst_gl_memory_get_texture_id (gl_tex));
-  gst_gl_shader_set_uniform_1i (shader, "s_texture", 0);
+    gl->Viewport (0, 0, 320, 240);
 
-  if (gl->GenVertexArrays)
-    gl->BindVertexArray (vao);
-  else
-    _bind_buffer (context);
+    gl->Clear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
 
-  gl->DrawElements (GL_TRIANGLES, 6, GL_UNSIGNED_SHORT, 0);
+    gl->MatrixMode (GL_PROJECTION);
+    gl->LoadIdentity ();
 
-  if (gl->GenVertexArrays)
-    gl->BindVertexArray (0);
-  else
-    _unbind_buffer (context);
+    gl->ActiveTexture (GL_TEXTURE_2D);
+    gl->BindTexture (GL_TEXTURE_2D, tex);
+
+    gl->EnableClientState (GL_VERTEX_ARRAY);
+    gl->VertexPointer (2, GL_FLOAT, 0, &verts);
+
+    gl->ClientActiveTexture (GL_TEXTURE0);
+    gl->EnableClientState (GL_TEXTURE_COORD_ARRAY);
+    gl->TexCoordPointer (2, GL_FLOAT, 0, &texcoords);
+
+    gl->DrawArrays (GL_TRIANGLE_FAN, 0, 4);
+
+    gl->DisableClientState (GL_VERTEX_ARRAY);
+    gl->DisableClientState (GL_TEXTURE_COORD_ARRAY);
+  }
+#endif
+#if GST_GL_HAVE_GLES2
+  if (gst_gl_context_get_gl_api (context) & GST_GL_API_GLES2) {
+    const GLfloat vVertices[] = { 1.0f, 1.0f, 0.0f,
+      1.0f, 0.0f,
+      -1.0f, 1.0f, 0.0f,
+      0.0f, 0.0f,
+      -1.0f, -1.0f, 0.0f,
+      0.0f, 1.0f,
+      1.0f, -1.0f, 0.0f,
+      1.0f, 1.0f
+    };
+
+    GLushort indices[] = { 0, 1, 2, 0, 2, 3 };
+
+    gl->Clear (GL_COLOR_BUFFER_BIT);
+
+    gst_gl_shader_use (shader);
+
+    /* Load the vertex position */
+    gl->VertexAttribPointer (shader_attr_position_loc, 3,
+        GL_FLOAT, GL_FALSE, 5 * sizeof (GLfloat), vVertices);
+
+    /* Load the texture coordinate */
+    gl->VertexAttribPointer (shader_attr_texture_loc, 2,
+        GL_FLOAT, GL_FALSE, 5 * sizeof (GLfloat), &vVertices[3]);
+
+    gl->EnableVertexAttribArray (shader_attr_position_loc);
+    gl->EnableVertexAttribArray (shader_attr_texture_loc);
+
+    gl->ActiveTexture (GL_TEXTURE0);
+    gl->BindTexture (GL_TEXTURE_2D, tex);
+    gst_gl_shader_set_uniform_1i (shader, "s_texture", 0);
+
+    gl->DrawElements (GL_TRIANGLES, 6, GL_UNSIGNED_SHORT, indices);
+  }
+#endif
 
   context_class->swap_buffers (context);
 }
@@ -276,9 +262,7 @@ GST_START_TEST (test_share)
   gst_gl_window_set_preferred_size (window, 320, 240);
   gst_gl_window_draw (window);
 
-  gst_gl_window_send_message (other_window, GST_GL_WINDOW_CB (init),
-      other_context);
-  gst_gl_window_send_message (window, GST_GL_WINDOW_CB (init_blit), context);
+  gst_gl_window_send_message (other_window, GST_GL_WINDOW_CB (init), context);
 
   while (i < 10) {
     gst_gl_window_send_message (other_window, GST_GL_WINDOW_CB (draw_tex),
@@ -288,9 +272,7 @@ GST_START_TEST (test_share)
     i++;
   }
 
-  gst_gl_window_send_message (other_window, GST_GL_WINDOW_CB (deinit),
-      other_context);
-  gst_gl_window_send_message (window, GST_GL_WINDOW_CB (deinit_blit), context);
+  gst_gl_window_send_message (other_window, GST_GL_WINDOW_CB (deinit), context);
 
   gst_object_unref (window);
   gst_object_unref (other_window);
@@ -332,7 +314,6 @@ check_wrapped (gpointer data)
   fail_if (!ret, "error received %s\n",
       error ? error->message : "Unknown error");
   fail_if (wrapped_context->gl_vtable->TexImage2D == NULL);
-  gst_gl_context_activate (wrapped_context, FALSE);
 }
 
 GST_START_TEST (test_wrapped_context)
@@ -383,9 +364,7 @@ GST_START_TEST (test_wrapped_context)
   gst_gl_window_set_preferred_size (window, 320, 240);
   gst_gl_window_draw (window);
 
-  gst_gl_window_send_message (other_window, GST_GL_WINDOW_CB (init),
-      other_context);
-  gst_gl_window_send_message (window, GST_GL_WINDOW_CB (init_blit), context);
+  gst_gl_window_send_message (other_window, GST_GL_WINDOW_CB (init), context);
 
   while (i < 10) {
     gst_gl_window_send_message (other_window, GST_GL_WINDOW_CB (draw_tex),
@@ -398,9 +377,7 @@ GST_START_TEST (test_wrapped_context)
   gst_gl_window_send_message (window, GST_GL_WINDOW_CB (check_wrapped),
       wrapped_context);
 
-  gst_gl_window_send_message (other_window, GST_GL_WINDOW_CB (deinit),
-      other_context);
-  gst_gl_window_send_message (window, GST_GL_WINDOW_CB (deinit_blit), context);
+  gst_gl_window_send_message (other_window, GST_GL_WINDOW_CB (deinit), context);
 
   gst_object_unref (other_context);
   gst_object_unref (other_window);
@@ -424,9 +401,7 @@ static void
 _fill_context_info (GstGLContext * context, struct context_info *info)
 {
   info->handle = gst_gl_context_get_current_gl_context (info->platform);
-  info->api =
-      gst_gl_context_get_current_gl_api (info->platform, &info->major,
-      &info->minor);
+  info->api = gst_gl_context_get_current_gl_api (&info->major, &info->minor);
 }
 
 GST_START_TEST (test_current_context)
@@ -467,165 +442,6 @@ GST_START_TEST (test_current_context)
 
 GST_END_TEST;
 
-GST_START_TEST (test_context_can_share)
-{
-  GstGLContext *c1, *c2, *c3;
-  GError *error = NULL;
-
-  c1 = gst_gl_context_new (display);
-  gst_gl_context_create (c1, NULL, &error);
-  fail_if (error != NULL, "Error creating context %s\n",
-      error ? error->message : "Unknown Error");
-
-  c2 = gst_gl_context_new (display);
-  gst_gl_context_create (c2, c1, &error);
-  fail_if (error != NULL, "Error creating context %s\n",
-      error ? error->message : "Unknown Error");
-
-  fail_unless (gst_gl_context_can_share (c1, c2));
-  fail_unless (gst_gl_context_can_share (c2, c1));
-
-  c3 = gst_gl_context_new (display);
-  gst_gl_context_create (c3, c2, &error);
-  fail_if (error != NULL, "Error creating context %s\n",
-      error ? error->message : "Unknown Error");
-
-  fail_unless (gst_gl_context_can_share (c1, c3));
-  fail_unless (gst_gl_context_can_share (c3, c1));
-  fail_unless (gst_gl_context_can_share (c2, c3));
-  fail_unless (gst_gl_context_can_share (c3, c2));
-
-  /* destroy the middle context */
-  gst_object_unref (c2);
-  c2 = NULL;
-
-  fail_unless (gst_gl_context_can_share (c1, c3));
-  fail_unless (gst_gl_context_can_share (c3, c1));
-
-  gst_object_unref (c1);
-  gst_object_unref (c3);
-}
-
-GST_END_TEST;
-
-GST_START_TEST (test_is_shared)
-{
-  GstGLContext *c1, *c2;
-  GError *error = NULL;
-
-  c1 = gst_gl_context_new (display);
-  gst_gl_context_create (c1, NULL, &error);
-  fail_if (error != NULL, "Error creating context %s\n",
-      error ? error->message : "Unknown Error");
-
-  c2 = gst_gl_context_new (display);
-  gst_gl_context_create (c2, c1, &error);
-  fail_if (error != NULL, "Error creating context %s\n",
-      error ? error->message : "Unknown Error");
-
-  fail_unless (gst_gl_context_is_shared (c1));
-  fail_unless (gst_gl_context_is_shared (c2));
-
-  gst_object_unref (c2);
-  c2 = NULL;
-
-  fail_unless (!gst_gl_context_is_shared (c1));
-
-  gst_object_unref (c1);
-}
-
-GST_END_TEST;
-
-GST_START_TEST (test_display_list)
-{
-  GstGLContext *c1, *c2;
-  GError *error = NULL;
-
-  c1 = gst_gl_context_new (display);
-  gst_gl_context_create (c1, NULL, &error);
-  fail_if (error != NULL, "Error creating context %s\n",
-      error ? error->message : "Unknown Error");
-
-  GST_OBJECT_LOCK (display);
-  {
-    /* no context added so get should return NULL */
-    GstGLContext *tmp =
-        gst_gl_display_get_gl_context_for_thread (display, NULL);
-    fail_unless (tmp == NULL);
-  }
-
-  fail_unless (gst_gl_display_add_context (display, c1));
-  /* re-adding the same context is a no-op */
-  fail_unless (gst_gl_display_add_context (display, c1));
-
-  {
-    GThread *thread;
-    GstGLContext *tmp;
-
-    thread = gst_gl_context_get_thread (c1);
-    fail_unless (thread != NULL);
-
-    tmp = gst_gl_display_get_gl_context_for_thread (display, thread);
-    fail_unless (tmp == c1);
-    g_thread_unref (thread);
-    gst_object_unref (tmp);
-
-    tmp = gst_gl_display_get_gl_context_for_thread (display, NULL);
-    fail_unless (tmp == c1);
-    gst_object_unref (tmp);
-  }
-
-  c2 = gst_gl_context_new (display);
-  gst_gl_context_create (c2, c1, &error);
-  fail_if (error != NULL, "Error creating context %s\n",
-      error ? error->message : "Unknown Error");
-
-  fail_unless (gst_gl_display_add_context (display, c2));
-  /* re-adding the same context is a no-op */
-  fail_unless (gst_gl_display_add_context (display, c2));
-
-  {
-    GThread *thread;
-    GstGLContext *tmp;
-
-    thread = gst_gl_context_get_thread (c2);
-    fail_unless (thread != NULL);
-
-    tmp = gst_gl_display_get_gl_context_for_thread (display, thread);
-    fail_unless (tmp == c2);
-    g_thread_unref (thread);
-    gst_object_unref (tmp);
-
-    /* undefined which context will be returned for the NULL thread */
-    tmp = gst_gl_display_get_gl_context_for_thread (display, NULL);
-    fail_unless (tmp != NULL);
-    gst_object_unref (tmp);
-  }
-
-  gst_object_unref (c1);
-  /* c1 is now dead */
-
-  {
-    GstGLContext *tmp;
-
-    tmp = gst_gl_display_get_gl_context_for_thread (display, NULL);
-    fail_unless (tmp == c2);
-    gst_object_unref (tmp);
-  }
-  GST_OBJECT_UNLOCK (display);
-
-  gst_object_unref (c2);
-  /* c2 is now dead */
-
-  {
-    /* no more contexts alive */
-    GstGLContext *tmp =
-        gst_gl_display_get_gl_context_for_thread (display, NULL);
-    fail_unless (tmp == NULL);
-  }
-}
-
-GST_END_TEST;
 
 static Suite *
 gst_gl_context_suite (void)
@@ -638,9 +454,6 @@ gst_gl_context_suite (void)
   tcase_add_test (tc_chain, test_share);
   tcase_add_test (tc_chain, test_wrapped_context);
   tcase_add_test (tc_chain, test_current_context);
-  tcase_add_test (tc_chain, test_context_can_share);
-  tcase_add_test (tc_chain, test_is_shared);
-  tcase_add_test (tc_chain, test_display_list);
 
   return s;
 }
